@@ -1,6 +1,6 @@
 /*
     pmacct (Promiscuous mode IP Accounting package)
-    pmacct is Copyright (C) 2003-2017 by Paolo Lucente
+    pmacct is Copyright (C) 2003-2019 by Paolo Lucente
 */
 
 /*
@@ -19,10 +19,9 @@
     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 
-#define __ADDR_C
-
 #include "pmacct.h"
 #include "addr.h"
+#include "jhash.h"
 
 static const char hex[] = "0123456789abcdef";
 
@@ -35,18 +34,17 @@ unsigned int str_to_addr(const char *str, struct host_addr *a)
     a->family = AF_INET;
     return a->family;
   }
-#if defined ENABLE_IPV6
+
   if (inet_pton(AF_INET6, str, &a->address.ipv6) > 0) {
     a->family = AF_INET6;
     return a->family;
   }
-#endif
 
-  return 0;
+  return FALSE;
 }
 
 /*
- * addr_to_str() converts a supported family addres into a string
+ * addr_to_str() converts a supported family address into a string
  * 'str' length is not checked and assumed to be INET6_ADDRSTRLEN 
  */
 unsigned int addr_to_str(char *str, const struct host_addr *a)
@@ -55,12 +53,12 @@ unsigned int addr_to_str(char *str, const struct host_addr *a)
     inet_ntop(AF_INET, &a->address.ipv4, str, INET6_ADDRSTRLEN); 
     return a->family;
   }
-#if defined ENABLE_IPV6
+
   if (a->family == AF_INET6) {
     inet_ntop(AF_INET6, &a->address.ipv6, str, INET6_ADDRSTRLEN); 
     return a->family;
   }
-#endif
+
 #if defined ENABLE_PLABEL
   if (a->family == AF_PLABEL) {
     strlcpy(str, a->address.plabel, INET6_ADDRSTRLEN);
@@ -70,7 +68,32 @@ unsigned int addr_to_str(char *str, const struct host_addr *a)
 
   memset(str, 0, INET6_ADDRSTRLEN);
 
-  return 0;
+  return FALSE;
+}
+
+/*
+ * addr_mask_to_str() converts a supported family address into a string
+ */
+unsigned int addr_mask_to_str(char *str, int len, const struct host_addr *a, const struct host_mask *m)
+{
+  char buf[INET6_ADDRSTRLEN];
+
+  if (a->family == m->family) { 
+    if (a->family == AF_INET) {
+      inet_ntop(AF_INET, &a->address.ipv4, buf, sizeof(buf));
+      snprintf(str, len, "%s/%u", buf, m->len);
+      return a->family;
+    }
+    else if (a->family == AF_INET6) {
+      inet_ntop(AF_INET6, &a->address.ipv6, buf, sizeof(buf));
+      snprintf(str, len, "%s/%u", buf, m->len);
+      return a->family;
+    }
+  }
+
+  memset(str, 0, len);
+
+  return FALSE;
 }
 
 /*
@@ -78,13 +101,11 @@ unsigned int addr_to_str(char *str, const struct host_addr *a)
  */
 unsigned int str_to_addr_mask(const char *str, struct host_addr *a, struct host_mask *m)
 {
-  char *delim = NULL, *net = NULL, *mask = NULL;
+  char *delim = NULL, *mask = NULL;
   unsigned int family = 0, index = 0, j;
 
   if (!str || !a || !m) return family;
 
-  net = (char *) str;
-
   delim = strchr(str, '/');
   if (delim) {
     *delim = '\0';
@@ -97,6 +118,8 @@ unsigned int str_to_addr_mask(const char *str, struct host_addr *a, struct host_
   if (family) {
     if (mask) {
       index = atoi(mask);
+      m->len = index;
+
       if (family == AF_INET) {
         if (index > 32) goto error;
         else {
@@ -104,24 +127,26 @@ unsigned int str_to_addr_mask(const char *str, struct host_addr *a, struct host_
 	  a->address.ipv4.s_addr &= m->mask.m4;
         }
       }
-#if defined ENABLE_IPV6
       else if (family == AF_INET6) {
         if (index > 128) goto error;
 
-        for (j = 0; j < 4 && index >= 32; j++, index -= 32) m->mask.m6[j] = 0xffffffffU;
-	if (j < 4 && index) m->mask.m6[j] = htonl(~(0xffffffffU >> index));
+        for (j = 0; j < 16 && index >= 8; j++, index -= 8) m->mask.m6[j] = 0xffU;
+	if (j < 16 && index) m->mask.m6[j] = htonl(~(0xffU >> index));
 
-        for (j = 0; j < 4; j++) a->address.ipv6.s6_addr[j] &= m->mask.m6[j];
+        for (j = 0; j < 16; j++) a->address.ipv6.s6_addr[j] &= m->mask.m6[j];
       }
-#endif
       else goto error;
     }
     /* if no mask: set ipv4 mask to /32 and ipv6 mask to /128 */
     else {
-      if (family == AF_INET) m->mask.m4 = 0xffffffffUL;
-#if defined ENABLE_IPV6
-      else if (family == AF_INET6) for (j = 0; j < 4; j++) m->mask.m6[j] = 0xffffffffU;
-#endif
+      if (family == AF_INET) {
+	m->len = 32;
+	m->mask.m4 = 0xffffffffUL;
+      }
+      else if (family == AF_INET6) {
+	m->len = 128;
+	for (j = 0; j < 16; j++) m->mask.m6[j] = 0xffU;
+      }
       else goto error;
     }
 
@@ -133,7 +158,8 @@ unsigned int str_to_addr_mask(const char *str, struct host_addr *a, struct host_
   error:
   a->family = 0;
   m->family = 0;
-  return 0;
+
+  return FALSE;
 }
 
 /*
@@ -143,9 +169,7 @@ unsigned int str_to_addr_mask(const char *str, struct host_addr *a, struct host_
 unsigned int addr_to_sa(struct sockaddr *sa, struct host_addr *a, u_int16_t port)
 {
   struct sockaddr_in *sa4 = (struct sockaddr_in *)sa;
-#if defined ENABLE_IPV6
   struct sockaddr_in6 *sa6 = (struct sockaddr_in6 *)sa;
-#endif
 
   if (a->family == AF_INET) {
     sa->sa_family = AF_INET;
@@ -153,17 +177,17 @@ unsigned int addr_to_sa(struct sockaddr *sa, struct host_addr *a, u_int16_t port
     sa4->sin_port = htons(port);
     return sizeof(struct sockaddr_in);
   }
-#if defined ENABLE_IPV6
+
   if (a->family == AF_INET6) {
     sa->sa_family = AF_INET6;
     ip6_addr_cpy(&sa6->sin6_addr, &a->address.ipv6);
     sa6->sin6_port = htons(port);
     return sizeof(struct sockaddr_in6); 
   }
-#endif
 
   memset(sa, 0, sizeof(struct sockaddr));
-  return 0;
+
+  return FALSE;
 }
 
 /*
@@ -173,9 +197,7 @@ unsigned int addr_to_sa(struct sockaddr *sa, struct host_addr *a, u_int16_t port
 unsigned int sa_to_addr(struct sockaddr *sa, struct host_addr *a, u_int16_t *port)
 {
   struct sockaddr_in *sa4 = (struct sockaddr_in *)sa;
-#if defined ENABLE_IPV6
   struct sockaddr_in6 *sa6 = (struct sockaddr_in6 *)sa;
-#endif
   
   if (sa->sa_family == AF_INET) {
     a->family = AF_INET;
@@ -183,59 +205,55 @@ unsigned int sa_to_addr(struct sockaddr *sa, struct host_addr *a, u_int16_t *por
     *port = ntohs(sa4->sin_port);
     return sizeof(struct sockaddr_in);
   }
-#if defined ENABLE_IPV6
+
   if (sa->sa_family == AF_INET6) {
     a->family = AF_INET6;
     ip6_addr_cpy(&a->address.ipv6, &sa6->sin6_addr);
     *port = ntohs(sa6->sin6_port);
     return sizeof(struct sockaddr_in6);
   }
-#endif
 
   memset(a, 0, sizeof(struct host_addr));
-  return 0;
+
+  return FALSE;
 }
 
 /*
- * sa_addr_cmp(): compare two IP addresses: the first encapsulated into a
- * 'struct sockaddr' and the second into a 'struct host_addr'.
- * returns 0 if they match; 1 if they don't match; -1 to signal a generic
- * error (e.g. unsupported family mismatch).
+ * sa_addr_cmp(): compare two IP addresses: the first encapsulated
+ * into a 'struct sockaddr' and the second into a 'struct host_addr'.
+ * returns 0 if they match, 1 if a1 is found greater than a2; -1 on
+ * the contrary. -1 is also used to flag a generic error (ie. family
+ * not supported).
  */
 int sa_addr_cmp(struct sockaddr *sa, struct host_addr *a)
 {
   struct sockaddr_in *sa4 = (struct sockaddr_in *)sa;
-#if defined ENABLE_IPV6
   struct sockaddr_in6 *sa6 = (struct sockaddr_in6 *)sa;
   struct sockaddr_in6 sa6_local;
-#endif
 
   if (a->family == AF_INET && sa->sa_family == AF_INET) {
     if (sa4->sin_addr.s_addr == a->address.ipv4.s_addr) return FALSE;
-    else return TRUE; 
+    else if (sa4->sin_addr.s_addr > a->address.ipv4.s_addr) return TRUE;
+    else return ERR;
   }
-#if defined ENABLE_IPV6
+
   if (a->family == AF_INET6 && sa->sa_family == AF_INET6) {
-    if (!ip6_addr_cmp(&sa6->sin6_addr, &a->address.ipv6)) return FALSE;
-    else return TRUE;
+    return ip6_addr_cmp(&sa6->sin6_addr, &a->address.ipv6);
   }
   else if (a->family == AF_INET && sa->sa_family == AF_INET6) {
     memset(&sa6_local, 0, sizeof(sa6_local));
     memset((u_int8_t *)&sa6_local.sin6_addr+10, 0xff, 2);
     memcpy((u_int8_t *)&sa6_local.sin6_addr+12, &a->address.ipv4.s_addr, 4);
-    if (!ip6_addr_cmp(&sa6->sin6_addr, &sa6_local.sin6_addr)) return FALSE;
-    else return TRUE;
+    return ip6_addr_cmp(&sa6->sin6_addr, &sa6_local.sin6_addr);
   }
   else if (a->family == AF_INET6 && sa->sa_family == AF_INET) {
     memset(&sa6_local, 0, sizeof(sa6_local));
     memset((u_int8_t *)&sa6_local.sin6_addr+10, 0xff, 2);
     memcpy((u_int8_t *)&sa6_local.sin6_addr+12, &sa4->sin_addr, 4);
-    if (!ip6_addr_cmp(&sa6_local.sin6_addr, &a->address.ipv6)) return FALSE;
-    else return TRUE;
+    return ip6_addr_cmp(&sa6_local.sin6_addr, &a->address.ipv6);
   }
-#endif
 
-  return -1;
+  return ERR;
 }
 
 /*
@@ -247,69 +265,160 @@ int sa_addr_cmp(struct sockaddr *sa, struct host_addr *a)
 int sa_port_cmp(struct sockaddr *sa, u_int16_t port)
 {
   struct sockaddr_in *sa4 = (struct sockaddr_in *)sa;
-#if defined ENABLE_IPV6
   struct sockaddr_in6 *sa6 = (struct sockaddr_in6 *)sa;
-  struct sockaddr_in6 sa6_local;
-#endif
 
   if (sa->sa_family == AF_INET) {
     if (sa4->sin_port == port) return FALSE;
     else return TRUE;
   }
-#if defined ENABLE_IPV6
+
   if (sa->sa_family == AF_INET6) {
     if (sa6->sin6_port == port) return FALSE;
     else return TRUE;
   }
-#endif
 
-  return -1;
+  return ERR;
 }
 
 /*
- * host_addr_mask_cmp() checks whether s1 falls in a1/m1
+ * host_addr_cmp() compares two IP addresses in a host_addr structure
+ * returns 0 if they match, 1 if a1 is found greater than a2; -1 on
+ * the contrary. -1 is also used to flag a generic error (ie. family
+ * not supported).
+ */
+int host_addr_cmp(struct host_addr *a1, struct host_addr *a2)
+{
+  struct host_addr ha_local;
+
+  if (a1->family == AF_INET && a2->family == AF_INET) {
+    if (a1->address.ipv4.s_addr == a2->address.ipv4.s_addr) return FALSE;
+    else if (a1->address.ipv4.s_addr > a2->address.ipv4.s_addr) return TRUE;
+    else return ERR;
+  }
+
+  if (a1->family == AF_INET6 && a2->family == AF_INET6) {
+    return ip6_addr_cmp(&a1->address.ipv6, &a2->address.ipv6);
+  }
+  else if (a1->family == AF_INET && a2->family == AF_INET6) {
+    memset(&ha_local, 0, sizeof(ha_local));
+    memset((u_int8_t *)&ha_local.address.ipv6.s6_addr[10], 0xff, 2);
+    memcpy((u_int8_t *)&ha_local.address.ipv6.s6_addr[12], &a1->address.ipv4.s_addr, 4);
+    return ip6_addr_cmp(&a2->address.ipv6, &ha_local.address.ipv6);
+  }
+  else if (a1->family == AF_INET6 && a2->family == AF_INET) {
+    memset(&ha_local, 0, sizeof(ha_local));
+    memset((u_int8_t *)&ha_local.address.ipv6.s6_addr[10], 0xff, 2);
+    memcpy((u_int8_t *)&ha_local.address.ipv6.s6_addr[12], &a2->address.ipv4.s_addr, 4);
+    return ip6_addr_cmp(&a1->address.ipv6, &ha_local.address.ipv6);
+  }
+
+  return ERR;
+}
+
+/*
+ * variant of host_addr_cmp(). In addition it returns FALSE if family
+ * is zero in both a1 and a2.  
+*/
+int host_addr_cmp2(struct host_addr *a1, struct host_addr *a2)
+{
+  struct host_addr ha_local;
+
+  if (a1->family == AF_INET && a2->family == AF_INET) {
+    if (a1->address.ipv4.s_addr == a2->address.ipv4.s_addr) return FALSE;
+    else if (a1->address.ipv4.s_addr > a2->address.ipv4.s_addr) return TRUE;
+    else return ERR;
+  }
+
+  if (a1->family == AF_INET6 && a2->family == AF_INET6) {
+    return ip6_addr_cmp(&a1->address.ipv6, &a2->address.ipv6);
+  }
+  else if (a1->family == AF_INET && a2->family == AF_INET6) {
+    if (a2->address.ipv6.s6_addr[10] == 0xff && a2->address.ipv6.s6_addr[11] == 0xff) {
+      memset(&ha_local, 0, sizeof(ha_local));
+      memset((u_int8_t *)&ha_local.address.ipv6.s6_addr[10], 0xff, 2);
+      memcpy((u_int8_t *)&ha_local.address.ipv6.s6_addr[12], &a1->address.ipv4.s_addr, 4);
+      return ip6_addr_cmp(&a2->address.ipv6, &ha_local.address.ipv6);
+    }
+  }
+  else if (a1->family == AF_INET6 && a2->family == AF_INET) {
+    if (a1->address.ipv6.s6_addr[10] == 0xff && a1->address.ipv6.s6_addr[11] == 0xff) {
+      memset(&ha_local, 0, sizeof(ha_local));
+      memset((u_int8_t *)&ha_local.address.ipv6.s6_addr[10], 0xff, 2);
+      memcpy((u_int8_t *)&ha_local.address.ipv6.s6_addr[12], &a2->address.ipv4.s_addr, 4);
+      return ip6_addr_cmp(&a1->address.ipv6, &ha_local.address.ipv6);
+    }
+  }
+
+  if (!a1->family && !a2->family) return FALSE;
+
+  return ERR;
+}
+
+/*
+ * host_addr_mask_sa_cmp() checks whether s1 falls in a1/m1
  * returns 0 if positive; 1 if negative; -1 to signal a generic error
  * (e.g. unsupported family).
  */
 int host_addr_mask_sa_cmp(struct host_addr *a1, struct host_mask *m1, struct sockaddr *s1)
 {
   struct sockaddr_in *sa4 = (struct sockaddr_in *)s1;
-#if defined ENABLE_IPV6
-  struct sockaddr_in6 *sa6 = (struct sockaddr_in6 *)s1;
   struct sockaddr_in6 sa6_local;
   int ret, j;
-#endif
 
-  if (!a1 || !m1 || !s1) return -1;
-  if (a1->family != s1->sa_family || a1->family != m1->family) return -1;
+  if (!a1 || !m1 || !s1) return ERR;
+  if (a1->family != s1->sa_family || a1->family != m1->family) return ERR;
 
   if (a1->family == AF_INET) {
-    if ((sa4->sin_addr.s_addr & m1->mask.m4) == a1->address.ipv4.s_addr) return 0;
-    else return 1;
+    if ((sa4->sin_addr.s_addr & m1->mask.m4) == a1->address.ipv4.s_addr) return FALSE;
+    else return TRUE;
   }
-#if defined ENABLE_IPV6
   else if (a1->family == AF_INET6) {
     memcpy(&sa6_local, s1, sizeof(struct sockaddr));
-    for (j = 0; j < 4; j++) sa6_local.sin6_addr.s6_addr[j] &= m1->mask.m6[j];
+    for (j = 0; j < 16; j++) sa6_local.sin6_addr.s6_addr[j] &= m1->mask.m6[j];
     ret = ip6_addr_cmp(a1, &sa6_local.sin6_addr);
-    if (!ret) return 0;
-    else return 1;
+    if (!ret) return FALSE;
+    else return TRUE;
   }
-#endif
 
-  return -1;
+  return ERR;
+}
+
+/*
+ * host_addr_mask_cmp() checks whether a2 falls in a1/m1
+ * returns 0 if positive; 1 if negative; -1 to signal a generic error
+ * (e.g. unsupported family).
+ */
+int host_addr_mask_cmp(struct host_addr *a1, struct host_mask *m1, struct host_addr *a2)
+{
+  struct host_addr ha_local;
+  int ret, j;
+
+  if (!a1 || !m1 || !a2) return ERR;
+  if (a1->family != a2->family || a1->family != m1->family) return ERR;
+
+  if (a1->family == AF_INET) {
+    if ((a2->address.ipv4.s_addr & m1->mask.m4) == a1->address.ipv4.s_addr) return FALSE;
+    else return TRUE;
+  }
+  else if (a1->family == AF_INET6) {
+    memcpy(&ha_local, a2, sizeof(struct host_addr));
+    for (j = 0; j < 16; j++) ha_local.address.ipv6.s6_addr[j] &= m1->mask.m6[j];
+    ret = ip6_addr_cmp(a1, &ha_local.address.ipv6);
+    if (!ret) return FALSE;
+    else return TRUE;
+  }
+
+  return ERR;
 }
 
 /*
  * raw_to_sa() converts a supported family address into a sockaddr 
  * structure 
  */
-unsigned int raw_to_sa(struct sockaddr *sa, char *src, u_int16_t port, u_int8_t v4v6)
+unsigned int raw_to_sa(struct sockaddr *sa, u_char *src, u_int16_t port, u_int8_t v4v6)
 {
   struct sockaddr_in *sa4 = (struct sockaddr_in *)sa;
-#if defined ENABLE_IPV6
   struct sockaddr_in6 *sa6 = (struct sockaddr_in6 *)sa;
-#endif
 
   if (v4v6 == AF_INET) {
     sa->sa_family = AF_INET;
@@ -317,44 +426,85 @@ unsigned int raw_to_sa(struct sockaddr *sa, char *src, u_int16_t port, u_int8_t
     sa4->sin_port = port;
     return sizeof(struct sockaddr_in);
   }
-#if defined ENABLE_IPV6
+
   if (v4v6 == AF_INET6) {
     sa->sa_family = AF_INET6;
     ip6_addr_cpy(&sa6->sin6_addr, src);
     sa6->sin6_port = port;
     return sizeof(struct sockaddr_in6);
   }
-#endif
 
   memset(sa, 0, sizeof(struct sockaddr));
-  return 0;
+
+  return FALSE;
 }
 
 /*
- * sa_to_str() converts a supported family addres into a string
- * 'str' length is not checked and assumed to be INET6_ADDRSTRLEN 
+ * raw_to_addr() converts a supported family address into a host_addr 
+ * structure 
  */
-unsigned int sa_to_str(char *str, const struct sockaddr *sa)
+unsigned int raw_to_addr(struct host_addr *ha, u_char *src, u_int8_t v4v6)
+{
+  if (v4v6 == AF_INET) {
+    ha->family = AF_INET;
+    memcpy(&ha->address.ipv4, src, 4);
+    return ha->family;
+  }
+
+  if (v4v6 == AF_INET6) {
+    ha->family = AF_INET6;
+    ip6_addr_cpy(&ha->address.ipv6, src);
+    return ha->family;
+  }
+
+  memset(ha, 0, sizeof(struct host_addr));
+
+  return FALSE;
+}
+
+/*
+ * sa_to_str() converts a supported family address into a string
+ */
+unsigned int sa_to_str(char *str, int len, const struct sockaddr *sa)
 {
   struct sockaddr_in *sa4 = (struct sockaddr_in *)sa;
-#if defined ENABLE_IPV6
   struct sockaddr_in6 *sa6 = (struct sockaddr_in6 *)sa;
-#endif
+  char sep[] = ":";
+  int off;
 
-  if (sa->sa_family == AF_INET) {
-    inet_ntop(AF_INET, &sa4->sin_addr.s_addr, str, INET6_ADDRSTRLEN);
-    return sa->sa_family;
+  if (len >= INET6_ADDRSTRLEN) {
+    if (sa->sa_family == AF_INET) {
+      inet_ntop(AF_INET, &sa4->sin_addr.s_addr, str, INET6_ADDRSTRLEN);
+
+      if (len >= (INET6_ADDRSTRLEN + PORT_STRLEN + 1) && sa4->sin_port) {
+	off = strlen(str);
+	snprintf(str + off, len - off, "%s", sep);
+
+	off = strlen(str);
+	snprintf(str + off, len - off, "%u", ntohs(sa4->sin_port));
+      }
+
+      return sa->sa_family;
+    }
+
+    if (sa->sa_family == AF_INET6) {
+      inet_ntop(AF_INET6, &sa6->sin6_addr, str, INET6_ADDRSTRLEN);
+
+      if (sa6->sin6_port) {
+        off = strlen(str);
+        snprintf(str + off, len - off, "%s", sep);
+
+        off = strlen(str);
+        snprintf(str + off, len - off, "%u", ntohs(sa6->sin6_port));
+      }
+
+      return sa->sa_family;
+    }
   }
-#if defined ENABLE_IPV6
-  if (sa->sa_family == AF_INET6) {
-    inet_ntop(AF_INET6, &sa6->sin6_addr, str, INET6_ADDRSTRLEN);
-    return sa->sa_family;
-  }
-#endif
 
-  memset(str, 0, INET6_ADDRSTRLEN);
+  memset(str, 0, len);
 
-  return 0;
+  return FALSE;
 }
 
 /*
@@ -401,7 +551,7 @@ int ip6_addr_cmp(void *addr1, void *addr2)
     if (ptr1[chunk] == ptr2[chunk]) continue;
     else {
       if (ptr1[chunk] > ptr2[chunk]) return TRUE;
-      else return -1; 
+      else return ERR; 
     }
   }
 
@@ -463,7 +613,7 @@ void etheraddr_string(const u_char *ep, char *buf)
  * to be ETH_ADDR_LEN long). TRUE is returned if any failure occurs;
  * TRUE if the routine completes the job successfully 
  */
-int string_etheraddr(const u_char *asc, char *addr)
+int string_etheraddr(const char *asc, u_char *addr)
 {
   int cnt;
 
@@ -473,19 +623,19 @@ int string_etheraddr(const u_char *asc, char *addr)
 
     ch = tolower (*asc++);
     if ((ch < '0' || ch > '9') && (ch < 'a' || ch > 'f'))
-      return 1;
+      return TRUE;
     number = isdigit (ch) ? (ch - '0') : (ch - 'a' + 10);
 
     ch = tolower(*asc);
     if ((cnt < 5 && ch != ':') || (cnt == 5 && ch != '\0' && !isspace (ch))) {
       ++asc;
       if ((ch < '0' || ch > '9') && (ch < 'a' || ch > 'f'))
-        return 1;
+        return TRUE;
       number <<= 4;
       number += isdigit (ch) ? (ch - '0') : (ch - 'a' + 10);
       ch = *asc;
       if (cnt < 5 && ch != ':')
-        return 1;
+        return TRUE;
     }
 
     /* Store result.  */
@@ -549,12 +699,11 @@ int is_multicast(struct host_addr *a)
     if (IS_IPV4_MULTICAST(a->address.ipv4.s_addr)) return a->family;
     else return FALSE;
   }
-#if defined ENABLE_IPV6
+
   if (a->family == AF_INET6) {
     if (IS_IPV6_MULTICAST(&a->address.ipv6)) return a->family;
     else return FALSE;
   }
-#endif
 
   return FALSE;
 }
@@ -575,12 +724,11 @@ int is_any(struct host_addr *a)
     if (!memcmp(&empty_host_addr.address.ipv4, &a->address.ipv4, 4)) return a->family;
     else return FALSE;
   }
-#if defined ENABLE_IPV6
+
   if (a->family == AF_INET6) {
     if (!memcmp(&empty_host_addr.address.ipv6, &a->address.ipv6, 16)) return a->family;
     else return FALSE;
   }
-#endif
 
   return FALSE;
 }
@@ -591,14 +739,10 @@ int is_any(struct host_addr *a)
 void clean_sin_addr(struct sockaddr *sa)
 {
   struct sockaddr_in *sa4 = (struct sockaddr_in *)sa;
-#if defined ENABLE_IPV6
   struct sockaddr_in6 *sa6 = (struct sockaddr_in6 *)sa;
-#endif
 
   if (sa->sa_family == AF_INET) sa4->sin_addr.s_addr = 0;
-#if defined ENABLE_IPV6
   if (sa->sa_family == AF_INET6) memset(&sa6->sin6_addr, 0, 16);
-#endif
 }
 
 #if defined ENABLE_PLABEL
@@ -610,14 +754,13 @@ unsigned int label_to_addr(const char *label, struct host_addr *a, int len)
   strlcpy(a->address.plabel, label, len);
   a->family = AF_PLABEL;
 
-  return 0;
+  return FALSE;
 }
 #endif
 
 /*
  * ipv4_mapped_to_ipv4() converts a label into a supported family address
  */
-#if defined ENABLE_IPV6
 void ipv4_mapped_to_ipv4(struct sockaddr_storage *sas)
 {
   struct sockaddr_storage sas_local;
@@ -653,14 +796,11 @@ void ipv4_to_ipv4_mapped(struct sockaddr_storage *sas)
   memcpy((u_int8_t *) &sa6->sin6_addr+12, &sa4->sin_addr, 4);
   sa6->sin6_port = sa4->sin_port;
 }
-#endif
 
 u_int8_t etype_to_af(u_int16_t etype)
 {
   if (etype == ETHERTYPE_IP) return AF_INET;
-#if defined ENABLE_IPV6
   else if (etype == ETHERTYPE_IPV6) return AF_INET6;
-#endif
 
   return FALSE;
 }
@@ -668,9 +808,50 @@ u_int8_t etype_to_af(u_int16_t etype)
 u_int16_t af_to_etype(u_int8_t af)
 {
   if (af == AF_INET) return ETHERTYPE_IP;
-#if defined ENABLE_IPV6
   else if (af == AF_INET6) return ETHERTYPE_IPV6;
-#endif
 
   return FALSE;
 }
+
+u_int32_t addr_hash(struct host_addr *ha, u_int32_t modulo)
+{
+  u_int32_t val = 0;
+
+  if (ha->family == AF_INET) {
+    val = jhash_1word(ha->address.ipv4.s_addr, 0);
+  }
+  else if (ha->family == AF_INET6) {
+    u_int32_t a, b, c;
+
+    memcpy(&a, &ha->address.ipv6.s6_addr[4], 4);
+    memcpy(&b, &ha->address.ipv6.s6_addr[8], 4);
+    memcpy(&c, &ha->address.ipv6.s6_addr[12], 4);
+    val = jhash_3words(a, b, c, 0);
+  }
+
+  return (val % modulo);
+}
+
+// XXX: basic implementation, to be improved
+u_int32_t addr_port_hash(struct host_addr *ha, u_int16_t port, u_int32_t modulo)
+{
+  u_int32_t val = 0;
+
+  if (ha->family == AF_INET) {
+    val = jhash_2words(ha->address.ipv4.s_addr, port, 0);
+  }
+  else if (ha->family == AF_INET6) {
+    u_int32_t a, b;
+
+    memcpy(&a, &ha->address.ipv6.s6_addr[8], 4);
+    memcpy(&b, &ha->address.ipv6.s6_addr[12], 4);
+    val = jhash_3words(port, a, b, 0);
+  }
+
+  return (val % modulo);
+}
+
+u_int16_t sa_has_family(struct sockaddr *sa)
+{
+  return sa->sa_family;
+}
