@@ -1,6 +1,6 @@
 /*
     pmacct (Promiscuous mode IP Accounting package)
-    pmacct is Copyright (C) 2003-2017 by Paolo Lucente
+    pmacct is Copyright (C) 2003-2019 by Paolo Lucente
 */
 
 /*
@@ -19,8 +19,6 @@
     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 
-#define __AMQP_PLUGIN_C
-
 /* includes */
 #include "pmacct.h"
 #include "pmacct-data.h"
@@ -33,6 +31,8 @@
 #ifndef WITH_JANSSON
 #error "--enable-rabbitmq requires --enable-jansson"
 #endif
+#include "net_aggr.h"
+#include "ports_aggr.h"
 
 /* Functions */
 void amqp_plugin(int pipe_fd, struct configuration *cfgptr, void *ptr) 
@@ -42,12 +42,11 @@ void amqp_plugin(int pipe_fd, struct configuration *cfgptr, void *ptr)
   unsigned char *pipebuf;
   struct pollfd pfd;
   struct insert_data idata;
-  time_t t, avro_schema_deadline = 0;
-  int timeout, refresh_timeout, amqp_timeout = 0, avro_schema_timeout = 0;
-  int ret, num; 
+  time_t avro_schema_deadline = 0;
+  int timeout, refresh_timeout, avro_schema_timeout = 0;
+  int ret, num, recv_budget, poll_bypass;
   struct ring *rg = &((struct channels_list_entry *)ptr)->rg;
   struct ch_status *status = ((struct channels_list_entry *)ptr)->status;
-  struct plugins_list_entry *plugin_data = ((struct channels_list_entry *)ptr)->plugin;
   int datasize = ((struct channels_list_entry *)ptr)->datasize;
   u_int32_t bufsz = ((struct channels_list_entry *)ptr)->bufsize;
   pid_t core_pid = ((struct channels_list_entry *)ptr)->core_pid;
@@ -59,14 +58,18 @@ void amqp_plugin(int pipe_fd, struct configuration *cfgptr, void *ptr)
 
   struct extra_primitives extras;
   struct primitives_ptrs prim_ptrs;
-  char *dataptr;
+  unsigned char *dataptr;
 
 #ifdef WITH_AVRO
-  char *avro_acct_schema_str;
+  char *avro_acct_data_schema_str = NULL;
+  char *avro_acct_init_schema_str = NULL;
+  char *avro_acct_close_schema_str = NULL;
 #endif
 
 #ifdef WITH_ZMQ
   struct p_zmq_host *zmq_host = &((struct channels_list_entry *)ptr)->zmq_host;
+#else
+  void *zmq_host = NULL;
 #endif
 
   memcpy(&config, cfgptr, sizeof(struct configuration));
@@ -90,38 +93,58 @@ void amqp_plugin(int pipe_fd, struct configuration *cfgptr, void *ptr)
   if (config.message_broker_output & PRINT_OUTPUT_JSON) {
     compose_json(config.what_to_count, config.what_to_count_2);
   }
-  else if (config.message_broker_output & PRINT_OUTPUT_AVRO) {
+  else if ((config.message_broker_output & PRINT_OUTPUT_AVRO_BIN) ||
+	   (config.message_broker_output & PRINT_OUTPUT_AVRO_JSON)) {
 #ifdef WITH_AVRO
-    avro_acct_schema = build_avro_schema(config.what_to_count, config.what_to_count_2);
+    avro_acct_schema = avro_schema_build_acct_data(config.what_to_count, config.what_to_count_2);
     avro_schema_add_writer_id(avro_acct_schema);
 
-    if (config.avro_schema_output_file) write_avro_schema_to_file(config.avro_schema_output_file, avro_acct_schema);
+    avro_acct_init_schema = avro_schema_build_acct_init();
+    avro_acct_close_schema = avro_schema_build_acct_close();
+
+    if (config.avro_schema_file) {
+      char avro_schema_file[SRVBUFLEN];
+
+      if (strlen(config.avro_schema_file) > (SRVBUFLEN - SUPERSHORTBUFLEN)) {
+	Log(LOG_ERR, "ERROR ( %s/%s ): 'avro_schema_file' too long. Exiting.\n", config.name, config.type);
+	exit_gracefully(1);
+      }
+
+      write_avro_schema_to_file_with_suffix(config.avro_schema_file, "-acct_data", avro_schema_file, avro_acct_schema);
+      write_avro_schema_to_file_with_suffix(config.avro_schema_file, "-acct_init", avro_schema_file, avro_acct_init_schema);
+      write_avro_schema_to_file_with_suffix(config.avro_schema_file, "-acct_close", avro_schema_file, avro_acct_close_schema);
+    }
 
     if (config.amqp_avro_schema_routing_key) {
-      if (!config.amqp_avro_schema_refresh_time)
+      if (!config.amqp_avro_schema_refresh_time) {
         config.amqp_avro_schema_refresh_time = DEFAULT_AVRO_SCHEMA_REFRESH_TIME;
+      }
 
       avro_schema_deadline = time(NULL);
       P_init_refresh_deadline(&avro_schema_deadline, config.amqp_avro_schema_refresh_time, 0, "m");
-      avro_acct_schema_str = compose_avro_purge_schema(avro_acct_schema, config.name);
+      avro_acct_data_schema_str = compose_avro_purge_schema(avro_acct_schema, config.name);
+      avro_acct_init_schema_str = compose_avro_purge_schema(avro_acct_init_schema, config.name);
+      avro_acct_close_schema_str = compose_avro_purge_schema(avro_acct_close_schema, config.name);
     }
     else {
       config.amqp_avro_schema_refresh_time = 0;
       avro_schema_deadline = 0;
       avro_schema_timeout = 0;
-      avro_acct_schema_str = NULL;
+      avro_acct_data_schema_str = NULL;
+      avro_acct_init_schema_str = NULL;
+      avro_acct_close_schema_str = NULL;
     }
 #endif
   }
 
   if ((config.sql_table && strchr(config.sql_table, '$')) && config.sql_multi_values) {
     Log(LOG_ERR, "ERROR ( %s/%s ): dynamic 'amqp_routing_key' is not compatible with 'amqp_multi_values'. Exiting.\n", config.name, config.type);
-    exit_plugin(1);
+    exit_gracefully(1);
   }
 
   if ((config.sql_table && strchr(config.sql_table, '$')) && config.amqp_routing_key_rr) {
     Log(LOG_ERR, "ERROR ( %s/%s ): dynamic 'amqp_routing_key' is not compatible with 'amqp_routing_key_rr'. Exiting.\n", config.name, config.type);
-    exit_plugin(1);
+    exit_gracefully(1);
   }
 
   p_amqp_init_host(&amqpp_amqp_host);
@@ -147,28 +170,12 @@ void amqp_plugin(int pipe_fd, struct configuration *cfgptr, void *ptr)
   set_net_funcs(&nt);
 
   if (config.ports_file) load_ports(config.ports_file, &pt);
-  if (config.pkt_len_distrib_bins_str) load_pkt_len_distrib_bins();
-  else {
-    if (config.what_to_count_2 & COUNT_PKT_LEN_DISTRIB) {
-      Log(LOG_ERR, "ERROR ( %s/%s ): 'aggregate' contains pkt_len_distrib but no 'pkt_len_distrib_bins' defined. Exiting.\n", config.name, config.type);
-      exit_plugin(1);
-    }
-  }
   
   memset(&idata, 0, sizeof(idata));
   memset(&prim_ptrs, 0, sizeof(prim_ptrs));
   set_primptrs_funcs(&extras);
 
-  if (config.pipe_zmq) {
-    plugin_pipe_zmq_compile_check();
-#ifdef WITH_ZMQ
-    p_zmq_plugin_pipe_init_plugin(zmq_host);
-    p_zmq_plugin_pipe_consume(zmq_host);
-    p_zmq_set_retry_timeout(zmq_host, config.pipe_zmq_retry);
-    pipe_fd = p_zmq_get_fd(zmq_host);
-    seq = 0;
-#endif
-  }
+  if (config.pipe_zmq) P_zmq_pipe_init(zmq_host, &pipe_fd, &seq);
   else setnonblocking(pipe_fd);
 
   idata.now = time(NULL);
@@ -192,6 +199,8 @@ void amqp_plugin(int pipe_fd, struct configuration *cfgptr, void *ptr)
   for(;;) {
     poll_again:
     status->wakeup = TRUE;
+    poll_bypass = FALSE;
+
     calc_refresh_timeout(refresh_deadline, idata.now, &refresh_timeout);
     if (config.amqp_avro_schema_routing_key) calc_refresh_timeout(avro_schema_deadline, idata.now, &avro_schema_timeout);
 
@@ -201,38 +210,45 @@ void amqp_plugin(int pipe_fd, struct configuration *cfgptr, void *ptr)
     ret = poll(&pfd, (pfd.fd == ERR ? 0 : 1), timeout);
 
     if (ret <= 0) {
-      if (getppid() == 1) {
+      if (getppid() != core_pid) {
         Log(LOG_ERR, "ERROR ( %s/%s ): Core process *seems* gone. Exiting.\n", config.name, config.type);
-        exit_plugin(1);
+        exit_gracefully(1);
       }
 
       if (ret < 0) goto poll_again;
     }
 
-    idata.now = time(NULL);
+    poll_ops:
+    P_update_time_reference(&idata);
 
-    if (config.sql_history) {
-      while (idata.now > (basetime.tv_sec + timeslot)) {
-	new_basetime.tv_sec = basetime.tv_sec;
-        basetime.tv_sec += timeslot;
-        if (config.sql_history == COUNT_MONTHLY)
-          timeslot = calc_monthly_timeslot(basetime.tv_sec, config.sql_history_howmany, ADD);
-      }
-    }
+    if (idata.now > refresh_deadline) P_cache_handle_flush_event(&pt);
 
 #ifdef WITH_AVRO
     if (idata.now > avro_schema_deadline) {
-      amqp_avro_schema_purge(avro_acct_schema_str);
+      amqp_avro_schema_purge(avro_acct_data_schema_str);
+      amqp_avro_schema_purge(avro_acct_init_schema_str);
+      amqp_avro_schema_purge(avro_acct_close_schema_str);
+
       avro_schema_deadline += config.amqp_avro_schema_refresh_time;
     }
 #endif
 
+    recv_budget = 0;
+    if (poll_bypass) {
+      poll_bypass = FALSE;
+      goto read_data;
+    }
+
     switch (ret) {
     case 0: /* timeout */
-      if (idata.now > refresh_deadline) P_cache_handle_flush_event(&pt);
       break;
     default: /* we received data */
       read_data:
+      if (recv_budget == DEFAULT_PLUGIN_COMMON_RECV_BUDGET) {
+	poll_bypass = TRUE;
+	goto poll_ops;
+      }
+
       if (config.pipe_homegrown) {
         if (!pollagain) {
           seq++;
@@ -241,7 +257,7 @@ void amqp_plugin(int pipe_fd, struct configuration *cfgptr, void *ptr)
         }
         else {
           if ((ret = read(pipe_fd, &rgptr, sizeof(rgptr))) == 0) 
-	    exit_plugin(1); /* we exit silently; something happened at the write end */
+	    exit_gracefully(1); /* we exit silently; something happened at the write end */
         }
 
         if ((rg->ptr + bufsz) > rg->end) rg->ptr = rg->base;
@@ -254,7 +270,7 @@ void amqp_plugin(int pipe_fd, struct configuration *cfgptr, void *ptr)
           else {
             rg_err_count++;
             if (config.debug || (rg_err_count > MAX_RG_COUNT_ERR)) {
-              Log(LOG_WARNING, "WARN ( %s/%s ): Missing data detected (plugin_buffer_size=%llu plugin_pipe_size=%llu).\n",
+              Log(LOG_WARNING, "WARN ( %s/%s ): Missing data detected (plugin_buffer_size=%" PRIu64 " plugin_pipe_size=%" PRIu64 ").\n",
 			config.name, config.type, config.buffer_size, config.pipe_size);
               Log(LOG_WARNING, "WARN ( %s/%s ): Increase values or look for plugin_buffer_size, plugin_pipe_size in CONFIG-KEYS document.\n\n",
 			config.name, config.type);
@@ -271,7 +287,7 @@ void amqp_plugin(int pipe_fd, struct configuration *cfgptr, void *ptr)
       }
 #ifdef WITH_ZMQ
       else if (config.pipe_zmq) {
-	ret = p_zmq_plugin_pipe_recv(zmq_host, pipebuf, config.buffer_size);
+	ret = p_zmq_topic_recv(zmq_host, pipebuf, config.buffer_size);
 	if (ret > 0) {
 	  if (seq && (((struct ch_buf_hdr *)pipebuf)->seq != ((seq + 1) % MAX_SEQNUM))) {
 	    Log(LOG_WARNING, "WARN ( %s/%s ): Missing data detected. Sequence received=%u expected=%u\n",
@@ -284,17 +300,13 @@ void amqp_plugin(int pipe_fd, struct configuration *cfgptr, void *ptr)
       }
 #endif
 
-      /* lazy refresh time handling */ 
-      if (idata.now > refresh_deadline) P_cache_handle_flush_event(&pt);
-
       data = (struct pkt_data *) (pipebuf+sizeof(struct ch_buf_hdr));
 
       if (config.debug_internal_msg)
-        Log(LOG_DEBUG, "DEBUG ( %s/%s ): buffer received cpid=%u len=%llu seq=%u num_entries=%u\n",
-                config.name, config.type, core_pid, ((struct ch_buf_hdr *)pipebuf)->len,
-                seq, ((struct ch_buf_hdr *)pipebuf)->num);
+        Log(LOG_DEBUG, "DEBUG ( %s/%s ): buffer received len=%" PRIu64 " seq=%u num_entries=%u\n",
+                config.name, config.type, ((struct ch_buf_hdr *)pipebuf)->len, seq,
+                ((struct ch_buf_hdr *)pipebuf)->num);
 
-      if (!config.pipe_check_core_pid || ((struct ch_buf_hdr *)pipebuf)->core_pid == core_pid) {
       while (((struct ch_buf_hdr *)pipebuf)->num > 0) {
         for (num = 0; primptrs_funcs[num]; num++)
           (*primptrs_funcs[num])((u_char *)data, &extras, &prim_ptrs);
@@ -307,10 +319,6 @@ void amqp_plugin(int pipe_fd, struct configuration *cfgptr, void *ptr)
           if (!pt.table[data->primitives.dst_port]) data->primitives.dst_port = 0;
         }
 
-        if (config.pkt_len_distrib_bins_str &&
-            config.what_to_count_2 & COUNT_PKT_LEN_DISTRIB)
-          evaluate_pkt_len_distrib(data);
-
         prim_ptrs.data = data;
         (*insert_func)(&prim_ptrs, &idata);
 
@@ -322,8 +330,8 @@ void amqp_plugin(int pipe_fd, struct configuration *cfgptr, void *ptr)
           data = (struct pkt_data *) dataptr;
 	}
       }
-      }
 
+      recv_budget++;
       goto read_data;
     }
   }
@@ -336,27 +344,38 @@ void amqp_cache_purge(struct chained_cache *queue[], int index, int safe_action)
   struct pkt_nat_primitives *pnat = NULL;
   struct pkt_mpls_primitives *pmpls = NULL;
   struct pkt_tunnel_primitives *ptun = NULL;
-  char *pcust = NULL;
+  u_char *pcust = NULL;
   struct pkt_vlen_hdr_primitives *pvlen = NULL;
   struct pkt_bgp_primitives empty_pbgp;
   struct pkt_nat_primitives empty_pnat;
   struct pkt_mpls_primitives empty_pmpls;
   struct pkt_tunnel_primitives empty_ptun;
-  char *empty_pcust = NULL;
-  char src_mac[18], dst_mac[18], src_host[INET6_ADDRSTRLEN], dst_host[INET6_ADDRSTRLEN], ip_address[INET6_ADDRSTRLEN];
-  char rd_str[SRVBUFLEN], misc_str[SRVBUFLEN], dyn_amqp_routing_key[SRVBUFLEN], *orig_amqp_routing_key = NULL;
-  int i, j, stop, batch_idx, is_routing_key_dyn = FALSE, qn = 0, ret, saved_index = index;
+  u_char *empty_pcust = NULL;
+  char dyn_amqp_routing_key[SRVBUFLEN], *orig_amqp_routing_key = NULL;
+  int j, stop, is_routing_key_dyn = FALSE, qn = 0, ret, saved_index = index;
   int mv_num = 0, mv_num_save = 0;
   time_t start, duration;
+  struct primitives_ptrs prim_ptrs;
+  struct pkt_data dummy_data;
   pid_t writer_pid = getpid();
 
   char *json_buf = NULL;
   int json_buf_off = 0;
 
+  //TODO sort out unused variables correctly
+  (void)pvlen;
+  (void)pcust;
+  (void)ptun;
+  (void)pmpls;
+  (void)pnat;
+  (void)pbgp;
+  (void)data;
+
 #ifdef WITH_AVRO
-  avro_writer_t avro_writer;
+  avro_writer_t avro_writer = {0};
   char *avro_buf = NULL;
   int avro_buffer_full = FALSE;
+  size_t avro_len;
 #endif
 
   /* setting some defaults */
@@ -387,10 +406,11 @@ void amqp_cache_purge(struct chained_cache *queue[], int index, int safe_action)
   p_amqp_set_frame_max(&amqpp_amqp_host, config.amqp_frame_max);
 
   if (config.message_broker_output & PRINT_OUTPUT_JSON) p_amqp_set_content_type_json(&amqpp_amqp_host);
-  else if (config.message_broker_output & PRINT_OUTPUT_AVRO) p_amqp_set_content_type_binary(&amqpp_amqp_host);
+  else if (config.message_broker_output & PRINT_OUTPUT_AVRO_BIN) p_amqp_set_content_type_binary(&amqpp_amqp_host);
+  else if (config.message_broker_output & PRINT_OUTPUT_AVRO_JSON) p_amqp_set_content_type_json(&amqpp_amqp_host);
   else {
     Log(LOG_ERR, "ERROR ( %s/%s ): Unsupported amqp_output value specified. Exiting.\n", config.name, config.type);
-    exit_plugin(1);
+    exit_gracefully(1);
   }
 
   p_amqp_init_routing_key_rr(&amqpp_amqp_host);
@@ -399,7 +419,7 @@ void amqp_cache_purge(struct chained_cache *queue[], int index, int safe_action)
   empty_pcust = malloc(config.cpptrs.len);
   if (!empty_pcust) {
     Log(LOG_ERR, "ERROR ( %s/%s ): Unable to malloc() empty_pcust. Exiting.\n", config.name, config.type);
-    exit_plugin(1);
+    exit_gracefully(1);
   }
 
   memset(&empty_pbgp, 0, sizeof(struct pkt_bgp_primitives));
@@ -417,10 +437,42 @@ void amqp_cache_purge(struct chained_cache *queue[], int index, int safe_action)
   Log(LOG_INFO, "INFO ( %s/%s ): *** Purging cache - START (PID: %u) ***\n", config.name, config.type, writer_pid);
   start = time(NULL);
 
+  if (config.message_broker_output & PRINT_OUTPUT_JSON) {
+    if (config.sql_multi_values) {
+      json_buf = malloc(config.sql_multi_values);
+
+      if (!json_buf) {
+	Log(LOG_ERR, "ERROR ( %s/%s ): malloc() failed (json_buf). Exiting ..\n", config.name, config.type);
+	exit_gracefully(1);
+      }
+      else memset(json_buf, 0, config.sql_multi_values);
+    }
+  }
+  else if ((config.message_broker_output & PRINT_OUTPUT_AVRO_BIN) ||
+           (config.message_broker_output & PRINT_OUTPUT_AVRO_JSON)) {
+#ifdef WITH_AVRO
+    if (!config.avro_buffer_size) config.avro_buffer_size = LARGEBUFLEN;
+
+    avro_buf = malloc(config.avro_buffer_size);
+
+    if (!avro_buf) {
+      Log(LOG_ERR, "ERROR ( %s/%s ): malloc() failed (avro_buf). Exiting ..\n", config.name, config.type);
+      exit_gracefully(1);
+    }
+    else {
+      memset(avro_buf, 0, config.avro_buffer_size);
+    }
+
+    if (config.message_broker_output & PRINT_OUTPUT_AVRO_BIN) {
+      avro_writer = avro_writer_memory(avro_buf, config.avro_buffer_size);
+    }
+#endif
+  }
+
   if (config.print_markers) {
-    if (config.message_broker_output & PRINT_OUTPUT_JSON || config.message_broker_output & PRINT_OUTPUT_AVRO) {
-      void *json_obj;
-      char *json_str;
+    if (config.message_broker_output & PRINT_OUTPUT_JSON) {
+      void *json_obj = NULL;
+      char *json_str = NULL;
 
       json_obj = compose_purge_init_json(config.name, writer_pid);
 
@@ -433,37 +485,34 @@ void amqp_cache_purge(struct chained_cache *queue[], int index, int safe_action)
         json_str = NULL;
       }
     }
-  }
-
-  if (config.message_broker_output & PRINT_OUTPUT_JSON) {
-    if (config.sql_multi_values) {
-      json_buf = malloc(config.sql_multi_values);
-
-      if (!json_buf) {
-	Log(LOG_ERR, "ERROR ( %s/%s ): malloc() failed (json_buf). Exiting ..\n", config.name, config.type);
-	exit_plugin(1);
-      }
-      else memset(json_buf, 0, config.sql_multi_values);
-    }
-  }
-  else if (config.message_broker_output & PRINT_OUTPUT_AVRO) {
+    else if ((config.message_broker_output & PRINT_OUTPUT_AVRO_BIN) || 
+	     (config.message_broker_output & PRINT_OUTPUT_AVRO_JSON)) {
 #ifdef WITH_AVRO
-    if (!config.avro_buffer_size) config.avro_buffer_size = LARGEBUFLEN;
+      avro_value_iface_t *avro_iface = avro_generic_class_from_schema(avro_acct_init_schema);
+      avro_value_t avro_value = compose_avro_acct_init(config.name, writer_pid, avro_iface);
 
-    avro_buf = malloc(config.avro_buffer_size);
+      if (config.message_broker_output & PRINT_OUTPUT_AVRO_BIN) {
+        avro_len = avro_writer_tell(avro_writer);
+        ret = p_amqp_publish_binary(&amqpp_amqp_host, avro_buf, avro_len);
+        avro_writer_reset(avro_writer);
+      }
+      else if (config.message_broker_output & PRINT_OUTPUT_AVRO_JSON) {
+	char *avro_locbuf = write_avro_json_record_to_buf(avro_value);
 
-    if (!avro_buf) {
-      Log(LOG_ERR, "ERROR ( %s/%s ): malloc() failed (avro_buf). Exiting ..\n", config.name, config.type);
-      exit_plugin(1);
-    }
+	if (avro_locbuf) {
+          ret = p_amqp_publish_string(&amqpp_amqp_host, avro_locbuf);
+	  free(avro_locbuf);
+	}
+      }
 
-    avro_writer = avro_writer_memory(avro_buf, config.avro_buffer_size);
+      avro_value_decref(&avro_value);
+      avro_value_iface_decref(avro_iface);
 #endif
+    }
   }
 
   for (j = 0; j < index; j++) {
-    void *json_obj;
-    char *json_str;
+    char *json_str = NULL;
 
     if (queue[j]->valid != PRINT_CACHE_COMMITTED) continue;
 
@@ -499,41 +548,65 @@ void amqp_cache_purge(struct chained_cache *queue[], int index, int safe_action)
       json_str = compose_json_str(json_obj);
 #endif
     }
-    else if (config.message_broker_output & PRINT_OUTPUT_AVRO) {
+    else if ((config.message_broker_output & PRINT_OUTPUT_AVRO_BIN) ||
+	     (config.message_broker_output & PRINT_OUTPUT_AVRO_JSON)) {
 #ifdef WITH_AVRO
       avro_value_iface_t *avro_iface = avro_generic_class_from_schema(avro_acct_schema);
-      avro_value_t avro_value = compose_avro(config.what_to_count, config.what_to_count_2, queue[j]->flow_type,
-                           &queue[j]->primitives, pbgp, pnat, pmpls, ptun, pcust, pvlen, queue[j]->bytes_counter,
-                           queue[j]->packet_counter, queue[j]->flow_counter, queue[j]->tcp_flags,
-                           &queue[j]->basetime, queue[j]->stitch, avro_iface);
-      size_t avro_value_size;
-
+      avro_value_t avro_value = compose_avro_acct_data(config.what_to_count, config.what_to_count_2,
+			   queue[j]->flow_type, &queue[j]->primitives, pbgp, pnat, pmpls, ptun, pcust,
+			   pvlen, queue[j]->bytes_counter, queue[j]->packet_counter,
+			   queue[j]->flow_counter, queue[j]->tcp_flags, &queue[j]->basetime,
+			   queue[j]->stitch, avro_iface);
       add_writer_name_and_pid_avro(avro_value, config.name, writer_pid);
-      avro_value_sizeof(&avro_value, &avro_value_size);
 
-      if (avro_value_size > config.avro_buffer_size) {
-        Log(LOG_ERR, "ERROR ( %s/%s ): AVRO; insufficient buffer size (avro_buffer_size=%u)\n",
-            config.name, config.type, config.avro_buffer_size);
-        Log(LOG_ERR, "ERROR ( %s/%s ): AVRO: increase value or look for avro_buffer_size in CONFIG-KEYS document.\n\n",
-            config.name, config.type);
-        exit_plugin(1);
+      if (config.message_broker_output & PRINT_OUTPUT_AVRO_BIN) {
+	size_t avro_value_size;
+
+	avro_value_sizeof(&avro_value, &avro_value_size);
+
+	if (avro_value_size > config.avro_buffer_size) {
+	  Log(LOG_ERR, "ERROR ( %s/%s ): amqp_cache_purge(): avro_buffer_size too small (%u)\n", config.name, config.type, config.avro_buffer_size);
+	  exit_gracefully(1);
+	}
+	else if (avro_value_size >= (config.avro_buffer_size - avro_writer_tell(avro_writer))) {
+	  avro_buffer_full = TRUE;
+	  j--;
+	}
+	else if (avro_value_write(avro_writer, &avro_value)) {
+	  Log(LOG_ERR, "ERROR ( %s/%s ): ARVO: unable to write value: %s\n", config.name, config.type, avro_strerror());
+	  exit_gracefully(1);
+	}
+	else {
+	  mv_num++;
+	}
       }
-      else if (avro_value_size >= (config.avro_buffer_size - avro_writer_tell(avro_writer))) {
-        avro_buffer_full = TRUE;
-        j--;
-      }
-      else if (avro_value_write(avro_writer, &avro_value)) {
-        Log(LOG_ERR, "ERROR ( %s/%s ): ARVO: unable to write value: %s\n", config.name, config.type, avro_strerror());
-        exit_plugin(1);
-      }
-      else {
-        mv_num++;
+      else if (config.message_broker_output & PRINT_OUTPUT_AVRO_JSON) {
+	char *avro_locbuf = write_avro_json_record_to_buf(avro_value);
+
+	if (avro_locbuf) {
+	  size_t avro_locbuf_len = strlen(avro_locbuf);
+
+	  if (avro_locbuf_len > config.avro_buffer_size) {
+	    Log(LOG_ERR, "ERROR ( %s/%s ): amqp_cache_purge(): avro_buffer_size too small (%u)\n", config.name, config.type, config.avro_buffer_size);
+	    exit_gracefully(1);
+	  }
+	  else if (avro_locbuf_len >= (config.avro_buffer_size - strlen(avro_buf))) {
+	    avro_buffer_full = TRUE;
+	    j--;
+	  }
+	  else {
+	    strcat(avro_buf, avro_locbuf);
+	    mv_num++;
+	  }
+
+	  free(avro_locbuf);
+	}
       }
 
       avro_value_decref(&avro_value);
       avro_value_iface_decref(avro_iface);
 #else
-      if (config.debug) Log(LOG_DEBUG, "DEBUG ( %s/%s ): compose_avro(): AVRO object not created due to missing --enable-avro\n", config.name, config.type);
+      if (config.debug) Log(LOG_DEBUG, "DEBUG ( %s/%s ): compose_avro_acct_data(): AVRO object not created due to missing --enable-avro\n", config.name, config.type);
 #endif
     }
 
@@ -546,7 +619,7 @@ void amqp_cache_purge(struct chained_cache *queue[], int index, int safe_action)
 	if (json_strlen >= (config.sql_multi_values - json_buf_off)) {
 	  if (json_strlen >= config.sql_multi_values) {
 	    Log(LOG_ERR, "ERROR ( %s/%s ): amqp_multi_values not large enough to store JSON elements. Exiting ..\n", config.name, config.type);
-	    exit(1);
+	    exit_gracefully(1);
 	  }
 
 	  tmp_str = json_str;
@@ -566,7 +639,10 @@ void amqp_cache_purge(struct chained_cache *queue[], int index, int safe_action)
 
       if (json_str) {
         if (is_routing_key_dyn) {
-          P_handle_table_dyn_strings(dyn_amqp_routing_key, SRVBUFLEN, orig_amqp_routing_key, queue[j]);
+	  prim_ptrs.data = &dummy_data;
+	  primptrs_set_all_from_chained_cache(&prim_ptrs, queue[j]);
+
+	  handle_dynname_internal_strings(dyn_amqp_routing_key, SRVBUFLEN, orig_amqp_routing_key, &prim_ptrs, DYN_STR_RABBITMQ_RK);
           p_amqp_set_routing_key(&amqpp_amqp_host, dyn_amqp_routing_key);
         }
 
@@ -599,11 +675,15 @@ void amqp_cache_purge(struct chained_cache *queue[], int index, int safe_action)
         else break;
       }
     }
-    else if (config.message_broker_output & PRINT_OUTPUT_AVRO) {
+    else if ((config.message_broker_output & PRINT_OUTPUT_AVRO_BIN) || 
+	     (config.message_broker_output & PRINT_OUTPUT_AVRO_JSON)) {
 #ifdef WITH_AVRO
       if (!config.sql_multi_values || (mv_num >= config.sql_multi_values) || avro_buffer_full) {
         if (is_routing_key_dyn) {
-          P_handle_table_dyn_strings(dyn_amqp_routing_key, SRVBUFLEN, orig_amqp_routing_key, queue[j]);
+          prim_ptrs.data = &dummy_data;
+          primptrs_set_all_from_chained_cache(&prim_ptrs, queue[j]);
+
+          handle_dynname_internal_strings(dyn_amqp_routing_key, SRVBUFLEN, orig_amqp_routing_key, &prim_ptrs, DYN_STR_RABBITMQ_RK);
           p_amqp_set_routing_key(&amqpp_amqp_host, dyn_amqp_routing_key);
         }
 
@@ -612,8 +692,15 @@ void amqp_cache_purge(struct chained_cache *queue[], int index, int safe_action)
           p_amqp_set_routing_key(&amqpp_amqp_host, dyn_amqp_routing_key);
         }
 
-        ret = p_amqp_publish_binary(&amqpp_amqp_host, avro_buf, avro_writer_tell(avro_writer));
-        avro_writer_reset(avro_writer);
+	if (config.message_broker_output & PRINT_OUTPUT_AVRO_BIN) {
+          ret = p_amqp_publish_binary(&amqpp_amqp_host, avro_buf, avro_writer_tell(avro_writer));
+          avro_writer_reset(avro_writer);
+	}
+	else if (config.message_broker_output & PRINT_OUTPUT_AVRO_JSON) {
+	  ret = p_amqp_publish_string(&amqpp_amqp_host, avro_buf);
+	  memset(avro_buf, 0, config.avro_buffer_size);
+	}
+
         avro_buffer_full = FALSE;
         mv_num_save = mv_num;
         mv_num = 0;
@@ -635,13 +722,21 @@ void amqp_cache_purge(struct chained_cache *queue[], int index, int safe_action)
 	if (!ret) qn += mv_num;
       }
     }
-    else if (config.message_broker_output & PRINT_OUTPUT_AVRO) {
+    else if ((config.message_broker_output & PRINT_OUTPUT_AVRO_BIN) || 
+	     (config.message_broker_output & PRINT_OUTPUT_AVRO_JSON)) {
 #ifdef WITH_AVRO
-      if (avro_writer_tell(avro_writer)) {
-        ret = p_amqp_publish_binary(&amqpp_amqp_host, avro_buf, avro_writer_tell(avro_writer));
-        avro_writer_free(avro_writer);
-
-        if (!ret) qn += mv_num;
+      if (config.message_broker_output & PRINT_OUTPUT_AVRO_BIN) {
+        if (avro_writer_tell(avro_writer)) {
+	  ret = p_amqp_publish_binary(&amqpp_amqp_host, avro_buf, avro_writer_tell(avro_writer));
+	  avro_writer_free(avro_writer);
+	  if (!ret) qn += mv_num;
+	}
+      }
+      else if (config.message_broker_output & PRINT_OUTPUT_AVRO_JSON) {
+	if (strlen(avro_buf)) {
+	  ret = p_amqp_publish_string(&amqpp_amqp_host, avro_buf);
+          if (!ret) qn += mv_num;
+	}
       }
 #endif
     }
@@ -650,9 +745,9 @@ void amqp_cache_purge(struct chained_cache *queue[], int index, int safe_action)
   duration = time(NULL)-start;
 
   if (config.print_markers) {
-    if (config.message_broker_output & PRINT_OUTPUT_JSON || config.message_broker_output & PRINT_OUTPUT_AVRO) {
-      void *json_obj;
-      char *json_str;
+    if (config.message_broker_output & PRINT_OUTPUT_JSON) {
+      void *json_obj = NULL;
+      char *json_str = NULL;
 
       json_obj = compose_purge_close_json(config.name, writer_pid, qn, saved_index, duration);
 
@@ -665,11 +760,35 @@ void amqp_cache_purge(struct chained_cache *queue[], int index, int safe_action)
         json_str = NULL;
       }
     }
+    else if ((config.message_broker_output & PRINT_OUTPUT_AVRO_BIN) || 
+	     (config.message_broker_output & PRINT_OUTPUT_AVRO_JSON)) {
+#ifdef WITH_AVRO
+      avro_value_iface_t *avro_iface = avro_generic_class_from_schema(avro_acct_close_schema);
+      avro_value_t avro_value = compose_avro_acct_close(config.name, writer_pid, qn, saved_index, duration, avro_iface);
+
+      if (config.message_broker_output & PRINT_OUTPUT_AVRO_BIN) {
+        avro_len = avro_writer_tell(avro_writer);
+        ret = p_amqp_publish_binary(&amqpp_amqp_host, avro_buf, avro_len);
+        avro_writer_reset(avro_writer);
+      }
+      else if (config.message_broker_output & PRINT_OUTPUT_AVRO_JSON) {
+	char *avro_locbuf = write_avro_json_record_to_buf(avro_value);
+
+	if (avro_locbuf) {
+	  ret = p_amqp_publish_string(&amqpp_amqp_host, avro_locbuf);
+	  free(avro_locbuf);
+	}
+      }
+
+      avro_value_decref(&avro_value);
+      avro_value_iface_decref(avro_iface);
+#endif
+    }
   }
 
   p_amqp_close(&amqpp_amqp_host, FALSE);
 
-  Log(LOG_INFO, "INFO ( %s/%s ): *** Purging cache - END (PID: %u, QN: %u/%u, ET: %u) ***\n",
+  Log(LOG_INFO, "INFO ( %s/%s ): *** Purging cache - END (PID: %u, QN: %u/%u, ET: %lu) ***\n",
 		config.name, config.type, writer_pid, qn, saved_index, duration);
 
   if (config.sql_trigger_exec && !safe_action) P_trigger_exec(config.sql_trigger_exec); 
