@@ -1,6 +1,6 @@
 /*  
     pmacct (Promiscuous mode IP Accounting package)
-    pmacct is Copyright (C) 2003-2017 by Paolo Lucente
+    pmacct is Copyright (C) 2003-2019 by Paolo Lucente
 */
 
 /*
@@ -19,9 +19,6 @@
     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 
-/* defines */
-#define __BGP_LOOKUP_C
-
 /* includes */
 #include "pmacct.h"
 #include "plugin_hooks.h"
@@ -29,6 +26,8 @@
 #include "pkt_handlers.h"
 #include "addr.h"
 #include "bgp.h"
+#include "pmbgpd.h"
+#include "rpki/rpki.h"
 
 void bgp_srcdst_lookup(struct packet_ptrs *pptrs, int type)
 {
@@ -38,15 +37,13 @@ void bgp_srcdst_lookup(struct packet_ptrs *pptrs, int type)
   struct xflow_status_entry *xs_entry = (struct xflow_status_entry *) pptrs->f_status;
   struct bgp_peer *peer;
   struct bgp_node *default_node, *result;
-  struct bgp_info *info;
+  struct bgp_info *info = NULL;
   struct node_match_cmp_term2 nmct2;
   struct prefix default_prefix;
-  int compare_bgp_port;
+  int compare_bgp_port = config.tmp_bgp_lookup_compare_ports;
   int follow_default = config.nfacctd_bgp_follow_default;
   struct in_addr pref4;
-#if defined ENABLE_IPV6
   struct in6_addr pref6;
-#endif
   safi_t safi;
   rd_t rd;
 
@@ -61,7 +58,6 @@ void bgp_srcdst_lookup(struct packet_ptrs *pptrs, int type)
   pptrs->bgp_dst_info = NULL;
   pptrs->bgp_peer = NULL;
   pptrs->bgp_nexthop_info = NULL;
-  compare_bgp_port = FALSE;
   safi = SAFI_UNICAST;
 
   memset(&rd, 0, sizeof(rd));
@@ -76,7 +72,6 @@ void bgp_srcdst_lookup(struct packet_ptrs *pptrs, int type)
 	compare_bgp_port = TRUE;
       }
     }
-#if defined ENABLE_IPV6
     else if (pptrs->bta_af == ETHERTYPE_IPV6) {
       sa->sa_family = AF_INET6;
       ip6_addr_32bit_cpy(&((struct sockaddr_in6 *)sa)->sin6_addr, &pptrs->bta, 0, 0, 1);
@@ -86,7 +81,6 @@ void bgp_srcdst_lookup(struct packet_ptrs *pptrs, int type)
 	compare_bgp_port = TRUE;
       }
     }
-#endif
   }
 
   start_again_follow_default:
@@ -129,9 +123,9 @@ void bgp_srcdst_lookup(struct packet_ptrs *pptrs, int type)
         memcpy(&pref4, &((struct pm_iphdr *)pptrs->iph_ptr)->ip_src, sizeof(struct in_addr));
 	bgp_node_match_ipv4(inter_domain_routing_db->rib[AFI_IP][safi],
 			    &pref4, (struct bgp_peer *) pptrs->bgp_peer,
-		     	    bgp_route_info_modulo_pathid,
+		     	    bms->route_info_modulo,
 			    bms->bgp_lookup_node_match_cmp, &nmct2,
-			    &result, &info);
+			    bms->bnv, &result, &info);
       }
 
       if (!pptrs->bgp_src_info && result) {
@@ -140,21 +134,26 @@ void bgp_srcdst_lookup(struct packet_ptrs *pptrs, int type)
         if (result->p.prefixlen >= pptrs->lm_mask_src) {
           pptrs->lm_mask_src = result->p.prefixlen;
           pptrs->lm_method_src = NF_NET_BGP;
+
+	  if (config.rpki_roas_file || config.rpki_rtr_cache) {
+	    pptrs->src_roa = rpki_vector_prefix_lookup(bms->bnv);
+	  }
         }
       }
 
       if (!pptrs->bgp_dst) {
         memset(&nmct2, 0, sizeof(struct node_match_cmp_term2));
         nmct2.peer = (struct bgp_peer *) pptrs->bgp_peer;
+	nmct2.safi = safi;
         nmct2.rd = &rd;
         nmct2.peer_dst_ip = &peer_dst_ip;
 
 	memcpy(&pref4, &((struct pm_iphdr *)pptrs->iph_ptr)->ip_dst, sizeof(struct in_addr));
 	bgp_node_match_ipv4(inter_domain_routing_db->rib[AFI_IP][safi],
 			    &pref4, (struct bgp_peer *) pptrs->bgp_peer,
-			    bgp_route_info_modulo_pathid,
+			    bms->route_info_modulo,
 			    bms->bgp_lookup_node_match_cmp, &nmct2,
-			    &result, &info);
+			    bms->bnv, &result, &info);
       }
 
       if (!pptrs->bgp_dst_info && result) {
@@ -163,23 +162,27 @@ void bgp_srcdst_lookup(struct packet_ptrs *pptrs, int type)
         if (result->p.prefixlen >= pptrs->lm_mask_dst) {
           pptrs->lm_mask_dst = result->p.prefixlen;
           pptrs->lm_method_dst = NF_NET_BGP;
+
+	  if (config.rpki_roas_file || config.rpki_rtr_cache) {
+	    pptrs->dst_roa = rpki_vector_prefix_lookup(bms->bnv);
+	  }
         }
       }
     }
-#if defined ENABLE_IPV6
     else if (pptrs->l3_proto == ETHERTYPE_IPV6) {
       if (!pptrs->bgp_src) {
         memset(&nmct2, 0, sizeof(struct node_match_cmp_term2));
         nmct2.peer = (struct bgp_peer *) pptrs->bgp_peer;
+	nmct2.safi = safi;
         nmct2.rd = &rd;
         nmct2.peer_dst_ip = NULL;
 
         memcpy(&pref6, &((struct ip6_hdr *)pptrs->iph_ptr)->ip6_src, sizeof(struct in6_addr));
 	bgp_node_match_ipv6(inter_domain_routing_db->rib[AFI_IP6][safi],
 		            &pref6, (struct bgp_peer *) pptrs->bgp_peer,
-		            bgp_route_info_modulo_pathid,
+		            bms->route_info_modulo,
 		            bms->bgp_lookup_node_match_cmp, &nmct2,
-		            &result, &info);
+		            bms->bnv, &result, &info);
       }
 
       if (!pptrs->bgp_src_info && result) {
@@ -188,6 +191,10 @@ void bgp_srcdst_lookup(struct packet_ptrs *pptrs, int type)
         if (result->p.prefixlen >= pptrs->lm_mask_src) {
           pptrs->lm_mask_src = result->p.prefixlen;
           pptrs->lm_method_src = NF_NET_BGP;
+
+	  if (config.rpki_roas_file || config.rpki_rtr_cache) {
+	    pptrs->src_roa = rpki_vector_prefix_lookup(bms->bnv); 
+	  }
         }
       }
 
@@ -200,9 +207,9 @@ void bgp_srcdst_lookup(struct packet_ptrs *pptrs, int type)
         memcpy(&pref6, &((struct ip6_hdr *)pptrs->iph_ptr)->ip6_dst, sizeof(struct in6_addr));
 	bgp_node_match_ipv6(inter_domain_routing_db->rib[AFI_IP6][safi],
 	     		    &pref6, (struct bgp_peer *) pptrs->bgp_peer,
-			    bgp_route_info_modulo_pathid,
+			    bms->route_info_modulo,
 			    bms->bgp_lookup_node_match_cmp, &nmct2,
-			    &result, &info);
+			    bms->bnv, &result, &info);
       }
 
       if (!pptrs->bgp_dst_info && result) {
@@ -211,22 +218,23 @@ void bgp_srcdst_lookup(struct packet_ptrs *pptrs, int type)
         if (result->p.prefixlen >= pptrs->lm_mask_dst) {
           pptrs->lm_mask_dst = result->p.prefixlen;
           pptrs->lm_method_dst = NF_NET_BGP;
+
+	  if (config.rpki_roas_file || config.rpki_rtr_cache) {
+	    pptrs->dst_roa = rpki_vector_prefix_lookup(bms->bnv);
+	  }
         }
       }
     }
-#endif
 
     if ((!pptrs->bgp_src || !pptrs->bgp_dst) && safi != SAFI_MPLS_LABEL) {
       if (pptrs->l3_proto == ETHERTYPE_IP && inter_domain_routing_db->rib[AFI_IP][SAFI_MPLS_LABEL]) {
         safi = SAFI_MPLS_LABEL;
         goto start_again_mpls_label;
       }
-#if defined ENABLE_IPV6
       else if (pptrs->l3_proto == ETHERTYPE_IPV6 && inter_domain_routing_db->rib[AFI_IP6][SAFI_MPLS_LABEL]) {
         safi = SAFI_MPLS_LABEL;
         goto start_again_mpls_label;
       }
-#endif
     }
 
     if (follow_default && safi != SAFI_MPLS_VPN) {
@@ -250,7 +258,6 @@ void bgp_srcdst_lookup(struct packet_ptrs *pptrs, int type)
 	  pptrs->bgp_dst_info = NULL;
         }
       }
-#if defined ENABLE_IPV6
       else if (pptrs->l3_proto == ETHERTYPE_IPV6) {
         memset(&default_prefix, 0, sizeof(default_prefix));
         default_prefix.family = AF_INET6;
@@ -269,7 +276,6 @@ void bgp_srcdst_lookup(struct packet_ptrs *pptrs, int type)
           pptrs->bgp_dst_info = NULL;
         }
       }
-#endif
       
       if (!pptrs->bgp_src || !pptrs->bgp_dst) {
 	follow_default--;
@@ -284,7 +290,6 @@ void bgp_srcdst_lookup(struct packet_ptrs *pptrs, int type)
               memcpy(&((struct sockaddr_in *)sa)->sin_addr, &info->attr->mp_nexthop.address.ipv4, 4);
 	      goto start_again_follow_default;
             }
-#if defined ENABLE_IPV6
             else if (info->attr->mp_nexthop.family == AF_INET6) {
               sa = &sa_local;
               memset(sa, 0, sizeof(struct sockaddr));
@@ -292,7 +297,6 @@ void bgp_srcdst_lookup(struct packet_ptrs *pptrs, int type)
               ip6_addr_cpy(&((struct sockaddr_in6 *)sa)->sin6_addr, &info->attr->mp_nexthop.address.ipv6);
               goto start_again_follow_default;
             }
-#endif
             else {
               sa = &sa_local;
               memset(sa, 0, sizeof(struct sockaddr));
@@ -317,17 +321,15 @@ void bgp_follow_nexthop_lookup(struct packet_ptrs *pptrs, int type)
   struct sockaddr *sa = (struct sockaddr *) pptrs->f_agent, sa_local;
   struct bgp_peer *nh_peer;
   struct bgp_node *result_node = NULL;
-  struct bgp_info *info;
+  struct bgp_info *info = NULL;
   struct node_match_cmp_term2 nmct2;
   char *saved_info = NULL;
   int peers_idx, ttl = MAX_HOPS_FOLLOW_NH, self = MAX_NH_SELF_REFERENCES;
   int nh_idx, matched = 0;
   struct prefix nh, ch;
   struct in_addr pref4;
-#if defined ENABLE_IPV6
   struct in6_addr pref6;
-#endif
-  char *saved_agent = pptrs->f_agent;
+  u_char *saved_agent = pptrs->f_agent;
   pm_id_t bta;
   u_int32_t modulo, local_modulo, modulo_idx, modulo_max;
 
@@ -349,7 +351,8 @@ void bgp_follow_nexthop_lookup(struct packet_ptrs *pptrs, int type)
   }
 
   for (nh_peer = NULL, peers_idx = 0; peers_idx < bms->max_peers; peers_idx++) {
-    if (!sa_addr_cmp(sa, &peers[peers_idx].addr) || !sa_addr_cmp(sa, &peers[peers_idx].id)) {
+    if (!sa_addr_cmp(sa, &peers[peers_idx].addr) ||
+	(!config.bgp_disable_router_id_check && !sa_addr_cmp(sa, &peers[peers_idx].id))) {
       nh_peer = &peers[peers_idx];
       break;
     }
@@ -383,19 +386,17 @@ void bgp_follow_nexthop_lookup(struct packet_ptrs *pptrs, int type)
       if (pptrs->l3_proto == ETHERTYPE_IP) {
         memcpy(&pref4, &((struct pm_iphdr *)pptrs->iph_ptr)->ip_dst, sizeof(struct in_addr));
         bgp_node_match_ipv4(inter_domain_routing_db->rib[AFI_IP][SAFI_UNICAST], &pref4, nh_peer,
-			    bgp_route_info_modulo_pathid,
+			    bms->route_info_modulo,
 			    bms->bgp_lookup_node_match_cmp, &nmct2,
-			    &result_node, &info);
+			    NULL, &result_node, &info);
       }
-#if defined ENABLE_IPV6
       else if (pptrs->l3_proto == ETHERTYPE_IPV6) {
         memcpy(&pref6, &((struct ip6_hdr *)pptrs->iph_ptr)->ip6_dst, sizeof(struct in6_addr));
         bgp_node_match_ipv6(inter_domain_routing_db->rib[AFI_IP6][SAFI_UNICAST], &pref6, nh_peer,
-			    bgp_route_info_modulo_pathid,
+			    bms->route_info_modulo,
 			    bms->bgp_lookup_node_match_cmp, &nmct2,
-			    &result_node, &info);
+			    NULL, &result_node, &info);
       }
-#endif
     }
 
     memset(&nh, 0, sizeof(nh));
@@ -424,7 +425,7 @@ void bgp_follow_nexthop_lookup(struct packet_ptrs *pptrs, int type)
 	if (matched && self > 0 && ttl > 0) { 
 	  if (prefix_match(&ch, &nh)) self--;
           sa = &sa_local;
-          pptrs->f_agent = (char *) &sa_local;
+          pptrs->f_agent = (u_char *) &sa_local;
           memset(sa, 0, sizeof(struct sockaddr));
           sa->sa_family = AF_INET;
           memcpy(&((struct sockaddr_in *)sa)->sin_addr, &info->attr->mp_nexthop.address.ipv4, 4);
@@ -437,7 +438,6 @@ void bgp_follow_nexthop_lookup(struct packet_ptrs *pptrs, int type)
 	  goto end;
 	}
       }
-#if defined ENABLE_IPV6
       else if (info->attr->mp_nexthop.family == AF_INET6) {
 	nh.family = AF_INET6;
 	nh.prefixlen = 128;
@@ -451,7 +451,7 @@ void bgp_follow_nexthop_lookup(struct packet_ptrs *pptrs, int type)
 	if (matched && self > 0 && ttl > 0) {
 	  if (prefix_match(&ch, &nh)) self--;
           sa = &sa_local;
-          pptrs->f_agent = (char *) &sa_local;
+          pptrs->f_agent = (u_char *) &sa_local;
           memset(sa, 0, sizeof(struct sockaddr));
           sa->sa_family = AF_INET6;
           ip6_addr_cpy(&((struct sockaddr_in6 *)sa)->sin6_addr, &info->attr->mp_nexthop.address.ipv6);
@@ -464,7 +464,6 @@ void bgp_follow_nexthop_lookup(struct packet_ptrs *pptrs, int type)
 	  goto end;
 	}
       }
-#endif
       else {
 	nh.family = AF_INET;
 	nh.prefixlen = 32;
@@ -478,7 +477,7 @@ void bgp_follow_nexthop_lookup(struct packet_ptrs *pptrs, int type)
 	if (matched && self > 0 && ttl > 0) {
 	  if (prefix_match(&ch, &nh)) self--;
           sa = &sa_local;
-          pptrs->f_agent = (char *) &sa_local;
+          pptrs->f_agent = (u_char *) &sa_local;
           memset(sa, 0, sizeof(struct sockaddr));
           sa->sa_family = AF_INET;
           memcpy(&((struct sockaddr_in *)sa)->sin_addr, &info->attr->nexthop, 4);
@@ -512,17 +511,16 @@ struct bgp_peer *bgp_lookup_find_bgp_peer(struct sockaddr *sa, struct xflow_stat
       peer_idx = xs_entry->peer_v4_idx; 
       peer_idx_ptr = &xs_entry->peer_v4_idx;
     }
-#if defined ENABLE_IPV6
     else if (l3_proto == ETHERTYPE_IPV6) {
       peer_idx = xs_entry->peer_v6_idx; 
       peer_idx_ptr = &xs_entry->peer_v6_idx;
     }
-#endif
   }
 
   if (xs_entry && peer_idx) {
-    if ((!sa_addr_cmp(sa, &peers[peer_idx].addr) || !sa_addr_cmp(sa, &peers[peer_idx].id)) &&
-        (!compare_bgp_port || !sa_port_cmp(sa, peers[peer_idx].tcp_port))) {
+    if ((!config.bgp_disable_router_id_check && !sa_addr_cmp(sa, &peers[peer_idx].id)) ||
+	(!sa_addr_cmp(sa, &peers[peer_idx].addr) &&
+	(!compare_bgp_port || !sa_port_cmp(sa, peers[peer_idx].tcp_port)))) {
       peer = &peers[peer_idx];
     }
     /* If no match then let's invalidate the entry */
@@ -533,8 +531,9 @@ struct bgp_peer *bgp_lookup_find_bgp_peer(struct sockaddr *sa, struct xflow_stat
   }
   else {
     for (peer = NULL, peers_idx = 0; peers_idx < config.nfacctd_bgp_max_peers; peers_idx++) {
-      if ((!sa_addr_cmp(sa, &peers[peers_idx].addr) || !sa_addr_cmp(sa, &peers[peers_idx].id)) && 
-	  (!compare_bgp_port || !sa_port_cmp(sa, peers[peer_idx].tcp_port))) {
+      if ((!config.bgp_disable_router_id_check && !sa_addr_cmp(sa, &peers[peers_idx].id)) ||
+	  (!sa_addr_cmp(sa, &peers[peers_idx].addr) && 
+	  (!compare_bgp_port || !sa_port_cmp(sa, peers[peer_idx].tcp_port)))) {
         peer = &peers[peers_idx];
         if (xs_entry && peer_idx_ptr) *peer_idx_ptr = peers_idx;
         break;
@@ -551,19 +550,23 @@ int bgp_lookup_node_match_cmp_bgp(struct bgp_info *info, struct node_match_cmp_t
 
   if (info->peer == nmct2->peer) {
     if (nmct2->safi == SAFI_MPLS_VPN) no_match++;
-    if (nmct2->peer->cap_add_paths && info->attr && nmct2->peer_dst_ip) no_match++;
+
+    if (nmct2->peer->cap_add_paths && info->extra && info->extra->path_id &&
+	info->attr && nmct2->peer_dst_ip) no_match++;
 
     if (nmct2->safi == SAFI_MPLS_VPN) {
-      if (info->extra && !memcmp(&info->extra->rd, &nmct2->rd, sizeof(rd_t))) no_match--;
+      if (info->extra && !memcmp(&info->extra->rd, nmct2->rd, sizeof(rd_t))) no_match--;
     }
 
     if (nmct2->peer->cap_add_paths) {
-      if (nmct2->peer_dst_ip && info->attr) {
-	if (info->attr->mp_nexthop.family == nmct2->peer_dst_ip->family) {
-	  if (!memcmp(&info->attr->mp_nexthop, nmct2->peer_dst_ip, HostAddrSz)) no_match--;
-	}
-	else if (info->attr->nexthop.s_addr && nmct2->peer_dst_ip->family == AF_INET) {
-	  if (info->attr->nexthop.s_addr == nmct2->peer_dst_ip->address.ipv4.s_addr) no_match--;
+      if (info->extra && info->extra->path_id) {
+        if (nmct2->peer_dst_ip && info->attr) {
+	  if (info->attr->mp_nexthop.family == nmct2->peer_dst_ip->family) {
+	    if (!memcmp(&info->attr->mp_nexthop, nmct2->peer_dst_ip, HostAddrSz)) no_match--;
+	  }
+	  else if (info->attr->nexthop.s_addr && nmct2->peer_dst_ip->family == AF_INET) {
+	    if (info->attr->nexthop.s_addr == nmct2->peer_dst_ip->address.ipv4.s_addr) no_match--;
+	  }
 	}
       }
     }
@@ -574,6 +577,37 @@ int bgp_lookup_node_match_cmp_bgp(struct bgp_info *info, struct node_match_cmp_t
   return TRUE;
 }
 
+int bgp_lookup_node_vector_unicast(struct prefix *p, struct bgp_peer *peer, struct bgp_node_vector *bnv)
+{
+  struct bgp_rt_structs *inter_domain_routing_db;
+  struct bgp_misc_structs *bms;
+  struct node_match_cmp_term2 nmct2;
+  struct bgp_node *result = NULL;
+  struct bgp_info *info = NULL;
+  afi_t afi;
+  safi_t safi;
+
+  if (!p || !peer || !bnv) return ERR;
+
+  bms = bgp_select_misc_db(peer->type);
+  inter_domain_routing_db = bgp_select_routing_db(peer->type);
+
+  if (!bms || !inter_domain_routing_db) return ERR;
+
+  afi = family2afi(p->family);
+  safi = SAFI_UNICAST;
+
+  memset(&nmct2, 0, sizeof(struct node_match_cmp_term2));
+  nmct2.peer = peer;
+  nmct2.safi = safi;
+  nmct2.p = p;
+
+  bgp_node_match(inter_domain_routing_db->rib[afi][safi], p, peer, bms->route_info_modulo,
+                 bms->bgp_lookup_node_match_cmp, &nmct2, bnv, &result, &info);
+
+  return SUCCESS;
+}
+
 void pkt_to_cache_legacy_bgp_primitives(struct cache_legacy_bgp_primitives *c, struct pkt_legacy_bgp_primitives *p,
 					pm_cfgreg_t what_to_count, pm_cfgreg_t what_to_count_2)
 {
@@ -746,3 +780,195 @@ u_int32_t bgp_route_info_modulo_pathid(struct bgp_peer *peer, path_id_t *path_id
           ((local_path_id - 1) % per_peer_buckets)) %
           (bms->table_peer_buckets * per_peer_buckets));
 }
+
+int bgp_lg_daemon_ip_lookup(struct bgp_lg_req_ipl_data *req, struct bgp_lg_rep *rep, int type)
+{
+  struct bgp_misc_structs *bms;
+  struct bgp_rt_structs *inter_domain_routing_db;
+  struct bgp_peer *peer;
+  struct bgp_node *result;
+  struct bgp_info *info;
+  struct node_match_cmp_term2 nmct2;
+  struct host_addr peer_ha;
+  u_int16_t l3_proto = 0, peer_port = 0;
+  u_int32_t bucket;
+  int ret = BGP_LOOKUP_OK;
+  struct rd_as4 *rd_as4_ptr;
+  safi_t safi;
+
+  bms = bgp_select_misc_db(type);
+  inter_domain_routing_db = bgp_select_routing_db(type);
+
+  if (!req || !rep || !bms || !inter_domain_routing_db) return BGP_LOOKUP_ERR;
+
+  memset(&peer_ha, 0, sizeof(peer_ha));
+  memset(rep, 0, sizeof(struct bgp_lg_rep));
+  safi = SAFI_UNICAST;
+
+  if (req->pref.family == AF_INET) l3_proto = ETHERTYPE_IP;
+  else if (req->pref.family == AF_INET6) l3_proto = ETHERTYPE_IPV6;
+
+  sa_to_addr(&req->peer, &peer_ha, &peer_port);
+
+  if (!peer_port) {
+    bucket = addr_hash(&peer_ha, bms->max_peers);
+    peer = bgp_peer_cache_search(bms->peers_cache, bucket, &peer_ha, FALSE);
+  }
+  else {
+    bucket = addr_port_hash(&peer_ha, peer_port, bms->max_peers);
+    peer = bgp_peer_cache_search(bms->peers_port_cache, bucket, &peer_ha, peer_port);
+  }
+
+  if (peer) {
+    // XXX: ADD-PATH code not currently supported
+
+    rd_as4_ptr = (struct rd_as4 *) &req->rd;
+    if (rd_as4_ptr->as) safi = SAFI_MPLS_VPN;
+
+    start_again_mpls_label:
+
+    memset(&nmct2, 0, sizeof(struct node_match_cmp_term2));
+    nmct2.peer = peer;
+    nmct2.safi = safi;
+    nmct2.rd = &req->rd;
+
+    if (l3_proto == ETHERTYPE_IP) {
+      bgp_node_match(inter_domain_routing_db->rib[AFI_IP][safi],
+			    &req->pref, peer, bgp_route_info_modulo_pathid,
+			    bms->bgp_lookup_node_match_cmp, &nmct2,
+			    NULL, &result, &info);
+
+      if (result) {
+	bgp_lg_rep_ipl_data_add(rep, AFI_IP, safi, &result->p, info);
+	ret = BGP_LOOKUP_OK;
+      }
+      else ret = BGP_LOOKUP_NOPREFIX; 
+    }
+    else if (l3_proto == ETHERTYPE_IPV6) {
+      bgp_node_match(inter_domain_routing_db->rib[AFI_IP6][safi],
+		            &req->pref, peer, bgp_route_info_modulo_pathid,
+			    bms->bgp_lookup_node_match_cmp, &nmct2,
+			    NULL, &result, &info);
+
+      if (result) {
+	bgp_lg_rep_ipl_data_add(rep, AFI_IP6, safi, &result->p, info);
+	ret = BGP_LOOKUP_OK;
+      }
+      else ret = BGP_LOOKUP_NOPREFIX; 
+    }
+
+    if (!result && safi != SAFI_MPLS_LABEL) {
+      if (l3_proto == ETHERTYPE_IP && inter_domain_routing_db->rib[AFI_IP][SAFI_MPLS_LABEL]) {
+        safi = SAFI_MPLS_LABEL;
+        goto start_again_mpls_label;
+      }
+      else if (l3_proto == ETHERTYPE_IPV6 && inter_domain_routing_db->rib[AFI_IP6][SAFI_MPLS_LABEL]) {
+        safi = SAFI_MPLS_LABEL;
+        goto start_again_mpls_label;
+      }
+    }
+
+    // XXX: bgp_follow_default code not currently supported
+  }
+  else ret = BGP_LOOKUP_NOPEER; 
+
+  return ret;
+}
+
+int bgp_lg_daemon_get_peers(struct bgp_lg_rep *rep, int type)
+{
+  struct bgp_misc_structs *bms;
+  struct bgp_peer *peer, *peers;
+  int idx, ret = BGP_LOOKUP_OK;
+
+  bms = bgp_select_misc_db(type);
+
+  if (!rep || !bms || !bms->peers) return BGP_LOOKUP_ERR;
+
+  for (peers = bms->peers, idx = 0; idx < bms->max_peers; idx++) {
+    peer = &peers[idx]; 
+    if (peer->fd) bgp_lg_rep_gp_data_add(rep, peer);
+  }
+
+  return ret;
+}
+
+void bgp_lg_rep_init(struct bgp_lg_rep *rep)
+{
+  struct bgp_lg_rep_data *data, *next;
+  u_int32_t idx;
+
+  assert(rep);
+
+  for (idx = 0, data = rep->data; idx < rep->results; idx++) {
+    assert(data);
+    assert(data->ptr);
+
+    next = data->next;
+    free(data->ptr);
+    free(data);
+
+    data = next;
+  }
+
+  memset(rep, 0, sizeof(struct bgp_lg_rep));
+}
+
+struct bgp_lg_rep_data *bgp_lg_rep_data_add(struct bgp_lg_rep *rep)
+{
+  struct bgp_lg_rep_data *data, *last;
+  u_int32_t idx;
+
+  assert(rep);
+
+  for (idx = 0, data = rep->data, last = NULL; idx < rep->results; idx++) {
+    last = data;
+    data = data->next;
+  }
+
+  data = malloc(sizeof(struct bgp_lg_rep_data)); 
+  assert(data);
+
+  data->ptr = NULL;
+  data->next = NULL;
+
+  if (last) last->next = data; 
+  else rep->data = data;
+
+  rep->results++;
+
+  return data;
+}
+
+void bgp_lg_rep_ipl_data_add(struct bgp_lg_rep *rep, afi_t afi, safi_t safi, struct prefix *pref, struct bgp_info *info)
+{
+  struct bgp_lg_rep_data *data;
+  struct bgp_lg_rep_ipl_data *ipl_data;
+
+  data = bgp_lg_rep_data_add(rep);
+
+  ipl_data = malloc(sizeof(struct bgp_lg_rep_ipl_data)); 
+  assert(ipl_data);
+
+  data->ptr = ipl_data;
+
+  ipl_data->afi = afi;
+  ipl_data->safi = safi;
+  ipl_data->pref = pref;
+  ipl_data->info = info;
+}
+
+void bgp_lg_rep_gp_data_add(struct bgp_lg_rep *rep, struct bgp_peer *peer)
+{
+  struct bgp_lg_rep_data *data;
+  struct bgp_lg_rep_gp_data *gp_data;
+
+  data = bgp_lg_rep_data_add(rep);
+
+  gp_data = malloc(sizeof(struct bgp_lg_rep_gp_data));
+  assert(gp_data);
+
+  data->ptr = gp_data;
+
+  gp_data->peer = peer;
+}
