@@ -1,6 +1,6 @@
 /*  
     pmacct (Promiscuous mode IP Accounting package)
-    pmacct is Copyright (C) 2003-2017 by Paolo Lucente
+    pmacct is Copyright (C) 2003-2019 by Paolo Lucente
 */
 
 /*
@@ -19,13 +19,11 @@
     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 
-/* defines */
-#define __BGP_MSG_C
-
 /* includes */
 #include "pmacct.h"
 #include "addr.h"
 #include "bgp.h"
+#include "bgp_blackhole.h"
 
 int bgp_parse_msg(struct bgp_peer *peer, time_t now, int online)
 {
@@ -87,7 +85,8 @@ int bgp_parse_msg(struct bgp_peer *peer, time_t now, int online)
       break;
     case BGP_NOTIFICATION:
       {
-	u_int8_t res_maj = 0, res_min = 0, shutdown_msglen = (BGP_NOTIFY_CEASE_SM_LEN + 1);
+	u_int16_t shutdown_msglen = (BGP_NOTIFY_CEASE_SM_LEN + 1);
+	u_int8_t res_maj = 0, res_min = 0;
         char shutdown_msg[shutdown_msglen];
 
 	memset(shutdown_msg, 0, shutdown_msglen);
@@ -132,7 +131,7 @@ int bgp_parse_msg(struct bgp_peer *peer, time_t now, int online)
       ret = bgp_parse_update_msg(&bmd, bgp_packet_ptr);
       if (ret < 0) {
         bgp_peer_print(peer, bgp_peer_str, INET6_ADDRSTRLEN);
-	Log(LOG_WARNING, "WARN ( %s/%s ): [%s] BGP UPDATE: malformed (%d).\n", config.name, bms->log_str, bgp_peer_str);
+	Log(LOG_WARNING, "WARN ( %s/%s ): [%s] BGP UPDATE: malformed.\n", config.name, bms->log_str, bgp_peer_str);
 	return BGP_NOTIFY_UPDATE_ERR;
       }
 
@@ -177,10 +176,19 @@ int bgp_parse_open_msg(struct bgp_msg_data *bmd, char *bgp_packet_ptr, time_t no
       peer->ht = MAX(5, ntohs(bopen->bgpo_holdtime));
       peer->id.family = AF_INET; 
       peer->id.address.ipv4.s_addr = bopen->bgpo_id;
+      peer->version = BGP_VERSION4;
+
+      /* Check: duplicate Router-IDs; BGP only, ie. no BMP */
+      if (!config.bgp_disable_router_id_check && bms->bgp_msg_open_router_id_check) {
+	int check_ret;
+
+	check_ret = bms->bgp_msg_open_router_id_check(bmd);
+	if (check_ret) return check_ret;
+      }
 
       /* OPEN options parsing */
       if (bopen->bgpo_optlen && bopen->bgpo_optlen >= 2) {
-	u_int8_t len, opt_type, opt_len, cap_type;
+	u_int8_t len, opt_type, opt_len;
 	char *ptr;
 
 	ptr = bgp_packet_ptr + BGP_MIN_OPEN_MSG_SIZE;
@@ -190,7 +198,7 @@ int bgp_parse_open_msg(struct bgp_msg_data *bmd, char *bgp_packet_ptr, time_t no
 	  opt_type = (u_int8_t) ptr[0];
 	  opt_len = (u_int8_t) ptr[1];
 
-	  if (opt_len > bopen->bgpo_optlen) {
+	  if (opt_len > len) {
             bgp_peer_print(peer, bgp_peer_str, INET6_ADDRSTRLEN);
 	    Log(LOG_INFO, "INFO ( %s/%s ): [%s] Received malformed BGP packet (option length).\n",
 		config.name, bms->log_str, bgp_peer_str);
@@ -202,14 +210,19 @@ int bgp_parse_open_msg(struct bgp_msg_data *bmd, char *bgp_packet_ptr, time_t no
  	   * some we are forced to support (ie. MP-BGP or 4-bytes AS support)
  	   */
 	  if (opt_type == BGP_OPTION_CAPABILITY) {
-	    char *optcap_ptr;
+	    char *optcap_ptr, *bgp_open_cap_len_reply_ptr, *bgp_open_cap_base_reply_ptr;
 	    int optcap_len;
 
 	    bgp_open_cap_ptr = ptr;
+	    memcpy(bgp_open_cap_reply_ptr, bgp_open_cap_ptr, 2);
+	    bgp_open_cap_len_reply_ptr = bgp_open_cap_reply_ptr + 1;
+	    bgp_open_cap_base_reply_ptr = bgp_open_cap_reply_ptr;
+	    bgp_open_cap_reply_ptr += 2;
+
 	    ptr += 2;
 	    len -= 2;
 	    optcap_ptr = ptr;
-	    optcap_len = len;
+	    optcap_len = opt_len;
 
 	    while (optcap_len > 0) {
 	      u_int8_t cap_len = optcap_ptr[1];
@@ -236,8 +249,8 @@ int bgp_parse_open_msg(struct bgp_msg_data *bmd, char *bgp_packet_ptr, time_t no
 		peer->cap_mp = TRUE;
 
 		if (online) {
-		  memcpy(bgp_open_cap_reply_ptr, bgp_open_cap_ptr, opt_len+2); 
-		  bgp_open_cap_reply_ptr += opt_len+2;
+		  memcpy(bgp_open_cap_reply_ptr, optcap_ptr, cap_len+2);
+		  bgp_open_cap_reply_ptr += cap_len+2;
 		}
 	      }
 	      else if (cap_type == BGP_CAPABILITY_4_OCTET_AS_NUMBER) {
@@ -258,9 +271,9 @@ int bgp_parse_open_msg(struct bgp_msg_data *bmd, char *bgp_packet_ptr, time_t no
 		  remote_as4 = ntohl(as4_ptr);
 
 		  if (online) {
-		    memcpy(bgp_open_cap_reply_ptr, bgp_open_cap_ptr, opt_len+2); 
-		    peer->cap_4as = bgp_open_cap_reply_ptr+4;
-		    bgp_open_cap_reply_ptr += opt_len+2;
+		    memcpy(bgp_open_cap_reply_ptr, optcap_ptr, cap_len+2); 
+		    peer->cap_4as = bgp_open_cap_reply_ptr+2;
+		    bgp_open_cap_reply_ptr += cap_len+2;
 		  }
 		  else peer->cap_4as = bgp_open_cap_ptr+4;
 		}
@@ -272,30 +285,53 @@ int bgp_parse_open_msg(struct bgp_msg_data *bmd, char *bgp_packet_ptr, time_t no
 		}
 	      }
 	      else if (cap_type == BGP_CAPABILITY_ADD_PATHS) {
-		char *cap_ptr = optcap_ptr+2;
+		char *cap_ptr = (optcap_ptr + 2);
 		struct capability_add_paths cap_data;
 
-		memcpy(&cap_data, cap_ptr, sizeof(cap_data));
-
-		if (online) {
-                  bgp_peer_print(peer, bgp_peer_str, INET6_ADDRSTRLEN);
-		  Log(LOG_INFO, "INFO ( %s/%s ): [%s] Capability: ADD-PATHs [%x] AFI [%x] SAFI [%x] SEND_RECEIVE [%x]\n",
-			config.name, bms->log_str, bgp_peer_str, cap_type, ntohs(cap_data.afi), cap_data.safi,
-			cap_data.sndrcv);
-		}
-
-		if (cap_data.sndrcv == 2 /* send */) {
-		  peer->cap_add_paths = TRUE; 
+		int cap_set = FALSE;
+		u_int8_t *cap_newlen_ptr; /* ptr to field of outbound cap_len */
+		int cap_idx;
+		
+		/* check for each AFI.SAFI included if remote can send - only then reply with receive */
+		for (cap_idx = 0; cap_idx < cap_len; cap_idx += sizeof(struct capability_add_paths)) {
+		  memcpy(&cap_data, cap_ptr+cap_idx, sizeof(cap_data));
 		  if (online) {
-		    memcpy(bgp_open_cap_reply_ptr, bgp_open_cap_ptr, opt_len+2);
-		    *(bgp_open_cap_reply_ptr+((opt_len+2)-1)) = 1; /* receive */
-		    bgp_open_cap_reply_ptr += opt_len+2;
+		    bgp_peer_print(peer, bgp_peer_str, INET6_ADDRSTRLEN);
+		    Log(LOG_INFO, "INFO ( %s/%s ): [%s] Capability: ADD-PATHs [%x] AFI [%x] SAFI [%x] SEND_RECEIVE [%x]\n",
+		  	config.name, bms->log_str, bgp_peer_str, cap_type, ntohs(cap_data.afi), cap_data.safi,
+			cap_data.sndrcv);
 		  }
-		}
+		  if ((cap_data.sndrcv == 2 /* send */) || (cap_data.sndrcv == 3 /* send and receive */)) {
+		    peer->cap_add_paths = TRUE; 
+		    if (online) {
+		      if (!cap_set) {
+			/* we need to send BGP_CAPABILITY_ADD_PATHS first */
+			cap_set = TRUE;
+			memcpy(bgp_open_cap_reply_ptr, optcap_ptr, 2);
+			cap_newlen_ptr = (u_char *)(bgp_open_cap_reply_ptr + 1);
+			(*cap_newlen_ptr) = 0;
+			bgp_open_cap_reply_ptr += 2;
+		      }
+		      cap_data.sndrcv = 1; /* we can receive */
+		      (*cap_newlen_ptr) += sizeof(cap_data);
+		      memcpy(bgp_open_cap_reply_ptr, &cap_data, sizeof(cap_data));
+		      bgp_open_cap_reply_ptr += sizeof(cap_data);
+		    }
+		  }
+	        }
 	      }
 
-	      optcap_ptr += cap_len+2;
-	      optcap_len -= cap_len+2;
+	      optcap_ptr += (cap_len + 2);
+	      optcap_len -= (cap_len + 2);
+	    }
+
+	    /* writing the new capability length */
+	    if (bgp_open_cap_reply_ptr > bgp_open_cap_base_reply_ptr + 2) {
+	      (*bgp_open_cap_len_reply_ptr) = ((bgp_open_cap_reply_ptr - bgp_open_cap_base_reply_ptr) - 2);
+	    }
+	    /* otherwise rollback */
+	    else {
+	      bgp_open_cap_reply_ptr = bgp_open_cap_base_reply_ptr;
 	    }
 	  }
 	  else {
@@ -434,7 +470,7 @@ int bgp_write_open_msg(char *msg, char *cp_msg, int cp_msglen, struct bgp_peer *
     }
   }
 
-  bopen_reply->bgpo_optlen = cp_msglen;
+  if (cp_msglen) bopen_reply->bgpo_optlen = cp_msglen;
   bopen_reply->bgpo_len = htons(BGP_MIN_OPEN_MSG_SIZE + bopen_reply->bgpo_optlen);
 
   if (config.nfacctd_bgp_ip) str_to_addr(config.nfacctd_bgp_ip, &bgp_ip);
@@ -476,7 +512,8 @@ int bgp_write_notification_msg(char *msg, int msglen, u_int8_t n_major, u_int8_t
 {
   struct bgp_notification *bn_reply = (struct bgp_notification *) msg;
   struct bgp_notification_shutdown_msg *bnsm_reply;
-  int ret = FALSE, shutdown_msglen;
+  u_int16_t shutdown_msglen;
+  int ret = FALSE;
   char *reply_msg_ptr;
 
   if (bn_reply && msglen >= BGP_MIN_NOTIFICATION_MSG_SIZE) {
@@ -515,7 +552,7 @@ int bgp_write_notification_msg(char *msg, int msglen, u_int8_t n_major, u_int8_t
   return ret;
 }
 
-int bgp_parse_notification_msg(struct bgp_msg_data *bmd, char *pkt, u_int8_t *res_maj, u_int8_t *res_min, char *shutdown_msg, u_int8_t shutdown_msglen)
+int bgp_parse_notification_msg(struct bgp_msg_data *bmd, char *pkt, u_int8_t *res_maj, u_int8_t *res_min, char *shutdown_msg, u_int16_t shutdown_msglen)
 {
   struct bgp_peer *peer = bmd->peer;
   struct bgp_notification *bn = (struct bgp_notification *) pkt;
@@ -539,8 +576,7 @@ int bgp_parse_notification_msg(struct bgp_msg_data *bmd, char *pkt, u_int8_t *re
       pkt_ptr = (pkt + BGP_MIN_NOTIFICATION_MSG_SIZE);
       bnsm = (struct bgp_notification_shutdown_msg *) pkt_ptr;
 
-      if (bnsm->bgpnsm_len <= rem_len && bnsm->bgpnsm_len <= BGP_NOTIFY_CEASE_SM_LEN &&
-	  bnsm->bgpnsm_len < shutdown_msglen) {
+      if (bnsm->bgpnsm_len <= rem_len && bnsm->bgpnsm_len < shutdown_msglen) {
 	memcpy(shutdown_msg, bnsm->bgpnsm_data, bnsm->bgpnsm_len);
 	shutdown_msg[bnsm->bgpnsm_len] = '\0';
 	
@@ -557,7 +593,6 @@ int bgp_parse_update_msg(struct bgp_msg_data *bmd, char *pkt)
 {
   struct bgp_peer *peer = bmd->peer;
   struct bgp_header bhdr;
-  u_char *startp, *endp;
   struct bgp_attr attr;
   u_int16_t attribute_len;
   u_int16_t update_len;
@@ -595,7 +630,7 @@ int bgp_parse_update_msg(struct bgp_msg_data *bmd, char *pkt)
   if (withdraw_len > 0) {
     withdraw.afi = AFI_IP;
     withdraw.safi = SAFI_UNICAST;
-    withdraw.nlri = pkt;
+    withdraw.nlri = (u_char *) pkt;
     withdraw.length = withdraw_len;
     pkt += withdraw_len;
   }
@@ -620,39 +655,37 @@ int bgp_parse_update_msg(struct bgp_msg_data *bmd, char *pkt)
   if (update_len > 0) {
     update.afi = AFI_IP;
     update.safi = SAFI_UNICAST;
-    update.nlri = pkt;
+    update.nlri = (u_char *) pkt;
     update.length = update_len;
   }
 
   /* NLRI parsing */
-  if (withdraw.length) bgp_nlri_parse(bmd, NULL, &withdraw);
-  if (update.length)  bgp_nlri_parse(bmd, &attr, &update);
+  if (withdraw.length) bgp_nlri_parse(bmd, NULL, &withdraw, BGP_NLRI_WITHDRAW);
+  if (update.length)  bgp_nlri_parse(bmd, &attr, &update, BGP_NLRI_UPDATE);
 	
   if (mp_update.length
 	  && mp_update.afi == AFI_IP
 	  && (mp_update.safi == SAFI_UNICAST || mp_update.safi == SAFI_MPLS_LABEL ||
 	      mp_update.safi == SAFI_MPLS_VPN))
-    bgp_nlri_parse(bmd, &attr, &mp_update);
+    bgp_nlri_parse(bmd, &attr, &mp_update, BGP_NLRI_UPDATE);
 
   if (mp_withdraw.length
 	  && mp_withdraw.afi == AFI_IP
 	  && (mp_withdraw.safi == SAFI_UNICAST || mp_withdraw.safi == SAFI_MPLS_LABEL ||
 	      mp_withdraw.safi == SAFI_MPLS_VPN))
-    bgp_nlri_parse (bmd, NULL, &mp_withdraw);
+    bgp_nlri_parse (bmd, NULL, &mp_withdraw, BGP_NLRI_WITHDRAW);
 
-#if defined ENABLE_IPV6
   if (mp_update.length
 	  && mp_update.afi == AFI_IP6
 	  && (mp_update.safi == SAFI_UNICAST || mp_update.safi == SAFI_MPLS_LABEL ||
 	      mp_update.safi == SAFI_MPLS_VPN))
-    bgp_nlri_parse(bmd, &attr, &mp_update);
+    bgp_nlri_parse(bmd, &attr, &mp_update, BGP_NLRI_UPDATE);
 
   if (mp_withdraw.length
 	  && mp_withdraw.afi == AFI_IP6
 	  && (mp_withdraw.safi == SAFI_UNICAST || mp_withdraw.safi == SAFI_MPLS_LABEL ||
 	      mp_withdraw.safi == SAFI_MPLS_VPN))
-    bgp_nlri_parse(bmd, NULL, &mp_withdraw);
-#endif
+    bgp_nlri_parse(bmd, NULL, &mp_withdraw, BGP_NLRI_WITHDRAW);
 
   /* Receipt of End-of-RIB can be processed here; being a silent
 	 BGP receiver only, honestly it doesn't matter to us */
@@ -676,7 +709,7 @@ int bgp_parse_update_msg(struct bgp_msg_data *bmd, char *pkt)
 int bgp_attr_parse(struct bgp_peer *peer, struct bgp_attr *attr, char *ptr, int len, struct bgp_nlri *mp_update, struct bgp_nlri *mp_withdraw)
 {
   int to_the_end = len, ret;
-  u_int8_t flag, type, *tmp, mp_nlri = 0;
+  u_int8_t flag, type, *tmp;
   u_int16_t tmp16, attr_len;
   struct aspath *as4_path = NULL;
 
@@ -728,11 +761,9 @@ int bgp_attr_parse(struct bgp_peer *peer, struct bgp_attr *attr, char *ptr, int
       break;
     case BGP_ATTR_MP_REACH_NLRI:
       ret = bgp_attr_parse_mp_reach(peer, attr_len, attr, ptr, mp_update);
-      mp_nlri = TRUE;
       break;
     case BGP_ATTR_MP_UNREACH_NLRI:
       ret = bgp_attr_parse_mp_unreach(peer, attr_len, attr, ptr, mp_withdraw);
-      mp_nlri = TRUE;
       break;
     default:
       ret = 0;
@@ -800,7 +831,7 @@ int bgp_attr_parse_community(struct bgp_peer *peer, u_int16_t len, struct bgp_at
 int bgp_attr_parse_ecommunity(struct bgp_peer *peer, u_int16_t len, struct bgp_attr *attr, char *ptr, u_int8_t flag)
 {
   if (len == 0) attr->ecommunity = NULL;
-  else attr->ecommunity = (struct ecommunity *) ecommunity_parse(peer, ptr, len);
+  else attr->ecommunity = (struct ecommunity *) ecommunity_parse(peer, (u_char *) ptr, len);
 
   return SUCCESS;
 }
@@ -808,7 +839,7 @@ int bgp_attr_parse_ecommunity(struct bgp_peer *peer, u_int16_t len, struct bgp_a
 int bgp_attr_parse_lcommunity(struct bgp_peer *peer, u_int16_t len, struct bgp_attr *attr, char *ptr, u_int8_t flag)
 {
   if (len == 0) attr->lcommunity = NULL;
-  else attr->lcommunity = (struct lcommunity *) lcommunity_parse(peer, ptr, len);
+  else attr->lcommunity = (struct lcommunity *) lcommunity_parse(peer, (u_char *) ptr, len);
 
   return SUCCESS;
 }
@@ -884,7 +915,6 @@ int bgp_attr_parse_mp_reach(struct bgp_peer *peer, u_int16_t len, struct bgp_att
 	attr->mp_nexthop.family = AF_INET;
 	memcpy(&attr->mp_nexthop.address.ipv4, ptr+8, 4);
 	break;
-#if defined ENABLE_IPV6
       case 16:
       case 32:
 	attr->mp_nexthop.family = AF_INET6;
@@ -895,7 +925,6 @@ int bgp_attr_parse_mp_reach(struct bgp_peer *peer, u_int16_t len, struct bgp_att
 	attr->mp_nexthop.family = AF_INET6;
 	memcpy(&attr->mp_nexthop.address.ipv6, ptr+8, 16);
 	break;
-#endif
       default:
 	memset(&attr->mp_nexthop, 0, sizeof(struct host_addr));
 	break;
@@ -920,7 +949,7 @@ int bgp_attr_parse_mp_reach(struct bgp_peer *peer, u_int16_t len, struct bgp_att
 
   mp_update->afi = afi;
   mp_update->safi = safi;
-  mp_update->nlri = ptr;
+  mp_update->nlri = (u_char *) ptr;
   mp_update->length = nlri_len;
 
   return SUCCESS;
@@ -945,22 +974,21 @@ int bgp_attr_parse_mp_unreach(struct bgp_peer *peer, u_int16_t len, struct bgp_a
 
   mp_withdraw->afi = afi;
   mp_withdraw->safi = safi;
-  mp_withdraw->nlri = ptr;
+  mp_withdraw->nlri = (u_char *) ptr;
   mp_withdraw->length = withdraw_len;
 
   return SUCCESS;
 }
 
-
 /* BGP UPDATE NLRI parsing */
-int bgp_nlri_parse(struct bgp_msg_data *bmd, void *attr, struct bgp_nlri *info)
+int bgp_nlri_parse(struct bgp_msg_data *bmd, void *attr, struct bgp_nlri *info, int type)
 {
   struct bgp_peer *peer = bmd->peer;
   u_char *pnt;
   u_char *lim;
   u_char safi, label[3];
   struct prefix p;
-  int psize, end;
+  int psize = 0, end;
   int ret;
   u_int32_t tmp32;
   u_int16_t tmp16;
@@ -1002,49 +1030,84 @@ int bgp_nlri_parse(struct bgp_msg_data *bmd, void *attr, struct bgp_nlri *info)
 
       /* Fetch prefix from NLRI packet. */
       memcpy(&p.u.prefix, pnt, psize);
-
-      // XXX: check address correctnesss now that we have it?
     }
     else if (info->safi == SAFI_MPLS_LABEL) { /* rfc3107 labeled unicast */
+      int labels_size = 0;
+      u_char *label_ptr = NULL;
+
       if ((info->afi == AFI_IP && p.prefixlen > 56) || (info->afi == AFI_IP6 && p.prefixlen > 152)) return ERR;
 
       psize = ((p.prefixlen+7)/8);
-      if (psize > end) return ERR;
+      if (psize > end || psize < 3 /* one label */) return ERR;
 
-      /* Fetch label (3) and prefix from NLRI packet */
-      memcpy(label, pnt, 3);
-      memcpy(&p.u.prefix, pnt+3, (psize-3));
-      p.prefixlen -= 24;
+      /* Fetch label(s) and prefix from NLRI packet */
+      label_ptr = pnt;
+
+      if (type == BGP_NLRI_UPDATE) {
+	while (((labels_size + 3) <= psize) && !check_bosbit(label_ptr)) {
+	  label_ptr += 3;
+	  labels_size += 3;
+	}
+      }
+
+      if ((labels_size + 3) <= psize) {
+        memcpy(label, label_ptr, 3);
+        label_ptr += 3;
+        labels_size += 3;
+      }
+      else return ERR;
+	
+      memcpy(&p.u.prefix, (pnt + labels_size), (psize - labels_size));
+      p.prefixlen -= (8 * labels_size);
     }
     else if (info->safi == SAFI_MPLS_VPN) { /* rfc4364 BGP/MPLS IP Virtual Private Networks */
+      int labels_size = 0;
+      u_char *label_ptr = NULL;
+
       if ((info->afi == AFI_IP && p.prefixlen > 120) || (info->afi == AFI_IP6 && p.prefixlen > 216)) return ERR;
 
       psize = ((p.prefixlen+7)/8);
-      if (psize > end) return ERR;
+      if (psize > end || psize < 3 /* one label */) return ERR;
 
       /* Fetch label (3), RD (8) and prefix from NLRI packet */
-      memcpy(label, pnt, 3);
+      label_ptr = pnt;
 
-      memcpy(&rd.type, pnt+3, 2);
+      if (type == BGP_NLRI_UPDATE) {
+	while (((labels_size + 3) <= psize) && !check_bosbit(label_ptr)) {
+	  label_ptr += 3;
+	  labels_size += 3;
+	}
+      }
+
+      if ((labels_size + 3) <= psize) {
+	memcpy(label, label_ptr, 3);
+	label_ptr += 3;
+	labels_size += 3;
+      }
+      else return ERR;
+
+      if (labels_size + 8 /* RD */ > psize) return ERR; 
+
+      memcpy(&rd.type, (pnt + labels_size), 2);
       rd.type = ntohs(rd.type);
       switch(rd.type) {
       case RD_TYPE_AS: 
 	rda = (struct rd_as *) &rd;
-	memcpy(&tmp16, pnt+5, 2);
-	memcpy(&tmp32, pnt+7, 4);
+	memcpy(&tmp16, (pnt + labels_size + 2 /* RD type */), 2);
+	memcpy(&tmp32, (pnt + labels_size + 2 /* RD type */ + 2 /* RD AS */), 4);
 	rda->as = ntohs(tmp16);
 	rda->val = ntohl(tmp32);
 	break;
       case RD_TYPE_IP: 
 	rdi = (struct rd_ip *) &rd;
-	memcpy(&rdi->ip.s_addr, pnt+5, 4);
-	memcpy(&tmp16, pnt+9, 2);
+	memcpy(&rdi->ip.s_addr, (pnt + labels_size + 2 /* RD type */), 4);
+	memcpy(&tmp16, (pnt + labels_size + 2 /* RD type */ + 4 /* RD IP */), 2);
 	rdi->val = ntohs(tmp16);
 	break;
       case RD_TYPE_AS4: 
 	rda4 = (struct rd_as4 *) &rd;
-	memcpy(&tmp32, pnt+5, 4);
-	memcpy(&tmp16, pnt+9, 2);
+	memcpy(&tmp32, (pnt + labels_size + 2 /* RD type */), 4);
+	memcpy(&tmp16, (pnt + labels_size + 2 /* RD type */ + 4 /* RD AS4 */), 2);
 	rda4->as = ntohl(tmp32);
 	rda4->val = ntohs(tmp16);
 	break;
@@ -1053,22 +1116,48 @@ int bgp_nlri_parse(struct bgp_msg_data *bmd, void *attr, struct bgp_nlri *info)
 	break;
       }
     
-      memcpy(&p.u.prefix, pnt+11, (psize-11));
-      p.prefixlen -= 88;
+      memcpy(&p.u.prefix, (pnt + labels_size + 8 /* RD */), (psize - (labels_size + 8 /* RD */)));
+      p.prefixlen -= (8 * (labels_size + 8 /* RD */));
     }
 
+    // XXX: check prefix correctnesss now that we have it?
+
+#if defined WITH_ZMQ
+    if (config.bgp_blackhole_stdcomm_list) {
+      bmd->is_blackhole = bgp_blackhole_evaluate_comms(attr);
+
+      /* let's process withdraws before withdrawing */  
+      if (!attr && bmd->is_blackhole) {
+	bgp_blackhole_instrument(peer, &p, attr, info->afi, safi);
+      }
+    }
+#endif
+
     /* Let's do our job now! */
-    if (attr)
+    if (attr) {
       ret = bgp_process_update(bmd, &p, attr, info->afi, safi, &rd, &path_id, label);
-    else
+    }
+    else {
       ret = bgp_process_withdraw(bmd, &p, attr, info->afi, safi, &rd, &path_id, label);
+      (void)ret; //Treat error?
+
+    }
+
+#if defined WITH_ZMQ
+    if (config.bgp_blackhole_stdcomm_list) {
+      /* let's process updates after installing */  
+      if (attr && bmd->is_blackhole) {
+	bgp_blackhole_instrument(peer, &p, attr, info->afi, safi);
+      }
+    }
+#endif
   }
 
   return SUCCESS;
 }
 
 int bgp_process_update(struct bgp_msg_data *bmd, struct prefix *p, void *attr, afi_t afi, safi_t safi,
-		       rd_t *rd, path_id_t *path_id, char *label)
+		       rd_t *rd, path_id_t *path_id, u_char *label)
 {
   struct bgp_peer *peer = bmd->peer;
   struct bgp_rt_structs *inter_domain_routing_db;
@@ -1191,7 +1280,7 @@ log_update:
   {
     char event_type[] = "log";
 
-    bgp_peer_log_msg(route, ri, afi, safi, event_type, bms->msglog_output, BGP_LOG_TYPE_UPDATE);
+    bgp_peer_log_msg(route, ri, afi, safi, event_type, bms->msglog_output, NULL, BGP_LOG_TYPE_UPDATE);
   }
 
   if (bms->skip_rib) {
@@ -1203,14 +1292,14 @@ log_update:
 }
 
 int bgp_process_withdraw(struct bgp_msg_data *bmd, struct prefix *p, void *attr, afi_t afi, safi_t safi,
-			 rd_t *rd, path_id_t *path_id, char *label)
+			 rd_t *rd, path_id_t *path_id, u_char *label)
 {
   struct bgp_peer *peer = bmd->peer;
   struct bgp_rt_structs *inter_domain_routing_db;
   struct bgp_misc_structs *bms;
   struct bgp_node *route = NULL, route_local;
   struct bgp_info *ri = NULL, ri_local;
-  u_int32_t modulo;
+  u_int32_t modulo = 0;
 
   if (!peer) return ERR;
 
@@ -1271,7 +1360,7 @@ int bgp_process_withdraw(struct bgp_msg_data *bmd, struct prefix *p, void *attr,
   if (ri && bms->msglog_backend_methods) {
     char event_type[] = "log";
 
-    bgp_peer_log_msg(route, ri, afi, safi, event_type, bms->msglog_output, BGP_LOG_TYPE_WITHDRAW);
+    bgp_peer_log_msg(route, ri, afi, safi, event_type, bms->msglog_output, NULL, BGP_LOG_TYPE_WITHDRAW);
   }
 
   if (!bms->skip_rib) {
