@@ -1,6 +1,6 @@
 /*  
     pmacct (Promiscuous mode IP Accounting package)
-    pmacct is Copyright (C) 2003-2017 by Paolo Lucente
+    pmacct is Copyright (C) 2003-2019 by Paolo Lucente
 */
 
 /*
@@ -19,14 +19,13 @@
     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 
-/* defines */
-#define __BGP_UTIL_C
-
 /* includes */
 #include "pmacct.h"
+#include "addr.h"
 #include "pmacct-data.h"
 #include "addr.h"
 #include "bgp.h"
+#include "thread_pool.h"
 #if defined WITH_RABBITMQ
 #include "amqp_common.h"
 #endif
@@ -39,10 +38,8 @@ int bgp_afi2family (int afi)
 {
   if (afi == AFI_IP)
     return AF_INET;
-#ifdef ENABLE_IPV6
   else if (afi == AFI_IP6)
     return AF_INET6;
-#endif 
   return SUCCESS;
 }
 
@@ -52,7 +49,7 @@ int bgp_rd2str(char *str, rd_t *rd)
   struct rd_as  *rda;
   struct rd_as4 *rda4;
   struct host_addr a;
-  u_char ip_address[INET6_ADDRSTRLEN];
+  char ip_address[INET6_ADDRSTRLEN];
 
   switch (rd->type) {
   case RD_TYPE_AS:
@@ -87,7 +84,6 @@ int bgp_str2rd(rd_t *output, char *value)
   struct host_addr a;
   char *endptr, *token;
   u_int32_t tmp32;
-  u_int16_t tmp16;
   struct rd_ip  *rdi;
   struct rd_as  *rda;
   struct rd_as4 *rda4;
@@ -170,7 +166,7 @@ int bgp_label2str(char *str, u_char *label)
 	(unsigned)(unsigned char)(label[2] >> 4));
   
   tmp = strtoul(str, &endp, 16);
-  snprintf(str, 8, "%u", tmp);
+  snprintf(str, 8, "%lu", tmp);
 
   return TRUE;
 }
@@ -190,7 +186,7 @@ struct bgp_info_extra *bgp_info_extra_new(struct bgp_info *ri)
   new = malloc(sizeof(struct bgp_info_extra));
   if (!new) {
     Log(LOG_ERR, "ERROR ( %s/%s ): malloc() failed (bgp_info_extra_new). Exiting ..\n", config.name, bms->log_str);
-    exit_all(1);
+    exit_gracefully(1);
   }
   else memset(new, 0, sizeof (struct bgp_info_extra));
 
@@ -223,7 +219,7 @@ struct bgp_info_extra *bgp_info_extra_get(struct bgp_info *ri)
 }
 
 struct bgp_info_extra *bgp_info_extra_process(struct bgp_peer *peer, struct bgp_info *ri, safi_t safi,
-		  			      path_id_t *path_id, rd_t *rd, char *label)
+		  			      path_id_t *path_id, rd_t *rd, u_char *label)
 {
   struct bgp_info_extra *rie = NULL;
 
@@ -261,7 +257,7 @@ struct bgp_info *bgp_info_new(struct bgp_peer *peer)
   new = malloc(sizeof(struct bgp_info));
   if (!new) {
     Log(LOG_ERR, "ERROR ( %s/%s ): malloc() failed (bgp_info_new). Exiting ..\n", config.name, bms->log_str);
-    exit_all(1);
+    exit_gracefully(1);
   }
   else memset(new, 0, sizeof (struct bgp_info));
   
@@ -329,10 +325,6 @@ unsigned int attrhash_key_make(void *p)
   key += attr->nexthop.s_addr;
   key += attr->med;
   key += attr->local_pref;
-  if (attr->pathlimit.as) {
-    key += attr->pathlimit.ttl;
-    key += attr->pathlimit.as;
-  }
 
   if (attr->aspath)
     key += aspath_key_make(attr->aspath);
@@ -362,9 +354,7 @@ int attrhash_cmp(const void *p1, const void *p2)
       && attr1->lcommunity == attr2->lcommunity
       && attr1->med == attr2->med
       && attr1->local_pref == attr2->local_pref
-      && attr1->pathlimit.ttl == attr2->pathlimit.ttl
-      && attr1->pathlimit.as == attr2->pathlimit.as
-      && !memcmp(&attr1->mp_nexthop, &attr2->mp_nexthop, sizeof(struct host_addr)))
+      && !host_addr_cmp2((struct host_addr *)&attr1->mp_nexthop, (struct host_addr *)&attr2->mp_nexthop))
     return TRUE;
 
   return FALSE;
@@ -471,7 +461,7 @@ void *bgp_attr_hash_alloc(void *p)
   attr = malloc(sizeof (struct bgp_attr));
   if (!attr) {
     Log(LOG_ERR, "ERROR ( %s/core/BGP ): malloc() failed (bgp_attr_hash_alloc). Exiting ..\n", config.name); // XXX
-    exit_all(1);
+    exit_gracefully(1);
   }
   else {
     memset(attr, 0, sizeof (struct bgp_attr));
@@ -482,12 +472,93 @@ void *bgp_attr_hash_alloc(void *p)
   return attr;
 }
 
+void bgp_peer_cache_init(struct bgp_peer_cache_bucket *cache, u_int32_t buckets)
+{
+  u_int32_t idx;
+
+  for (idx = 0; idx < buckets; idx++) {
+    memset(&cache[idx], 0, sizeof(struct bgp_peer_cache_bucket));
+    pthread_mutex_init(&cache[idx].mutex, NULL);
+  }
+}
+
+struct bgp_peer_cache *bgp_peer_cache_insert(struct bgp_peer_cache_bucket *cache, u_int32_t bucket, struct bgp_peer *peer)
+{
+  struct bgp_peer_cache *cursor, *last, *new, *ret = NULL;
+
+  pthread_mutex_lock(&peers_cache[bucket].mutex);  
+
+  for (cursor = peers_cache[bucket].e, last = NULL; cursor; cursor = cursor->next) last = cursor;
+
+  new = malloc(sizeof(struct bgp_peer_cache));
+  if (new) {
+    new->ptr = peer;
+    new->next = NULL;
+
+    if (!last) peers_cache[bucket].e = new;
+    else last->next = new; 
+
+    ret = new;
+  }
+
+  pthread_mutex_unlock(&peers_cache[bucket].mutex);  
+
+  return ret;
+}
+
+int bgp_peer_cache_delete(struct bgp_peer_cache_bucket *cache, u_int32_t bucket, struct bgp_peer *peer)
+{
+  struct bgp_peer_cache *cursor, *last;
+  int ret = ERR;
+
+  pthread_mutex_lock(&peers_cache[bucket].mutex);  
+
+  for (cursor = peers_cache[bucket].e, last = NULL; cursor; cursor = cursor->next) {
+    if (cursor->ptr == peer) {
+      if (!last) peers_cache[bucket].e = cursor->next;
+      else last->next = cursor->next;
+
+      free(cursor);
+
+      ret = SUCCESS;
+      break;
+    }
+
+    last = cursor;
+  }
+
+  pthread_mutex_unlock(&peers_cache[bucket].mutex);
+
+  return ret;
+}
+
+struct bgp_peer *bgp_peer_cache_search(struct bgp_peer_cache_bucket *cache, u_int32_t bucket, struct host_addr *ha, u_int16_t port)
+{
+  struct bgp_peer_cache *cursor;
+  struct bgp_peer *ret = NULL;
+
+  pthread_mutex_lock(&peers_cache[bucket].mutex);
+
+  for (cursor = peers_cache[bucket].e; cursor; cursor = cursor->next) {
+    if (port) {
+      if (cursor->ptr->tcp_port != port) continue;
+    }
+
+    if (!host_addr_cmp(&cursor->ptr->addr, ha)) {
+      ret = cursor->ptr;
+      break;
+    }
+  }
+
+  pthread_mutex_unlock(&peers_cache[bucket].mutex);
+
+  return ret;
+}
+
 int bgp_peer_init(struct bgp_peer *peer, int type)
 {
   struct bgp_misc_structs *bms;
   int ret = TRUE;
-  afi_t afi;
-  safi_t safi;
 
   bms = bgp_select_misc_db(type);
 
@@ -500,7 +571,7 @@ int bgp_peer_init(struct bgp_peer *peer, int type)
   peer->buf.base = malloc(peer->buf.len);
   if (!peer->buf.base) {
     Log(LOG_ERR, "ERROR ( %s/%s ): malloc() failed (bgp_peer_init). Exiting ..\n", config.name, bms->log_str);
-    exit_all(1);
+    exit_gracefully(1);
   }
   else {
     memset(peer->buf.base, 0, peer->buf.len);
@@ -513,8 +584,6 @@ int bgp_peer_init(struct bgp_peer *peer, int type)
 void bgp_peer_close(struct bgp_peer *peer, int type, int no_quiet, int send_notification, u_int8_t n_major, u_int8_t n_minor, char *shutdown_msg)
 {
   struct bgp_misc_structs *bms;
-  afi_t afi;
-  safi_t safi;
 
   if (!peer) return;
 
@@ -522,23 +591,40 @@ void bgp_peer_close(struct bgp_peer *peer, int type, int no_quiet, int send_noti
 
   if (!bms) return;
 
-  if (send_notification) {
-    int ret, notification_msglen = (BGP_MIN_NOTIFICATION_MSG_SIZE + BGP_NOTIFY_CEASE_SM_LEN + 1);
-    char notification_msg[notification_msglen];
+  if (!config.bgp_xconnect_map) {
+    if (send_notification) {
+      int ret, notification_msglen = (BGP_MIN_NOTIFICATION_MSG_SIZE + BGP_NOTIFY_CEASE_SM_LEN + 1);
+      char notification_msg[notification_msglen];
 
-    ret = bgp_write_notification_msg(notification_msg, notification_msglen, n_major, n_minor, shutdown_msg);
-    if (ret) send(peer->fd, notification_msg, ret, 0);
+      ret = bgp_write_notification_msg(notification_msg, notification_msglen, n_major, n_minor, shutdown_msg);
+      if (ret) send(peer->fd, notification_msg, ret, 0);
+    }
+
+    /* be quiet if we are in a signal handler and already set to exit */
+    if (!no_quiet) bgp_peer_info_delete(peer);
+
+    if (bms->msglog_file || bms->msglog_amqp_routing_key || bms->msglog_kafka_topic)
+      bgp_peer_log_close(peer, bms->msglog_output, peer->type);
+
+    if (bms->peers_cache && bms->peers_port_cache) {
+      u_int32_t bucket;
+
+      bucket = addr_hash(&peer->addr, bms->max_peers);
+      bgp_peer_cache_delete(bms->peers_cache, bucket, peer);
+
+      bucket = addr_port_hash(&peer->addr, peer->tcp_port, bms->max_peers);
+      bgp_peer_cache_delete(bms->peers_port_cache, bucket, peer);
+    }
+  }
+  else {
+    if (peer->xconnect_fd && peer->xconnect_fd != ERR) close(peer->xconnect_fd);
   }
 
-  /* be quiet if we are in a signal handler and already set to exit() */
-  if (!no_quiet) bgp_peer_info_delete(peer);
-
-  if (bms->msglog_file || bms->msglog_amqp_routing_key || bms->msglog_kafka_topic)
-    bgp_peer_log_close(peer, bms->msglog_output, peer->type);
-
-  if (peer->fd != ERR) close(peer->fd);
+  if (peer->fd && peer->fd != ERR) close(peer->fd);
 
   peer->fd = 0;
+  memset(&peer->xc, 0, sizeof(peer->xc));
+  peer->xconnect_fd = 0;
   memset(&peer->id, 0, sizeof(peer->id));
   memset(&peer->addr, 0, sizeof(peer->addr));
   memset(&peer->addr_str, 0, sizeof(peer->addr_str));
@@ -549,6 +635,60 @@ void bgp_peer_close(struct bgp_peer *peer, int type, int no_quiet, int send_noti
     write_neighbors_file(bms->neighbors_file, peer->type);
 }
 
+int bgp_peer_xconnect_init(struct bgp_peer *peer, int type)
+{
+  char peer_str[INET6_ADDRSTRLEN], xconnect_str[BGP_XCONNECT_STRLEN];
+  struct bgp_misc_structs *bms;
+  struct bgp_xconnects *bxm; 
+  int ret = TRUE, idx, fd;
+
+  assert(!peer->xconnect_fd);
+
+  bms = bgp_select_misc_db(type);
+
+  if (!peer || !bms) return ERR;
+
+  bxm = bms->xconnects;
+
+  if (bxm) {
+    for (idx = 0; idx < bxm->num; idx++) {
+      if (!sa_addr_cmp((struct sockaddr *) &bxm->pool[idx].src, &peer->addr) ||
+	  !host_addr_mask_cmp(&bxm->pool[idx].src_addr, &bxm->pool[idx].src_mask, &peer->addr)) { 
+	struct sockaddr *sa = (struct sockaddr *) &bxm->pool[idx].dst;
+
+	memcpy(&peer->xc, &bxm->pool[idx], sizeof(struct bgp_xconnect));
+	bgp_peer_xconnect_print(peer, xconnect_str, BGP_XCONNECT_STRLEN);
+
+	fd = socket(sa->sa_family, SOCK_STREAM, 0);
+	if (fd == ERR) {
+	  Log(LOG_WARNING, "WARN ( %s/%s ): [%s] bgp_peer_xconnect_init(): socket() failed.\n", config.name, bms->log_str, xconnect_str);
+	  memset(&peer->xc, 0, sizeof(peer->xc));
+	  peer->xconnect_fd = 0;
+	  return ERR;
+	}
+
+	ret = connect(fd, sa, bxm->pool[idx].dst_len);
+	if (ret == ERR) {
+	  Log(LOG_WARNING, "WARN ( %s/%s ): [%s] bgp_peer_xconnect_init(): connect() failed.\n", config.name, bms->log_str, xconnect_str);
+	  memset(&peer->xc, 0, sizeof(peer->xc));
+	  peer->xconnect_fd = 0;
+	  return ERR;
+	}
+
+	peer->xconnect_fd = fd;
+	break;
+      }
+    }
+
+    if (!peer->xconnect_fd) {
+      bgp_peer_print(peer, peer_str, INET6_ADDRSTRLEN);
+      Log(LOG_WARNING, "WARN ( %s/%s ): [%s] unable to xconnect BGP peer.\n", config.name, bgp_misc_db->log_str, peer_str);
+    }
+  }
+
+  return ret;
+}
+
 void bgp_peer_print(struct bgp_peer *peer, char *buf, int len)
 {
   char dumb_buf[] = "0.0.0.0";
@@ -567,12 +707,29 @@ void bgp_peer_print(struct bgp_peer *peer, char *buf, int len)
   if (!ret) strcpy(buf, dumb_buf);
 }
 
+void bgp_peer_xconnect_print(struct bgp_peer *peer, char *buf, int len)
+{
+  char src[INET6_ADDRSTRLEN + PORT_STRLEN + 1], dst[INET6_ADDRSTRLEN + PORT_STRLEN + 1];
+  struct sockaddr *sa_src;
+  struct sockaddr *sa_dst;
+
+  if (peer && buf && len >= BGP_XCONNECT_STRLEN) {
+    sa_src = (struct sockaddr *) &peer->xc.src;
+    sa_dst = (struct sockaddr *) &peer->xc.dst;
+
+    if (sa_src->sa_family) sa_to_str(src, sizeof(src), sa_src);
+    else addr_mask_to_str(src, sizeof(src), &peer->xc.src_addr, &peer->xc.src_mask);
+
+    sa_to_str(dst, sizeof(dst), sa_dst);
+
+    snprintf(buf, len, "%s x %s", src, dst);
+  }
+}
+
 void bgp_peer_info_delete(struct bgp_peer *peer)
 {
   struct bgp_rt_structs *inter_domain_routing_db = bgp_select_routing_db(peer->type);
-  struct bgp_misc_structs *bms = bgp_select_misc_db(peer->type);
   struct bgp_table *table;
-  struct bgp_node *node;
   afi_t afi;
   safi_t safi;
 
@@ -581,33 +738,44 @@ void bgp_peer_info_delete(struct bgp_peer *peer)
   for (afi = AFI_IP; afi < AFI_MAX; afi++) {
     for (safi = SAFI_UNICAST; safi < SAFI_MAX; safi++) {
       table = inter_domain_routing_db->rib[afi][safi];
-      node = bgp_table_top(peer, table);
+      bgp_table_info_delete(peer, table, afi, safi);
+    }
+  }
+}
 
-      while (node) {
-        u_int32_t modulo = bms->route_info_modulo(peer, NULL, bms->table_per_peer_buckets);
-        u_int32_t peer_buckets;
-        struct bgp_info *ri;
-        struct bgp_info *ri_next;
+void bgp_table_info_delete(struct bgp_peer *peer, struct bgp_table *table, afi_t afi, safi_t safi)
+{
+  struct bgp_misc_structs *bms = bgp_select_misc_db(peer->type);
+  struct bgp_node *node;
 
-        for (peer_buckets = 0; peer_buckets < bms->table_per_peer_buckets; peer_buckets++) {
-          for (ri = node->info[modulo+peer_buckets]; ri; ri = ri_next) {
-            if (ri->peer == peer) {
-	      if (bms->msglog_backend_methods) {
-		char event_type[] = "log";
+  node = bgp_table_top(peer, table);
 
-		bgp_peer_log_msg(node, ri, afi, safi, event_type, bms->msglog_output, BGP_LOG_TYPE_DELETE);
-	      }
+  while (node) {
+    u_int32_t modulo;
+    u_int32_t peer_buckets;
+    struct bgp_info *ri;
+    struct bgp_info *ri_next;
 
-	      ri_next = ri->next; /* let's save pointer to next before free up */
-              bgp_info_delete(peer, node, ri, modulo+peer_buckets);
-            }
-	    else ri_next = ri->next;
-          }
+    if (bms->route_info_modulo) modulo = bms->route_info_modulo(peer, NULL, bms->table_per_peer_buckets);
+    else modulo = 0;
+
+    for (peer_buckets = 0; peer_buckets < bms->table_per_peer_buckets; peer_buckets++) {
+      for (ri = node->info[modulo + peer_buckets]; ri; ri = ri_next) {
+	if (ri->peer == peer) {
+	  if (bms->msglog_backend_methods) {
+	    char event_type[] = "log";
+
+	    bgp_peer_log_msg(node, ri, afi, safi, event_type, bms->msglog_output, NULL, BGP_LOG_TYPE_DELETE);
+	  }
+
+	  ri_next = ri->next; /* let's save pointer to next before free up */
+          bgp_info_delete(peer, node, ri, (modulo + peer_buckets));
         }
-
-        node = bgp_route_next(peer, node);
+	else ri_next = ri->next;
       }
     }
+
+    node = bgp_route_next(peer, node);
   }
 }
 
@@ -621,14 +789,14 @@ int bgp_attr_munge_as4path(struct bgp_peer *peer, struct bgp_attr *attr, struct
   /* pre-requisite for AS4_PATH is AS_PATH indeed */ 
   // XXX if (as4path && !attr->aspath) return ERR;
 
-  newpath = aspath_reconcile_as4(peer, attr->aspath, as4path);
+  newpath = aspath_reconcile_as4(attr->aspath, as4path);
   aspath_unintern(peer, attr->aspath);
   attr->aspath = aspath_intern(peer, newpath);
 
   return SUCCESS;
 }
 
-void load_comm_patterns(char **stdcomm, char **extcomm, char **lrgcomm, char **stdcomm_to_asn)
+void load_comm_patterns(char **stdcomm, char **extcomm, char **lrgcomm, char **stdcomm_to_asn, char **lrgcomm_to_asn)
 {
   int idx;
   char *token;
@@ -637,6 +805,7 @@ void load_comm_patterns(char **stdcomm, char **extcomm, char **lrgcomm, char **s
   memset(ext_comm_patterns, 0, sizeof(ext_comm_patterns));
   memset(lrg_comm_patterns, 0, sizeof(lrg_comm_patterns));
   memset(std_comm_patterns_to_asn, 0, sizeof(std_comm_patterns_to_asn));
+  memset(lrg_comm_patterns_to_asn, 0, sizeof(lrg_comm_patterns_to_asn));
 
   if (*stdcomm) {
     idx = 0;
@@ -673,17 +842,25 @@ void load_comm_patterns(char **stdcomm, char **extcomm, char **lrgcomm, char **s
       idx++;
     }
   }
-} 
+
+  if (*lrgcomm_to_asn) {
+    idx = 0;
+    while ( (token = extract_token(lrgcomm_to_asn, ',')) && idx < MAX_BGP_COMM_PATTERNS ) {
+      lrg_comm_patterns_to_asn[idx] = token;
+      trim_spaces(lrg_comm_patterns_to_asn[idx]);
+      idx++;
+    }
+  }
+}
 
 void evaluate_comm_patterns(char *dst, char *src, char **patterns, int dstlen)
 {
   char *ptr, *haystack, *delim_src, *delim_ptn;
   char local_ptr[MAX_BGP_STD_COMMS], *auxptr;
-  int idx, i, j, srclen;
+  int idx, i, j;
 
   if (!src || !dst || !dstlen) return;
 
-  srclen = strlen(src);
   memset(dst, 0, dstlen);
 
   for (idx = 0, j = 0; patterns[idx]; idx++) {
@@ -744,7 +921,7 @@ void evaluate_comm_patterns(char *dst, char *src, char **patterns, int dstlen)
 
 as_t evaluate_last_asn(struct aspath *as)
 {
-  if (!as) return SUCCESS;
+  if (!as) return 0;
 
   return as->last_as;
 }
@@ -836,6 +1013,24 @@ void copy_stdcomm_to_asn(char *stdcomm, as_t *asn, int is_origin)
   else *asn = atoi(p1);
 }
 
+void copy_lrgcomm_to_asn(char *lrgcomm, as_t *asn, int is_origin)
+{
+  char *delim, *delim2;
+  char *p1, *p2, *endptr;
+
+  if (!lrgcomm || !strlen(lrgcomm) || (delim = strchr(lrgcomm, ':')) == NULL) return;
+  if (validate_truefalse(is_origin)) return;
+
+  delim2 = strchr(lrgcomm, ':');
+  *delim = '\0';
+  *delim2 = '\0';
+  p1 = lrgcomm;
+  p2 = delim+1;
+
+  if (is_origin) *asn = strtoul(p2, &endptr, 10);
+  else *asn = strtoul(p1, &endptr, 10);
+}
+
 /* XXX: currently only BGP is supported due to use of peers struct */
 void write_neighbors_file(char *filename, int type)
 {
@@ -871,7 +1066,6 @@ void write_neighbors_file(char *filename, int type)
 	  neighbor[len] = '\0';
           fwrite(neighbor, len, 1, file);
         }
-#if defined ENABLE_IPV6
 	else if (peers[idx].addr.family == AF_INET6) {
           inet_ntop(AF_INET6, &peers[idx].addr.address.ipv6, neighbor, INET6_ADDRSTRLEN);
           len = strlen(neighbor);
@@ -879,7 +1073,6 @@ void write_neighbors_file(char *filename, int type)
           neighbor[len] = '\0';
           fwrite(neighbor, len, 1, file);
         }
-#endif
       }
     }
 
@@ -906,7 +1099,7 @@ void bgp_config_checks(struct configuration *c)
       printf("       peer_src_as     =>  bgp_peer_src_as_type\n");
       printf("       src_local_pref  =>  bgp_src_local_pref_type\n");
       printf("       src_med         =>  bgp_src_med_type\n");
-      exit(1);
+      exit_gracefully(1);
     }
 
     c->data_type |= PIPE_TYPE_BGP;
@@ -925,7 +1118,7 @@ void bgp_config_checks(struct configuration *c)
       printf("       src_std_comm    =>  bgp_src_std_comm_type\n");
       printf("       src_ext_comm    =>  bgp_src_ext_comm_type\n");
       printf("       src_lrg_comm    =>  bgp_src_lrg_comm_type\n");
-      exit(1);
+      exit_gracefully(1);
     }
 
     if (c->type_id == PLUGIN_ID_MEMORY) c->data_type |= PIPE_TYPE_LBGP;
@@ -949,7 +1142,7 @@ void bgp_md5_file_load(char *filename, struct bgp_md5_table *t)
 
     if ((file = fopen(filename, "r")) == NULL) {
       Log(LOG_ERR, "ERROR ( %s/core/BGP ): [%s] file not found.\n", config.name, filename);
-      exit(1);
+      exit_gracefully(1);
     }
 
     while (!feof(file)) {
@@ -960,7 +1153,7 @@ void bgp_md5_file_load(char *filename, struct bgp_md5_table *t)
       memset(buf, 0, SRVBUFLEN);
       if (fgets(buf, SRVBUFLEN, file)) {
         if (!sanitize_buf(buf)) {
-          char *endptr, *token;
+          char *token;
           int tk_idx = 0, ret = 0, len = 0;
 
           ptr = buf;
@@ -1003,10 +1196,12 @@ void bgp_md5_file_unload(struct bgp_md5_table *t)
 
 void bgp_md5_file_process(int sock, struct bgp_md5_table *bgp_md5)
 {
+  char peer_str[INET6_ADDRSTRLEN + PORT_STRLEN + 1];
   struct pm_tcp_md5sig md5sig;
   struct sockaddr_storage ss_md5sig, ss_server;
   struct sockaddr *sa_md5sig = (struct sockaddr *)&ss_md5sig, *sa_server = (struct sockaddr *)&ss_server;
-  int rc, keylen, idx = 0, ss_md5sig_len, ss_server_len;
+  int rc, keylen, idx = 0, ss_md5sig_len;
+  socklen_t ss_server_len;
 
   if (!bgp_md5) return;
 
@@ -1019,12 +1214,14 @@ void bgp_md5_file_process(int sock, struct bgp_md5_table *bgp_md5)
     ss_server_len = sizeof(ss_server);
     getsockname(sock, (struct sockaddr *)&ss_server, &ss_server_len);
 
-#if defined ENABLE_IPV6
-    if (sa_md5sig->sa_family == AF_INET6 && sa_server->sa_family == AF_INET)
+    if (sa_md5sig->sa_family == AF_INET6 && sa_server->sa_family == AF_INET) {
       ipv4_mapped_to_ipv4(&ss_md5sig);
-    else if (sa_md5sig->sa_family == AF_INET && sa_server->sa_family == AF_INET6)
+      ss_md5sig_len = sizeof(struct sockaddr_in);
+    }
+    else if (sa_md5sig->sa_family == AF_INET && sa_server->sa_family == AF_INET6) {
       ipv4_to_ipv4_mapped(&ss_md5sig);
-#endif
+      ss_md5sig_len = sizeof(struct sockaddr_in6);
+    }
 
     memcpy(&md5sig.tcpm_addr, &ss_md5sig, ss_md5sig_len);
     keylen = strlen(bgp_md5->table[idx].key);
@@ -1033,8 +1230,15 @@ void bgp_md5_file_process(int sock, struct bgp_md5_table *bgp_md5)
       memcpy(md5sig.tcpm_key, &bgp_md5->table[idx].key, keylen);
     }
 
-    rc = setsockopt(sock, IPPROTO_TCP, TCP_MD5SIG, &md5sig, sizeof(md5sig));
-    if (rc < 0) Log(LOG_ERR, "WARN ( %s/core/BGP ): setsockopt() failed for TCP_MD5SIG (errno: %d).\n", config.name, errno);
+    sa_to_str(peer_str, sizeof(peer_str), sa_md5sig);
+
+    rc = setsockopt(sock, IPPROTO_TCP, TCP_MD5SIG, &md5sig, (socklen_t) sizeof(md5sig));
+    if (rc < 0) {
+      Log(LOG_WARNING, "WARN ( %s/core/BGP ): setsockopt() failed for TCP_MD5SIG peer=%s (errno: %d)\n", config.name, peer_str, errno);
+    }
+    else { 
+      Log(LOG_DEBUG, "DEBUG ( %s/core/BGP ): setsockopt() set TCP_MD5SIG peer=%s\n", config.name, peer_str);
+    }
 
     idx++;
   }
@@ -1140,22 +1344,28 @@ void bgp_link_misc_structs(struct bgp_misc_structs *bms)
   bms->msglog_kafka_host = &bgp_daemon_msglog_kafka_host;
 #endif
   bms->max_peers = config.nfacctd_bgp_max_peers;
+  bms->peers = peers;
+  bms->peers_cache = peers_cache;
+  bms->peers_port_cache = peers_port_cache;
+  bms->xconnects = &bgp_xcs_map;
   bms->neighbors_file = config.nfacctd_bgp_neighbors_file; 
   bms->dump_file = config.bgp_table_dump_file; 
   bms->dump_amqp_routing_key = config.bgp_table_dump_amqp_routing_key; 
   bms->dump_amqp_routing_key_rr = config.bgp_table_dump_amqp_routing_key_rr;
   bms->dump_kafka_topic = config.bgp_table_dump_kafka_topic;
   bms->dump_kafka_topic_rr = config.bgp_table_dump_kafka_topic_rr;
+  bms->dump_kafka_avro_schema_registry = config.bgp_table_dump_kafka_avro_schema_registry;
   bms->msglog_file = config.nfacctd_bgp_msglog_file;
   bms->msglog_output = config.nfacctd_bgp_msglog_output;
   bms->msglog_amqp_routing_key = config.nfacctd_bgp_msglog_amqp_routing_key;
   bms->msglog_amqp_routing_key_rr = config.nfacctd_bgp_msglog_amqp_routing_key_rr;
   bms->msglog_kafka_topic = config.nfacctd_bgp_msglog_kafka_topic;
   bms->msglog_kafka_topic_rr = config.nfacctd_bgp_msglog_kafka_topic_rr;
+  bms->msglog_kafka_avro_schema_registry = config.nfacctd_bgp_msglog_kafka_avro_schema_registry;
   bms->peer_str = malloc(strlen("peer_ip_src") + 1);
   strcpy(bms->peer_str, "peer_ip_src");
-  bms->peer_port_str = malloc(strlen("peer_ip_src_port") + 1);
-  strcpy(bms->peer_port_str, "peer_ip_src_port");
+  bms->peer_port_str = malloc(strlen("peer_tcp_port") + 1);
+  strcpy(bms->peer_port_str, "peer_tcp_port");
   bms->bgp_peer_log_msg_extras = NULL;
   bms->bgp_peer_logdump_initclose_extras = NULL;
 
@@ -1167,17 +1377,36 @@ void bgp_link_misc_structs(struct bgp_misc_structs *bms)
   bms->bgp_lookup_find_peer = bgp_lookup_find_bgp_peer;
   bms->bgp_lookup_node_match_cmp = bgp_lookup_node_match_cmp_bgp;
 
-  if (!bms->is_thread && !bms->dump_backend_methods) bms->skip_rib = TRUE;
+  bms->bgp_msg_open_router_id_check = bgp_router_id_check; 
+
+  if (!bms->is_thread && !bms->dump_backend_methods && !bms->has_lglass && !bms->has_blackhole) {
+    bms->skip_rib = TRUE;
+  }
+}
+
+void bgp_blackhole_link_misc_structs(struct bgp_misc_structs *m_data)
+{
+  m_data->table_peer_buckets = 1; /* saving on DEFAULT_BGP_INFO_HASH for now */
+  m_data->table_per_peer_buckets = DEFAULT_BGP_INFO_PER_PEER_HASH;
+  m_data->table_attr_hash_buckets = HASHTABSIZE;
+  m_data->table_per_peer_hash = BGP_ASPATH_HASH_PATHID;
+  m_data->route_info_modulo = NULL;
+  m_data->bgp_lookup_node_match_cmp = NULL;
 }
 
 int bgp_peer_cmp(const void *a, const void *b)
 {
-  return memcmp(&((struct bgp_peer *)a)->addr, &((struct bgp_peer *)b)->addr, sizeof(struct host_addr));
+  return host_addr_cmp(&((struct bgp_peer *)a)->addr, &((struct bgp_peer *)b)->addr);
 }
 
 int bgp_peer_host_addr_cmp(const void *a, const void *b)
 {
-  return memcmp(a, &((struct bgp_peer *)b)->addr, sizeof(struct host_addr));
+  return host_addr_cmp((struct host_addr *)a, &((struct bgp_peer *)b)->addr);
+}
+
+int bgp_peer_sa_addr_cmp(const void *a, const void *b)
+{
+  return sa_addr_cmp((struct sockaddr *) a, &((struct bgp_peer *)b)->addr);
 }
 
 void bgp_peer_free(void *a)
@@ -1227,3 +1456,84 @@ int bgp_peers_bintree_walk_delete(const void *nodep, const pm_VISIT which, const
 
   return TRUE;
 }
+
+int bgp_router_id_check(struct bgp_msg_data *bmd)
+{
+  struct bgp_peer *peer = bmd->peer;
+  struct bgp_misc_structs *bms;
+  struct bgp_peer *peers_check;
+  int peers_check_idx = 0;
+
+  bms = bgp_select_misc_db(peer->type);
+
+  if (!bms) return ERR;
+
+  peers_check = bms->peers;
+
+  for (; peers_check_idx < bms->max_peers; peers_check_idx++) {
+    if (peers_check_idx != peer->idx && !memcmp(&peers_check[peers_check_idx].id, &peer->id, sizeof(peers_check[peers_check_idx].id))) {
+      char bgp_peer_str[INET6_ADDRSTRLEN];
+
+      bgp_peer_print(&peers_check[peers_check_idx], bgp_peer_str, INET6_ADDRSTRLEN);
+      Log(LOG_WARNING, "WARN ( %s/%s ): [%s] Refusing new connection from existing Router-ID.\n", config.name, bms->log_str, bgp_peer_str);
+
+      return ERR;
+    }
+  }
+
+  return FALSE;
+}
+
+/*
+  utility function when an ASN is prepended by 'AS', 'AS ', etc. strings
+  that need to be stripped.
+*/
+int bgp_str2asn(char *asn_str, as_t *asn)
+{
+  char *endptr, *asn_ptr = asn_str;
+  int len, cnt;
+
+  if (!asn_str || !asn) return ERR;
+
+  len = strlen(asn_str);
+
+  for (cnt = 0; !isdigit(asn_str[cnt]) && (cnt < len); cnt++);
+
+  asn_ptr = &asn_str[cnt];
+
+  (*asn) = strtoul(asn_ptr, &endptr, 10);
+  if (endptr == asn_ptr || (*endptr) != '\0') return ERR;
+  if (errno) {
+    errno = FALSE;
+    return ERR;
+  }
+
+  return SUCCESS;
+}
+
+const char *bgp_origin_print(u_int8_t origin)
+{
+  if (origin <= BGP_ORIGIN_MAX) return bgp_origin[origin];
+  else return bgp_origin[BGP_ORIGIN_UNKNOWN];
+}
+
+u_int8_t bgp_str2origin(char *origin_str)
+{
+  if (!strcmp(origin_str, "i")) return BGP_ORIGIN_IGP;
+  else if (!strcmp(origin_str, "e")) return BGP_ORIGIN_EGP;
+  else if (!strcmp(origin_str, "u")) return BGP_ORIGIN_INCOMPLETE;
+
+  return BGP_ORIGIN_UNKNOWN;
+}
+
+u_int16_t bgp_get_packet_len(char *pkt)
+{
+  struct bgp_header *bhdr = (struct bgp_header *) pkt;
+  u_int16_t blen = 0;
+
+  if (bgp_marker_check(bhdr, BGP_MARKER_SIZE) != ERR) {
+    blen = ntohs(bhdr->bgpo_len);
+  }
+
+  return blen;
+}
