@@ -1,6 +1,6 @@
 /*  
     pmacct (Promiscuous mode IP Accounting package)
-    pmacct is Copyright (C) 2003-2017 by Paolo Lucente
+    pmacct is Copyright (C) 2003-2019 by Paolo Lucente
 */
 
 /*
@@ -19,14 +19,12 @@
     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 
-/* defines */
-#define __BMP_C
-
 /* includes */
 #include "pmacct.h"
 #include "addr.h"
-#include "../bgp/bgp.h"
+#include "bgp/bgp.h"
 #include "bmp.h"
+#include "rpki/rpki.h"
 #include "thread_pool.h"
 #if defined WITH_RABBITMQ
 #include "amqp_common.h"
@@ -34,12 +32,14 @@
 #ifdef WITH_KAFKA
 #include "kafka_common.h"
 #endif
+#if defined WITH_AVRO
+#include "plugin_cmn_avro.h"
+#endif
 
 /* variables to be exported away */
 thread_pool_t *bmp_pool;
 
 /* Functions */
-#if defined ENABLE_THREADS
 void nfacctd_bmp_wrapper()
 {
   /* initialize variables */
@@ -54,39 +54,38 @@ void nfacctd_bmp_wrapper()
   /* giving a kick to the BMP thread */
   send_to_pool(bmp_pool, skinny_bmp_daemon, NULL);
 }
-#endif
 
 void skinny_bmp_daemon()
 {
-  int slen, clen, ret, rc, peers_idx, allowed, yes=1, no=0;
+  int ret, rc, peers_idx, allowed, yes=1;
   int peers_idx_rr = 0, max_peers_idx = 0;
   u_int32_t pkt_remaining_len=0;
   time_t now;
   afi_t afi;
   safi_t safi;
+  socklen_t slen, clen;
 
   struct bmp_peer *bmpp = NULL;
   struct bgp_peer *peer = NULL;
 
-#if defined ENABLE_IPV6
   struct sockaddr_storage server, client;
-#else
-  struct sockaddr server, client;
-#endif
   struct hosts_table allow;
   struct host_addr addr;
   struct bgp_peer_batch bp_batch;
 
+  sigset_t signal_set;
+
   /* select() stuff */
   fd_set read_descs, bkp_read_descs;
   int fd, select_fd, bkp_select_fd, recalc_fds, select_num;
 
   /* logdump time management */
-  time_t dump_refresh_deadline;
+  time_t dump_refresh_deadline = {0};
   struct timeval dump_refresh_timeout, *drt_ptr;
 
 
   /* initial cleanups */
+  reload_map_bmp_thread = FALSE;
   reload_log_bmp_thread = FALSE;
   memset(&server, 0, sizeof(server));
   memset(&client, 0, sizeof(client));
@@ -97,7 +96,6 @@ void skinny_bmp_daemon()
   memset(bmp_routing_db, 0, sizeof(struct bgp_rt_structs));
 
   /* socket creation for BMP server: IPv4 only */
-#if (defined ENABLE_IPV6)
   if (!config.nfacctd_bmp_ip) {
     struct sockaddr_in6 *sa6 = (struct sockaddr_in6 *)&server;
 
@@ -105,22 +103,12 @@ void skinny_bmp_daemon()
     sa6->sin6_port = htons(config.nfacctd_bmp_port);
     slen = sizeof(struct sockaddr_in6);
   }
-#else
-  if (!config.nfacctd_bmp_ip) {
-    struct sockaddr_in *sa4 = (struct sockaddr_in *)&server;
-
-    sa4->sin_family = AF_INET;
-    sa4->sin_addr.s_addr = htonl(0);
-    sa4->sin_port = htons(config.nfacctd_bmp_port);
-    slen = sizeof(struct sockaddr_in);
-  }
-#endif
   else {
     trim_spaces(config.nfacctd_bmp_ip);
     ret = str_to_addr(config.nfacctd_bmp_ip, &addr);
     if (!ret) {
       Log(LOG_ERR, "ERROR ( %s/%s ): 'bmp_daemon_ip' value is not a valid IPv4/IPv6 address. Terminating thread.\n", config.name, bmp_misc_db->log_str);
-      exit_all(1);
+      exit_gracefully(1);
     }
     slen = addr_to_sa((struct sockaddr *)&server, &addr, config.nfacctd_bmp_port);
   }
@@ -131,10 +119,17 @@ void skinny_bmp_daemon()
   bmp_peers = malloc(config.nfacctd_bmp_max_peers*sizeof(struct bmp_peer));
   if (!bmp_peers) {
     Log(LOG_ERR, "ERROR ( %s/%s ): Unable to malloc() BMP peers structure. Terminating thread.\n", config.name, bmp_misc_db->log_str);
-    exit_all(1);
+    exit_gracefully(1);
   }
   memset(bmp_peers, 0, config.nfacctd_bmp_max_peers*sizeof(struct bmp_peer));
 
+  if (config.rpki_roas_file || config.rpki_rtr_cache) {
+    rpki_daemon_wrapper();
+
+    /* Let's give the RPKI thread some advantage to create its structures */
+    sleep(DEFAULT_SLOTH_SLEEP_TIME);
+  }
+
   if (config.nfacctd_bmp_msglog_file || config.nfacctd_bmp_msglog_amqp_routing_key || config.nfacctd_bmp_msglog_kafka_topic) {
     if (config.nfacctd_bmp_msglog_file) bmp_misc_db->msglog_backend_methods++;
     if (config.nfacctd_bmp_msglog_amqp_routing_key) bmp_misc_db->msglog_backend_methods++;
@@ -142,7 +137,7 @@ void skinny_bmp_daemon()
 
     if (bmp_misc_db->msglog_backend_methods > 1) {
       Log(LOG_ERR, "ERROR ( %s/%s ): bmp_daemon_msglog_file, bmp_daemon_msglog_amqp_routing_key and bmp_daemon_msglog_kafka_topic are mutually exclusive. Terminating thread.\n", config.name, bmp_misc_db->log_str);
-      exit_all(1);
+      exit_gracefully(1);
     }
   }
 
@@ -153,7 +148,7 @@ void skinny_bmp_daemon()
 
     if (bmp_misc_db->dump_backend_methods > 1) {
       Log(LOG_ERR, "ERROR ( %s/%s ): bmp_dump_file, bmp_dump_amqp_routing_key and bmp_dump_kafka_topic are mutually exclusive. Terminating thread.\n", config.name, bmp_misc_db->log_str);
-      exit_all(1);
+      exit_gracefully(1);
     }
   }
 
@@ -164,7 +159,7 @@ void skinny_bmp_daemon()
     bmp_misc_db->peers_log = malloc(config.nfacctd_bmp_max_peers*sizeof(struct bgp_peer_log));
     if (!bmp_misc_db->peers_log) {
       Log(LOG_ERR, "ERROR ( %s/%s ): Unable to malloc() BMP peers log structure. Terminating thread.\n", config.name, bmp_misc_db->log_str);
-      exit_all(1);
+      exit_gracefully(1);
     }
     memset(bmp_misc_db->peers_log, 0, config.nfacctd_bmp_max_peers*sizeof(struct bgp_peer_log));
 
@@ -199,12 +194,11 @@ void skinny_bmp_daemon()
     bmp_route_info_modulo = bmp_route_info_modulo_pathid;
   else {
     Log(LOG_ERR, "ERROR ( %s/%s ): Unknown 'bmp_table_per_peer_hash' value. Terminating thread.\n", config.name, bmp_misc_db->log_str);
-    exit_all(1);
+    exit_gracefully(1);
   }
 
   config.bmp_sock = socket(((struct sockaddr *)&server)->sa_family, SOCK_STREAM, 0);
   if (config.bmp_sock < 0) {
-#if (defined ENABLE_IPV6)
     /* retry with IPv4 */
     if (!config.nfacctd_bmp_ip) {
       struct sockaddr_in *sa4 = (struct sockaddr_in *)&server;
@@ -216,34 +210,42 @@ void skinny_bmp_daemon()
 
       config.bmp_sock = socket(((struct sockaddr *)&server)->sa_family, SOCK_STREAM, 0);
     }
-#endif
 
     if (config.bmp_sock < 0) {
       Log(LOG_ERR, "ERROR ( %s/%s ): thread socket() failed. Terminating thread.\n", config.name, bmp_misc_db->log_str);
-      exit_all(1);
+      exit_gracefully(1);
     }
   }
   if (config.nfacctd_bmp_ipprec) {
     int opt = config.nfacctd_bmp_ipprec << 5;
 
-    rc = setsockopt(config.bmp_sock, IPPROTO_IP, IP_TOS, &opt, sizeof(opt));
+    rc = setsockopt(config.bmp_sock, IPPROTO_IP, IP_TOS, &opt, (socklen_t) sizeof(opt));
     if (rc < 0) Log(LOG_ERR, "WARN ( %s/%s ): setsockopt() failed for IP_TOS (errno: %d).\n", config.name, bmp_misc_db->log_str, errno);
   }
 
-  rc = setsockopt(config.bmp_sock, SOL_SOCKET, SO_REUSEADDR, (char *)&yes, sizeof(yes));
+#if (defined LINUX) && (defined HAVE_SO_REUSEPORT)
+  rc = setsockopt(config.bmp_sock, SOL_SOCKET, SO_REUSEADDR|SO_REUSEPORT, (char *)&yes, (socklen_t) sizeof(yes));
+  if (rc < 0) Log(LOG_ERR, "WARN ( %s/%s ): setsockopt() failed for SO_REUSEADDR|SO_REUSEPORT (errno: %d).\n", config.name, bmp_misc_db->log_str, errno);
+#else
+  rc = setsockopt(config.bmp_sock, SOL_SOCKET, SO_REUSEADDR, (char *) &yes, (socklen_t) sizeof(yes));
   if (rc < 0) Log(LOG_ERR, "WARN ( %s/%s ): setsockopt() failed for SO_REUSEADDR (errno: %d).\n", config.name, bmp_misc_db->log_str, errno);
+#endif
 
-#if (defined ENABLE_IPV6) && (defined IPV6_BINDV6ONLY)
-  rc = setsockopt(config.bmp_sock, IPPROTO_IPV6, IPV6_BINDV6ONLY, (char *) &no, (socklen_t) sizeof(no));
-  if (rc < 0) Log(LOG_ERR, "WARN ( %s/%s ): setsockopt() failed for IPV6_BINDV6ONLY (errno: %d).\n", config.name, bmp_misc_db->log_str, errno);
+#if (defined IPV6_BINDV6ONLY)
+  {
+    int no=0;
+
+    rc = setsockopt(config.bmp_sock, IPPROTO_IPV6, IPV6_BINDV6ONLY, (char *) &no, (socklen_t) sizeof(no));
+    if (rc < 0) Log(LOG_ERR, "WARN ( %s/%s ): setsockopt() failed for IPV6_BINDV6ONLY (errno: %d).\n", config.name, bmp_misc_db->log_str, errno);
+  }
 #endif
 
   if (config.nfacctd_bmp_pipe_size) {
-    int l = sizeof(config.nfacctd_bmp_pipe_size);
+    socklen_t l = sizeof(config.nfacctd_bmp_pipe_size);
     int saved = 0, obtained = 0;
 
     getsockopt(config.bmp_sock, SOL_SOCKET, SO_RCVBUF, &saved, &l);
-    Setsocksize(config.bmp_sock, SOL_SOCKET, SO_RCVBUF, &config.nfacctd_bmp_pipe_size, sizeof(config.nfacctd_bmp_pipe_size));
+    Setsocksize(config.bmp_sock, SOL_SOCKET, SO_RCVBUF, &config.nfacctd_bmp_pipe_size, (socklen_t) sizeof(config.nfacctd_bmp_pipe_size));
     getsockopt(config.bmp_sock, SOL_SOCKET, SO_RCVBUF, &obtained, &l);
 
     Setsocksize(config.bmp_sock, SOL_SOCKET, SO_RCVBUF, &saved, l);
@@ -258,13 +260,13 @@ void skinny_bmp_daemon()
 
     ip_address = config.nfacctd_bmp_ip ? config.nfacctd_bmp_ip : null_ip_address;
     Log(LOG_ERR, "ERROR ( %s/%s ): bind() to ip=%s port=%d/tcp failed (errno: %d).\n", config.name, bmp_misc_db->log_str, ip_address, config.nfacctd_bmp_port, errno);
-    exit_all(1);
+    exit_gracefully(1);
   }
 
   rc = listen(config.bmp_sock, 1);
   if (rc < 0) {
     Log(LOG_ERR, "ERROR ( %s/%s ): listen() failed (errno: %d).\n", config.name, bmp_misc_db->log_str, errno);
-    exit_all(1);
+    exit_gracefully(1);
   }
 
   /* Preparing for syncronous I/O multiplexing */
@@ -307,6 +309,113 @@ void skinny_bmp_daemon()
 #else
     Log(LOG_WARNING, "WARN ( %s/%s ): bmp_daemon_msglog_output set to json but will produce no output (missing --enable-jansson).\n", config.name, bmp_misc_db->log_str);
 #endif
+
+#ifdef WITH_AVRO
+    if ((config.nfacctd_bmp_msglog_output == PRINT_OUTPUT_AVRO_BIN) ||
+	(config.nfacctd_bmp_msglog_output == PRINT_OUTPUT_AVRO_JSON)) {
+      assert(BMP_MSG_TYPE_MAX < BMP_LOG_TYPE_LOGINIT);
+      assert(BMP_LOG_TYPE_MAX < MAX_AVRO_SCHEMA);
+
+      bmp_misc_db->msglog_avro_schema[BMP_MSG_ROUTE_MONITOR] = avro_schema_build_bmp_rm(BGP_LOGDUMP_ET_LOG, "bmp_msglog_rm");
+      bmp_misc_db->msglog_avro_schema[BMP_MSG_STATS] = avro_schema_build_bmp_stats("bmp_stats");
+      bmp_misc_db->msglog_avro_schema[BMP_MSG_PEER_DOWN] = avro_schema_build_bmp_peer_down("bmp_peer_down");
+      bmp_misc_db->msglog_avro_schema[BMP_MSG_PEER_UP] = avro_schema_build_bmp_peer_up("bmp_peer_up");
+      bmp_misc_db->msglog_avro_schema[BMP_MSG_INIT] = avro_schema_build_bmp_init("bmp_init");
+      bmp_misc_db->msglog_avro_schema[BMP_MSG_TERM] = avro_schema_build_bmp_term("bmp_term");
+
+      bmp_misc_db->msglog_avro_schema[BMP_LOG_TYPE_LOGINIT] = avro_schema_build_bmp_log_initclose(BGP_LOGDUMP_ET_LOG, "bmp_loginit");
+      bmp_misc_db->msglog_avro_schema[BMP_LOG_TYPE_LOGCLOSE] = avro_schema_build_bmp_log_initclose(BGP_LOGDUMP_ET_LOG, "bmp_logclose");
+
+      if (config.nfacctd_bmp_msglog_avro_schema_file) {
+	char avro_schema_file[SRVBUFLEN];
+
+	if (strlen(config.nfacctd_bmp_msglog_avro_schema_file) > (SRVBUFLEN - SUPERSHORTBUFLEN)) {
+	  Log(LOG_ERR, "ERROR ( %s/%s ): 'bmp_daemon_msglog_avro_schema_file' too long. Exiting.\n", config.name, bmp_misc_db->log_str);
+	  exit_gracefully(1);
+	}
+
+	write_avro_schema_to_file_with_suffix(config.nfacctd_bmp_msglog_avro_schema_file, "-bmp_msglog_rm",
+					      avro_schema_file, bmp_misc_db->msglog_avro_schema[BMP_MSG_ROUTE_MONITOR]);
+
+	write_avro_schema_to_file_with_suffix(config.nfacctd_bmp_msglog_avro_schema_file, "-bmp_stats",
+					      avro_schema_file, bmp_misc_db->msglog_avro_schema[BMP_MSG_STATS]);
+
+	write_avro_schema_to_file_with_suffix(config.nfacctd_bmp_msglog_avro_schema_file, "-bmp_peer_down",
+					      avro_schema_file, bmp_misc_db->msglog_avro_schema[BMP_MSG_PEER_DOWN]);
+
+	write_avro_schema_to_file_with_suffix(config.nfacctd_bmp_msglog_avro_schema_file, "-bmp_peer_up",
+					      avro_schema_file, bmp_misc_db->msglog_avro_schema[BMP_MSG_PEER_UP]);
+
+	write_avro_schema_to_file_with_suffix(config.nfacctd_bmp_msglog_avro_schema_file, "-bmp_init",
+					      avro_schema_file, bmp_misc_db->msglog_avro_schema[BMP_MSG_INIT]);
+
+	write_avro_schema_to_file_with_suffix(config.nfacctd_bmp_msglog_avro_schema_file, "-bmp_term",
+					      avro_schema_file, bmp_misc_db->msglog_avro_schema[BMP_MSG_TERM]);
+
+	write_avro_schema_to_file_with_suffix(config.nfacctd_bmp_msglog_avro_schema_file, "-bmp_loginit",
+					      avro_schema_file, bmp_misc_db->msglog_avro_schema[BMP_LOG_TYPE_LOGINIT]);
+
+	write_avro_schema_to_file_with_suffix(config.nfacctd_bmp_msglog_avro_schema_file, "-bmp_logclose",
+					      avro_schema_file, bmp_misc_db->msglog_avro_schema[BMP_LOG_TYPE_LOGCLOSE]);
+      }
+
+      if (config.nfacctd_bmp_msglog_kafka_avro_schema_registry) {
+#ifdef WITH_SERDES
+        if (strchr(config.nfacctd_bmp_msglog_kafka_topic, '$')) {
+	  Log(LOG_ERR, "ERROR ( %s/%s ): dynamic 'bmp_daemon_msglog_kafka_topic' is not compatible with 'bmp_daemon_msglog_kafka_avro_schema_registry'. Exiting.\n",
+	      config.name, bmp_misc_db->log_str);
+	  exit_gracefully(1);
+	}
+
+	if (config.nfacctd_bmp_msglog_output == PRINT_OUTPUT_AVRO_JSON) {
+	  Log(LOG_ERR, "ERROR ( %s/%s ): 'avro_json' output is not compatible with 'bmp_daemon_msglog_kafka_avro_schema_registry'. Exiting.\n",
+	      config.name, bmp_misc_db->log_str);
+	  exit_gracefully(1);
+        }
+	
+        bmp_daemon_msglog_kafka_host.sd_schema[BMP_MSG_ROUTE_MONITOR] = compose_avro_schema_registry_name_2(config.nfacctd_bmp_msglog_kafka_topic, FALSE,
+												     bmp_misc_db->msglog_avro_schema[BMP_MSG_ROUTE_MONITOR],
+												     "bmp", "msglog_rm",
+												     config.nfacctd_bmp_msglog_kafka_avro_schema_registry);
+
+        bmp_daemon_msglog_kafka_host.sd_schema[BMP_MSG_STATS] = compose_avro_schema_registry_name_2(config.nfacctd_bmp_msglog_kafka_topic, FALSE,
+												     bmp_misc_db->msglog_avro_schema[BMP_MSG_STATS],
+												     "bmp", "stats",
+												     config.nfacctd_bmp_msglog_kafka_avro_schema_registry);
+
+        bmp_daemon_msglog_kafka_host.sd_schema[BMP_MSG_PEER_UP] = compose_avro_schema_registry_name_2(config.nfacctd_bmp_msglog_kafka_topic, FALSE,
+												     bmp_misc_db->msglog_avro_schema[BMP_MSG_PEER_UP],
+												     "bmp", "peer_up",
+												     config.nfacctd_bmp_msglog_kafka_avro_schema_registry);
+
+        bmp_daemon_msglog_kafka_host.sd_schema[BMP_MSG_PEER_DOWN] = compose_avro_schema_registry_name_2(config.nfacctd_bmp_msglog_kafka_topic, FALSE,
+												     bmp_misc_db->msglog_avro_schema[BMP_MSG_PEER_DOWN],
+												     "bmp", "peer_down",
+												     config.nfacctd_bmp_msglog_kafka_avro_schema_registry);
+
+        bmp_daemon_msglog_kafka_host.sd_schema[BMP_MSG_INIT] = compose_avro_schema_registry_name_2(config.nfacctd_bmp_msglog_kafka_topic, FALSE,
+												     bmp_misc_db->msglog_avro_schema[BMP_MSG_INIT],
+												     "bmp", "init",
+												     config.nfacctd_bmp_msglog_kafka_avro_schema_registry);
+
+        bmp_daemon_msglog_kafka_host.sd_schema[BMP_MSG_TERM] = compose_avro_schema_registry_name_2(config.nfacctd_bmp_msglog_kafka_topic, FALSE,
+												     bmp_misc_db->msglog_avro_schema[BMP_MSG_TERM],
+												     "bmp", "term",
+												     config.nfacctd_bmp_msglog_kafka_avro_schema_registry);
+
+	bmp_daemon_msglog_kafka_host.sd_schema[BMP_LOG_TYPE_LOGINIT] = compose_avro_schema_registry_name_2(config.nfacctd_bmp_msglog_kafka_topic, FALSE,
+												     bmp_misc_db->msglog_avro_schema[BMP_LOG_TYPE_LOGINIT],
+												     "bmp", "loginit",
+												     config.nfacctd_bmp_msglog_kafka_avro_schema_registry);
+
+	bmp_daemon_msglog_kafka_host.sd_schema[BMP_LOG_TYPE_LOGCLOSE] = compose_avro_schema_registry_name_2(config.nfacctd_bmp_msglog_kafka_topic, FALSE,
+												     bmp_misc_db->msglog_avro_schema[BMP_LOG_TYPE_LOGCLOSE],
+												     "bmp", "logclose",
+												     config.nfacctd_bmp_msglog_kafka_avro_schema_registry);
+#endif
+      }
+    }
+#endif
   }
 
   if (bmp_misc_db->dump_backend_methods) {
@@ -315,6 +424,57 @@ void skinny_bmp_daemon()
 #else
     Log(LOG_WARNING, "WARN ( %s/%s ): bmp_table_dump_output set to json but will produce no output (missing --enable-jansson).\n", config.name, bmp_misc_db->log_str);
 #endif
+
+#ifdef WITH_AVRO
+    if ((config.bmp_dump_output == PRINT_OUTPUT_AVRO_BIN) || 
+	(config.bmp_dump_output == PRINT_OUTPUT_AVRO_JSON)) {
+      assert(BMP_MSG_TYPE_MAX < BMP_LOG_TYPE_LOGINIT);
+      assert(BMP_LOG_TYPE_MAX < MAX_AVRO_SCHEMA);
+
+      bmp_misc_db->dump_avro_schema[BMP_MSG_ROUTE_MONITOR] = avro_schema_build_bmp_rm(BGP_LOGDUMP_ET_DUMP, "bmp_dump_rm");
+      bmp_misc_db->dump_avro_schema[BMP_MSG_STATS] = avro_schema_build_bmp_stats("bmp_stats");
+      bmp_misc_db->dump_avro_schema[BMP_MSG_PEER_DOWN] = avro_schema_build_bmp_peer_down("bmp_peer_down");
+      bmp_misc_db->dump_avro_schema[BMP_MSG_PEER_UP] = avro_schema_build_bmp_peer_up("bmp_peer_up");
+      bmp_misc_db->dump_avro_schema[BMP_MSG_INIT] = avro_schema_build_bmp_init("bmp_init");
+      bmp_misc_db->dump_avro_schema[BMP_MSG_TERM] = avro_schema_build_bmp_term("bmp_term");
+
+      bmp_misc_db->dump_avro_schema[BMP_LOG_TYPE_DUMPINIT] = avro_schema_build_bmp_dump_init(BGP_LOGDUMP_ET_DUMP, "bmp_dumpinit");
+      bmp_misc_db->dump_avro_schema[BMP_LOG_TYPE_DUMPCLOSE] = avro_schema_build_bmp_dump_close(BGP_LOGDUMP_ET_DUMP, "bmp_dumpclose");
+
+      if (config.bmp_dump_avro_schema_file) {
+	char avro_schema_file[SRVBUFLEN];
+
+	if (strlen(config.bmp_dump_avro_schema_file) > (SRVBUFLEN - SUPERSHORTBUFLEN)) {
+	  Log(LOG_ERR, "ERROR ( %s/%s ): 'bmp_table_dump_avro_schema_file' too long. Exiting ..\n", config.name, bmp_misc_db->log_str);
+	  exit_gracefully(1);
+	}
+
+	write_avro_schema_to_file_with_suffix(config.bmp_dump_avro_schema_file, "-bmp_dump_rm",
+					      avro_schema_file, bmp_misc_db->dump_avro_schema[BMP_MSG_ROUTE_MONITOR]);
+
+	write_avro_schema_to_file_with_suffix(config.bmp_dump_avro_schema_file, "-bmp_stats",
+					      avro_schema_file, bmp_misc_db->dump_avro_schema[BMP_MSG_STATS]);
+
+	write_avro_schema_to_file_with_suffix(config.bmp_dump_avro_schema_file, "-bmp_peer_down",
+					      avro_schema_file, bmp_misc_db->dump_avro_schema[BMP_MSG_PEER_DOWN]);
+
+	write_avro_schema_to_file_with_suffix(config.bmp_dump_avro_schema_file, "-bmp_peer_up",
+					      avro_schema_file, bmp_misc_db->dump_avro_schema[BMP_MSG_PEER_UP]);
+
+	write_avro_schema_to_file_with_suffix(config.bmp_dump_avro_schema_file, "-bmp_init",
+					      avro_schema_file, bmp_misc_db->dump_avro_schema[BMP_MSG_INIT]);
+
+	write_avro_schema_to_file_with_suffix(config.bmp_dump_avro_schema_file, "-bmp_term",
+					      avro_schema_file, bmp_misc_db->dump_avro_schema[BMP_MSG_TERM]);
+
+	write_avro_schema_to_file_with_suffix(config.bmp_dump_avro_schema_file, "-bmp_dumpinit",
+					      avro_schema_file, bmp_misc_db->dump_avro_schema[BMP_LOG_TYPE_DUMPINIT]);
+
+	write_avro_schema_to_file_with_suffix(config.bmp_dump_avro_schema_file, "-bmp_dumpclose",
+					      avro_schema_file, bmp_misc_db->dump_avro_schema[BMP_LOG_TYPE_DUMPCLOSE]);
+      }
+    }
+#endif
   }
 
   if (bmp_misc_db->dump_backend_methods) {
@@ -341,14 +501,45 @@ void skinny_bmp_daemon()
     if (config.bmp_dump_kafka_topic) bmp_dump_init_kafka_host();
   }
 
+#ifdef WITH_AVRO
+  bmp_misc_db->avro_buf = malloc(LARGEBUFLEN);
+  if (!bmp_misc_db->avro_buf) {
+    Log(LOG_ERR, "ERROR ( %s/%s ): malloc() failed (avro_buf). Exiting ..\n", config.name, bmp_misc_db->log_str);
+    exit_gracefully(1);
+  }
+  else memset(bmp_misc_db->avro_buf, 0, LARGEBUFLEN);
+#endif
+
+  if (config.nfacctd_bmp_msglog_kafka_avro_schema_registry || config.bmp_dump_kafka_avro_schema_registry) {
+#ifndef WITH_SERDES
+      Log(LOG_ERR, "ERROR ( %s/%s ): 'bmp_*_kafka_avro_schema_registry' require --enable-serdes. Exiting.\n", config.name, bmp_misc_db->log_str);
+      exit_gracefully(1);
+#endif
+    }
+
   select_fd = bkp_select_fd = (config.bmp_sock + 1);
   recalc_fds = FALSE;
 
   bmp_link_misc_structs(bmp_misc_db);
 
+  sigemptyset(&signal_set);
+  sigaddset(&signal_set, SIGCHLD);
+  sigaddset(&signal_set, SIGHUP);
+  sigaddset(&signal_set, SIGUSR1);
+  sigaddset(&signal_set, SIGUSR2);
+  sigaddset(&signal_set, SIGTERM);
+  if (config.daemon) {
+    sigaddset(&signal_set, SIGINT);
+  }
+
   for (;;) {
     select_again:
 
+    if (!bmp_misc_db->is_thread) {
+      sigprocmask(SIG_UNBLOCK, &signal_set, NULL);
+      sigprocmask(SIG_BLOCK, &signal_set, NULL);
+    }
+
     if (recalc_fds) {
       select_fd = config.bmp_sock;
       max_peers_idx = -1; /* .. since valid indexes include 0 */
@@ -380,6 +571,12 @@ void skinny_bmp_daemon()
     select_num = select(select_fd, &read_descs, NULL, NULL, drt_ptr);
     if (select_num < 0) goto select_again;
 
+    if (reload_map_bmp_thread) {
+      if (config.nfacctd_bmp_allow_file) load_allow_file(config.nfacctd_bmp_allow_file, &allow);
+
+      reload_map_bmp_thread = FALSE;
+    }
+
     if (reload_log_bmp_thread) {
       for (peers_idx = 0; peers_idx < config.nfacctd_bmp_max_peers; peers_idx++) {
         if (bmp_misc_db->peers_log[peers_idx].fd) {
@@ -393,18 +590,35 @@ void skinny_bmp_daemon()
       reload_log_bmp_thread = FALSE;
     }
 
+    if (reload_log && !bmp_misc_db->is_thread) {
+      reload_logs();
+      reload_log = FALSE;
+    }
+
     if (bmp_misc_db->msglog_backend_methods || bmp_misc_db->dump_backend_methods) {
       gettimeofday(&bmp_misc_db->log_tstamp, NULL);
-      compose_timestamp(bmp_misc_db->log_tstamp_str, SRVBUFLEN, &bmp_misc_db->log_tstamp, TRUE, config.timestamps_since_epoch);
+      compose_timestamp(bmp_misc_db->log_tstamp_str, SRVBUFLEN, &bmp_misc_db->log_tstamp, TRUE,
+			config.timestamps_since_epoch, config.timestamps_rfc3339, config.timestamps_utc);
+
+      /* if dumping, let's reset log sequence at the next dump event */
+      if (!bmp_misc_db->dump_backend_methods) {
+	if (bgp_peer_log_seq_has_ro_bit(&bmp_misc_db->log_seq))
+	  bgp_peer_log_seq_init(&bmp_misc_db->log_seq);
+      }
 
       if (bmp_misc_db->dump_backend_methods) {
         while (bmp_misc_db->log_tstamp.tv_sec > dump_refresh_deadline) {
           bmp_misc_db->dump.tstamp.tv_sec = dump_refresh_deadline;
           bmp_misc_db->dump.tstamp.tv_usec = 0;
-          compose_timestamp(bmp_misc_db->dump.tstamp_str, SRVBUFLEN, &bmp_misc_db->dump.tstamp, FALSE, config.timestamps_since_epoch);
+          compose_timestamp(bmp_misc_db->dump.tstamp_str, SRVBUFLEN, &bmp_misc_db->dump.tstamp, FALSE,
+			    config.timestamps_since_epoch, config.timestamps_rfc3339, config.timestamps_utc);
 	  bmp_misc_db->dump.period = config.bmp_dump_refresh_time;
 
+	  if (bgp_peer_log_seq_has_ro_bit(&bmp_misc_db->log_seq))
+	    bgp_peer_log_seq_init(&bmp_misc_db->log_seq);
+
           bmp_handle_dump_event();
+
           dump_refresh_deadline += config.bmp_dump_refresh_time;
         }
       }
@@ -444,9 +658,7 @@ void skinny_bmp_daemon()
       fd = accept(config.bmp_sock, (struct sockaddr *) &client, &clen);
       if (fd == ERR) goto read_data;
 
-#if defined ENABLE_IPV6
       ipv4_mapped_to_ipv4(&client);
-#endif
 
       /* If an ACL is defined, here we check against and enforce it */
       if (allow.num) allowed = check_allow(&allow, (struct sockaddr *)&client);
@@ -500,8 +712,6 @@ void skinny_bmp_daemon()
       }
 
       if (!peer) {
-        int fd;
-
         /* We briefly accept the new connection to be able to drop it */
         Log(LOG_ERR, "ERROR ( %s/%s ): Insufficient number of BMP peers has been configured by 'bmp_daemon_max_peers' (%d).\n",
                         config.name, bmp_misc_db->log_str, config.nfacctd_bmp_max_peers);
@@ -516,12 +726,10 @@ void skinny_bmp_daemon()
         peer->addr.address.ipv4.s_addr = ((struct sockaddr_in *)&client)->sin_addr.s_addr;
         peer->tcp_port = ntohs(((struct sockaddr_in *)&client)->sin_port);
       }
-#if defined ENABLE_IPV6
       else if (peer->addr.family == AF_INET6) {
         memcpy(&peer->addr.address.ipv6, &((struct sockaddr_in6 *)&client)->sin6_addr, 16);
         peer->tcp_port = ntohs(((struct sockaddr_in6 *)&client)->sin6_port);
       }
-#endif
       addr_to_str(peer->addr_str, &peer->addr);
       memcpy(&peer->id, &peer->addr, sizeof(struct host_addr)); /* XXX: some inet_ntoa()'s could be around against peer->id */
 
@@ -594,6 +802,12 @@ void bmp_prepare_thread()
   memset(bmp_misc_db, 0, sizeof(struct bgp_misc_structs));
 
   bmp_misc_db->is_thread = TRUE;
+
+  if (config.rpki_roas_file || config.rpki_rtr_cache) {
+    bmp_misc_db->bnv = malloc(sizeof(struct bgp_node_vector));
+    memset(bmp_misc_db->bnv, 0, sizeof(struct bgp_node_vector));
+  }
+
   bmp_misc_db->log_str = malloc(strlen("core/BMP") + 1);
   strcpy(bmp_misc_db->log_str, "core/BMP");
 }
@@ -603,7 +817,13 @@ void bmp_prepare_daemon()
   bmp_misc_db = &inter_domain_misc_dbs[FUNC_TYPE_BMP];
   memset(bmp_misc_db, 0, sizeof(struct bgp_misc_structs));
 
- bmp_misc_db->is_thread = FALSE;
- bmp_misc_db->log_str = malloc(strlen("core") + 1);
- strcpy(bmp_misc_db->log_str, "core");
+  bmp_misc_db->is_thread = FALSE;
+
+  if (config.rpki_roas_file || config.rpki_rtr_cache) {
+    bmp_misc_db->bnv = malloc(sizeof(struct bgp_node_vector));
+    memset(bmp_misc_db->bnv, 0, sizeof(struct bgp_node_vector));
+  }
+
+  bmp_misc_db->log_str = malloc(strlen("core") + 1);
+  strcpy(bmp_misc_db->log_str, "core");
 }
