@@ -1,6 +1,6 @@
 /*
     pmacct (Promiscuous mode IP Accounting package)
-    pmacct is Copyright (C) 2003-2017 by Paolo Lucente
+    pmacct is Copyright (C) 2003-2019 by Paolo Lucente
 */
 
 /*
@@ -18,6 +18,8 @@
     along with this program; if not, write to the Free Software
     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
+#ifndef CFG_H
+#define CFG_H
 
 #include "cfg_handlers.h"
 #include "bgp/bgp_prefix.h"
@@ -42,7 +44,7 @@ struct packet_data_ptr {
 
 struct custom_primitive_entry {
   /* compiled from map */
-  u_char name[MAX_CUSTOM_PRIMITIVE_NAMELEN];
+  char name[MAX_CUSTOM_PRIMITIVE_NAMELEN];
   struct packet_data_ptr pd_ptr[MAX_CUSTOM_PRIMITIVE_PD_PTRS];
   u_int32_t pen;
   u_int16_t field_type;
@@ -82,6 +84,7 @@ struct configuration {
   char *name;
   char *type;
   int type_id;
+  int is_forked;
   int pmacctd_nonroot;
   char *proc_name;
   int proc_priority;
@@ -93,10 +96,11 @@ struct configuration {
   u_int64_t pipe_size;
   u_int64_t buffer_size;
   int buffer_immediate;
-  int pipe_check_core_pid;
   int pipe_zmq;
   int pipe_zmq_retry;
   int pipe_zmq_profile;
+  int pipe_zmq_hwm;
+  int plugin_exit_any;
   int files_umask;
   int files_uid;
   int files_gid;
@@ -119,7 +123,9 @@ struct configuration {
   char *sql_table_type;
   char *sql_user;
   char *sql_passwd;
+  char *sql_conn_ca_file;
   char *sql_host;
+  int sql_port;
   char *sql_data;
   char *sql_backup_host;
   int sql_optimize_clauses;
@@ -141,12 +147,14 @@ struct configuration {
   char *sql_locking_style;
   int sql_use_copy;
   char *sql_delimiter;
+  int timestamps_rfc3339;
+  int timestamps_utc;
   int timestamps_secs;
   int timestamps_since_epoch;
   int mongo_insert_batch;
   int message_broker_output;
   int avro_buffer_size;
-  char *avro_schema_output_file;
+  char *avro_schema_file;
   char *amqp_exchange_type;
   int amqp_persistent_msg;
   u_int32_t amqp_frame_max;
@@ -157,21 +165,31 @@ struct configuration {
   int amqp_avro_schema_refresh_time;
   int kafka_broker_port;
   int kafka_partition;
+  int kafka_partition_dynamic;
   char *kafka_partition_key;
   int kafka_partition_keylen;
   char *kafka_avro_schema_topic;
   int kafka_avro_schema_refresh_time;
+  char *kafka_avro_schema_registry;
   char *kafka_config_file;
   int print_cache_entries;
   int print_markers;
   int print_output;
   int print_output_file_append;
+  int print_write_empty_file;
   char *print_output_lock_file;
   char *print_output_separator;
   char *print_output_file;
+  char *print_output_custom_lib;
+  char *print_output_custom_cfg_file;
   char *print_latest_file;
   int nfacctd_port;
   char *nfacctd_ip;
+  char *nfacctd_kafka_broker_host;
+  int nfacctd_kafka_broker_port;
+  char *nfacctd_kafka_topic;
+  char *nfacctd_kafka_config_file;
+  char *nfacctd_zmq_address;
   char *nfacctd_allow_file;
   int nfacctd_time;
   int nfacctd_time_new;
@@ -212,9 +230,11 @@ struct configuration {
   int telemetry_port_tcp;
   int telemetry_port_udp;
   char *telemetry_ip;
+  char *telemetry_zmq_address;
   char *telemetry_decoder;
+  int telemetry_decoder_id;
   int telemetry_max_peers;
-  int telemetry_udp_timeout;
+  int telemetry_peer_timeout;
   char *telemetry_allow_file;
   int telemetry_pipe_size;
   int telemetry_ipprec;
@@ -267,6 +287,7 @@ struct configuration {
   int nfacctd_bgp;
   int nfacctd_bgp_msglog_output;
   char *nfacctd_bgp_msglog_file;
+  char *nfacctd_bgp_msglog_avro_schema_file;
   char *nfacctd_bgp_msglog_amqp_host;
   char *nfacctd_bgp_msglog_amqp_vhost;
   char *nfacctd_bgp_msglog_amqp_user;
@@ -288,6 +309,7 @@ struct configuration {
   int nfacctd_bgp_msglog_kafka_broker_port;
   int nfacctd_bgp_msglog_kafka_retry;
   char *nfacctd_bgp_msglog_kafka_config_file;
+  char *nfacctd_bgp_msglog_kafka_avro_schema_registry;
   char *nfacctd_bgp_id;
   char *nfacctd_bgp_ip;
   as_t nfacctd_bgp_as;
@@ -301,6 +323,8 @@ struct configuration {
   char *nfacctd_bgp_extcomm_pattern;
   char *nfacctd_bgp_lrgcomm_pattern;
   char *nfacctd_bgp_stdcomm_pattern_to_asn;
+  char *nfacctd_bgp_lrgcomm_pattern_to_asn;
+  char *bgp_blackhole_stdcomm_list;
   int nfacctd_bgp_peer_as_src_type;
   int nfacctd_bgp_src_std_comm_type;
   int nfacctd_bgp_src_ext_comm_type;
@@ -308,6 +332,7 @@ struct configuration {
   int nfacctd_bgp_src_as_path_type;
   int nfacctd_bgp_src_local_pref_type;
   int nfacctd_bgp_src_med_type;
+  int nfacctd_bgp_src_roa_type;
   int nfacctd_bgp_peer_as_skip_subas;
   int nfacctd_bgp_batch;
   int nfacctd_bgp_batch_interval;
@@ -321,9 +346,6 @@ struct configuration {
   int nfacctd_bgp_follow_nexthop_external;
   char *nfacctd_bgp_neighbors_file;
   char *nfacctd_bgp_md5_file;
-  int nfacctd_bgp_offline_input;
-  char *nfacctd_bgp_offline_file_spool;
-  int nfacctd_bgp_offline_file_refresh_time;
   int bgp_table_peer_buckets;
   int bgp_table_per_peer_buckets;
   int bgp_table_attr_hash_buckets;
@@ -331,6 +353,7 @@ struct configuration {
   int bgp_table_dump_output;
   char *bgp_table_dump_file;
   char *bgp_table_dump_latest_file;
+  char *bgp_table_dump_avro_schema_file;
   int bgp_table_dump_refresh_time;
   char *bgp_table_dump_amqp_host;
   char *bgp_table_dump_amqp_vhost;
@@ -351,6 +374,15 @@ struct configuration {
   int bgp_table_dump_kafka_partition_keylen;
   int bgp_table_dump_kafka_broker_port;
   char *bgp_table_dump_kafka_config_file;
+  char *bgp_table_dump_kafka_avro_schema_registry;
+  int bgp_lg;
+  char *bgp_lg_ip;
+  int bgp_lg_port;
+  int bgp_lg_threads;
+  char *bgp_lg_user;
+  char *bgp_lg_passwd;
+  char *bgp_xconnect_map;
+  int bgp_disable_router_id_check;
   int bmp_sock;
   int nfacctd_bmp;
   char *nfacctd_bmp_ip;
@@ -363,6 +395,7 @@ struct configuration {
   int nfacctd_bmp_batch_interval;
   int nfacctd_bmp_msglog_output;
   char *nfacctd_bmp_msglog_file;
+  char *nfacctd_bmp_msglog_avro_schema_file;
   char *nfacctd_bmp_msglog_amqp_host;
   char *nfacctd_bmp_msglog_amqp_vhost;
   char *nfacctd_bmp_msglog_amqp_user;
@@ -384,6 +417,7 @@ struct configuration {
   int nfacctd_bmp_msglog_kafka_broker_port;
   int nfacctd_bmp_msglog_kafka_retry;
   char *nfacctd_bmp_msglog_kafka_config_file;
+  char *nfacctd_bmp_msglog_kafka_avro_schema_registry;
   int bmp_table_peer_buckets;
   int bmp_table_per_peer_buckets;
   int bmp_table_attr_hash_buckets;
@@ -391,6 +425,7 @@ struct configuration {
   int bmp_dump_output;
   char *bmp_dump_file;
   char *bmp_dump_latest_file;
+  char *bmp_dump_avro_schema_file;
   int bmp_dump_refresh_time;
   char *bmp_dump_amqp_host;
   char *bmp_dump_amqp_vhost;
@@ -411,6 +446,7 @@ struct configuration {
   int bmp_dump_kafka_partition_keylen;
   int bmp_dump_kafka_broker_port;
   char *bmp_dump_kafka_config_file;
+  char *bmp_dump_kafka_avro_schema_registry;
   int nfacctd_isis;
   char *nfacctd_isis_ip;
   char *nfacctd_isis_net;
@@ -423,9 +459,7 @@ struct configuration {
   char *geoip_ipv6_file;
 #if defined WITH_GEOIP
   GeoIP *geoip_ipv4;
-#if defined ENABLE_IPV6
   GeoIP *geoip_ipv6;
-#endif
 #endif
   char *geoipv2_file;
 #if defined WITH_GEOIPV2
@@ -435,9 +469,14 @@ struct configuration {
   char *clbuf; /* pcap filter */
   int pcap_protocol;
   char *pcap_savefile;
-  char *dev;
-  int if_wait;
-  int sf_wait;
+  int pcap_direction;
+  int pcap_ifindex;
+  char *pcap_interfaces_map;
+  char *pcap_if;
+  int pcap_if_wait;
+  int pcap_sf_wait;
+  int pcap_sf_delay;
+  int pcap_sf_replay;
   int num_memory_pools;
   int memory_pool_size;
   int buckets;
@@ -503,7 +542,9 @@ struct configuration {
   int nfprobe_ipprec;
   int nfprobe_direction;
   u_int32_t nfprobe_ifindex;
+  int nfprobe_ifindex_override;
   int nfprobe_ifindex_type;
+  int nfprobe_dont_cache;
   char *sfprobe_receiver;
   char *sfprobe_agentip;
   int sfprobe_agentsubid;
@@ -513,43 +554,43 @@ struct configuration {
   int tee_max_receiver_pools;
   char *tee_receivers;
   int tee_pipe_size;
-  int tee_dissect_send_full_pkt;
+  char *tee_kafka_config_file;
   int uacctd_group;
   int uacctd_nl_size;
   int uacctd_threshold;
   char *tunnel0;
-  char *pkt_len_distrib_bins_str;
-  char *pkt_len_distrib_bins[MAX_PKT_LEN_DISTRIB_BINS];
-  u_int16_t pkt_len_distrib_bins_lookup[ETHER_JUMBO_MTU+1];
   int use_ip_next_hop;
+  int decode_arista_trailer;
   int dump_max_writers;
   int tmp_asa_bi_flow;
+  int tmp_bgp_lookup_compare_ports;
   size_t thread_stack;
+  char *rpki_roas_file;
+  char *rpki_rtr_cache;
+  int rpki_rtr_cache_version;
+  int rpki_rtr_cache_pipe_size;
+  int rpki_rtr_cache_ipprec;
 };
 
 /* prototypes */ 
-#if (!defined __CFG_C)
-#define EXT extern
-#else
-#define EXT
-#endif
-EXT void evaluate_configuration(char *, int);
-EXT int parse_configuration_file(char *);
-EXT int parse_plugin_names(char *, int, int);
-EXT void parse_core_process_name(char *, int, int);
-EXT void compose_default_plugin_name(char *, int, char *);
-EXT int create_plugin(char *, char *, char *);
-EXT int delete_plugin_by_id(int);
-EXT struct plugins_list_entry *search_plugin_by_pipe(int);
-EXT struct plugins_list_entry *search_plugin_by_pid(pid_t);
-EXT void sanitize_cfg(int, char *);
-EXT void set_default_values();
+extern void evaluate_configuration(char *, int);
+extern int parse_configuration_file(char *);
+extern int parse_plugin_names(char *, int, int);
+extern void parse_core_process_name(char *, int, int);
+extern void compose_default_plugin_name(char *, int, char *);
+extern int create_plugin(char *, char *, char *);
+extern int delete_plugin_by_id(int);
+extern struct plugins_list_entry *search_plugin_by_pipe(int);
+extern struct plugins_list_entry *search_plugin_by_pid(pid_t);
+extern void sanitize_cfg(int, char *);
+extern void set_default_values();
 
 /* global vars */
-EXT char *cfg[LARGEBUFLEN], *cfg_cmdline[SRVBUFLEN];
-EXT struct custom_primitives custom_primitives_registry;
-EXT pm_cfgreg_t custom_primitives_type;
-EXT int rows;
+extern char *cfg[LARGEBUFLEN], *cfg_cmdline[SRVBUFLEN];
+extern struct custom_primitives custom_primitives_registry;
+extern pm_cfgreg_t custom_primitives_type;
+extern int rows;
 
-static char default_proc_name[] = "default";
-#undef EXT
+extern char default_proc_name[];
+
+#endif //CFG_H
