@@ -1,6 +1,6 @@
 /*
     pmacct (Promiscuous mode IP Accounting package)
-    pmacct is Copyright (C) 2003-2017 by Paolo Lucente
+    pmacct is Copyright (C) 2003-2019 by Paolo Lucente
 */
 
 /*
@@ -19,15 +19,24 @@
     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 
-#define __IMT_PLUGIN_C
-
 /* includes */
 #include "pmacct.h"
 #include "plugin_hooks.h"
+#include "plugin_common.h"
 #include "imt_plugin.h"
+#include "bgp/bgp.h"
 #include "net_aggr.h"
 #include "ports_aggr.h"
-#include "bgp/bgp.h"
+
+//Global variables
+void (*imt_insert_func)(struct primitives_ptrs *);
+unsigned char *mpd;
+unsigned char *a;
+struct memory_pool_desc *current_pool;
+struct acc **lru_elem_ptr;
+int no_more_space;
+struct timeval cycle_stamp;
+struct timeval table_reset_stamp;
 
 /* Functions */
 void imt_plugin(int pipe_fd, struct configuration *cfgptr, void *ptr) 
@@ -51,17 +60,17 @@ void imt_plugin(int pipe_fd, struct configuration *cfgptr, void *ptr)
   int pollagain = 0;
   u_int32_t seq = 0;
   int rg_err_count = 0;
-  int ret, lock = FALSE, cLen, num, sd, sd2;
+  int ret, lock = FALSE, num, sd, sd2;
   struct pkt_bgp_primitives *pbgp, empty_pbgp;
   struct pkt_legacy_bgp_primitives *plbgp, empty_plbgp;
   struct pkt_nat_primitives *pnat, empty_pnat;
   struct pkt_mpls_primitives *pmpls, empty_pmpls;
   struct pkt_tunnel_primitives *ptun, empty_ptun;
-  char *pcust, empty_pcust[] = "";
+  unsigned char *pcust, empty_pcust[] = "";
   struct pkt_vlen_hdr_primitives *pvlen, empty_pvlen;
   struct networks_file_data nfd;
   struct primitives_ptrs prim_ptrs;
-  struct plugins_list_entry *plugin_data = ((struct channels_list_entry *)ptr)->plugin;
+  socklen_t cLen;
 
   /* poll() stuff */
   struct pollfd poll_fd[2]; /* pipe + server */
@@ -69,6 +78,8 @@ void imt_plugin(int pipe_fd, struct configuration *cfgptr, void *ptr)
 
 #ifdef WITH_ZMQ
   struct p_zmq_host *zmq_host = &((struct channels_list_entry *)ptr)->zmq_host;
+#else
+  void *zmq_host = NULL;
 #endif
 
   memcpy(&config, cfgptr, sizeof(struct configuration));
@@ -77,8 +88,6 @@ void imt_plugin(int pipe_fd, struct configuration *cfgptr, void *ptr)
   pm_setproctitle("%s [%s]", "IMT Plugin", config.name);
 
   if (config.proc_priority) {
-    int ret;
-
     ret = setpriority(PRIO_PROCESS, 0, config.proc_priority);
     if (ret) Log(LOG_WARNING, "WARN ( %s/%s ): proc_priority failed (errno: %d)\n", config.name, config.type, errno);
     else Log(LOG_INFO, "INFO ( %s/%s ): proc_priority set to %d\n", config.name, config.type, getpriority(PRIO_PROCESS, 0));
@@ -92,7 +101,7 @@ void imt_plugin(int pipe_fd, struct configuration *cfgptr, void *ptr)
 
   if (extras.off_pkt_vlen_hdr_primitives) {
     Log(LOG_ERR, "ERROR ( %s/%s ): variable-length primitives, ie. label, are not supported in IMT plugin. Exiting ..\n", config.name, config.type);
-    exit_plugin(1);
+    exit_gracefully(1);
   }
 
   reload_map = FALSE;
@@ -102,19 +111,10 @@ void imt_plugin(int pipe_fd, struct configuration *cfgptr, void *ptr)
   pipebuf = (unsigned char *) malloc(config.buffer_size);
   if (!pipebuf) {
     Log(LOG_ERR, "ERROR ( %s/%s ): malloc() failed (pipebuf). Exiting ..\n", config.name, config.type);
-    exit_plugin(1);
+    exit_gracefully(1);
   }
 
-  if (config.pipe_zmq) {
-    plugin_pipe_zmq_compile_check();
-#ifdef WITH_ZMQ
-    p_zmq_plugin_pipe_init_plugin(zmq_host);
-    p_zmq_plugin_pipe_consume(zmq_host);
-    p_zmq_set_retry_timeout(zmq_host, config.pipe_zmq_retry);
-    pipe_fd = p_zmq_get_fd(zmq_host);
-    seq = 0;
-#endif
-  }
+  if (config.pipe_zmq) P_zmq_pipe_init(zmq_host, &pipe_fd, &seq);
   else setnonblocking(pipe_fd);
 
   memset(pipebuf, 0, config.buffer_size);
@@ -137,19 +137,12 @@ void imt_plugin(int pipe_fd, struct configuration *cfgptr, void *ptr)
   set_net_funcs(&nt);
 
   if (config.ports_file) load_ports(config.ports_file, &pt);
-  if (config.pkt_len_distrib_bins_str) load_pkt_len_distrib_bins();
-  else {
-    if (config.what_to_count_2 & COUNT_PKT_LEN_DISTRIB) {
-      Log(LOG_ERR, "ERROR ( %s/%s ): 'aggregate' contains pkt_len_distrib but no 'pkt_len_distrib_bins' defined. Exiting.\n", config.name, config.type);
-      exit_plugin(1); 
-    }
-  }
 
   if (!config.num_memory_pools) config.num_memory_pools = NUM_MEMORY_POOLS;
   if (!config.memory_pool_size) config.memory_pool_size = MEMORY_POOL_SIZE;  
   else {
     if (config.memory_pool_size < sizeof(struct acc)) {
-      Log(LOG_WARNING, "WARN ( %s/%s ): enforcing memory pool's minimum size, %d bytes.\n", config.name, config.type, sizeof(struct acc));
+      Log(LOG_WARNING, "WARN ( %s/%s ): enforcing memory pool's minimum size, %d bytes.\n", config.name, config.type, (int)sizeof(struct acc));
       config.memory_pool_size = MEMORY_POOL_SIZE;
     }
   }
@@ -160,27 +153,27 @@ void imt_plugin(int pipe_fd, struct configuration *cfgptr, void *ptr)
   init_memory_pool_table(config);
   if (mpd == NULL) {
     Log(LOG_ERR, "ERROR ( %s/%s ): unable to allocate memory pools table\n", config.name, config.type);
-    exit_plugin(1);
+    exit_gracefully(1);
   }
 
   current_pool = request_memory_pool(config.buckets*sizeof(struct acc));
   if (current_pool == NULL) {
     Log(LOG_ERR, "ERROR ( %s/%s ): unable to allocate first memory pool, try with larger value.\n", config.name, config.type);
-    exit_plugin(1);
+    exit_gracefully(1);
   }
   a = current_pool->base_ptr;
 
   lru_elem_ptr = malloc(config.buckets*sizeof(struct acc *));
   if (lru_elem_ptr == NULL) {
     Log(LOG_ERR, "ERROR ( %s/%s ): unable to allocate LRU element pointers.\n", config.name, config.type);
-    exit_plugin(1);
+    exit_gracefully(1);
   }
   else memset(lru_elem_ptr, 0, config.buckets*sizeof(struct acc *));
 
   current_pool = request_memory_pool(config.memory_pool_size);
   if (current_pool == NULL) {
     Log(LOG_ERR, "ERROR ( %s/%s ): unable to allocate more memory pools, try with larger value.\n", config.name, config.type);
-    exit_plugin(1);
+    exit_gracefully(1);
   }
 
   signal(SIGHUP, reload); /* handles reopening of syslog channel */
@@ -221,9 +214,9 @@ void imt_plugin(int pipe_fd, struct configuration *cfgptr, void *ptr)
     gettimeofday(&cycle_stamp, NULL);
 
     if (num <= 0) {
-      if (getppid() == 1) {
+      if (getppid() != core_pid) {
 	Log(LOG_ERR, "ERROR ( %s/%s ): Core process *seems* gone. Exiting.\n", config.name, config.type);
-	exit_plugin(1);
+	exit_gracefully(1);
       } 
 
       if (num < 0) goto poll_again;  
@@ -232,7 +225,6 @@ void imt_plugin(int pipe_fd, struct configuration *cfgptr, void *ptr)
     /* doing server tasks */
     if (poll_fd[1].revents & POLLIN) {
       struct pollfd pfd;
-      int ret;
 
       sd2 = accept(sd, &cAddr, &cLen);
       setblocking(sd2);
@@ -305,11 +297,6 @@ void imt_plugin(int pipe_fd, struct configuration *cfgptr, void *ptr)
         else Log(LOG_DEBUG, "DEBUG ( %s/%s ): %d incoming bytes. ERRNO: %d\n", config.name, config.type, num, errno);
         Log(LOG_DEBUG, "DEBUG ( %s/%s ): Closing connection with client ...\n", config.name, config.type);
       }
-      else if (request == WANT_PKT_LEN_DISTRIB_TABLE) {
-        if (num > 0) process_query_data(sd2, srvbuf, num, &extras, datasize, FALSE);
-        else Log(LOG_DEBUG, "DEBUG ( %s/%s ): %d incoming bytes. ERRNO: %d\n", config.name, config.type, num, errno);
-        Log(LOG_DEBUG, "DEBUG ( %s/%s ): Closing connection with client ...\n", config.name, config.type);
-      }
       else {
 	if (lock) {
 	  if (num > 0) process_query_data(sd2, srvbuf, num, &extras, datasize, FALSE);
@@ -323,12 +310,17 @@ void imt_plugin(int pipe_fd, struct configuration *cfgptr, void *ptr)
 	    break;
           case 0: /* Child */
             close(sd);
+
 	    pm_setproctitle("%s [%s]", "IMT Plugin -- serving client", config.name);
+	    config.is_forked = TRUE;
+
             if (num > 0) process_query_data(sd2, srvbuf, num, &extras, datasize, TRUE);
 	    else Log(LOG_DEBUG, "DEBUG ( %s/%s ): %d incoming bytes. Errno: %d\n", config.name, config.type, num, errno);
+
             Log(LOG_DEBUG, "DEBUG ( %s/%s ): Closing connection with client ...\n", config.name, config.type);
             close(sd2);
-            exit(0);
+
+            exit_gracefully(0);
           default: /* Parent */
             break;
           } 
@@ -349,14 +341,14 @@ void imt_plugin(int pipe_fd, struct configuration *cfgptr, void *ptr)
       current_pool = request_memory_pool(config.buckets*sizeof(struct acc));
       if (current_pool == NULL) {
         Log(LOG_ERR, "ERROR ( %s/%s ): Cannot allocate my first memory pool, try with larger value.\n", config.name, config.type);
-        exit_plugin(1);
+        exit_gracefully(1);
       }
       a = current_pool->base_ptr;
 
       current_pool = request_memory_pool(config.memory_pool_size);
       if (current_pool == NULL) {
         Log(LOG_ERR, "ERROR ( %s/%s ): Cannot allocate more memory pools, try with larger value.\n", config.name, config.type);
-        exit_plugin(1);
+        exit_gracefully(1);
       }
       go_to_clear = FALSE;
       no_more_space = FALSE;
@@ -369,8 +361,15 @@ void imt_plugin(int pipe_fd, struct configuration *cfgptr, void *ptr)
       reload_map = FALSE;
     }
 
+    if (reload_log) {
+      reload_logs();
+      reload_log = FALSE;
+    }
+
     if (poll_fd[0].revents & POLLIN) {
+#ifdef WITH_ZMQ
       read_data:
+#endif
       if (config.pipe_homegrown) {
         if (!pollagain) {
           seq++;
@@ -379,7 +378,7 @@ void imt_plugin(int pipe_fd, struct configuration *cfgptr, void *ptr)
 
         pollagain = FALSE;
         if ((num = read(pipe_fd, &rgptr, sizeof(rgptr))) == 0)
-          exit_plugin(1); /* we exit silently; something happened at the write end */
+          exit_gracefully(1); /* we exit silently; something happened at the write end */
 
         if (num < 0) {
           pollagain = TRUE;
@@ -390,7 +389,7 @@ void imt_plugin(int pipe_fd, struct configuration *cfgptr, void *ptr)
         if (((struct ch_buf_hdr *)pipebuf)->seq != seq) {
           rg_err_count++;
           if (config.debug || (rg_err_count > MAX_RG_COUNT_ERR)) {
-	    Log(LOG_WARNING, "WARN ( %s/%s ): Missing data detected (plugin_buffer_size=%llu plugin_pipe_size=%llu).\n",
+	    Log(LOG_WARNING, "WARN ( %s/%s ): Missing data detected (plugin_buffer_size=%" PRIu64 " plugin_pipe_size=%" PRIu64 ").\n",
 		config.name, config.type, config.buffer_size, config.pipe_size);
 	    Log(LOG_WARNING, "WARN ( %s/%s ): Increase values or look for plugin_buffer_size, plugin_pipe_size in CONFIG-KEYS document.\n\n",
 		config.name, config.type);
@@ -401,7 +400,7 @@ void imt_plugin(int pipe_fd, struct configuration *cfgptr, void *ptr)
       }
 #ifdef WITH_ZMQ
       else if (config.pipe_zmq) {
-	ret = p_zmq_plugin_pipe_recv(zmq_host, pipebuf, config.buffer_size);
+	ret = p_zmq_topic_recv(zmq_host, pipebuf, config.buffer_size);
 	if (ret > 0) {
 	  if (((struct ch_buf_hdr *)pipebuf)->seq != ((seq + 1) % MAX_SEQNUM)) {
 	    Log(LOG_WARNING, "WARN ( %s/%s ): Missing data detected. Sequence received=%u expected=%u\n",
@@ -418,11 +417,10 @@ void imt_plugin(int pipe_fd, struct configuration *cfgptr, void *ptr)
 	data = (struct pkt_data *) (pipebuf+sizeof(struct ch_buf_hdr));
 
 	if (config.debug_internal_msg) 
-	  Log(LOG_DEBUG, "DEBUG ( %s/%s ): buffer received cpid=%u len=%llu seq=%u num_entries=%u\n",
-		config.name, config.type, core_pid, ((struct ch_buf_hdr *)pipebuf)->len,
-		seq, ((struct ch_buf_hdr *)pipebuf)->num);
+	  Log(LOG_DEBUG, "DEBUG ( %s/%s ): buffer received len=%" PRIu64 " seq=%u num_entries=%u\n",
+		config.name, config.type, ((struct ch_buf_hdr *)pipebuf)->len, seq,
+		((struct ch_buf_hdr *)pipebuf)->num);
 
-	if (!config.pipe_check_core_pid || ((struct ch_buf_hdr *)pipebuf)->core_pid == core_pid) {
 	while (((struct ch_buf_hdr *)pipebuf)->num > 0) {
 
           if (extras.off_pkt_bgp_primitives)
@@ -455,10 +453,6 @@ void imt_plugin(int pipe_fd, struct configuration *cfgptr, void *ptr)
 	    if (!pt.table[data->primitives.dst_port]) data->primitives.dst_port = 0;
 	  }
 
-	  if (config.pkt_len_distrib_bins_str &&
-	      config.what_to_count_2 & COUNT_PKT_LEN_DISTRIB)
-	    evaluate_pkt_len_distrib(data);
-
 	  prim_ptrs.data = data; 
 	  prim_ptrs.pbgp = pbgp; 
 	  prim_ptrs.plbgp = plbgp; 
@@ -477,7 +471,6 @@ void imt_plugin(int pipe_fd, struct configuration *cfgptr, void *ptr)
             data = (struct pkt_data *) dataptr;
 	  }
         }
-	}
       }
 
 #ifdef WITH_ZMQ
@@ -491,7 +484,7 @@ void exit_now(int signum)
 {
   if (config.imt_plugin_path) unlink(config.imt_plugin_path);
   if (config.pidfile) remove_pid_file(config.pidfile);
-  exit_plugin(0);
+  exit_gracefully(0);
 }
 
 void sum_host_insert(struct primitives_ptrs *prim_ptrs)
