@@ -21,8 +21,6 @@
  * 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  */
 
-#define __ISIS_ADJACENCY_C
-
 #include "pmacct.h"
 #include "isis.h"
 
@@ -58,7 +56,6 @@ isis_new_adj (u_char * id, u_char * snpa, int level,
 	      struct isis_circuit *circuit)
 {
   struct isis_adjacency *adj;
-  int i;
 
   adj = adj_alloc (id);		/* P2P kludge */
 
@@ -121,10 +118,8 @@ isis_delete_adj (struct isis_adjacency *adj, struct list *adjdb)
 
   if (adj->ipv4_addrs)
     isis_list_delete (adj->ipv4_addrs);
-#ifdef ENABLE_IPV6
   if (adj->ipv6_addrs)
     isis_list_delete (adj->ipv6_addrs);
-#endif
   
   free(adj);
   return;
@@ -135,7 +130,6 @@ isis_adj_state_change (struct isis_adjacency *adj, enum isis_adj_state state,
 		       const char *reason)
 {
   int old_state;
-  int level = adj->level;
   struct isis_circuit *circuit;
 
   old_state = adj->adj_state;
@@ -167,13 +161,11 @@ isis_adj_state_change (struct isis_adjacency *adj, enum isis_adj_state state,
 int
 isis_adj_expire (struct isis_adjacency *adj)
 {
-  int level;
 
   /*
    * Get the adjacency
    */
   assert (adj);
-  level = adj->level;
   memset(&adj->expire, 0, sizeof(struct timeval));
 
   /* trigger the adj expire event */
@@ -182,25 +174,6 @@ isis_adj_expire (struct isis_adjacency *adj)
   return 0;
 }
 
-static const char *
-adj_state2string (int state)
-{
-
-  switch (state)
-    {
-    case ISIS_ADJ_INITIALIZING:
-      return "Initializing";
-    case ISIS_ADJ_UP:
-      return "Up";
-    case ISIS_ADJ_DOWN:
-      return "Down";
-    default:
-      return "Unknown";
-    }
-
-  return NULL;			/* not reached */
-}
-
 void
 isis_adjdb_iterate (struct list *adjdb, void (*func) (struct isis_adjacency *,
 						      void *), void *arg)
