@@ -20,8 +20,6 @@
  * 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  */
 
-#define __ISIS_CIRCUIT_C
-
 #include "pmacct.h"
 #include "isis.h"
 
@@ -169,12 +167,10 @@ isis_circuit_del (struct isis_circuit *circuit)
     }
   if (circuit->ip_addrs)
     isis_list_delete (circuit->ip_addrs);
-#ifdef ENABLE_IPV6
   if (circuit->ipv6_link)
     isis_list_delete (circuit->ipv6_link);
   if (circuit->ipv6_non_link)
     isis_list_delete (circuit->ipv6_non_link);
-#endif /* ENABLE_IPV6 */
 
   /* and lastly the circuit itself */
   free(circuit);
@@ -245,12 +241,10 @@ circuit_update_nlpids (struct isis_circuit *circuit)
       circuit->nlpids.nlpids[0] = NLPID_IP;
       circuit->nlpids.count++;
     }
-#ifdef ENABLE_IPV6
   if (circuit->ipv6_router)
     {
       circuit->nlpids.nlpids[circuit->nlpids.count] = NLPID_IPV6;
       circuit->nlpids.count++;
     }
-#endif /* ENABLE_IPV6 */
   return;
 }
