@@ -20,8 +20,6 @@
  * 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  */
 
-#define __ISIS_DYNHN_C
-
 #include "pmacct.h"
 #include "isis.h"
 
@@ -38,6 +36,7 @@
 #include "isis_misc.h"
 #include "isis_constants.h"
 
+struct list *dyn_cache;
 extern struct host host;
 
 void
