@@ -20,8 +20,6 @@
  * 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  */
 
-#define __ISIS_EVENTS_C
-
 #include "pmacct.h"
 #include "linklist.h"
 #include "dict.h"
@@ -146,8 +144,6 @@ circuit_commence_level (struct isis_circuit *circuit, int level)
 static void
 circuit_resign_level (struct isis_circuit *circuit, int level)
 {
-  int idx = level - 1;
-
 //  THREAD_TIMER_OFF (circuit->t_send_csnp[idx]);
 //  THREAD_TIMER_OFF (circuit->t_send_psnp[idx]);
 
