@@ -21,8 +21,6 @@
  * 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  */
 
-#define __ISIS_LSP_C
-
 #include "pmacct.h"
 #include "isis.h"
 
@@ -113,12 +111,10 @@ lsp_clear_data (struct isis_lsp *lsp)
     isis_list_delete (lsp->tlv_data.ipv4_ext_reachs);
   if (lsp->tlv_data.te_ipv4_reachs)
     isis_list_delete (lsp->tlv_data.te_ipv4_reachs);
-#ifdef ENABLE_IPV6
   if (lsp->tlv_data.ipv6_addrs)
     isis_list_delete (lsp->tlv_data.ipv6_addrs);
   if (lsp->tlv_data.ipv6_reachs)
     isis_list_delete (lsp->tlv_data.ipv6_reachs);
-#endif /* ENABLE_IPV6 */
 
   memset (&lsp->tlv_data, 0, sizeof (struct tlvs));
 
@@ -371,15 +367,14 @@ lsp_update_data (struct isis_lsp *lsp, struct stream *stream,
   expected |= TLVFLAG_IPV4_ADDR;
   expected |= TLVFLAG_IPV4_INT_REACHABILITY;
   expected |= TLVFLAG_IPV4_EXT_REACHABILITY;
-#ifdef ENABLE_IPV6
   expected |= TLVFLAG_IPV6_ADDR;
   expected |= TLVFLAG_IPV6_REACHABILITY;
-#endif /* ENABLE_IPV6 */
 
   retval = parse_tlvs (area->area_tag, lsp->pdu->data +
 		       ISIS_FIXED_HDR_LEN + ISIS_LSP_HDR_LEN,
 		       ntohs (lsp->lsp_header->pdu_len) - ISIS_FIXED_HDR_LEN
 		       - ISIS_LSP_HDR_LEN, &expected, &found, &lsp->tlv_data);
+  (void)retval; //TODO treat error
 
   if (found & TLVFLAG_DYN_HOSTNAME)
     {
@@ -591,33 +586,6 @@ lsp_build_isis_list_ssn (struct isis_circuit *circuit, struct list *list,
   return;
 }
 
-static void
-lsp_set_time (struct isis_lsp *lsp)
-{
-  assert (lsp);
-
-  if (lsp->lsp_header->rem_lifetime == 0)
-    {
-      if (lsp->age_out != 0)
-	lsp->age_out--;
-      return;
-    }
-
-  /* If we are turning 0 */
-  /* ISO 10589 - 7.3.16.4 first paragraph */
-
-  if (ntohs (lsp->lsp_header->rem_lifetime) == 1)
-    {
-      /* 7.3.16.4 a) set SRM flags on all */
-      ISIS_FLAGS_SET_ALL (lsp->SRMflags);
-      /* 7.3.16.4 b) retain only the header FIXME  */
-      /* 7.3.16.4 c) record the time to purge FIXME (other way to do it) */
-    }
-
-  lsp->lsp_header->rem_lifetime =
-    htons (ntohs (lsp->lsp_header->rem_lifetime) - 1);
-}
-
 /* Convert the lsp attribute bits to attribute string */
 const char *
 lsp_bits2string (u_char * lsp_bits)
@@ -700,8 +668,10 @@ lsp_next_frag (u_char frag_num, struct isis_lsp *lsp0, struct isis_area *area,
       lsp_clear_data (lsp);
       if (lsp0->tlv_data.auth_info.type)
 	{
-	  memcpy (&lsp->tlv_data.auth_info, &lsp->tlv_data.auth_info,
+	  /* FIXME: this is broken. Detected by GCC 8.3
+           memcpy (&lsp->tlv_data.auth_info, &lsp->tlv_data.auth_info,
 		  sizeof (struct isis_passwd));
+          */
 	  tlv_add_authinfo (lsp->tlv_data.auth_info.type,
 			    lsp->tlv_data.auth_info.len,
 			    lsp->tlv_data.auth_info.passwd, lsp->pdu);
@@ -719,8 +689,10 @@ lsp_next_frag (u_char frag_num, struct isis_lsp *lsp0, struct isis_area *area,
    */
   if (lsp0->tlv_data.auth_info.type)
     {
+      /* FIXME: this is broken. Detected by GCC 8.3
       memcpy (&lsp->tlv_data.auth_info, &lsp->tlv_data.auth_info,
 	      sizeof (struct isis_passwd));
+      */
       tlv_add_authinfo (lsp->tlv_data.auth_info.type,
 			lsp->tlv_data.auth_info.len,
 			lsp->tlv_data.auth_info.passwd, lsp->pdu);
@@ -744,10 +716,8 @@ lsp_build_nonpseudo (struct isis_lsp *lsp, struct isis_area *area)
   struct ipv4_reachability *ipreach;
   struct te_ipv4_reachability *te_ipreach;
   struct isis_adjacency *nei;
-#ifdef ENABLE_IPV6
   struct prefix_ipv6 *ipv6, *ip6prefix;
   struct ipv6_reachability *ip6reach;
-#endif /* ENABLE_IPV6 */
   struct tlvs tlv_data;
   struct isis_lsp *lsp0 = lsp;
   struct isis_passwd *passwd;
@@ -762,11 +732,7 @@ lsp_build_nonpseudo (struct isis_lsp *lsp, struct isis_area *area)
     lsp->tlv_data.area_addrs = isis_list_new ();
   isis_list_add_list (lsp->tlv_data.area_addrs, area->area_addrs);
   /* Protocols Supported */
-  if (area->ip_circuits > 0
-#ifdef ENABLE_IPV6
-      || area->ipv6_circuits > 0
-#endif /* ENABLE_IPV6 */
-    )
+  if (area->ip_circuits > 0 || area->ipv6_circuits > 0)
     {
       lsp->tlv_data.nlpids = calloc(1, sizeof (struct nlpids));
       lsp->tlv_data.nlpids->count = 0;
@@ -775,14 +741,12 @@ lsp_build_nonpseudo (struct isis_lsp *lsp, struct isis_area *area)
 	  lsp->tlv_data.nlpids->count++;
 	  lsp->tlv_data.nlpids->nlpids[0] = NLPID_IP;
 	}
-#ifdef ENABLE_IPV6
       if (area->ipv6_circuits > 0)
 	{
 	  lsp->tlv_data.nlpids->count++;
 	  lsp->tlv_data.nlpids->nlpids[lsp->tlv_data.nlpids->count - 1] =
 	    NLPID_IPV6;
 	}
-#endif /* ENABLE_IPV6 */
     }
   /* XXX: Dynamic Hostname */
 /*
@@ -908,7 +872,6 @@ lsp_build_nonpseudo (struct isis_lsp *lsp, struct isis_area *area)
 		}
 	    }
 	}
-#ifdef ENABLE_IPV6
       /*
        * Add IPv6 reachability of this circuit
        */
@@ -940,7 +903,6 @@ lsp_build_nonpseudo (struct isis_lsp *lsp, struct isis_area *area)
 	      isis_listnode_add (tlv_data.ipv6_reachs, ip6reach);
 	    }
 	}
-#endif /* ENABLE_IPV6 */
 
       switch (circuit->circ_type)
 	{
@@ -1067,7 +1029,6 @@ lsp_build_nonpseudo (struct isis_lsp *lsp, struct isis_area *area)
 			     lsp0, area, level);
     }
 
-#ifdef  ENABLE_IPV6
   while (tlv_data.ipv6_reachs && listcount (tlv_data.ipv6_reachs))
     {
       if (lsp->tlv_data.ipv6_reachs == NULL)
@@ -1080,7 +1041,6 @@ lsp_build_nonpseudo (struct isis_lsp *lsp, struct isis_area *area)
 	lsp = lsp_next_frag (LSP_FRAGMENT (lsp->lsp_header->lsp_id) + 1,
 			     lsp0, area, level);
     }
-#endif /* ENABLE_IPV6 */
 
   while (tlv_data.is_neighs && listcount (tlv_data.is_neighs))
     {
@@ -1232,10 +1192,9 @@ lsp_non_pseudo_regenerate (struct isis_area *area, int level)
 
   if (area->ip_circuits)
     isis_spf_schedule (area, level);
-#ifdef ENABLE_IPV6
   if (area->ipv6_circuits)
     isis_spf_schedule6 (area, level);
-#endif
+
   return ISIS_OK;
 }
 
