@@ -21,8 +21,6 @@
  * 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  */
 
-#define __ISIS_PDU_C
-
 #include "pmacct.h"
 #include "isis.h"
 
@@ -63,7 +61,7 @@ extern struct isis *isis;
 #endif /* PNBBY */
 
 /* Utility mask array. */
-static const u_char maskbit[] = {
+static const u_char __attribute__((unused)) maskbit[] = {
   0x00, 0x80, 0xc0, 0xe0, 0xf0, 0xf8, 0xfc, 0xfe, 0xff
 };
 
@@ -100,69 +98,6 @@ area_match (struct list *left, struct list *right)
   return 0;			/* mismatch */
 }
 
-/*
- * Check if ip2 is in the ip1's network (function like Prefix.h:prefix_match() )
- * param ip1            the IS interface ip address structure
- * param ip2            the IIH's ip address
- * return  0            the IIH's IP is not in the IS's subnetwork
- *         1            the IIH's IP is in the IS's subnetwork
- */
-static int
-ip_same_subnet (struct prefix_ipv4 *ip1, struct in_addr *ip2)
-{
-  u_char *addr1, *addr2;
-  int shift, offset, offsetloop;
-  int len;
-
-  addr1 = (u_char *) & ip1->prefix.s_addr;
-  addr2 = (u_char *) & ip2->s_addr;
-  len = ip1->prefixlen;
-
-  shift = len % PNBBY;
-  offsetloop = offset = len / PNBBY;
-
-  while (offsetloop--)
-    if (addr1[offsetloop] != addr2[offsetloop])
-      return 0;
-
-  if (shift)
-    if (maskbit[shift] & (addr1[offset] ^ addr2[offset]))
-      return 0;
-
-  return 1;			/* match  */
-}
-
-/*
- * Compares two set of ip addresses
- * param left     the local interface's ip addresses
- * param right    the iih interface's ip address
- * return         0   no match;
- *                1   match;
- */
-static int
-ip_match (struct list *left, struct list *right)
-{
-  struct prefix_ipv4 *ip1;
-  struct in_addr *ip2;
-  struct listnode *node1, *node2;
-
-  if ((left == NULL) || (right == NULL))
-    return 0;
-  
-  for (ALL_LIST_ELEMENTS_RO (left, node1, ip1))
-  {
-    for (ALL_LIST_ELEMENTS_RO (right, node2, ip2))
-    {
-      if (ip_same_subnet (ip1, ip2))
-	{
-	  return 1;		/* match */
-	}
-    }
-
-  }
-  return 0;
-}
-
 /*
  * Checks whether we should accept a PDU of given level 
  */
@@ -248,7 +183,6 @@ tlvs_to_adj_ipv4_addrs (struct tlvs *tlvs, struct isis_adjacency *adj)
     }
 }
 
-#ifdef ENABLE_IPV6
 static void
 tlvs_to_adj_ipv6_addrs (struct tlvs *tlvs, struct isis_adjacency *adj)
 {
@@ -272,7 +206,6 @@ tlvs_to_adj_ipv6_addrs (struct tlvs *tlvs, struct isis_adjacency *adj)
     }
 
 }
-#endif /* ENABLE_IPV6 */
 
 /*
  *  RECEIVE SIDE                           
@@ -388,10 +321,8 @@ process_p2p_hello (struct isis_circuit *circuit)
   if (found & TLVFLAG_IPV4_ADDR)
     tlvs_to_adj_ipv4_addrs (&tlvs, adj);
 
-#ifdef ENABLE_IPV6
   if (found & TLVFLAG_IPV6_ADDR)
     tlvs_to_adj_ipv6_addrs (&tlvs, adj);
-#endif /* ENABLE_IPV6 */
 
   /* lets take care of the expiry */
   adj->expire.tv_sec = isis_now.tv_sec + adj->hold_time; 
@@ -680,6 +611,7 @@ process_lsp (int level, struct isis_circuit *circuit, u_char * ssnpa)
 
   /* Checksum sanity check - FIXME: move to correct place */
   /* 12 = sysid+pdu+remtime */
+/*
   if (iso_csum_verify (STREAM_PNT (circuit->rcv_stream) + 4,
 		       ntohs (hdr->pdu_len) - 12, &hdr->checksum))
     {
@@ -689,6 +621,7 @@ process_lsp (int level, struct isis_circuit *circuit, u_char * ssnpa)
 
       return ISIS_WARNING;
     }
+*/
 
   /* 7.3.15.1 a) 1 - external domain circuit will discard lsps */
   if (circuit->ext_domain)
@@ -960,9 +893,7 @@ process_snp (int snp_type, int level, struct isis_circuit *circuit,
 {
   int retval = ISIS_OK;
   int cmp, own_lsp;
-  char typechar = ' ';
   int len;
-  struct isis_adjacency *adj;
   struct isis_complete_seqnum_hdr *chdr = NULL;
   struct isis_partial_seqnum_hdr *phdr = NULL;
   uint32_t found = 0, expected = 0;
@@ -977,7 +908,6 @@ process_snp (int snp_type, int level, struct isis_circuit *circuit,
   if (snp_type == ISIS_SNP_CSNP_FLAG)
     {
       /* getting the header info */
-      typechar = 'C';
       chdr =
 	(struct isis_complete_seqnum_hdr *) STREAM_PNT (circuit->rcv_stream);
       circuit->rcv_stream->getp += ISIS_CSNP_HDRLEN;
@@ -990,7 +920,6 @@ process_snp (int snp_type, int level, struct isis_circuit *circuit,
     }
   else
     {
-      typechar = 'P';
       phdr =
 	(struct isis_partial_seqnum_hdr *) STREAM_PNT (circuit->rcv_stream);
       circuit->rcv_stream->getp += ISIS_PSNP_HDRLEN;
@@ -1492,12 +1421,12 @@ send_hello (struct isis_circuit *circuit, int level)
   else
     {
       hello_hdr.prio = circuit->u.bc.priority[level - 1];
-      if (level == 1 && circuit->u.bc.l1_desig_is)
+      if (level == 1 /* && circuit->u.bc.l1_desig_is */)
 	{
 	  memcpy (hello_hdr.lan_id, circuit->u.bc.l1_desig_is,
 		  ISIS_SYS_ID_LEN + 1);
 	}
-      else if (level == 2 && circuit->u.bc.l2_desig_is)
+      else if (level == 2 /* && circuit->u.bc.l2_desig_is */)
 	{
 	  memcpy (hello_hdr.lan_id, circuit->u.bc.l2_desig_is,
 		  ISIS_SYS_ID_LEN + 1);
@@ -1528,13 +1457,11 @@ send_hello (struct isis_circuit *circuit, int level)
     if (tlv_add_ip_addrs (circuit->ip_addrs, circuit->snd_stream))
       return ISIS_WARNING;
 
-#ifdef ENABLE_IPV6
   /* IPv6 Interface Address TLV */
   if (circuit->ipv6_router && circuit->ipv6_link &&
       circuit->ipv6_link->count > 0)
     if (tlv_add_ipv6_addrs (circuit->ipv6_link, circuit->snd_stream))
       return ISIS_WARNING;
-#endif /* ENABLE_IPV6 */
 
   /* We should always pad hellos, even on p2p links */
   /* if (circuit->u.bc.pad_hellos) */
@@ -1637,6 +1564,7 @@ int isis_send_pdu_p2p (struct isis_circuit *circuit, int level)
                     stream_get_endp (circuit->snd_stream), 0,
                     (struct sockaddr *) &sa,
                     sizeof (struct sockaddr_ll));
+  (void)written; //TODO treat error?
 
   return ISIS_OK;
 }
