@@ -22,8 +22,6 @@
  * 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  */
 
-#define __ISIS_ROUTE_C
-
 #include "pmacct.h"
 #include "isis.h"
 
@@ -74,10 +72,12 @@ isis_nexthop_create (struct in_addr *ip, unsigned int ifindex)
       Log(LOG_ERR, "ERROR ( %s/core/ISIS ): ISIS-Rte: isis_nexthop_create: out of memory!\n", config.name);
     }
 
-  nexthop->ifindex = ifindex;
-  memcpy (&nexthop->ip, ip, sizeof (struct in_addr));
-  isis_listnode_add (isis->nexthops, nexthop);
-  nexthop->lock++;
+  if (nexthop) {
+    nexthop->ifindex = ifindex;
+    memcpy (&nexthop->ip, ip, sizeof (struct in_addr));
+    isis_listnode_add (isis->nexthops, nexthop);
+    nexthop->lock++;
+  }
 
   return nexthop;
 }
@@ -112,7 +112,6 @@ nexthoplookup (struct list *nexthops, struct in_addr *ip,
   return 0;
 }
 
-#ifdef ENABLE_IPV6
 static struct isis_nexthop6 *
 isis_nexthop6_new (struct in6_addr *ip6, unsigned int ifindex)
 {
@@ -184,8 +183,6 @@ nexthop6lookup (struct list *nexthops6, struct in6_addr *ip6,
   return 0;
 }
 
-#endif /* ENABLE_IPV6 */
-
 static void
 adjinfo2nexthop (struct list *nexthops, struct isis_adjacency *adj)
 {
@@ -206,7 +203,6 @@ adjinfo2nexthop (struct list *nexthops, struct isis_adjacency *adj)
     }
 }
 
-#ifdef ENABLE_IPV6
 static void
 adjinfo2nexthop6 (struct list *nexthops6, struct isis_adjacency *adj)
 {
@@ -228,7 +224,6 @@ adjinfo2nexthop6 (struct list *nexthops6, struct isis_adjacency *adj)
 	}
     }
 }
-#endif
 
 static struct isis_route_info *
 isis_route_info_new (uint32_t cost, uint32_t depth, u_char family,
@@ -251,7 +246,6 @@ isis_route_info_new (uint32_t cost, uint32_t depth, u_char family,
       for (ALL_LIST_ELEMENTS_RO (adjacencies, node, adj))
         adjinfo2nexthop (rinfo->nexthops, adj);
     }
-#ifdef ENABLE_IPV6
   if (family == AF_INET6)
     {
       rinfo->nexthops6 = isis_list_new ();
@@ -259,8 +253,6 @@ isis_route_info_new (uint32_t cost, uint32_t depth, u_char family,
         adjinfo2nexthop6 (rinfo->nexthops6, adj);
     }
 
-#endif
-
   rinfo->cost = cost;
   rinfo->depth = depth;
 
@@ -276,13 +268,11 @@ isis_route_info_delete (struct isis_route_info *route_info)
       isis_list_delete (route_info->nexthops);
     }
 
-#ifdef ENABLE_IPV6
   if (route_info->nexthops6)
     {
       route_info->nexthops6->del = (void (*)(void *)) isis_nexthop6_delete;
       isis_list_delete (route_info->nexthops6);
     }
-#endif /* ENABLE_IPV6 */
 
   free(route_info);
 }
@@ -305,9 +295,7 @@ isis_route_info_same (struct isis_route_info *new,
 {
   struct listnode *node;
   struct isis_nexthop *nexthop;
-#ifdef ENABLE_IPV6
   struct isis_nexthop6 *nexthop6;
-#endif /* ENABLE_IPV6 */
   if (!isis_route_info_same_attrib (new, old))
     return 0;
 
@@ -323,7 +311,6 @@ isis_route_info_same (struct isis_route_info *new,
              == 0)
           return 0;
     }
-#ifdef ENABLE_IPV6
   else if (family == AF_INET6)
     {
       for (ALL_LIST_ELEMENTS_RO (new->nexthops6, node, nexthop6))
@@ -336,7 +323,6 @@ isis_route_info_same (struct isis_route_info *new,
                             nexthop6->ifindex) == 0)
           return 0;
     }
-#endif /* ENABLE_IPV6 */
 
   return 1;
 }
@@ -356,7 +342,6 @@ isis_nexthops_merge (struct list *new, struct list *old)
     }
 }
 
-#ifdef ENABLE_IPV6
 static void
 isis_nexthops6_merge (struct list *new, struct list *old)
 {
@@ -371,7 +356,6 @@ isis_nexthops6_merge (struct list *new, struct list *old)
       nexthop6->lock++;
     }
 }
-#endif /* ENABLE_IPV6 */
 
 static void
 isis_route_info_merge (struct isis_route_info *new,
@@ -379,10 +363,8 @@ isis_route_info_merge (struct isis_route_info *new,
 {
   if (family == AF_INET)
     isis_nexthops_merge (new->nexthops, old->nexthops);
-#ifdef ENABLE_IPV6
   else if (family == AF_INET6)
     isis_nexthops6_merge (new->nexthops6, old->nexthops6);
-#endif /* ENABLE_IPV6 */
 
   return;
 }
@@ -424,12 +406,13 @@ isis_route_create (struct isis_prefix *prefix, u_int32_t cost, u_int32_t depth,
 
   if (family == AF_INET)
     route_node = route_node_get (area->route_table[level - 1], prefix);
-#ifdef ENABLE_IPV6
   else if (family == AF_INET6)
     route_node = route_node_get (area->route_table6[level - 1], prefix);
-#endif /* ENABLE_IPV6 */
-  else
+  else {
+    if (rinfo_new) free(rinfo_new);
     return NULL;
+  }
+
   rinfo_old = route_node->info;
   if (!rinfo_old)
     {
@@ -563,7 +546,6 @@ void isis_route_validate_table (struct isis_area *area, struct route_table *tabl
 		drnode->info = NULL;
 	    }
 
-#ifdef ENABLE_IPV6
 	  if (rnode->p.family == AF_INET6)
 	    {
 	      drnode = route_node_get (area->route_table6[0], &rnode->p);
@@ -573,7 +555,6 @@ void isis_route_validate_table (struct isis_area *area, struct route_table *tabl
 	      if (drnode->info == rnode->info)
 		drnode->info = NULL;
 	    }
-#endif
 	      
 	  isis_route_delete (&rnode->p, table);
 	}
@@ -600,10 +581,8 @@ void isis_route_validate_merge (struct isis_area *area, int family)
 
   if (family == AF_INET)
     table = area->route_table[0];
-#ifdef ENABLE_IPV6
   else if (family == AF_INET6)
     table = area->route_table6[0];
-#endif
 
   for (rnode = route_top (table); rnode; rnode = route_next (rnode))
     {
@@ -615,10 +594,8 @@ void isis_route_validate_merge (struct isis_area *area, int family)
 
   if (family == AF_INET)
     table = area->route_table[1];
-#ifdef ENABLE_IPV6
   else if (family == AF_INET6)
     table = area->route_table6[1];
-#endif
 
   for (rnode = route_top (table); rnode; rnode = route_next (rnode))
     {
@@ -657,7 +634,6 @@ isis_route_validate (struct thread *thread)
   isis_route_validate_merge (area, AF_INET);
 
 validate_ipv6:
-#ifdef ENABLE_IPV6
   if (area->is_type == IS_LEVEL_1)
     {
       isis_route_validate_table (area, area->route_table6[0]);
@@ -670,7 +646,6 @@ validate_ipv6:
     }
 
   isis_route_validate_merge (area, AF_INET6);
-#endif
 
   return ISIS_OK;
 }
