@@ -62,22 +62,20 @@ struct list
 #define listgetdata(X) (assert((X)->data != NULL), (X)->data)
 
 /* Prototypes. */
-#if (!defined __LINKLIST_C)
-#define EXT extern
-#else
-#define EXT
-#endif
-EXT struct list *isis_list_new(void); /* encouraged: set list.del callback on new lists */
-EXT void isis_list_free (struct list *);
-EXT void isis_listnode_add (struct list *, void *);
-EXT void isis_listnode_add_sort (struct list *, void *);
-EXT void isis_listnode_add_after (struct list *, struct listnode *, void *);
-EXT void isis_listnode_delete (struct list *, void *);
-EXT struct listnode *isis_listnode_lookup (struct list *, void *);
-EXT void *isis_listnode_head (struct list *);
-EXT void isis_list_delete (struct list *);
-EXT void isis_list_delete_all_node (struct list *);
-#undef EXT
+extern struct list *isis_list_new(void); /* encouraged: set list.del callback on new lists */
+extern void isis_list_free (struct list *);
+extern void isis_listnode_add (struct list *, void *);
+extern void isis_listnode_add_sort (struct list *, void *);
+extern void isis_listnode_add_after (struct list *, struct listnode *, void *);
+extern void isis_listnode_delete (struct list *, void *);
+extern struct listnode *isis_listnode_lookup (struct list *, void *);
+extern void *isis_listnode_head (struct list *);
+extern void isis_list_delete (struct list *);
+extern void isis_list_delete_all_node (struct list *);
+extern void isis_list_delete_node (struct list *, struct listnode *);
+extern void isis_list_add_node_prev (struct list *, struct listnode *, void *);
+extern void isis_list_add_node_next (struct list *, struct listnode *, void *);
+extern void isis_list_add_list (struct list *, struct list *);
 
 /* List iteration macro. 
  * Usage: for (ALL_LIST_ELEMENTS (...) { ... }
