@@ -1,6 +1,6 @@
 /*
     pmacct (Promiscuous mode IP Accounting package)
-    pmacct is Copyright (C) 2003-2017 by Paolo Lucente
+    pmacct is Copyright (C) 2003-2019 by Paolo Lucente
 */
 
 /*
@@ -31,8 +31,6 @@
 
 /* $Id$ */
 
-#define __NFPROBE_NETFLOW9_C
-
 #include "common.h"
 #include "treetype.h"
 #include "nfprobe_plugin.h"
@@ -140,6 +138,9 @@ struct NF9_DATA_FLOWSET_HEADER {
 #define NF9_FLOW_APPLICATION_ID		95
 #define NF9_FLOW_APPLICATION_NAME	96
 /* ... */
+#define NF9_EXPORTER_IPV4_ADDRESS       130
+#define NF9_EXPORTER_IPV6_ADDRESS       131
+/* ... */
 #define NF9_FLOW_EXPORTER		144
 /* ... */
 #define NF9_FIRST_SWITCHED_MSEC         152
@@ -207,7 +208,6 @@ static struct NF9_SOFTFLOWD_TEMPLATE v4_template_out;
 static struct IPFIX_PEN_TEMPLATE_ADDENDUM v4_pen_template_out;
 static struct NF9_INTERNAL_TEMPLATE v4_int_template_out;
 static struct NF9_INTERNAL_TEMPLATE v4_pen_int_template_out;
-#if defined ENABLE_IPV6
 static struct NF9_SOFTFLOWD_TEMPLATE v6_template;
 static struct IPFIX_PEN_TEMPLATE_ADDENDUM v6_pen_template;
 static struct NF9_INTERNAL_TEMPLATE v6_int_template;
@@ -216,11 +216,12 @@ static struct NF9_SOFTFLOWD_TEMPLATE v6_template_out;
 static struct IPFIX_PEN_TEMPLATE_ADDENDUM v6_pen_template_out;
 static struct NF9_INTERNAL_TEMPLATE v6_int_template_out;
 static struct NF9_INTERNAL_TEMPLATE v6_pen_int_template_out;
-#endif
 static struct NF9_OPTIONS_TEMPLATE sampling_option_template;
 static struct NF9_INTERNAL_OPTIONS_TEMPLATE sampling_option_int_template;
 static struct NF9_OPTIONS_TEMPLATE class_option_template;
 static struct NF9_INTERNAL_OPTIONS_TEMPLATE class_option_int_template;
+static struct NF9_OPTIONS_TEMPLATE exporter_option_template;
+static struct NF9_INTERNAL_OPTIONS_TEMPLATE exporter_option_int_template;
 static char ftoft_buf_0[NF9_SOFTFLOWD_MAX_PACKET_SIZE*2];
 static char ftoft_buf_1[NF9_SOFTFLOWD_MAX_PACKET_SIZE*2];
 static char packet[NF9_SOFTFLOWD_MAX_PACKET_SIZE];
@@ -229,6 +230,7 @@ static int nf9_pkts_until_template = -1;
 static u_int8_t send_options = FALSE;
 static u_int8_t send_sampling_option = FALSE;
 static u_int8_t send_class_option = FALSE;
+static u_int8_t send_exporter_option = FALSE;
 
 /*
  * XXX: pmXXX_htonll(): similar to htonl() for 64 bits integers; no checks are done
@@ -443,9 +445,10 @@ flow_to_flowset_vlan_handler(char *flowset, const struct FLOW *flow, int idx, in
 }
 
 static int
-flow_to_flowset_mpls_handler(char *flowset, const struct FLOW *flow, int idx, int size)
+flow_to_flowset_mpls_label_top_handler(char *flowset, const struct FLOW *flow, int idx, int size)
 {
-  memcpy(flowset, &flow->mpls_label[idx], size);
+  encode_mpls_label(flowset, flow->mpls_label[idx]);
+  // memcpy(flowset, &flow->mpls_label[idx], size);
 
   return 0;
 }
@@ -539,10 +542,17 @@ flow_to_flowset_cp_handler(char *flowset, const struct FLOW *flow, int idx, int
 
     if (!cp_entry->ptr->pen) {
       if (cp_entry->ptr->len != PM_VARIABLE_LENGTH) {
-        if (flow->pcust[idx] && cp_entry->ptr->field_type)
-          memcpy(flowset, (flow->pcust[idx]+cp_entry->off), cp_entry->ptr->len);
-        else 
+        if (flow->pcust[idx] && cp_entry->ptr->field_type) {
+          if (cp_entry->ptr->semantics == CUSTOM_PRIMITIVE_TYPE_RAW) {
+            serialize_bin((flow->pcust[idx] + cp_entry->off), (u_char *)flowset, strlen((char *)(flow->pcust[idx] + cp_entry->off)));
+          }
+          else {
+	    memcpy(flowset, (flow->pcust[idx] + cp_entry->off), cp_entry->ptr->len);
+	  }
+	}
+        else {
           memset(flowset, 0, cp_entry->ptr->len);
+	}
 
         flowset += cp_entry->ptr->len;
       }
@@ -584,10 +594,17 @@ flow_to_flowset_cp_pen_handler(char *flowset, const struct FLOW *flow, int idx,
 
     if (config.nfprobe_version == 10 && cp_entry->ptr->pen) {
       if (cp_entry->ptr->len != PM_VARIABLE_LENGTH) {
-        if (flow->pcust[idx] && cp_entry->ptr->field_type)
-          memcpy(flowset, (flow->pcust[idx]+cp_entry->off), cp_entry->ptr->len);
-        else
+        if (flow->pcust[idx] && cp_entry->ptr->field_type) {
+	  if (cp_entry->ptr->semantics == CUSTOM_PRIMITIVE_TYPE_RAW) {
+	    serialize_bin((flow->pcust[idx] + cp_entry->off), (u_char *)flowset, strlen((char *)(flow->pcust[idx] + cp_entry->off)));
+	  }
+          else {
+	    memcpy(flowset, (flow->pcust[idx] + cp_entry->off), cp_entry->ptr->len);
+	  }
+	}
+        else {
           memset(flowset, 0, cp_entry->ptr->len);
+	}
 
         flowset += cp_entry->ptr->len;
       }
@@ -1014,6 +1031,17 @@ nf9_init_template(void)
           v4_int_template_out.r[rcount].length = 2;
 	  rcount++;
 	}
+        if (config.nfprobe_what_to_count_2 & COUNT_MPLS_LABEL_TOP) {
+          v4_template.r[rcount].type = htons(NF9_MPLS_LABEL_1);
+          v4_template.r[rcount].length = htons(3);
+          v4_int_template.r[rcount].handler = flow_to_flowset_mpls_label_top_handler;
+          v4_int_template.r[rcount].length = 3;
+          v4_template_out.r[rcount].type = htons(NF9_MPLS_LABEL_1);
+          v4_template_out.r[rcount].length = htons(3);
+          v4_int_template_out.r[rcount].handler = flow_to_flowset_mpls_label_top_handler;
+          v4_int_template_out.r[rcount].length = 3;
+          rcount++;
+        }
 	if (config.nfprobe_version == 10 && config.nfprobe_what_to_count & COUNT_TAG) {
 	  int rlen = sizeof(pm_id_t);
 
@@ -1132,7 +1160,6 @@ nf9_init_template(void)
 	  else v4_pen_int_template_out.tot_rec_len += v4_pen_int_template_out.r[idx].length;
         }
 
-#if defined ENABLE_IPV6
 	rcount = 0; rcount_pen = 0;
 	bzero(&v6_template, sizeof(v6_template));
 	bzero(&v6_pen_template, sizeof(v6_pen_template));
@@ -1420,6 +1447,17 @@ nf9_init_template(void)
           v6_int_template_out.r[rcount].length = 2;
 	  rcount++;
 	}
+        if (config.nfprobe_what_to_count_2 & COUNT_MPLS_LABEL_TOP) {
+          v6_template.r[rcount].type = htons(NF9_MPLS_LABEL_1);
+          v6_template.r[rcount].length = htons(3);
+          v6_int_template.r[rcount].handler = flow_to_flowset_mpls_label_top_handler;
+          v6_int_template.r[rcount].length = 3;
+          v6_template_out.r[rcount].type = htons(NF9_MPLS_LABEL_1);
+          v6_template_out.r[rcount].length = htons(3);
+          v6_int_template_out.r[rcount].handler = flow_to_flowset_mpls_label_top_handler;
+          v6_int_template_out.r[rcount].length = 3;
+          rcount++;
+        }
         if (config.nfprobe_version == 10 && config.nfprobe_what_to_count & COUNT_TAG) {
           int rlen = sizeof(pm_id_t);
 
@@ -1537,7 +1575,6 @@ nf9_init_template(void)
 	  if (config.nfprobe_version == 10 && v6_pen_int_template_out.r[idx].length == IPFIX_VARIABLE_LENGTH);
           else v6_pen_int_template_out.tot_rec_len += v6_pen_int_template_out.r[idx].length;
         }
-#endif
 }
 
 static void
@@ -1633,6 +1670,47 @@ nf9_init_options_template(void)
 
         for (idx = 0, class_option_int_template.tot_rec_len = 0; idx < rcount; idx++)
           class_option_int_template.tot_rec_len += class_option_int_template.r[idx].length;
+
+        rcount = 0;
+        bzero(&exporter_option_template, sizeof(exporter_option_template));
+        bzero(&exporter_option_int_template, sizeof(exporter_option_int_template));
+
+        if (config.nfprobe_version == 9) {
+	  flowset_id = NF9_OPTIONS_FLOWSET_ID;
+	  scope = NF9_OPT_SCOPE_SYSTEM;
+	}
+        else if (config.nfprobe_version == 10) {
+	  flowset_id = IPFIX_OPTIONS_FLOWSET_ID;
+	  scope = NF9_FLOW_EXPORTER;
+	}
+ 
+        exporter_option_template.r[rcount].type = htons(scope);
+        exporter_option_template.r[rcount].length = htons(slen);
+        exporter_option_int_template.r[rcount].length = slen;
+        rcount++;
+        exporter_option_template.r[rcount].type = htons(NF9_EXPORTER_IPV4_ADDRESS);
+        exporter_option_template.r[rcount].length = htons(4);
+        exporter_option_int_template.r[rcount].length = 4;
+        rcount++;
+        exporter_option_template.r[rcount].type = htons(NF9_EXPORTER_IPV6_ADDRESS);
+        exporter_option_template.r[rcount].length = htons(16);
+        exporter_option_int_template.r[rcount].length = 16;
+        rcount++;
+        exporter_option_template.h.c.flowset_id = htons(flowset_id);
+        exporter_option_template.h.c.length = htons( sizeof(struct NF9_OPTIONS_TEMPLATE_FLOWSET_HEADER) + (sizeof(struct NF9_TEMPLATE_FLOWSET_RECORD) * rcount) );
+        exporter_option_template.h.template_id = htons(NF9_OPTIONS_TEMPLATE_ID + 2 + config.nfprobe_id );
+        if (config.nfprobe_version == 9) {
+          exporter_option_template.h.scope_len = htons(4); /* NF9_OPT_SCOPE_SYSTEM */
+          exporter_option_template.h.option_len = htons(8); /* NF9_EXPORTER_IPV4_ADDRESS + NF9_EXPORTER_IPV6_ADDRESS */
+	}
+	else if (config.nfprobe_version == 10) {
+          exporter_option_template.h.scope_len = htons(2+1); /* IPFIX twist: NF9_EXPORTER_IPV4_ADDRESS + NF9_EXPORTER_IPV6_ADDRESS + NF9_OPT_SCOPE_SYSTEM */ 
+          exporter_option_template.h.option_len = htons(1); /* IPFIX twist: NF9_OPT_SCOPE_SYSTEM */
+	}
+        exporter_option_template.tot_len = sizeof(struct NF9_OPTIONS_TEMPLATE_FLOWSET_HEADER) + (sizeof(struct NF9_TEMPLATE_FLOWSET_RECORD) * rcount);
+
+        for (idx = 0, exporter_option_int_template.tot_rec_len = 0; idx < rcount; idx++)
+          exporter_option_int_template.tot_rec_len += exporter_option_int_template.r[idx].length;
 }
 
 static void
@@ -1651,9 +1729,9 @@ static int
 nf_flow_to_flowset(const struct FLOW *flow, u_char *packet, u_int len,
     const struct timeval *system_boot_time, u_int *len_used, int direction)
 {
-	u_int freclen_0, freclen_1, ret_len, nflows, idx;
+	u_int freclen_0 = 0, freclen_1 = 0, ret_len, nflows, idx;
 	u_int64_t rec64;
-	u_int32_t rec32;
+	u_int32_t rec32 = 0;
 	u_int8_t rec8;
 	char *ftoft_ptr_0 = ftoft_buf_0;
 	char *ftoft_ptr_1 = ftoft_buf_1;
@@ -1743,7 +1821,6 @@ nf_flow_to_flowset(const struct FLOW *flow, u_char *packet, u_int len,
                   freclen_0 += (v4_pen_int_template_out.tot_rec_len + add_len);
 		}
                 break;
-#if defined ENABLE_IPV6
           case AF_INET6:
                 rec8 = 6;
                 memcpy(ftoft_ptr_0, &rec8, 1);
@@ -1775,7 +1852,6 @@ nf_flow_to_flowset(const struct FLOW *flow, u_char *packet, u_int len,
                   freclen_0 += (v6_pen_int_template_out.tot_rec_len + add_len);
 		}
                 break;
-#endif
           default:
                 return (-1);
           }
@@ -1859,7 +1935,6 @@ nf_flow_to_flowset(const struct FLOW *flow, u_char *packet, u_int len,
                   freclen_1 += (v4_pen_int_template_out.tot_rec_len + add_len);
 		}
                 break;
-#if defined ENABLE_IPV6
           case AF_INET6:
                 rec8 = 6;
                 memcpy(ftoft_ptr_1, &rec8, 1);
@@ -1891,7 +1966,6 @@ nf_flow_to_flowset(const struct FLOW *flow, u_char *packet, u_int len,
                   freclen_1 += (v6_pen_int_template_out.tot_rec_len + add_len);
 		}
                 break;
-#endif
           default:
                 return (-1);
           }
@@ -1920,7 +1994,7 @@ static int
 nf_sampling_option_to_flowset(u_char *packet, u_int len, const struct timeval *system_boot_time, u_int *len_used)
 {
         u_int freclen, ret_len, nflows;
-        u_int32_t rec32;
+        u_int32_t rec32 = 0;
         u_int8_t rec8;
         char *ftoft_ptr_0 = ftoft_buf_0;
 
@@ -1933,12 +2007,10 @@ nf_sampling_option_to_flowset(u_char *packet, u_int len, const struct timeval *s
           memcpy(ftoft_ptr_0, &config.nfprobe_source_ha.address.ipv4, 4);
           ftoft_ptr_0 += 4;
           break;
-#if defined ENABLE_IPV6
         case AF_INET6:
           memcpy(ftoft_ptr_0, &config.nfprobe_source_ha.address.ipv6, 16);
           ftoft_ptr_0 += 16;
           break;
-#endif
         default:
           memset(ftoft_ptr_0, 0, 4);
           ftoft_ptr_0 += 4;
@@ -1988,12 +2060,10 @@ nf_class_option_to_flowset(u_int idx, u_char *packet, u_int len, const struct ti
           memcpy(ftoft_ptr_0, &config.nfprobe_source_ha.address.ipv4, 4);
           ftoft_ptr_0 += 4;
           break;
-#if defined ENABLE_IPV6
         case AF_INET6:
           memcpy(ftoft_ptr_0, &config.nfprobe_source_ha.address.ipv6, 16);
           ftoft_ptr_0 += 16;
           break;
-#endif
         default:
           memset(ftoft_ptr_0, 0, 4);
           ftoft_ptr_0 += 4;
@@ -2022,6 +2092,72 @@ nf_class_option_to_flowset(u_int idx, u_char *packet, u_int len, const struct ti
         return (nflows);
 }
 
+static int
+nf_exporter_option_to_flowset(u_char *packet, u_int len, const struct timeval *system_boot_time, u_int *len_used)
+{
+  u_int freclen, ret_len, nflows;
+  char *ftoft_ptr_0 = ftoft_buf_0;
+
+  memset(ftoft_buf_0, 0, sizeof(ftoft_buf_0));
+  *len_used = nflows = ret_len = 0;
+
+  /* NF9_OPT_SCOPE_SYSTEM */
+  switch (config.nfprobe_source_ha.family) {
+  case AF_INET:
+    memcpy(ftoft_ptr_0, &config.nfprobe_source_ha.address.ipv4, 4);
+    ftoft_ptr_0 += 4;
+    break;
+  case AF_INET6:
+    memcpy(ftoft_ptr_0, &config.nfprobe_source_ha.address.ipv6, 16);
+    ftoft_ptr_0 += 16;
+    break;
+  default:
+    memset(ftoft_ptr_0, 0, 4);
+    ftoft_ptr_0 += 4;
+    break;
+  }
+
+  switch (config.nfprobe_source_ha.family) {
+  /* NF9_EXPORTER_IPV4_ADDRESS */
+  case AF_INET:
+    memcpy(ftoft_ptr_0, &config.nfprobe_source_ha.address.ipv4, 4);
+    ftoft_ptr_0 += 4;
+
+    memset(ftoft_ptr_0, 0, 16);
+    ftoft_ptr_0 += 16;
+
+    break;
+  /* NF9_EXPORTER_IPV6_ADDRESS */
+  case AF_INET6:
+    memset(ftoft_ptr_0, 0, 4);
+    ftoft_ptr_0 += 4;
+
+    memcpy(ftoft_ptr_0, &config.nfprobe_source_ha.address.ipv6, 16);
+    ftoft_ptr_0 += 16;
+
+    break;
+  default:
+    memset(ftoft_ptr_0, 0, 4);
+    ftoft_ptr_0 += 4;
+
+    memset(ftoft_ptr_0, 0, 16);
+    ftoft_ptr_0 += 16;
+
+    break;
+  }
+
+  freclen = exporter_option_int_template.tot_rec_len;
+
+  if (ret_len + freclen > len) return (ERR);
+
+  memcpy(packet + ret_len, ftoft_buf_0, freclen);
+  ret_len += freclen;
+  nflows++;
+
+  *len_used = ret_len;
+  return (nflows);
+}
+
 /*
  * Given an array of expired flows, send netflow v9 report packets
  * Returns number of packets sent or -1 on error
@@ -2029,7 +2165,7 @@ nf_class_option_to_flowset(u_int idx, u_char *packet, u_int len, const struct ti
 int
 send_netflow_v9(struct FLOW **flows, int num_flows, int nfsock,
     u_int64_t *flows_exported, struct timeval *system_boot_time,
-    int verbose_flag, u_int8_t engine_type, u_int8_t engine_id)
+    int verbose_flag, u_int8_t unused, u_int32_t source_id)
 {
 	struct NF9_HEADER *nf9;
 	struct IPFIX_HEADER *nf10;
@@ -2040,7 +2176,6 @@ send_netflow_v9(struct FLOW **flows, int num_flows, int nfsock,
 	int direction, new_direction;
 	socklen_t errsz;
 	int err, r, flow_i, class_i;
-	u_int8_t *sid_ptr;
 
 	memset(packet, 0, sizeof(packet));
 	gettimeofday(&now, NULL);
@@ -2051,6 +2186,9 @@ send_netflow_v9(struct FLOW **flows, int num_flows, int nfsock,
 		nf9_pkts_until_template = 0;
 	}		
 
+        
+        offset = 0;
+        r = 0;
 	num_packets = 0;
 	num_class = pmct_find_first_free(); 
 
@@ -2067,11 +2205,7 @@ send_netflow_v9(struct FLOW **flows, int num_flows, int nfsock,
 		  nf9->uptime_ms = htonl(timeval_sub_ms(&now, system_boot_time));
 		  nf9->time_sec = htonl(time(NULL));
 		  nf9->package_sequence = htonl(++(*flows_exported));
-
-		  nf9->source_id = 0;
-		  sid_ptr = (u_int8_t *) &nf9->source_id;
-		  sid_ptr[2] = engine_type; 
-		  sid_ptr[3] = engine_id; 
+		  nf9->source_id = htonl(source_id);
 
 		  offset = sizeof(*nf9);
 		}
@@ -2082,11 +2216,7 @@ send_netflow_v9(struct FLOW **flows, int num_flows, int nfsock,
                   nf10->len = 0;
                   nf10->time_sec = htonl(time(NULL));
                   nf10->package_sequence = htonl(*flows_exported);
-
-                  nf10->source_id = 0;
-                  sid_ptr = (u_int8_t *) &nf10->source_id;
-                  sid_ptr[2] = engine_type;
-                  sid_ptr[3] = engine_id;
+                  nf10->source_id = htonl(source_id);
 
                   offset = sizeof(*nf10);
 		}
@@ -2108,7 +2238,6 @@ send_netflow_v9(struct FLOW **flows, int num_flows, int nfsock,
                         offset += v4_pen_template_out.tot_len;
                         flows++;
 			tot_len += v4_template_out.tot_len + v4_pen_template_out.tot_len;
-#if defined ENABLE_IPV6
 			memcpy(packet + offset, &v6_template, v6_template.tot_len);
 			offset += v6_template.tot_len; 
 			memcpy(packet + offset, &v6_pen_template, v6_pen_template.tot_len);
@@ -2122,7 +2251,7 @@ send_netflow_v9(struct FLOW **flows, int num_flows, int nfsock,
                         offset += v6_pen_template_out.tot_len;
                         flows++;
 			tot_len += v6_template_out.tot_len + v6_pen_template_out.tot_len; 
-#endif
+
 			if (config.sampling_rate || config.ext_sampling_rate) {
                           memcpy(packet + offset, &sampling_option_template, sampling_option_template.tot_len);
                           offset += sampling_option_template.tot_len;
@@ -2144,6 +2273,14 @@ send_netflow_v9(struct FLOW **flows, int num_flows, int nfsock,
 			  send_options = TRUE;
                           send_class_option = TRUE;
 			}
+                        if (config.nfprobe_source_ip) {
+                          memcpy(packet + offset, &exporter_option_template, exporter_option_template.tot_len);
+                          offset += exporter_option_template.tot_len;
+                          flows++;
+                          tot_len += exporter_option_template.tot_len;
+                          send_options = TRUE;
+                          send_exporter_option = TRUE;
+                        }
 			nf9_pkts_until_template = NF9_DEFAULT_TEMPLATE_INTERVAL;
 
 			if (config.nfprobe_version == 9) nf9->flows = flows;
@@ -2154,7 +2291,7 @@ send_netflow_v9(struct FLOW **flows, int num_flows, int nfsock,
 		for (flow_i = 0, class_i = 0; flow_i + flow_j < num_flows; flow_i++) {
 			/* Shall we send a new flowset header? */
 			if (dh == NULL || (!send_options && (flows[flow_i + flow_j]->af != last_af || new_direction)) ||
-			    send_sampling_option || (send_class_option && !class_i) ) {
+			    send_sampling_option || send_exporter_option || (send_class_option && !class_i) ) {
 				if (dh != NULL) {
 					if (offset % 4 != 0) {
 						/* Pad to multiple of 4 */
@@ -2180,6 +2317,10 @@ send_netflow_v9(struct FLOW **flows, int num_flows, int nfsock,
 				    dh->c.flowset_id = class_option_template.h.template_id;
 				    // last_af = 0; new_direction = TRUE;
 				  }
+				  else if (send_exporter_option) {
+				    dh->c.flowset_id = exporter_option_template.h.template_id;
+				    // last_af = 0; new_direction = TRUE;
+				  }
 				}
 				else {
 				  if (flows[flow_i + flow_j]->af == AF_INET) {
@@ -2188,14 +2329,12 @@ send_netflow_v9(struct FLOW **flows, int num_flows, int nfsock,
 				    else if (direction == DIRECTION_OUT)
 				      dh->c.flowset_id = v4_template_out.h.template_id;
 				  }
-#if defined ENABLE_IPV6
 				  else if (flows[flow_i + flow_j]->af == AF_INET6) {
 				    if (direction == DIRECTION_IN)
 				      dh->c.flowset_id = v6_template.h.template_id;
 				    else if (direction == DIRECTION_OUT)
 				      dh->c.flowset_id = v6_template_out.h.template_id;
 				  }
-#endif
 				  // last_af = flows[flow_i + flow_j]->af; /* XXX */
 				}
 				last_valid = offset;
@@ -2207,20 +2346,25 @@ send_netflow_v9(struct FLOW **flows, int num_flows, int nfsock,
 			/* Send flowset data over */
 			if (send_options) {
 			  if (send_sampling_option) {
-                            r = nf_sampling_option_to_flowset(packet + offset,
+                            r = nf_sampling_option_to_flowset((u_char *)(packet + offset),
                               sizeof(packet) - offset, system_boot_time, &inc);
 			    send_sampling_option = FALSE;
 			  }
 			  else if (send_class_option) {
-                            r = nf_class_option_to_flowset(class_i + class_j, packet + offset,
+                            r = nf_class_option_to_flowset(class_i + class_j, (u_char *)(packet + offset),
                               sizeof(packet) - offset, system_boot_time, &inc);
 
 			    if (r > 0) class_i += r;
 			    if (class_i + class_j >= num_class) send_class_option = FALSE;
 			  }
+			  else if (send_exporter_option) {
+                            r = nf_exporter_option_to_flowset((u_char *)(packet + offset),
+                              sizeof(packet) - offset, system_boot_time, &inc);
+			    send_exporter_option = FALSE;
+			  }
 			}
 			else 
-			  r = nf_flow_to_flowset(flows[flow_i + flow_j], packet + offset,
+			  r = nf_flow_to_flowset(flows[flow_i + flow_j], (u_char *)(packet + offset),
 			    sizeof(packet) - offset, system_boot_time, &inc, direction);
 
 			/* Wrap up */
@@ -2251,7 +2395,8 @@ send_netflow_v9(struct FLOW **flows, int num_flows, int nfsock,
 
 			  if (send_options) {
 			    if (!send_sampling_option &&
-				!send_class_option) {
+				!send_class_option &&
+				!send_exporter_option) {
 			      send_options = FALSE;
 			    }
 			    flow_i--;
