@@ -1,6 +1,6 @@
 /*
     pmacct (Promiscuous mode IP Accounting package)
-    pmacct is Copyright (C) 2003-2017 by Paolo Lucente
+    pmacct is Copyright (C) 2003-2019 by Paolo Lucente
 */
 
 /*
@@ -29,8 +29,6 @@
  * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  */
 
-#define __NFPROBE_PLUGIN_C
-
 /*
  * This is software implementation of Cisco's NetFlow(tm) traffic 
  * reporting system. It operates by listening (via libpcap) on a 
@@ -49,19 +47,19 @@
  * As this implementation watches traffic promiscuously, it is likely to 
  * place significant load on hosts or gateways on which it is installed.
  */
-
 #include "common.h"
 #include "addr.h"
 #include "sys-tree.h"
 #include "convtime.h"
-#include "../nfacctd.h"
+#include "nfacctd.h"
 #include "nfprobe_plugin.h"
 #include "treetype.h"
 
 #include "pmacct-data.h"
-#include "plugin_hooks.h"
 #include "net_aggr.h"
 #include "ports_aggr.h"
+#include "plugin_hooks.h"
+#include "plugin_common.h"
 
 /* Global variables */
 static int verbose_flag = 0;		/* Debugging flag */
@@ -83,8 +81,8 @@ struct CB_CTXT {
 };
 
 /* Netflow send functions */
-typedef int (netflow_send_func_t)(struct FLOW **, int, int, u_int64_t *,
-    struct timeval *, int, u_int8_t, u_int8_t);
+typedef int (netflow_send_func_t)(struct FLOW **, int, int, u_int64_t *, struct timeval *, int, u_int8_t, u_int32_t);
+
 struct NETFLOW_SENDER {
 	int version;
 	netflow_send_func_t *func;
@@ -94,7 +92,6 @@ struct NETFLOW_SENDER {
 /* Array of NetFlow export function that we know of. NB. nf[0] is default */
 static const struct NETFLOW_SENDER nf[] = {
 	{ 5, send_netflow_v5, 0 },
-	{ 1, send_netflow_v1, 0 },
 	{ 9, send_netflow_v9, 1 },
 	{ 10, send_netflow_v9, 1 },
 	{ -1, NULL, 0 },
@@ -184,14 +181,11 @@ EXPIRY_GENERATE(EXPIRIES, EXPIRY, trp, expiry_compare);
 static const char *
 format_time(time_t t)
 {
-	struct tm *tm;
-	static char buf[20];
+	static char buf[32];
 
-	tm = localtime(&t);
-	strftime(buf, sizeof(buf), "%Y-%m-%dT%H:%M:%S", tm);
+	pm_strftime(buf, sizeof(buf), "%Y-%m-%dT%H:%M:%S$tzone", &t, config.timestamps_utc);
 
 	return (buf);
-
 }
 
 /* Format a flow in a verbose and ugly way */
@@ -207,8 +201,8 @@ format_flow(struct FLOW *flow)
 	snprintf(stime, sizeof(ftime), "%s", format_time(flow->flow_start.tv_sec));
 	snprintf(ftime, sizeof(ftime), "%s", format_time(flow->flow_last.tv_sec));
 
-	snprintf(buf, sizeof(buf),  "seq:%llu [%s]:%u <> [%s]:%u proto:%u "
-	    "octets>:%llu packets>:%llu octets<:%llu packets<:%llu "
+	snprintf(buf, sizeof(buf),  "seq:%" PRIu64 " [%s]:%u <> [%s]:%u proto:%u "
+	    "octets>:%" PRIu64 " packets>:%" PRIu64 " octets<:%" PRIu64 " packets<:%" PRIu64 " "
 	    "start:%s.%03u finish:%s.%03u tcp>:%02x tcp<:%02x "
 	    "flowlabel>:%08x flowlabel<:%08x ",
 	    flow->flow_seq,
@@ -216,8 +210,8 @@ format_flow(struct FLOW *flow)
 	    (int)flow->protocol, 
 	    flow->octets[0], flow->packets[0], 
 	    flow->octets[1], flow->packets[1], 
-	    stime, (flow->flow_start.tv_usec + 500) / 1000, 
-	    ftime, (flow->flow_last.tv_usec + 500) / 1000,
+	    stime, (unsigned int)((flow->flow_start.tv_usec + 500) / 1000), 
+	    ftime, (unsigned int)((flow->flow_last.tv_usec + 500) / 1000),
 	    flow->tcp_flags[0], flow->tcp_flags[1],
 	    flow->ip6_flowlabel[0], flow->ip6_flowlabel[1]);
 
@@ -235,7 +229,7 @@ format_flow_brief(struct FLOW *flow)
 	inet_ntop(flow->af, &flow->addr[1], addr2, sizeof(addr2));
 
 	snprintf(buf, sizeof(buf), 
-	    "seq:%llu [%s]:%hu <> [%s]:%hu proto:%u",
+	    "seq:%" PRIu64 " [%s]:%hu <> [%s]:%hu proto:%u",
 	    flow->flow_seq,
 	    addr1, ntohs(flow->port[0]), addr2, ntohs(flow->port[1]),
 	    (int)flow->protocol);
@@ -287,8 +281,10 @@ transport_to_flowrec(struct FLOW *flow, struct pkt_data *data, struct pkt_extras
 }
 
 static int
-l2_to_flowrec(struct FLOW *flow, struct pkt_data *data, struct pkt_extras *extras, int ndx)
+l2_to_flowrec(struct FLOW *flow, struct primitives_ptrs *prim_ptrs, int ndx)
 {
+  struct pkt_data *data = prim_ptrs->data;
+  struct pkt_mpls_primitives *pmpls = prim_ptrs->pmpls;
   struct pkt_primitives *p = &data->primitives;
   int direction = 0;
 
@@ -322,41 +318,77 @@ l2_to_flowrec(struct FLOW *flow, struct pkt_data *data, struct pkt_extras *extra
   memcpy(&flow->mac[ndx][0], &p->eth_shost, 6);
   memcpy(&flow->mac[ndx ^ 1][0], &p->eth_dhost, 6);
   flow->vlan = p->vlan_id;
-  flow->mpls_label[ndx] = extras->mpls_top_label;
 #endif
 
-  if (!p->ifindex_in && !p->ifindex_out) {
-    if (config.nfprobe_ifindex_type) {
+  if (pmpls) flow->mpls_label[ndx] = pmpls->mpls_label_top;
+
+  /* handling input interface */
+  flow->ifindex[ndx] = 0;
+
+  if (p->ifindex_in) flow->ifindex[ndx] = p->ifindex_in;
+
+  if (!flow->ifindex[ndx] || config.nfprobe_ifindex_override) {
+    if (config.nfprobe_ifindex_type && direction == DIRECTION_IN) {
       switch (config.nfprobe_ifindex_type) {
       case IFINDEX_STATIC:
-        flow->ifindex[ndx] = (direction == DIRECTION_IN) ? config.nfprobe_ifindex : 0;
-        flow->ifindex[ndx ^ 1] = (direction == DIRECTION_OUT) ? config.nfprobe_ifindex : 0;
-        break;
+	if (config.nfprobe_ifindex) {
+	  flow->ifindex[ndx] = config.nfprobe_ifindex;
+	}
+
+	break;
       case IFINDEX_TAG:
-        flow->ifindex[ndx] = (direction == DIRECTION_IN) ? p->tag : 0;
-	flow->ifindex[ndx ^ 1] = (direction == DIRECTION_OUT) ? p->tag : 0;
+	if (p->tag) {
+	  flow->ifindex[ndx] = p->tag;
+	}
+
 	break;
       case IFINDEX_TAG2:
-        flow->ifindex[ndx] = (direction == DIRECTION_IN) ? p->tag2 : 0;
-	flow->ifindex[ndx ^ 1] = (direction == DIRECTION_OUT) ? p->tag2 : 0;
-        break;
-      default:
-        flow->ifindex[ndx] = 0;
-	flow->ifindex[ndx ^ 1] = 0;
+	if (p->tag2) {
+	  flow->ifindex[ndx] = p->tag2;
+	}
+
+	break;
       }
     }
   }
-  else {
-    flow->ifindex[ndx] = p->ifindex_in;
-    flow->ifindex[ndx ^ 1] = p->ifindex_out;
+
+  /* handling output interface */
+  flow->ifindex[ndx ^ 1] = 0;
+
+  if (p->ifindex_out) flow->ifindex[ndx ^ 1] = p->ifindex_out;
+
+  if (!flow->ifindex[ndx ^ 1] || config.nfprobe_ifindex_override) {
+    if (config.nfprobe_ifindex_type && direction == DIRECTION_OUT) {
+      switch (config.nfprobe_ifindex_type) {
+      case IFINDEX_STATIC:
+	if (config.nfprobe_ifindex) {
+	  flow->ifindex[ndx ^ 1] = config.nfprobe_ifindex; 
+	}
+
+	break;
+      case IFINDEX_TAG:
+	if (p->tag) {
+	  flow->ifindex[ndx ^ 1] = p->tag;
+	}
+
+	break;
+      case IFINDEX_TAG2:
+	if (p->tag2) {
+	  flow->ifindex[ndx ^ 1] = p->tag2;
+	}
+
+	break;
+      }
+    }
   }
 
   return (0);
 }
 
 static int
-l2_to_flowrec_update(struct FLOW *flow, struct pkt_data *data, struct pkt_extras *extras, int ndx)
+l2_to_flowrec_update(struct FLOW *flow, struct primitives_ptrs *prim_ptrs, int ndx)
 {
+  struct pkt_data *data = prim_ptrs->data;
   struct pkt_primitives *p = &data->primitives;
   int direction = 0;
 
@@ -395,7 +427,7 @@ ASN_to_flowrec(struct FLOW *flow, struct pkt_data *data, int ndx)
 }
 
 static int
-cust_to_flowrec(struct FLOW *flow, char *pcust, int ndx)
+cust_to_flowrec(struct FLOW *flow, u_char *pcust, int ndx)
 {
   if (pcust) {
     if (!flow->pcust[ndx]) flow->pcust[ndx] = malloc(config.cpptrs.len);
@@ -434,8 +466,9 @@ static int
 ipv4_to_flowrec(struct FLOW *flow, struct primitives_ptrs *prim_ptrs, int *isfrag, int af)
 {
   struct pkt_data *data = prim_ptrs->data;
+  struct pkt_bgp_primitives *pbgp = prim_ptrs->pbgp;
   struct pkt_extras *extras = prim_ptrs->pextras;
-  char *pcust = prim_ptrs->pcust;
+  u_char *pcust = prim_ptrs->pcust;
   struct pkt_vlen_hdr_primitives *pvlen = prim_ptrs->pvlen;
   struct pkt_primitives *p = &data->primitives;
   int ndx;
@@ -446,7 +479,7 @@ ipv4_to_flowrec(struct FLOW *flow, struct primitives_ptrs *prim_ptrs, int *isfra
   flow->af = af;
   flow->addr[ndx].v4 = p->src_ip.address.ipv4;
   flow->addr[ndx ^ 1].v4 = p->dst_ip.address.ipv4;
-  flow->bgp_next_hop[ndx].v4 = extras->bgp_next_hop.address.ipv4;
+  if (pbgp) flow->bgp_next_hop[ndx].v4 = pbgp->peer_dst_ip.address.ipv4;
   flow->mask[ndx] = p->src_nmask;
   flow->mask[ndx ^ 1] = p->dst_nmask;
   flow->tos[ndx] = p->tos;
@@ -463,7 +496,7 @@ ipv4_to_flowrec(struct FLOW *flow, struct primitives_ptrs *prim_ptrs, int *isfra
 
   *isfrag = 0;
 
-  l2_to_flowrec(flow, data, extras, ndx);
+  l2_to_flowrec(flow, prim_ptrs, ndx);
   ASN_to_flowrec(flow, data, ndx);
   cust_to_flowrec(flow, pcust, ndx);
   vlen_to_flowrec(flow, pvlen, ndx);
@@ -475,8 +508,8 @@ static int
 ipv4_to_flowrec_update(struct FLOW *flow, struct primitives_ptrs *prim_ptrs, int *isfrag, int af)
 {
   struct pkt_data *data = prim_ptrs->data;
-  struct pkt_extras *extras = prim_ptrs->pextras;
-  char *pcust = prim_ptrs->pcust;
+  struct pkt_bgp_primitives *pbgp = prim_ptrs->pbgp;
+  u_char *pcust = prim_ptrs->pcust;
   struct pkt_vlen_hdr_primitives *pvlen = prim_ptrs->pvlen;
   struct pkt_primitives *p = &data->primitives;
   int ndx;
@@ -484,26 +517,27 @@ ipv4_to_flowrec_update(struct FLOW *flow, struct primitives_ptrs *prim_ptrs, int
   /* Prepare to store flow in canonical format */
   ndx = memcmp(&p->src_ip.address.ipv4, &p->dst_ip.address.ipv4, sizeof(p->src_ip.address.ipv4)) > 0 ? 1 : 0;
 
-  if (!flow->bgp_next_hop[ndx].v4.s_addr) flow->bgp_next_hop[ndx].v4 = extras->bgp_next_hop.address.ipv4;
+  if (pbgp && !flow->bgp_next_hop[ndx].v4.s_addr)
+    flow->bgp_next_hop[ndx].v4 = pbgp->peer_dst_ip.address.ipv4;
 
-  l2_to_flowrec_update(flow, data, extras, ndx);
+  l2_to_flowrec_update(flow, prim_ptrs, ndx);
   cust_to_flowrec(flow, pcust, ndx);
   vlen_to_flowrec(flow, pvlen, ndx);
 
   return (0);
 }
 
-#if defined ENABLE_IPV6
 /* Convert a IPv6 packet to a partial flow record (used for comparison) */
 static int
 ipv6_to_flowrec(struct FLOW *flow, struct primitives_ptrs *prim_ptrs, int *isfrag, int af)
 {
   struct pkt_data *data = prim_ptrs->data;
+  struct pkt_bgp_primitives *pbgp = prim_ptrs->pbgp;
   struct pkt_extras *extras = prim_ptrs->pextras;
-  char *pcust = prim_ptrs->pcust;
+  u_char *pcust = prim_ptrs->pcust;
   struct pkt_vlen_hdr_primitives *pvlen = prim_ptrs->pvlen;
   struct pkt_primitives *p = &data->primitives;
-  int ndx, nxt;
+  int ndx;
 
   /* Prepare to store flow in canonical format */
   ndx = memcmp(&p->src_ip.address.ipv6, &p->dst_ip.address.ipv6, sizeof(p->src_ip.address.ipv6)) > 0 ? 1 : 0; 
@@ -513,7 +547,7 @@ ipv6_to_flowrec(struct FLOW *flow, struct primitives_ptrs *prim_ptrs, int *isfra
   flow->ip6_flowlabel[ndx] = 0;
   flow->addr[ndx].v6 = p->src_ip.address.ipv6; 
   flow->addr[ndx ^ 1].v6 = p->dst_ip.address.ipv6; 
-  flow->bgp_next_hop[ndx].v6 = extras->bgp_next_hop.address.ipv6;
+  if (pbgp) flow->bgp_next_hop[ndx].v6 = pbgp->peer_dst_ip.address.ipv6;
   flow->mask[ndx] = p->src_nmask;
   flow->mask[ndx ^ 1] = p->dst_nmask;
   flow->protocol = p->proto;
@@ -529,7 +563,7 @@ ipv6_to_flowrec(struct FLOW *flow, struct primitives_ptrs *prim_ptrs, int *isfra
 
   *isfrag = 0;
 
-  l2_to_flowrec(flow, data, extras, ndx);
+  l2_to_flowrec(flow, prim_ptrs, ndx);
   ASN_to_flowrec(flow, data, ndx);
   cust_to_flowrec(flow, pcust, ndx);
   vlen_to_flowrec(flow, pvlen, ndx);
@@ -541,8 +575,8 @@ static int
 ipv6_to_flowrec_update(struct FLOW *flow, struct primitives_ptrs *prim_ptrs, int *isfrag, int af)
 {
   struct pkt_data *data = prim_ptrs->data;
-  struct pkt_extras *extras = prim_ptrs->pextras;
-  char *pcust = prim_ptrs->pcust;
+  struct pkt_bgp_primitives *pbgp = prim_ptrs->pbgp;
+  u_char *pcust = prim_ptrs->pcust;
   struct pkt_vlen_hdr_primitives *pvlen = prim_ptrs->pvlen;
   struct pkt_primitives *p = &data->primitives;
   struct in6_addr dummy_ipv6; 
@@ -552,16 +586,15 @@ ipv6_to_flowrec_update(struct FLOW *flow, struct primitives_ptrs *prim_ptrs, int
   memset(&dummy_ipv6, 0, sizeof(dummy_ipv6));
   ndx = memcmp(&p->src_ip.address.ipv6, &p->dst_ip.address.ipv6, sizeof(p->src_ip.address.ipv6)) > 0 ? 1 : 0;
 
-  if (!memcmp(&dummy_ipv6, &flow->bgp_next_hop[ndx].v6, sizeof(dummy_ipv6)))
-    flow->bgp_next_hop[ndx].v6 = extras->bgp_next_hop.address.ipv6;
+  if (pbgp && !memcmp(&dummy_ipv6, &flow->bgp_next_hop[ndx].v6, sizeof(dummy_ipv6)))
+    flow->bgp_next_hop[ndx].v6 = pbgp->peer_dst_ip.address.ipv6;
 
-  l2_to_flowrec_update(flow, data, extras, ndx);
+  l2_to_flowrec_update(flow, prim_ptrs, ndx);
   cust_to_flowrec(flow, pcust, ndx);
   vlen_to_flowrec(flow, pvlen, ndx);
 
   return (0);
 }
-#endif 
 
 static void
 flow_update_expiry(struct FLOWTRACK *ft, struct FLOW *flow)
@@ -640,9 +673,7 @@ flow_update_expiry(struct FLOWTRACK *ft, struct FLOW *flow)
 
 	if (ft->icmp_timeout != 0 &&
 	    ((flow->af == AF_INET && flow->protocol == IPPROTO_ICMP)
-#if defined ENABLE_IPV6	   
 	    || ((flow->af == AF_INET6 && flow->protocol == IPPROTO_ICMPV6))
-#endif
 	   )) {
 		/* UDP flows */
 		flow->expiry->expires_at = flow->flow_last.tv_sec + 
@@ -682,7 +713,7 @@ process_packet(struct FLOWTRACK *ft, struct primitives_ptrs *prim_ptrs, const st
   struct pkt_data *data = prim_ptrs->data;
 
   struct FLOW tmp, *flow;
-  int frag, af, dont_summarize = (config.acct_type == ACCT_NF ? 1 : 0);
+  int frag, af;
 
   ft->total_packets += data->pkt_num;
   af = (data->primitives.src_ip.family == 0 ? AF_INET : data->primitives.src_ip.family);
@@ -695,12 +726,10 @@ process_packet(struct FLOWTRACK *ft, struct primitives_ptrs *prim_ptrs, const st
     if (ipv4_to_flowrec(&tmp, prim_ptrs, &frag, af) == -1)
       goto bad;
     break;
-#if defined ENABLE_IPV6
   case AF_INET6:
     if (ipv6_to_flowrec(&tmp, prim_ptrs, &frag, af) == -1)
       goto bad;
     break;
-#endif
   default:
   bad: 
     ft->bad_packets += data->pkt_num;
@@ -713,7 +742,7 @@ process_packet(struct FLOWTRACK *ft, struct primitives_ptrs *prim_ptrs, const st
     ft->frag_packets += data->pkt_num;
 
   /* If a matching flow does not exist, create and insert one */
-  if (dont_summarize || ((flow = FLOW_FIND(FLOWS, &ft->flows, &tmp)) == NULL)) {
+  if (config.nfprobe_dont_cache || ((flow = FLOW_FIND(FLOWS, &ft->flows, &tmp)) == NULL)) {
     /* Allocate and fill in the flow */
     if ((flow = malloc(sizeof(*flow))) == NULL) return (PP_MALLOC_FAIL);
     memcpy(flow, &tmp, sizeof(*flow));
@@ -725,9 +754,8 @@ process_packet(struct FLOWTRACK *ft, struct primitives_ptrs *prim_ptrs, const st
     if ((flow->expiry = malloc(sizeof(*flow->expiry))) == NULL)
       return (PP_MALLOC_FAIL);
     flow->expiry->flow = flow;
-    /* Expiration note: 0 means expire immediately; we prefer this to happen 
-       when attaching to nfacctd - ie. dont_summarize is TRUE */
-    if (!dont_summarize) flow->expiry->expires_at = 1;
+    /* Expiration note: 0 means expire immediately */
+    if (!config.nfprobe_dont_cache) flow->expiry->expires_at = 1;
     else flow->expiry->expires_at = 0;
     flow->expiry->reason = R_GENERAL;
     EXPIRY_INSERT(EXPIRIES, &ft->expiries, flow->expiry);
@@ -755,11 +783,9 @@ process_packet(struct FLOWTRACK *ft, struct primitives_ptrs *prim_ptrs, const st
     case AF_INET:
       ipv4_to_flowrec_update(flow, prim_ptrs, &frag, af);
     break;
-#if defined ENABLE_IPV6
     case AF_INET6:
       ipv6_to_flowrec_update(flow, prim_ptrs, &frag, af);
     break;
-#endif
     }
     if (!flow->class) flow->class = tmp.class;
 #if defined (WITH_NDPI)
@@ -922,7 +948,7 @@ next_expire(struct FLOWTRACK *ft)
 #define CE_EXPIRE_ALL		-1 /* Expire all flows immediately */
 #define CE_EXPIRE_FORCED	1  /* Only expire force-expired flows */
 static int
-check_expired(struct FLOWTRACK *ft, struct NETFLOW_TARGET *target, int ex, u_int8_t engine_type, u_int8_t engine_id)
+check_expired(struct FLOWTRACK *ft, struct NETFLOW_TARGET *target, int ex, u_int8_t engine_type, u_int32_t engine_id)
 {
 	struct FLOW **expired_flows, **oldexp;
 	int num_expired, i, r;
@@ -948,7 +974,7 @@ check_expired(struct FLOWTRACK *ft, struct NETFLOW_TARGET *target, int ex, u_int
 		    (expiry->expires_at < now.tv_sec))) {
 			/* Flow has expired */
 			if (verbose_flag)
-				Log(LOG_DEBUG, "DEBUG ( %s/%s ): Queuing flow seq:%llu (%p) for expiry\n",
+				Log(LOG_DEBUG, "DEBUG ( %s/%s ): Queuing flow seq:%" PRIu64 " (%p) for expiry\n",
 				   config.name, config.type, expiry->flow->flow_seq, expiry->flow);
 
 			/* Add to array of expired flows */
@@ -1086,31 +1112,6 @@ force_expire(struct FLOWTRACK *ft, u_int32_t num_to_expire)
 	/* XXX - this is overcomplicated, perhaps use a separate queue */
 }
 
-/* Delete all flows that we know about without processing */
-static int
-delete_all_flows(struct FLOWTRACK *ft)
-{
-	struct FLOW *flow, *nflow;
-	int i;
-	
-	i = 0;
-	for(flow = FLOW_MIN(FLOWS, &ft->flows); flow != NULL; flow = nflow) {
-		nflow = FLOW_NEXT(FLOWS, &ft->flows, flow);
-		FLOW_REMOVE(FLOWS, &ft->flows, flow);
-		
-		EXPIRY_REMOVE(EXPIRIES, &ft->expiries, flow->expiry);
-		free(flow->expiry);
-
-		ft->num_flows--;
-
-		free_flow_allocs(flow);
-		free(flow);
-		i++;
-	}
-	
-	return (i);
-}
-
 /*
  * Per-packet callback function from libpcap. Pass the packet (if it is IP)
  * sans datalink headers to process_packet.
@@ -1150,9 +1151,7 @@ connsock(struct sockaddr_storage *addr, socklen_t len, int hoplimit)
   unsigned int h6;
   unsigned char h4;
   struct sockaddr_in *in4 = (struct sockaddr_in *)addr;
-#if defined ENABLE_IPV6
   struct sockaddr_in6 *in6 = (struct sockaddr_in6 *)addr;
-#endif
   struct sockaddr ssource_ip;
 
   if (config.nfprobe_source_ip) {
@@ -1162,14 +1161,14 @@ connsock(struct sockaddr_storage *addr, socklen_t len, int hoplimit)
 
   if ((s = socket(addr->ss_family, SOCK_DGRAM, 0)) == -1) {
     Log(LOG_ERR, "ERROR ( %s/%s ): socket() failed: %s\n", config.name, config.type, strerror(errno));
-    exit_plugin(1);
+    exit_gracefully(1);
   }
 
   if (config.nfprobe_ipprec) {
     int opt = config.nfprobe_ipprec << 5;
     int rc;
 
-    rc = setsockopt(s, IPPROTO_IP, IP_TOS, &opt, sizeof(opt));
+    rc = setsockopt(s, IPPROTO_IP, IP_TOS, &opt, (socklen_t) sizeof(opt));
     if (rc < 0) Log(LOG_WARNING, "WARN ( %s/%s ): setsockopt() failed for IP_TOS: %s\n", config.name, config.type, strerror(errno));
   }
 
@@ -1177,13 +1176,13 @@ connsock(struct sockaddr_storage *addr, socklen_t len, int hoplimit)
     int rc, value;
 
     value = MIN(config.pipe_size, INT_MAX); 
-    rc = Setsocksize(s, SOL_SOCKET, SO_SNDBUF, &value, sizeof(value));
+    rc = Setsocksize(s, SOL_SOCKET, SO_SNDBUF, &value, (socklen_t) sizeof(value));
     if (rc < 0) Log(LOG_WARNING, "WARN ( %s/%s ): setsockopt() failed for SOL_SNDBUF: %s\n", config.name, config.type, strerror(errno));
   }
 
   if (ret && bind(s, (struct sockaddr *) &ssource_ip, sizeof(ssource_ip)) == -1) {
     Log(LOG_ERR, "ERROR ( %s/%s ): bind() failed: %s\n", config.name, config.type, strerror(errno));
-    exit_plugin(1);
+    exit_gracefully(1);
   }
 
   if (connect(s, (struct sockaddr*)addr, len) == -1) {
@@ -1192,7 +1191,7 @@ connsock(struct sockaddr_storage *addr, socklen_t len, int hoplimit)
       close(s);
       return -1;
     }
-    else exit_plugin(1);
+    else exit_gracefully(1);
   }
 
   switch (addr->ss_family) {
@@ -1203,12 +1202,11 @@ connsock(struct sockaddr_storage *addr, socklen_t len, int hoplimit)
     if (hoplimit == -1)
       break;
     h4 = hoplimit;
-    if (setsockopt(s, IPPROTO_IP, IP_MULTICAST_TTL, &h4, sizeof(h4)) == -1) {
+    if (setsockopt(s, IPPROTO_IP, IP_MULTICAST_TTL, &h4, (socklen_t) sizeof(h4)) == -1) {
       Log(LOG_ERR, "ERROR ( %s/%s ): setsockopt() failed for IP_MULTICAST_TTL: %s\n", config.name, config.type, strerror(errno));
-      exit_plugin(1);
+      exit_gracefully(1);
     }
     break;
-#if defined ENABLE_IPV6
   case AF_INET6:
     /* Default to link-local hoplimit for multicast addresses */
     if (hoplimit == -1 && IN6_IS_ADDR_MULTICAST(&in6->sin6_addr))
@@ -1216,11 +1214,10 @@ connsock(struct sockaddr_storage *addr, socklen_t len, int hoplimit)
     if (hoplimit == -1)
       break;
     h6 = hoplimit;
-    if (setsockopt(s, IPPROTO_IPV6, IPV6_MULTICAST_HOPS, &h6, sizeof(h6)) == -1) {
+    if (setsockopt(s, IPPROTO_IPV6, IPV6_MULTICAST_HOPS, &h6, (socklen_t) sizeof(h6)) == -1) {
       Log(LOG_ERR, "ERROR ( %s/%s ): setsockopt() failed for IPV6_MULTICAST_HOPS: %s\n", config.name, config.type, strerror(errno));
-      exit_plugin(1);
+      exit_gracefully(1);
     }
-#endif
   }
 
   return(s);
@@ -1299,7 +1296,7 @@ handle_timeouts(struct FLOWTRACK *ft, char *to_spec)
   char *sep, *current = to_spec;
 
   trim_spaces(current);
-  while (sep = strchr(current, ':')) {
+  while ((sep = strchr(current, ':'))) {
     *sep = '\0';
     set_timeout(ft, current);
     *sep = ':';
@@ -1310,73 +1307,39 @@ handle_timeouts(struct FLOWTRACK *ft, char *to_spec)
 }
 
 static void
-parse_hostport(const char *s, struct sockaddr *addr, socklen_t *len)
-{
-  char *orig, *host, *port;
-  struct addrinfo hints, *res;
-  int herr;
-
-  if ((host = orig = strdup(s)) == NULL) {
-    Log(LOG_ERR, "ERROR ( %s/%s ): parse_hostport(), strdup() out of memory\n", config.name, config.type);
-    exit_plugin(1);
-  }
-
-  trim_spaces(host);
-  trim_spaces(orig);
-
-  if ((port = strrchr(host, ':')) == NULL || *(++port) == '\0' || *host == '\0') {
-    Log(LOG_ERR, "ERROR ( %s/%s ): parse_hostport(), invalid 'nfprobe_receiver' argument\n", config.name, config.type);
-    exit_plugin(1);
-  }
-  *(port - 1) = '\0';
-	
-  /* Accept [host]:port for numeric IPv6 addresses */
-  if (*host == '[' && *(port - 2) == ']') {
-    host++;
-    *(port - 2) = '\0';
-  }
-
-  memset(&hints, '\0', sizeof(hints));
-  hints.ai_socktype = SOCK_DGRAM;
-
-  if ((herr = getaddrinfo(host, port, &hints, &res)) == -1) {
-    Log(LOG_ERR, "ERROR ( %s/%s ): parse_hostport(), address lookup failed\n", config.name, config.type);
-    exit_plugin(1);
-  }
-
-  if (res == NULL || res->ai_addr == NULL) {
-    Log(LOG_ERR, "ERROR ( %s/%s ): parse_hostport(), no addresses found for [%s]:%s\n", config.name, config.type, host, port);
-    exit_plugin(1);
-  }
-
-  if (res->ai_addrlen > *len) {
-    Log(LOG_ERR, "ERROR ( %s/%s ): parse_hostport(), address too long.\n", config.name, config.type);
-    exit_plugin(1);
-  }
-
-  memcpy(addr, res->ai_addr, res->ai_addrlen);
-  free(orig);
-  *len = res->ai_addrlen;
-}
-
-static void
-parse_engine(char *s, u_int8_t *engine_type, u_int8_t *engine_id)
+parse_engine(char *s, u_int8_t *engine_type, u_int32_t *engine_id)
 {
   char *delim, *ptr;
 
+  if (config.nfprobe_version == 1) {
+    Log(LOG_ERR, "ERROR ( %s/%s ): parse_engine(): NetFlow v1 export does not support nfprobe_engine.\n", config.name, config.type);
+    exit_gracefully(1);
+  }
+
   trim_spaces(s);
   delim = strchr(s, ':');
+  
+  /* NetFlow v5 case */
   if (delim) {
+    if (config.nfprobe_version != 5) {
+      Log(LOG_ERR, "ERROR ( %s/%s ): parse_engine(): engine_type:engine_id is only supported on NetFlow v5 export.\n", config.name, config.type);
+      exit_gracefully(1);
+    }
+
     *delim = '\0';
     ptr = delim+1;
     *engine_type = atoi(s);
     *engine_id = atoi(ptr);
     *delim = ':';
   }
+  /* NetFlow v9 / IPFIX case */
   else {
-    *engine_type = 0;
-    *engine_id = 0;
-    Log(LOG_WARNING, "WARN ( %s/%s ): Engine Type/ID '%s' is not valid. Ignoring.\n", config.name, config.type, s);
+    if (config.nfprobe_version != 9 && config.nfprobe_version != 10) {
+      Log(LOG_ERR, "ERROR ( %s/%s ): parse_engine(): source_id is only supported on NetFlow v9/IPFIX exports.\n", config.name, config.type);
+      exit_gracefully(1);
+    }
+
+    *engine_id = strtoul(s, &delim, 10);
   }
 }
 
@@ -1386,18 +1349,14 @@ void nfprobe_plugin(int pipe_fd, struct configuration *cfgptr, void *ptr)
   struct pkt_bgp_primitives dummy_pbgp;
   struct ports_table pt;
   struct pollfd pfd;
-  struct timezone tz;
   unsigned char *pipebuf;
-  time_t now, refresh_deadline;
-  int refresh_timeout, ret, num;
+  int refresh_timeout, ret, num, recv_budget, poll_bypass;
   char default_receiver[] = "127.0.0.1:2100";
-  char default_engine[] = "0:0";
+  char default_engine_v5[] = "0:0", default_engine_v9[] = "0";
   struct ring *rg = &((struct channels_list_entry *)ptr)->rg;
   struct ch_status *status = ((struct channels_list_entry *)ptr)->status;
-  struct plugins_list_entry *plugin_data = ((struct channels_list_entry *)ptr)->plugin;
   int datasize = ((struct channels_list_entry *)ptr)->datasize;
   u_int32_t bufsz = ((struct channels_list_entry *)ptr)->bufsize;
-  pid_t core_pid = ((struct channels_list_entry *)ptr)->core_pid;
   struct networks_file_data nfd;
 
   unsigned char *rgptr, *dataptr;
@@ -1405,20 +1364,23 @@ void nfprobe_plugin(int pipe_fd, struct configuration *cfgptr, void *ptr)
   u_int32_t seq = 1, rg_err_count = 0;
 
   char *capfile = NULL, dest_addr[256], dest_serv[256];
-  int ch, linktype, ctlsock, i, r, err, always_v6;
-  int max_flows, stop_collection_flag, exit_request, hoplimit;
+  int linktype = 0, i, r, err, always_v6;
+  int max_flows, hoplimit;
   struct sockaddr_storage dest;
   struct FLOWTRACK flowtrack;
   socklen_t dest_len;
   struct NETFLOW_TARGET target;
   struct CB_CTXT cb_ctxt;
-  u_int8_t engine_type, engine_id;
+  u_int8_t engine_type = 0;
+  u_int32_t engine_id;
 
   struct extra_primitives extras;
   struct primitives_ptrs prim_ptrs;
 
 #ifdef WITH_ZMQ
   struct p_zmq_host *zmq_host = &((struct channels_list_entry *)ptr)->zmq_host;
+#else
+  void *zmq_host = NULL;
 #endif
 
   memcpy(&config, cfgptr, sizeof(struct configuration));
@@ -1477,9 +1439,7 @@ void nfprobe_plugin(int pipe_fd, struct configuration *cfgptr, void *ptr)
 
   dest_len = sizeof(dest);
   if (!config.nfprobe_receiver) config.nfprobe_receiver = default_receiver;
-  if (!config.nfprobe_engine) config.nfprobe_engine = default_engine;
   parse_hostport(config.nfprobe_receiver, (struct sockaddr *)&dest, &dest_len);
-  parse_engine(config.nfprobe_engine, &engine_type, &engine_id);
 
 sort_version:
   for (i = 0, r = config.nfprobe_version; nf[i].version != -1; i++) {
@@ -1492,13 +1452,21 @@ sort_version:
   }
   target.dialect = &nf[i];
 
+  if (!config.nfprobe_engine) {
+    if (config.nfprobe_version == 5)
+      config.nfprobe_engine = default_engine_v5;
+    else if (config.nfprobe_version == 9 || config.nfprobe_version == 10)
+      config.nfprobe_engine = default_engine_v9;
+  }
+  parse_engine(config.nfprobe_engine, &engine_type, &engine_id);
+
   /* Netflow send socket */
   if (dest.ss_family != 0) {
     if ((err = getnameinfo((struct sockaddr *)&dest,
 	    dest_len, dest_addr, sizeof(dest_addr), 
 	    dest_serv, sizeof(dest_serv), NI_NUMERICHOST)) == -1) {
       Log(LOG_ERR, "ERROR ( %s/%s ): getnameinfo: %d\n", config.name, config.type, err);
-      exit_plugin(1);
+      exit_gracefully(1);
     }
     target.fd = connsock(&dest, dest_len, hoplimit);
 	
@@ -1509,7 +1477,6 @@ sort_version:
 
   /* Main processing loop */
   gettimeofday(&flowtrack.system_boot_time, NULL);
-  stop_collection_flag = 0;
   cb_ctxt.ft = &flowtrack;
   cb_ctxt.linktype = linktype;
   cb_ctxt.want_v6 = target.dialect->v6_capable || always_v6;
@@ -1528,16 +1495,7 @@ sort_version:
   pipebuf = (unsigned char *) pm_malloc(config.buffer_size);
   memset(pipebuf, 0, config.buffer_size);
 
-  if (config.pipe_zmq) {
-    plugin_pipe_zmq_compile_check();
-#ifdef WITH_ZMQ
-    p_zmq_plugin_pipe_init_plugin(zmq_host);
-    p_zmq_plugin_pipe_consume(zmq_host);
-    p_zmq_set_retry_timeout(zmq_host, config.pipe_zmq_retry);
-    pipe_fd = p_zmq_get_fd(zmq_host);
-    seq = 0;
-#endif
-  }
+  if (config.pipe_zmq) P_zmq_pipe_init(zmq_host, &pipe_fd, &seq);
   else setnonblocking(pipe_fd);
 
   memset(&prim_ptrs, 0, sizeof(prim_ptrs));
@@ -1553,13 +1511,14 @@ sort_version:
 
       if (!cp_entry->ptr->field_type) {
         Log(LOG_ERR, "ERROR ( %s/%s ): custom primitive '%s' has null field_type\n", config.name, config.type, cp_entry->ptr->name);
-        exit_plugin(1);
+        exit_gracefully(1);
       }
     }
   }
 
   for(;;) {
     status->wakeup = TRUE;
+    poll_bypass = FALSE;
 
     pfd.fd = pipe_fd;
     pfd.events = POLLIN;
@@ -1581,16 +1540,31 @@ sort_version:
       break;
     }
 
+    poll_ops:
     if (reload_map) {
       load_networks(config.networks_file, &nt, &nc);
       load_ports(config.ports_file, &pt);
       reload_map = FALSE;
     }
 
-    now = time(NULL);
+    if (reload_log) {
+      reload_logs();
+      reload_log = FALSE;
+    }
+
+    recv_budget = 0;
+    if (poll_bypass) {
+      poll_bypass = FALSE;
+      goto read_data;
+    }
 
     if (ret > 0) { /* we received data */
-read_data:
+      read_data:
+      if (recv_budget == DEFAULT_PLUGIN_COMMON_RECV_BUDGET) {
+	poll_bypass = TRUE;
+	goto poll_ops;
+      }
+
       if (config.pipe_homegrown) {
         if (!pollagain) {
           seq++;
@@ -1599,7 +1573,7 @@ read_data:
         }
         else {
           if ((ret = read(pipe_fd, &rgptr, sizeof(rgptr))) == 0)
-            exit_plugin(1); /* we exit silently; something happened at the write end */
+            exit_gracefully(1); /* we exit silently; something happened at the write end */
         }
   
         if ((rg->ptr + bufsz) > rg->end) rg->ptr = rg->base;
@@ -1612,7 +1586,7 @@ read_data:
   	  else {
   	    rg_err_count++;
   	    if (config.debug || (rg_err_count > MAX_RG_COUNT_ERR)) {
-              Log(LOG_WARNING, "WARN ( %s/%s ): Missing data detected (plugin_buffer_size=%llu plugin_pipe_size=%llu).\n",
+              Log(LOG_WARNING, "WARN ( %s/%s ): Missing data detected (plugin_buffer_size=%" PRIu64 " plugin_pipe_size=%" PRIu64 ").\n",
                         config.name, config.type, config.buffer_size, config.pipe_size);
               Log(LOG_WARNING, "WARN ( %s/%s ): Increase values or look for plugin_buffer_size, plugin_pipe_size in CONFIG-KEYS document.\n\n",
                         config.name, config.type);
@@ -1629,7 +1603,7 @@ read_data:
       }
 #ifdef WITH_ZMQ
       else if (config.pipe_zmq) {
-	ret = p_zmq_plugin_pipe_recv(zmq_host, pipebuf, config.buffer_size);
+	ret = p_zmq_topic_recv(zmq_host, pipebuf, config.buffer_size);
 	if (ret > 0) {
 	  if (seq && (((struct ch_buf_hdr *)pipebuf)->seq != ((seq + 1) % MAX_SEQNUM))) {
 	    Log(LOG_WARNING, "WARN ( %s/%s ): Missing data detected. Sequence received=%u expected=%u\n",
@@ -1645,23 +1619,16 @@ read_data:
       data = (struct pkt_data *) (pipebuf+sizeof(struct ch_buf_hdr));
 
       if (config.debug_internal_msg) 
-        Log(LOG_DEBUG, "DEBUG ( %s/%s ): buffer received cpid=%u len=%llu seq=%u num_entries=%u\n",
-                config.name, config.type, core_pid, ((struct ch_buf_hdr *)pipebuf)->len,
-                seq, ((struct ch_buf_hdr *)pipebuf)->num);
+        Log(LOG_DEBUG, "DEBUG ( %s/%s ): buffer received len=%" PRIu64 " seq=%u num_entries=%u\n",
+                config.name, config.type, ((struct ch_buf_hdr *)pipebuf)->len, seq,
+                ((struct ch_buf_hdr *)pipebuf)->num);
 
-      if (!config.pipe_check_core_pid || ((struct ch_buf_hdr *)pipebuf)->core_pid == core_pid) {
       while (((struct ch_buf_hdr *)pipebuf)->num > 0) {
         for (num = 0; primptrs_funcs[num]; num++)
           (*primptrs_funcs[num])((u_char *)data, &extras, &prim_ptrs);
 
         for (num = 0; net_funcs[num]; num++)
-	  (*net_funcs[num])(&nt, &nc, &data->primitives, &dummy_pbgp, &nfd);
-
-	/* hacky: bgp next-hop */
-        if (config.nfacctd_net & NF_NET_NEW && dummy_pbgp.peer_dst_ip.family) {
-          memcpy(&prim_ptrs.pextras->bgp_next_hop, &dummy_pbgp.peer_dst_ip, sizeof(struct host_addr));
-          memset(&dummy_pbgp, 0, sizeof(dummy_pbgp));
-        }
+	  (*net_funcs[num])(&nt, &nc, &data->primitives, prim_ptrs.pbgp, &nfd);
 
 	if (config.ports_file) {
 	  if (!pt.table[data->primitives.src_port]) data->primitives.src_port = 0;
@@ -1680,8 +1647,8 @@ read_data:
 	  data = (struct pkt_data *) dataptr;
 	}
       }
-      }
 
+      recv_budget++;
       goto read_data;
     }
 
