@@ -1,6 +1,6 @@
 /*
     pmacct (Promiscuous mode IP Accounting package)
-    pmacct is Copyright (C) 2003-2014 by Paolo Lucente
+    pmacct is Copyright (C) 2003-2019 by Paolo Lucente
 */
 
 /*
@@ -176,7 +176,7 @@ struct FLOW {
 	u_int8_t mac[2][6];			/* Endpoint L2/Ethernet MAC addresses */
 	u_int16_t vlan;				/* VLAN ID */
 	u_int32_t mpls_label[2];		/* MPLS top label */
-        u_int16_t ifindex[2];			/* input/output ifindex */
+        u_int32_t ifindex[2];			/* input/output ifindex */
 
 	/* classification stuff */
 	pm_class_t class;			/* Classification internal ID */
@@ -202,7 +202,7 @@ struct FLOW {
 	u_int32_t flows[2];			/* Flows so far */
 #endif
 
-	char *pcust[2];				/* space for custom-defined primitives */
+	u_char *pcust[2];			/* space for custom-defined primitives */
 	struct pkt_vlen_hdr_primitives *pvlen[2]; 	/* space for vlen primitives */
 };
 
@@ -232,17 +232,10 @@ struct EXPIRY {
 };
 
 /* Prototype for functions shared from softflowd.c */
-u_int32_t timeval_sub_ms(const struct timeval *t1, const struct timeval *t2);
+u_int32_t timeval_sub_ms(const struct timeval *, const struct timeval *);
 
 /* Prototypes for functions to send NetFlow packets, from netflow*.c */
-int send_netflow_v1(struct FLOW **flows, int num_flows, int nfsock,
-    u_int64_t *flows_exported, struct timeval *system_boot_time, 
-    int verbose_flag, u_int8_t engine_type, u_int8_t engine_id);
-int send_netflow_v5(struct FLOW **flows, int num_flows, int nfsock,
-    u_int64_t *flows_exported, struct timeval *system_boot_time,
-    int verbose_flag, u_int8_t engine_type, u_int8_t engine_id);
-int send_netflow_v9(struct FLOW **flows, int num_flows, int nfsock,
-    u_int64_t *flows_exported, struct timeval *system_boot_time,
-    int verbose_flag, u_int8_t engine_type, u_int8_t engine_id);
-
+int send_netflow_v1(struct FLOW **, int, int, u_int64_t *, struct timeval *,  int, u_int8_t, u_int32_t);
+int send_netflow_v5(struct FLOW **, int, int, u_int64_t *, struct timeval *,  int, u_int8_t, u_int32_t);
+int send_netflow_v9(struct FLOW **, int, int, u_int64_t *, struct timeval *,  int, u_int8_t, u_int32_t);
 #endif /* _SOFTFLOWD_H */
