@@ -1,6 +1,6 @@
 /*
     pmacct (Promiscuous mode IP Accounting package)
-    pmacct is Copyright (C) 2003-2017 by Paolo Lucente
+    pmacct is Copyright (C) 2003-2019 by Paolo Lucente
 */
 
 /*
@@ -19,8 +19,6 @@
     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 
-#define __PLUGIN_CMN_JSON_C
-
 /* includes */
 #include "pmacct.h"
 #include "addr.h"
@@ -29,12 +27,18 @@
 #include "plugin_cmn_json.h"
 #include "ip_flow.h"
 #include "classifier.h"
+#include "bgp/bgp.h"
+#include "rpki/rpki.h"
 #if defined (WITH_NDPI)
 #include "ndpi/ndpi.h"
 #endif
 
-/* Functions */
 #ifdef WITH_JANSSON
+
+/* Global variables */
+compose_json_handler cjhandler[N_PRIMITIVES];
+
+/* Functions */
 void compose_json(u_int64_t wtc, u_int64_t wtc_2)
 {
   int idx = 0;
@@ -140,6 +144,11 @@ void compose_json(u_int64_t wtc, u_int64_t wtc_2)
     idx++;
   }
 
+  if (wtc_2 & COUNT_DST_ROA) {
+    cjhandler[idx] = compose_json_dst_roa;
+    idx++;
+  }
+
   if (wtc & COUNT_PEER_SRC_AS) {
     cjhandler[idx] = compose_json_peer_src_as;
     idx++;
@@ -190,6 +199,11 @@ void compose_json(u_int64_t wtc, u_int64_t wtc_2)
     idx++;
   }
 
+  if (wtc_2 & COUNT_SRC_ROA) {
+    cjhandler[idx] = compose_json_src_roa;
+    idx++;
+  }
+
   if (wtc & COUNT_IN_IFACE) {
     cjhandler[idx] = compose_json_in_iface;
     idx++;
@@ -205,6 +219,11 @@ void compose_json(u_int64_t wtc, u_int64_t wtc_2)
     idx++;
   }
 
+  if (wtc_2 & COUNT_MPLS_PW_ID) {
+    cjhandler[idx] = compose_json_mpls_pw_id;
+    idx++;
+  }
+
   if (wtc & (COUNT_SRC_HOST|COUNT_SUM_HOST)) {
     cjhandler[idx] = compose_json_src_host;
     idx++;
@@ -276,6 +295,17 @@ void compose_json(u_int64_t wtc, u_int64_t wtc_2)
     cjhandler[idx] = compose_json_dst_host_pocode;
     idx++;
   }
+
+  if (wtc_2 & COUNT_SRC_HOST_COORDS) {
+    cjhandler[idx] = compose_json_src_host_coords;
+    idx++;
+  }
+
+  if (wtc_2 & COUNT_DST_HOST_COORDS) {
+    cjhandler[idx] = compose_json_dst_host_coords;
+    idx++;
+  }
+
 #endif
 
   if (wtc & COUNT_TCPFLAGS) {
@@ -298,8 +328,8 @@ void compose_json(u_int64_t wtc, u_int64_t wtc_2)
     idx++;
   }
 
-  if (wtc_2 & COUNT_PKT_LEN_DISTRIB) {
-    cjhandler[idx] = compose_json_pkt_len_distrib;
+  if (wtc_2 & COUNT_SAMPLING_DIRECTION) {
+    cjhandler[idx] = compose_json_sampling_direction;
     idx++;
   }
 
@@ -343,6 +373,16 @@ void compose_json(u_int64_t wtc, u_int64_t wtc_2)
     idx++;
   }
 
+  if (wtc_2 & COUNT_TUNNEL_SRC_MAC) {
+    cjhandler[idx] = compose_json_tunnel_src_mac;
+    idx++;
+  }
+
+  if (wtc_2 & COUNT_TUNNEL_DST_MAC) {
+    cjhandler[idx] = compose_json_tunnel_dst_mac;
+    idx++;
+  }
+
   if (wtc_2 & COUNT_TUNNEL_SRC_HOST) {
     cjhandler[idx] = compose_json_tunnel_src_host;
     idx++;
@@ -363,6 +403,21 @@ void compose_json(u_int64_t wtc, u_int64_t wtc_2)
     idx++;
   }
 
+  if (wtc_2 & COUNT_TUNNEL_SRC_PORT) {
+    cjhandler[idx] = compose_json_tunnel_src_port;
+    idx++;
+  }
+
+  if (wtc_2 & COUNT_TUNNEL_DST_PORT) {
+    cjhandler[idx] = compose_json_tunnel_dst_port;
+    idx++;
+  }
+
+  if (wtc_2 & COUNT_VXLAN) {
+    cjhandler[idx] = compose_json_vxlan;
+    idx++;
+  }
+
   if (wtc_2 & COUNT_TIMESTAMP_START) {
     cjhandler[idx] = compose_json_timestamp_start;
     idx++;
@@ -393,6 +448,11 @@ void compose_json(u_int64_t wtc, u_int64_t wtc_2)
     idx++;
   }
 
+  if (wtc_2 & COUNT_EXPORT_PROTO_SYSID) {
+    cjhandler[idx] = compose_json_export_proto_sysid;
+    idx++;
+  }
+
   if (config.cpptrs.num) {
     cjhandler[idx] = compose_json_custom_primitives;
     idx++;
@@ -440,7 +500,6 @@ void compose_json_label(json_t *obj, struct chained_cache *cc)
 
 void compose_json_class(json_t *obj, struct chained_cache *cc)
 {
-  char empty_string[] = "", *str_ptr;
   struct pkt_primitives *pbase = &cc->primitives;
 
   json_object_set_new_nocheck(obj, "class", json_string((pbase->class && class[(pbase->class)-1].id) ? class[(pbase->class)-1].protocol : "unknown"));
@@ -460,6 +519,7 @@ void compose_json_ndpi_class(json_t *obj, struct chained_cache *cc)
 }
 #endif
 
+#if defined (HAVE_L2)
 void compose_json_src_mac(json_t *obj, struct chained_cache *cc)
 {
   char mac[18];
@@ -493,6 +553,7 @@ void compose_json_etype(json_t *obj, struct chained_cache *cc)
   sprintf(misc_str, "%x", cc->primitives.etype);
   json_object_set_new_nocheck(obj, "etype", json_string(misc_str));
 }
+#endif
 
 void compose_json_src_as(json_t *obj, struct chained_cache *cc)
 {
@@ -582,6 +643,11 @@ void compose_json_med(json_t *obj, struct chained_cache *cc)
   json_object_set_new_nocheck(obj, "med", json_integer((json_int_t)cc->pbgp->med));
 }
 
+void compose_json_dst_roa(json_t *obj, struct chained_cache *cc)
+{
+  json_object_set_new_nocheck(obj, "roa_dst", json_string(rpki_roa_print(cc->pbgp->dst_roa)));
+}
+
 void compose_json_peer_src_as(json_t *obj, struct chained_cache *cc)
 {
   json_object_set_new_nocheck(obj, "peer_as_src", json_integer((json_int_t)cc->pbgp->peer_src_as));
@@ -622,7 +688,7 @@ void compose_json_src_std_comm(json_t *obj, struct chained_cache *cc)
   }
   else str_ptr = empty_string;
 
-  json_object_set_new_nocheck(obj, "src_comms", json_string(str_ptr));
+  json_object_set_new_nocheck(obj, "comms_src", json_string(str_ptr));
 }
 
 void compose_json_src_ext_comm(json_t *obj, struct chained_cache *cc)
@@ -639,7 +705,7 @@ void compose_json_src_ext_comm(json_t *obj, struct chained_cache *cc)
   }
   else str_ptr = empty_string;
 
-  json_object_set_new_nocheck(obj, "src_ecomms", json_string(str_ptr));
+  json_object_set_new_nocheck(obj, "ecomms_src", json_string(str_ptr));
 }
 
 void compose_json_src_lrg_comm(json_t *obj, struct chained_cache *cc)
@@ -656,7 +722,7 @@ void compose_json_src_lrg_comm(json_t *obj, struct chained_cache *cc)
   }
   else str_ptr = empty_string;
 
-  json_object_set_new_nocheck(obj, "src_lcomms", json_string(str_ptr));
+  json_object_set_new_nocheck(obj, "lcomms_src", json_string(str_ptr));
 }
 
 void compose_json_src_as_path(json_t *obj, struct chained_cache *cc)
@@ -673,17 +739,22 @@ void compose_json_src_as_path(json_t *obj, struct chained_cache *cc)
   }
   else str_ptr = empty_string;
 
-  json_object_set_new_nocheck(obj, "src_as_path", json_string(str_ptr));
+  json_object_set_new_nocheck(obj, "as_path_src", json_string(str_ptr));
 }
 
 void compose_json_src_local_pref(json_t *obj, struct chained_cache *cc)
 {
-  json_object_set_new_nocheck(obj, "src_local_pref", json_integer((json_int_t)cc->pbgp->src_local_pref));
+  json_object_set_new_nocheck(obj, "local_pref_src", json_integer((json_int_t)cc->pbgp->src_local_pref));
 }
 
 void compose_json_src_med(json_t *obj, struct chained_cache *cc)
 {
-  json_object_set_new_nocheck(obj, "src_med", json_integer((json_int_t)cc->pbgp->src_med));
+  json_object_set_new_nocheck(obj, "med_src", json_integer((json_int_t)cc->pbgp->src_med));
+}
+
+void compose_json_src_roa(json_t *obj, struct chained_cache *cc)
+{
+  json_object_set_new_nocheck(obj, "roa_src", json_string(rpki_roa_print(cc->pbgp->src_roa)));
 }
 
 void compose_json_in_iface(json_t *obj, struct chained_cache *cc)
@@ -704,6 +775,11 @@ void compose_json_mpls_vpn_rd(json_t *obj, struct chained_cache *cc)
   json_object_set_new_nocheck(obj, "mpls_vpn_rd", json_string(rd_str));
 }
 
+void compose_json_mpls_pw_id(json_t *obj, struct chained_cache *cc)
+{
+  json_object_set_new_nocheck(obj, "mpls_pw_id", json_integer((json_int_t)cc->pbgp->mpls_pw_id));
+}
+
 void compose_json_src_host(json_t *obj, struct chained_cache *cc)
 {
   char ip_address[INET6_ADDRSTRLEN];
@@ -817,6 +893,18 @@ void compose_json_dst_host_pocode(json_t *obj, struct chained_cache *cc)
   else
     json_object_set_new_nocheck(obj, "pocode_ip_dst", json_string(empty_string));
 }
+
+void compose_json_src_host_coords(json_t *obj, struct chained_cache *cc)
+{
+  json_object_set_new_nocheck(obj, "lat_ip_src", json_real(cc->primitives.src_ip_lat));
+  json_object_set_new_nocheck(obj, "lon_ip_src", json_real(cc->primitives.src_ip_lon));
+}
+
+void compose_json_dst_host_coords(json_t *obj, struct chained_cache *cc)
+{
+  json_object_set_new_nocheck(obj, "lat_ip_dst", json_real(cc->primitives.dst_ip_lat));
+  json_object_set_new_nocheck(obj, "lon_ip_dst", json_real(cc->primitives.dst_ip_lon));
+}
 #endif
 
 void compose_json_tcp_flags(json_t *obj, struct chained_cache *cc)
@@ -829,12 +917,9 @@ void compose_json_tcp_flags(json_t *obj, struct chained_cache *cc)
 
 void compose_json_proto(json_t *obj, struct chained_cache *cc)
 {
-  char misc_str[VERYSHORTBUFLEN];
+  char proto[PROTO_NUM_STRLEN];
 
-  if (!config.num_protos && (cc->primitives.proto < protocols_number))
-    json_object_set_new_nocheck(obj, "ip_proto", json_string(_protocols[cc->primitives.proto].name));
-  else
-    json_object_set_new_nocheck(obj, "ip_proto", json_integer((json_int_t)cc->primitives.proto));
+  json_object_set_new_nocheck(obj, "ip_proto", json_string(ip_proto_print(cc->primitives.proto, proto, PROTO_NUM_STRLEN)));
 }
 
 void compose_json_tos(json_t *obj, struct chained_cache *cc)
@@ -847,9 +932,9 @@ void compose_json_sampling_rate(json_t *obj, struct chained_cache *cc)
   json_object_set_new_nocheck(obj, "sampling_rate", json_integer((json_int_t)cc->primitives.sampling_rate));
 }
 
-void compose_json_pkt_len_distrib(json_t *obj, struct chained_cache *cc)
+void compose_json_sampling_direction(json_t *obj, struct chained_cache *cc)
 {
-  json_object_set_new_nocheck(obj, "pkt_len_distrib", json_string(config.pkt_len_distrib_bins[cc->primitives.pkt_len_distrib]));
+  json_object_set_new_nocheck(obj, "sampling_direction", json_string(cc->primitives.sampling_direction));
 }
 
 void compose_json_post_nat_src_host(json_t *obj, struct chained_cache *cc)
@@ -898,6 +983,22 @@ void compose_json_mpls_stack_depth(json_t *obj, struct chained_cache *cc)
   json_object_set_new_nocheck(obj, "mpls_stack_depth", json_integer((json_int_t)cc->pmpls->mpls_stack_depth));
 }
 
+void compose_json_tunnel_src_mac(json_t *obj, struct chained_cache *cc)
+{
+  char mac[18];
+
+  etheraddr_string(cc->ptun->tunnel_eth_shost, mac);
+  json_object_set_new_nocheck(obj, "tunnel_mac_src", json_string(mac));
+}
+
+void compose_json_tunnel_dst_mac(json_t *obj, struct chained_cache *cc)
+{
+  char mac[18];
+
+  etheraddr_string(cc->ptun->tunnel_eth_dhost, mac);
+  json_object_set_new_nocheck(obj, "tunnel_mac_dst", json_string(mac));
+}
+
 void compose_json_tunnel_src_host(json_t *obj, struct chained_cache *cc)
 {
   char ip_address[INET6_ADDRSTRLEN];
@@ -916,12 +1017,9 @@ void compose_json_tunnel_dst_host(json_t *obj, struct chained_cache *cc)
 
 void compose_json_tunnel_proto(json_t *obj, struct chained_cache *cc)
 {
-  char misc_str[VERYSHORTBUFLEN];
+  char proto[PROTO_NUM_STRLEN];
 
-  if (!config.num_protos && (cc->ptun->tunnel_proto < protocols_number))
-    json_object_set_new_nocheck(obj, "tunnel_ip_proto", json_string(_protocols[cc->ptun->tunnel_proto].name));
-  else
-    json_object_set_new_nocheck(obj, "tunnel_ip_proto", json_integer((json_int_t)cc->ptun->tunnel_proto));
+  json_object_set_new_nocheck(obj, "tunnel_ip_proto", json_string(ip_proto_print(cc->ptun->tunnel_proto, proto, PROTO_NUM_STRLEN)));
 }
 
 void compose_json_tunnel_tos(json_t *obj, struct chained_cache *cc)
@@ -929,11 +1027,28 @@ void compose_json_tunnel_tos(json_t *obj, struct chained_cache *cc)
   json_object_set_new_nocheck(obj, "tunnel_tos", json_integer((json_int_t)cc->ptun->tunnel_tos));
 }
 
+void compose_json_tunnel_src_port(json_t *obj, struct chained_cache *cc)
+{
+  json_object_set_new_nocheck(obj, "tunnel_port_src", json_integer((json_int_t)cc->ptun->tunnel_src_port));
+}
+
+void compose_json_tunnel_dst_port(json_t *obj, struct chained_cache *cc)
+{
+  json_object_set_new_nocheck(obj, "tunnel_port_dst", json_integer((json_int_t)cc->ptun->tunnel_dst_port));
+}
+
+void compose_json_vxlan(json_t *obj, struct chained_cache *cc)
+{
+  json_object_set_new_nocheck(obj, "vxlan", json_integer((json_int_t)cc->ptun->tunnel_id));
+}
+
 void compose_json_timestamp_start(json_t *obj, struct chained_cache *cc)
 {
   char tstamp_str[VERYSHORTBUFLEN];
 
-  compose_timestamp(tstamp_str, VERYSHORTBUFLEN, &cc->pnat->timestamp_start, TRUE, config.timestamps_since_epoch);
+  compose_timestamp(tstamp_str, VERYSHORTBUFLEN, &cc->pnat->timestamp_start, TRUE,
+		    config.timestamps_since_epoch, config.timestamps_rfc3339,
+		    config.timestamps_utc);
   json_object_set_new_nocheck(obj, "timestamp_start", json_string(tstamp_str));
 }
 
@@ -941,7 +1056,9 @@ void compose_json_timestamp_end(json_t *obj, struct chained_cache *cc)
 {
   char tstamp_str[VERYSHORTBUFLEN];
 
-  compose_timestamp(tstamp_str, VERYSHORTBUFLEN, &cc->pnat->timestamp_end, TRUE, config.timestamps_since_epoch);
+  compose_timestamp(tstamp_str, VERYSHORTBUFLEN, &cc->pnat->timestamp_end, TRUE,
+		    config.timestamps_since_epoch, config.timestamps_rfc3339,
+		    config.timestamps_utc);
   json_object_set_new_nocheck(obj, "timestamp_end", json_string(tstamp_str));
 }
 
@@ -949,7 +1066,9 @@ void compose_json_timestamp_arrival(json_t *obj, struct chained_cache *cc)
 {
   char tstamp_str[VERYSHORTBUFLEN];
 
-  compose_timestamp(tstamp_str, VERYSHORTBUFLEN, &cc->pnat->timestamp_arrival, TRUE, config.timestamps_since_epoch);
+  compose_timestamp(tstamp_str, VERYSHORTBUFLEN, &cc->pnat->timestamp_arrival, TRUE,
+		    config.timestamps_since_epoch, config.timestamps_rfc3339,
+		    config.timestamps_utc);
   json_object_set_new_nocheck(obj, "timestamp_arrival", json_string(tstamp_str));
 }
 
@@ -957,10 +1076,14 @@ void compose_json_timestamp_stitching(json_t *obj, struct chained_cache *cc)
 {
   char tstamp_str[VERYSHORTBUFLEN];
 
-  compose_timestamp(tstamp_str, VERYSHORTBUFLEN, &cc->stitch->timestamp_min, TRUE, config.timestamps_since_epoch);
+  compose_timestamp(tstamp_str, VERYSHORTBUFLEN, &cc->stitch->timestamp_min, TRUE,
+		    config.timestamps_since_epoch, config.timestamps_rfc3339,
+		    config.timestamps_utc);
   json_object_set_new_nocheck(obj, "timestamp_min", json_string(tstamp_str));
 
-  compose_timestamp(tstamp_str, VERYSHORTBUFLEN, &cc->stitch->timestamp_max, TRUE, config.timestamps_since_epoch);
+  compose_timestamp(tstamp_str, VERYSHORTBUFLEN, &cc->stitch->timestamp_max, TRUE,
+		    config.timestamps_since_epoch, config.timestamps_rfc3339,
+		    config.timestamps_utc);
   json_object_set_new_nocheck(obj, "timestamp_max", json_string(tstamp_str));
 }
 
@@ -974,6 +1097,11 @@ void compose_json_export_proto_version(json_t *obj, struct chained_cache *cc)
   json_object_set_new_nocheck(obj, "export_proto_version", json_integer((json_int_t)cc->primitives.export_proto_version));
 }
 
+void compose_json_export_proto_sysid(json_t *obj, struct chained_cache *cc)
+{
+  json_object_set_new_nocheck(obj, "export_proto_sysid", json_integer((json_int_t)cc->primitives.export_proto_sysid));
+}
+
 void compose_json_custom_primitives(json_t *obj, struct chained_cache *cc)
 {
   char empty_string[] = "";
@@ -1004,12 +1132,16 @@ void compose_json_history(json_t *obj, struct chained_cache *cc)
 
     tv.tv_sec = cc->basetime.tv_sec;
     tv.tv_usec = 0;
-    compose_timestamp(tstamp_str, VERYSHORTBUFLEN, &tv, FALSE, config.timestamps_since_epoch);
+    compose_timestamp(tstamp_str, VERYSHORTBUFLEN, &tv, FALSE,
+		      config.timestamps_since_epoch, config.timestamps_rfc3339,
+		      config.timestamps_utc);
     json_object_set_new_nocheck(obj, "stamp_inserted", json_string(tstamp_str));
 
     tv.tv_sec = time(NULL);
     tv.tv_usec = 0;
-    compose_timestamp(tstamp_str, VERYSHORTBUFLEN, &tv, FALSE, config.timestamps_since_epoch);
+    compose_timestamp(tstamp_str, VERYSHORTBUFLEN, &tv, FALSE,
+		      config.timestamps_since_epoch, config.timestamps_rfc3339,
+		      config.timestamps_utc);
     json_object_set_new_nocheck(obj, "stamp_updated", json_string(tstamp_str));
   }
 }
@@ -1066,10 +1198,14 @@ void compose_json(u_int64_t wtc, u_int64_t wtc_2)
 void *compose_purge_init_json(char *writer_name, pid_t writer_pid)
 {
   if (config.debug) Log(LOG_DEBUG, "DEBUG ( %s/%s ): compose_purge_init_json(): JSON object not created due to missing --enable-jansson\n", config.name, config.type);
+
+  return NULL;
 }
 
 void *compose_purge_close_json(char *writer_name, pid_t writer_pid, int purged_entries, int total_entries, int duration)
 {
   if (config.debug) Log(LOG_DEBUG, "DEBUG ( %s/%s ): compose_purge_close_json(): JSON object not created due to missing --enable-jansson\n", config.name, config.type);
+
+  return NULL;
 }
 #endif
