@@ -1,6 +1,6 @@
 /*
     pmacct (Promiscuous mode IP Accounting package)
-    pmacct is Copyright (C) 2003-2017 by Paolo Lucente
+    pmacct is Copyright (C) 2003-2019 by Paolo Lucente
 */
 
 /*
@@ -19,16 +19,34 @@
     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 
-#define __PLUGIN_COMMON_C
-
 /* includes */
 #include "pmacct.h"
+#include "plugin_common.h"
 #include "addr.h"
 #include "pmacct-data.h"
-#include "plugin_common.h"
+#include "plugin_hooks.h"
 #include "ip_flow.h"
 #include "classifier.h"
 #include "crc32.h"
+#include "preprocess-internal.h"
+
+/* Global variables */
+void (*insert_func)(struct primitives_ptrs *, struct insert_data *); /* pointer to INSERT function */
+void (*purge_func)(struct chained_cache *[], int, int); /* pointer to purge function */ 
+struct scratch_area sa;
+struct chained_cache *cache;
+struct chained_cache **queries_queue, **pending_queries_queue, *pqq_container;
+struct timeval flushtime;
+int qq_ptr, pqq_ptr, pp_size, pb_size, pn_size, pm_size, pt_size, pc_size;
+int dbc_size, quit; 
+time_t refresh_deadline;
+
+void (*basetime_init)(time_t);
+void (*basetime_eval)(struct timeval *, struct timeval *, time_t);
+int (*basetime_cmp)(struct timeval *, struct timeval *);
+struct timeval basetime, ibasetime, new_basetime;
+time_t timeslot;
+int dyn_table, dyn_table_time_only;
 
 /* Functions */
 void P_set_signals()
@@ -85,7 +103,7 @@ void P_init_default_values()
   sa.num = config.print_cache_entries*AVERAGE_CHAIN_LEN;
   sa.size = sa.num*dbc_size;
 
-  Log(LOG_INFO, "INFO ( %s/%s ): cache entries=%llu base cache memory=%llu bytes\n", config.name, config.type,
+  Log(LOG_INFO, "INFO ( %s/%s ): cache entries=%d base cache memory=%" PRIu64 " bytes\n", config.name, config.type,
 	config.print_cache_entries, ((config.print_cache_entries * dbc_size) + (2 * ((sa.num +
 	config.print_cache_entries) * sizeof(struct chained_cache *))) + sa.size));
 
@@ -116,7 +134,7 @@ void P_config_checks()
   return;
 
 exit_lane:
-  exit_plugin(1);
+  exit_gracefully(1);
 }
 
 unsigned int P_cache_modulo(struct primitives_ptrs *prim_ptrs)
@@ -127,7 +145,7 @@ unsigned int P_cache_modulo(struct primitives_ptrs *prim_ptrs)
   struct pkt_nat_primitives *pnat = prim_ptrs->pnat;
   struct pkt_mpls_primitives *pmpls = prim_ptrs->pmpls;
   struct pkt_tunnel_primitives *ptun = prim_ptrs->ptun;
-  char *pcust = prim_ptrs->pcust;
+  u_char *pcust = prim_ptrs->pcust;
   struct pkt_vlen_hdr_primitives *pvlen = prim_ptrs->pvlen;
   register unsigned int modulo;
 
@@ -150,7 +168,7 @@ struct chained_cache *P_cache_search(struct primitives_ptrs *prim_ptrs)
   struct pkt_nat_primitives *pnat = prim_ptrs->pnat;
   struct pkt_mpls_primitives *pmpls = prim_ptrs->pmpls;
   struct pkt_tunnel_primitives *ptun = prim_ptrs->ptun;
-  char *pcust = prim_ptrs->pcust;
+  u_char *pcust = prim_ptrs->pcust;
   struct pkt_vlen_hdr_primitives *pvlen = prim_ptrs->pvlen;
   unsigned int modulo = P_cache_modulo(prim_ptrs);
   struct chained_cache *cache_ptr = &cache[modulo];
@@ -215,7 +233,7 @@ void P_cache_insert(struct primitives_ptrs *prim_ptrs, struct insert_data *idata
   struct pkt_nat_primitives *pnat = prim_ptrs->pnat;
   struct pkt_mpls_primitives *pmpls = prim_ptrs->pmpls;
   struct pkt_tunnel_primitives *ptun = prim_ptrs->ptun;
-  char *pcust = prim_ptrs->pcust;
+  u_char *pcust = prim_ptrs->pcust;
   struct pkt_vlen_hdr_primitives *pvlen = prim_ptrs->pvlen;
   unsigned int modulo = P_cache_modulo(prim_ptrs);
   struct chained_cache *cache_ptr = &cache[modulo];
@@ -520,8 +538,11 @@ void P_cache_insert(struct primitives_ptrs *prim_ptrs, struct insert_data *idata
       switch (ret = fork()) {
       case 0: /* Child */
 	pm_setproctitle("%s %s [%s]", config.type, "Plugin -- Writer (urgent)", config.name);
+	config.is_forked = TRUE;
+
         (*purge_func)(queries_queue, qq_ptr, TRUE);
-        exit(0);
+
+        exit_gracefully(0);
       default: /* Parent */
         if (ret == -1) Log(LOG_WARNING, "WARN ( %s/%s ): Unable to fork writer: %s\n", config.name, config.type, strerror(errno));
         else dump_writers_add(ret);
@@ -620,8 +641,11 @@ void P_cache_handle_flush_event(struct ports_table *pt)
     switch (ret = fork()) {
     case 0: /* Child */
       pm_setproctitle("%s %s [%s]", config.type, "Plugin -- Writer", config.name);
+      config.is_forked = TRUE;
+
       (*purge_func)(queries_queue, qq_ptr, FALSE);
-      exit(0);
+
+      exit_gracefully(0);
     default: /* Parent */
       if (ret == -1) Log(LOG_WARNING, "WARN ( %s/%s ): Unable to fork writer: %s\n", config.name, config.type, strerror(errno));
       else dump_writers_add(ret);
@@ -648,6 +672,11 @@ void P_cache_handle_flush_event(struct ports_table *pt)
     load_ports(config.ports_file, pt);
     reload_map = FALSE;
   }
+
+  if (reload_log) {
+    reload_logs();
+    reload_log = FALSE;
+  }
 }
 
 void P_cache_mark_flush(struct chained_cache *queue[], int index, int exiting)
@@ -677,7 +706,13 @@ void P_cache_mark_flush(struct chained_cache *queue[], int index, int exiting)
       else queue[j]->valid = PRINT_CACHE_COMMITTED;
     }
 
-    if (pqq_ptr) pqq_container = (struct chained_cache *) malloc(pqq_ptr*dbc_size); 
+    if (pqq_ptr) {
+      pqq_container = (struct chained_cache *) malloc(pqq_ptr*dbc_size); 
+      if (!pqq_container) {
+	Log(LOG_ERR, "ERROR ( %s/%s ): P_cache_mark_flush() cannot allocate pqq_container. Exiting ..\n", config.name, config.type);
+	exit_gracefully(1); 
+      }
+    }
     
     /* we copy un-committed elements to a container structure for re-insertion
        in cache. As we copy elements out of the cache we mark entries as free */
@@ -717,12 +752,12 @@ void P_cache_flush(struct chained_cache *queue[], int index)
 
 struct chained_cache *P_cache_attach_new_node(struct chained_cache *elem)
 {
-  if ((sa.ptr+sizeof(struct chained_cache)) <= (sa.base+sa.size)) {
+  if ((sa.ptr + (2 * sizeof(struct chained_cache))) <= (sa.base + sa.size)) {
     sa.ptr += sizeof(struct chained_cache);
     elem->next = (struct chained_cache *) sa.ptr;
     return (struct chained_cache *) sa.ptr;
   }
-  else return NULL; /* XXX */
+  else return NULL;
 }
 
 void P_sum_host_insert(struct primitives_ptrs *prim_ptrs, struct insert_data *idata)
@@ -786,22 +821,20 @@ void P_exit_now(int signum)
   if (config.pidfile) remove_pid_file(config.pidfile);
 
   wait(NULL);
-  exit_plugin(0);
+  exit_gracefully(0);
 }
 
 int P_trigger_exec(char *filename)
 {
-  char *args[1];
+  char *args[2] = { filename, NULL };
   int pid;
 
-  memset(args, 0, sizeof(args));
-
   switch (pid = vfork()) {
   case -1:
     return -1;
   case 0:
     execv(filename, args);
-    exit(0);
+    _exit(0);
   }
 
   return 0;
@@ -835,7 +868,7 @@ void P_init_historical_acct(time_t now)
   if (config.sql_history_offset) {
     if (config.sql_history_offset >= timeslot) {
       Log(LOG_ERR, "ERROR ( %s/%s ): History offset (ie. sql_history_offset) must be < history (ie. sql_history).\n", config.name, config.type);
-      exit(1);
+      exit_gracefully(1);
     }
 
     t = t - (timeslot + config.sql_history_offset);
@@ -893,26 +926,6 @@ int P_cmp_historical_acct(struct timeval *entry_basetime, struct timeval *insert
   return ret;
 }
 
-int P_test_zero_elem(struct chained_cache *elem)
-{
-  if (elem) {
-    if (elem->flow_type == NF9_FTYPE_NAT_EVENT) {
-      if (elem->pnat && elem->pnat->nat_event) return FALSE;
-      else return TRUE;
-    }
-    else if (elem->flow_type == NF9_FTYPE_OPTION) {
-      /* not really much we can test */
-      return FALSE;
-    }
-    else {
-      if (elem->bytes_counter || elem->packet_counter || elem->flow_counter) return FALSE;
-      else return TRUE;
-    }
-  }
-
-  return TRUE;
-}
-
 void primptrs_set_all_from_chained_cache(struct primitives_ptrs *prim_ptrs, struct chained_cache *entry)
 {
   struct pkt_data *data;
@@ -951,86 +964,6 @@ void P_handle_table_dyn_rr(char *new, int newlen, char *old, struct p_table_rr *
   rk_rr->next %= rk_rr->max;
 }
 
-void P_handle_table_dyn_strings(char *new, int newlen, char *old, struct chained_cache *elem)
-{
-  int oldlen, ptr_len;
-  char peer_src_ip_string[] = "$peer_src_ip", post_tag_string[] = "$post_tag";
-  char pre_tag_string[] = "$pre_tag";
-  char *ptr_start, *ptr_end;
-
-  oldlen = strlen(old);
-  if (oldlen <= newlen) strcpy(new, old);
-  else {
-    strncpy(new, old, newlen);
-    return;
-  }
-
-  if (!strchr(new, '$')) return;
-  ptr_start = strstr(new, peer_src_ip_string);
-  if (ptr_start) {
-    char ip_address[INET6_ADDRSTRLEN];
-    char buf[newlen];
-    int len;
-
-    if (!elem || !elem->pbgp) goto out_peer_src_ip; 
-
-    len = strlen(ptr_start);
-    ptr_end = ptr_start;
-    ptr_len = strlen(peer_src_ip_string);
-    ptr_end += ptr_len;
-    len -= ptr_len;
-
-    addr_to_str(ip_address, &elem->pbgp->peer_src_ip);
-    snprintf(buf, newlen, "%s", ip_address);
-    strncat(buf, ptr_end, len);
-
-    len = strlen(buf);
-    *ptr_start = '\0';
-    strncat(new, buf, len);
-  }
-  out_peer_src_ip:
-
-  if (!strchr(new, '$')) return;
-  ptr_start = strstr(new, post_tag_string);
-  if (ptr_start) {
-    char buf[newlen];
-    int len;
-
-    len = strlen(ptr_start);
-    ptr_end = ptr_start;
-    ptr_len = strlen(post_tag_string);
-    ptr_end += ptr_len;
-    len -= ptr_len;
-
-    snprintf(buf, newlen, "%u", config.post_tag);
-    strncat(buf, ptr_end, len);
-
-    len = strlen(buf);
-    *ptr_start = '\0';
-    strncat(new, buf, len);
-  }
-
-  if (!strchr(new, '$')) return;
-  ptr_start = strstr(new, pre_tag_string);
-  if (ptr_start) {
-    char buf[newlen];
-    int len;
-
-    len = strlen(ptr_start);
-    ptr_end = ptr_start;
-    ptr_len = strlen(pre_tag_string);
-    ptr_end += ptr_len;
-    len -= ptr_len;
-
-    snprintf(buf, newlen, "%u", elem->primitives.tag);
-    strncat(buf, ptr_end, len);
-
-    len = strlen(buf);
-    *ptr_start = '\0';
-    strncat(new, buf, len);
-  }
-}
-
 void P_update_time_reference(struct insert_data *idata)
 {
   idata->now = time(NULL);
@@ -1044,32 +977,3 @@ void P_update_time_reference(struct insert_data *idata)
     }
   }
 }
-
-void P_broker_timers_set_last_fail(struct p_broker_timers *btimers, time_t timestamp)
-{
-  if (btimers) btimers->last_fail = timestamp;
-}
-
-time_t P_broker_timers_get_last_fail(struct p_broker_timers *btimers)
-{
-  if (btimers) return btimers->last_fail;
-
-  return FALSE;
-}
-
-void P_broker_timers_unset_last_fail(struct p_broker_timers *btimers)
-{
-  if (btimers) btimers->last_fail = FALSE;
-}
-
-void P_broker_timers_set_retry_interval(struct p_broker_timers *btimers, int interval)
-{
-  if (btimers) btimers->retry_interval = interval;
-}
-
-int P_broker_timers_get_retry_interval(struct p_broker_timers *btimers)
-{
-  if (btimers) return btimers->retry_interval;
-
-  return ERR;
-}
