@@ -1,6 +1,6 @@
 /*
     pmacct (Promiscuous mode IP Accounting package)
-    pmacct is Copyright (C) 2003-2017 by Paolo Lucente
+    pmacct is Copyright (C) 2003-2019 by Paolo Lucente
 */
 
 /*
@@ -19,8 +19,6 @@
     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 
-#define __PLUGIN_HOOKS_C
-
 /* includes */
 #include "pmacct.h"
 #include "pmacct-data.h"
@@ -40,22 +38,32 @@
 void load_plugins(struct plugin_requests *req)
 {
   u_int64_t buf_pipe_ratio_sz = 0, pipe_idx = 0;
-  int snd_buflen = 0, rcv_buflen = 0, socklen = 0, target_buflen = 0, ret;
+  int snd_buflen = 0, rcv_buflen = 0, socklen = 0, target_buflen = 0;
 
-  int nfprobe_id = 0, min_sz = 0, extra_sz = 0;
+  int nfprobe_id = 0, min_sz = 0, extra_sz = 0, offset = 0;
   struct plugins_list_entry *list = plugins_list;
-  int l = sizeof(list->cfg.pipe_size), offset = 0;
+  socklen_t l = sizeof(list->cfg.pipe_size);
   struct channels_list_entry *chptr = NULL;
 
+ 
   init_random_seed(); 
   init_pipe_channels();
+ 
+#ifdef WITH_ZMQ
+  char username[SHORTBUFLEN], password[SHORTBUFLEN];
+  memset(username, 0, sizeof(username));
+  memset(password, 0, sizeof(password));
+
+  generate_random_string(username, (sizeof(username) - 1));
+  generate_random_string(password, (sizeof(password) - 1));
+#endif
 
   while (list) {
     if ((*list->type.func)) {
       if (list->cfg.data_type & (PIPE_TYPE_METADATA|PIPE_TYPE_PAYLOAD|PIPE_TYPE_MSG));
       else {
 	Log(LOG_ERR, "ERROR ( %s/%s ): Data type not supported: %d\n", list->name, list->type.string, list->cfg.data_type);
-	exit(1);
+	exit_gracefully(1);
       }
 
       min_sz = ChBufHdrSz;
@@ -117,9 +125,10 @@ void load_plugins(struct plugin_requests *req)
 
         buf_pipe_ratio_sz = (list->cfg.pipe_size/list->cfg.buffer_size)*sizeof(char *);
         if (buf_pipe_ratio_sz > INT_MAX) {
-	  Log(LOG_ERR, "ERROR ( %s/%s ): Current plugin_buffer_size elems per plugin_pipe_size: %d. Max: %d.\nExiting.\n",
-		list->name, list->type.string, (list->cfg.pipe_size/list->cfg.buffer_size), (INT_MAX/sizeof(char *)));
-          exit_all(1);
+	  Log(LOG_ERR, "ERROR ( %s/%s ): Current plugin_buffer_size elems per plugin_pipe_size: %llu. Max: %d.\nExiting.\n",
+		list->name, list->type.string, (unsigned long long)(list->cfg.pipe_size/list->cfg.buffer_size),
+		(INT_MAX/(int)sizeof(char *)));
+          exit_gracefully(1);
         }
         else target_buflen = buf_pipe_ratio_sz;
 
@@ -142,7 +151,7 @@ void load_plugins(struct plugin_requests *req)
         }
 
         if (list->cfg.debug || (list->cfg.pipe_size > WARNING_PIPE_SIZE)) {
-	  Log(LOG_INFO, "INFO ( %s/%s ): plugin_pipe_size=%llu bytes plugin_buffer_size=%llu bytes\n", 
+	  Log(LOG_INFO, "INFO ( %s/%s ): plugin_pipe_size=%" PRIu64 " bytes plugin_buffer_size=%" PRIu64 " bytes\n", 
 		list->name, list->type.string, list->cfg.pipe_size, list->cfg.buffer_size);
 	  if (target_buflen <= snd_buflen) 
             Log(LOG_INFO, "INFO ( %s/%s ): ctrl channel: obtained=%d bytes target=%d bytes\n",
@@ -165,7 +174,7 @@ void load_plugins(struct plugin_requests *req)
       chptr = insert_pipe_channel(list->type.id, &list->cfg, list->pipe[1]);
       if (!chptr) {
 	Log(LOG_ERR, "ERROR ( %s/%s ): Unable to setup a new Core Process <-> Plugin channel.\nExiting.\n", list->name, list->type.string);
-	exit_all(1);
+	exit_gracefully(1);
       }
       else chptr->plugin = list;
 
@@ -244,8 +253,12 @@ void load_plugins(struct plugin_requests *req)
       /* ZMQ inits, if required */
 #ifdef WITH_ZMQ
       if (list->cfg.pipe_zmq) {
-	p_zmq_plugin_pipe_init_core(&chptr->zmq_host, list->id);
-	p_zmq_plugin_pipe_publish(&chptr->zmq_host);
+	char log_id[SHORTBUFLEN];
+
+	p_zmq_plugin_pipe_init_core(&chptr->zmq_host, list->id, username, password);
+	snprintf(log_id, sizeof(log_id), "%s/%s", list->name, list->type.string);
+	p_zmq_set_log_id(&chptr->zmq_host, log_id);
+	p_zmq_pub_setup(&chptr->zmq_host);
       }
 #endif
       
@@ -257,7 +270,7 @@ void load_plugins(struct plugin_requests *req)
       case 0: /* Child */
 	/* SIGCHLD handling issue: SysV avoids zombies by ignoring SIGCHLD; to emulate
 	   such semantics on BSD systems, we need an handler like handle_falling_child() */
-#if defined (IRIX) || (SOLARIS)
+#if defined (SOLARIS)
 	signal(SIGCHLD, SIG_IGN);
 #else
 	signal(SIGCHLD, ignore_falling_child);
@@ -267,11 +280,12 @@ void load_plugins(struct plugin_requests *req)
         mallopt(M_CHECK_ACTION, 0);
 #endif
 
+	if (device.dev_desc) pcap_close(device.dev_desc);
 	close(config.sock);
 	close(config.bgp_sock);
 	if (!list->cfg.pipe_zmq) close(list->pipe[1]);
 	(*list->type.func)(list->pipe[0], &list->cfg, chptr);
-	exit(0);
+	exit_gracefully(0);
       default: /* Parent */
 	if (!list->cfg.pipe_zmq) {
 	  close(list->pipe[0]);
@@ -338,14 +352,14 @@ void exec_plugins(struct packet_ptrs *pptrs, struct plugin_requests *req)
 {
   int saved_have_tag = FALSE, saved_have_tag2 = FALSE, saved_have_label = FALSE;
   pm_id_t saved_tag = 0, saved_tag2 = 0;
-  pt_label_t saved_label;
+  pt_label_t *saved_label = malloc(sizeof(pt_label_t));
 
-  int num, ret, fixed_size;
+  int num, fixed_size;
   u_int32_t savedptr;
   char *bptr;
   int index, got_tags = FALSE;
 
-  pretag_init_label(&saved_label);
+  pretag_init_label(saved_label);
 
 #if defined WITH_GEOIPV2
   if (reload_geoipv2_file && config.geoipv2_file) {
@@ -363,36 +377,49 @@ void exec_plugins(struct packet_ptrs *pptrs, struct plugin_requests *req)
 
     if (p->cfg.pre_tag_map && find_id_func) {
       if (p->cfg.type_id == PLUGIN_ID_TEE) {
-	if ((req->ptm_c.exec_ptm_res && !p->cfg.ptm_complex) ||
-	    ((!req->ptm_c.exec_ptm_res && p->cfg.ptm_complex) && !p->cfg.tee_dissect_send_full_pkt)) 
+	/*
+	   replicate and compute tagging if:
+	   - a dissected flow hits a complex pre_tag_map or
+	   - a non-dissected (full) packet hits a simple pre_tag_map
+	*/
+	if ((req->ptm_c.exec_ptm_res && !p->cfg.ptm_complex) || (!req->ptm_c.exec_ptm_res && p->cfg.ptm_complex))
 	  continue;
       }
 
       if (p->cfg.ptm_global && got_tags) {
         pptrs->tag = saved_tag;
         pptrs->tag2 = saved_tag2;
-	pretag_copy_label(&pptrs->label, &saved_label);
+	    pretag_copy_label(&pptrs->label, saved_label);
 
         pptrs->have_tag = saved_have_tag;
         pptrs->have_tag2 = saved_have_tag2;
         pptrs->have_label = saved_have_label;
       }
       else {
-        find_id_func(&p->cfg.ptm, pptrs, &pptrs->tag, &pptrs->tag2);
+	if (p->cfg.type_id == PLUGIN_ID_TEE && req->ptm_c.exec_ptm_res && pptrs->tee_dissect_bcast) /* noop */;
+        else {
+	  find_id_func(&p->cfg.ptm, pptrs, &pptrs->tag, &pptrs->tag2);
 
-	if (p->cfg.ptm_global) {
-	  saved_tag = pptrs->tag;
-	  saved_tag2 = pptrs->tag2;
-	  pretag_copy_label(&saved_label, &pptrs->label);
+	  if (p->cfg.ptm_global) {
+	    saved_tag = pptrs->tag;
+	    saved_tag2 = pptrs->tag2;
+	    pretag_copy_label(saved_label, &pptrs->label);
 
-	  saved_have_tag = pptrs->have_tag;
-	  saved_have_tag2 = pptrs->have_tag2;
-	  saved_have_label = pptrs->have_label;
+	    saved_have_tag = pptrs->have_tag;
+	    saved_have_tag2 = pptrs->have_tag2;
+	    saved_have_label = pptrs->have_label;
 
-          got_tags = TRUE;
+            got_tags = TRUE;
+	  }
         }
       }
     }
+    else {
+      if (p->cfg.type_id == PLUGIN_ID_TEE) {
+        /* stop dissected flows from being replicated in case of no pre_tag_map */
+        if (req->ptm_c.exec_ptm_res) continue;
+      }
+    }
 
     if (evaluate_filters(&channels_list[index].agg_filter, pptrs->packet_ptr, pptrs->pkthdr) &&
         !evaluate_tags(&channels_list[index].tag_filter, pptrs->tag) && 
@@ -431,7 +458,7 @@ reprocess:
           struct plugins_list_entry *list = channels_list[index].plugin;
 
           Log(LOG_ERR, "ERROR ( %s/%s ): plugin_buffer_size is too short.\n", list->name, list->type.string);
-          exit_all(1);
+          exit_gracefully(1);
         }
 
         channels_list[index].already_reprocessed = TRUE;
@@ -453,15 +480,14 @@ reprocess:
 	((struct ch_buf_hdr *)channels_list[index].rg.ptr)->len = channels_list[index].bufptr;
 	((struct ch_buf_hdr *)channels_list[index].rg.ptr)->seq = channels_list[index].hdr.seq;
 	((struct ch_buf_hdr *)channels_list[index].rg.ptr)->num = channels_list[index].hdr.num;
-	((struct ch_buf_hdr *)channels_list[index].rg.ptr)->core_pid = channels_list[index].core_pid;
 
 	channels_list[index].status->last_buf_off = (u_int64_t)(channels_list[index].rg.ptr - channels_list[index].rg.base);
 
         if (config.debug_internal_msg) {
 	  struct plugins_list_entry *list = channels_list[index].plugin;
-	  Log(LOG_DEBUG, "DEBUG ( %s/%s ): buffer released cpid=%u len=%llu seq=%u num_entries=%u off=%llu\n",
-		list->name, list->type.string, channels_list[index].core_pid, channels_list[index].bufptr,
-		channels_list[index].hdr.seq, channels_list[index].hdr.num, channels_list[index].status->last_buf_off);
+	  Log(LOG_DEBUG, "DEBUG ( %s/%s ): buffer released len=%" PRIu64 " seq=%u num_entries=%u off=%" PRIu64 "\n",
+		list->name, list->type.string, channels_list[index].bufptr, channels_list[index].hdr.seq,
+		channels_list[index].hdr.num, channels_list[index].status->last_buf_off);
 	}
 
 	/* sending buffer to connected ZMQ subscriber(s) */
@@ -469,7 +495,8 @@ reprocess:
 #ifdef WITH_ZMQ
           struct channels_list_entry *chptr = &channels_list[index];
 
-	  ret = p_zmq_plugin_pipe_send(&chptr->zmq_host, chptr->rg.ptr, chptr->bufsize);
+	  int ret = p_zmq_topic_send(&chptr->zmq_host, chptr->rg.ptr, chptr->bufsize);
+          (void)ret; //Check error?
 #endif
 	}
 	else {
@@ -490,7 +517,6 @@ reprocess:
 	/* let's protect the buffer we are going to write */
         ((struct ch_buf_hdr *)channels_list[index].rg.ptr)->seq = -1;
         ((struct ch_buf_hdr *)channels_list[index].rg.ptr)->num = 0;
-        ((struct ch_buf_hdr *)channels_list[index].rg.ptr)->core_pid = 0;
 
         /* rewind pointer */
         channels_list[index].bufptr = channels_list[index].buf;
@@ -537,13 +563,14 @@ reprocess:
 
   /* cleanups */
   reload_map_exec_plugins = FALSE;
-  pretag_free_label(&saved_label);
+  pretag_free_label(saved_label);
+  if (saved_label) free(saved_label);
 }
 
 struct channels_list_entry *insert_pipe_channel(int plugin_type, struct configuration *cfg, int pipe)
 {
   struct channels_list_entry *chptr; 
-  int index = 0, x;  
+  int index = 0;  
 
   while (index < MAX_N_PLUGINS) {
     chptr = &channels_list[index]; 
@@ -578,7 +605,7 @@ struct channels_list_entry *insert_pipe_channel(int plugin_type, struct configur
       chptr->rg.base = map_shared(0, cfg->pipe_size+PKT_MSG_SIZE, PROT_READ|PROT_WRITE, MAP_SHARED|MAP_ANONYMOUS, -1, 0);
       if (chptr->rg.base == MAP_FAILED) {
         Log(LOG_ERR, "ERROR ( %s/%s ): unable to allocate pipe buffer. Exiting ...\n", cfg->name, cfg->type); 
-	exit_all(1);
+	exit_gracefully(1);
       }
       memset(chptr->rg.base, 0, cfg->pipe_size);
       chptr->rg.ptr = chptr->rg.base;
@@ -587,7 +614,7 @@ struct channels_list_entry *insert_pipe_channel(int plugin_type, struct configur
       chptr->status = map_shared(0, sizeof(struct ch_status), PROT_READ|PROT_WRITE, MAP_SHARED|MAP_ANONYMOUS, -1, 0);
       if (chptr->status == MAP_FAILED) {
         Log(LOG_ERR, "ERROR ( %s/%s ): unable to allocate status buffer. Exiting ...\n", cfg->name, cfg->type);
-        exit_all(1);
+        exit_gracefully(1);
       }
       memset(chptr->status, 0, sizeof(struct ch_status));
 
@@ -740,7 +767,7 @@ pm_counter_t take_simple_systematic_skip(pm_counter_t mean)
    TRUE: We want it!
    FALSE: Discard it!
 */
-int evaluate_filters(struct aggregate_filter *filter, char *pkt, struct pcap_pkthdr *pkthdr)
+int evaluate_filters(struct aggregate_filter *filter, u_char *pkt, struct pcap_pkthdr *pkthdr)
 {
   int index;
 
@@ -789,11 +816,10 @@ void fill_pipe_buffer()
 
     ((struct ch_buf_hdr *)chptr->rg.ptr)->seq = chptr->hdr.seq;
     ((struct ch_buf_hdr *)chptr->rg.ptr)->num = chptr->hdr.num;
-    ((struct ch_buf_hdr *)chptr->rg.ptr)->core_pid = chptr->core_pid;
 
     if (chptr->plugin->cfg.pipe_zmq) {
 #ifdef WITH_ZMQ
-      p_zmq_plugin_pipe_send(&chptr->zmq_host, chptr->rg.ptr, chptr->bufsize);
+      p_zmq_topic_send(&chptr->zmq_host, chptr->rg.ptr, chptr->bufsize);
 #endif
     }
     else {
@@ -869,7 +895,7 @@ void load_plugin_filters(int link_type)
 
 	dev_desc = pcap_open_dead(link_type, 128); /* 128 bytes should be long enough */
 
-	if (config.dev) pcap_lookupnet(config.dev, &localnet, &netmask, errbuf);
+	if (config.pcap_if) pcap_lookupnet(config.pcap_if, &localnet, &netmask, errbuf);
 
 	list->cfg.bpfp_a_table[idx] = malloc(sizeof(struct bpf_program));
 	while ( (count_token = extract_token(&list->cfg.a_filter, ',')) && idx < AGG_FILTER_ENTRIES ) {
@@ -922,7 +948,7 @@ void plugin_pipe_zmq_compile_check()
 {
 #ifndef WITH_ZMQ
   Log(LOG_ERR, "ERROR ( %s/%s ): 'plugin_pipe_zmq' requires compiling with --enable-zmq. Exiting ..\n", config.name, config.type);
-  exit_plugin(1);
+  exit_gracefully(1);
 #endif
 }
 
@@ -930,3 +956,27 @@ void plugin_pipe_check(struct configuration *cfg)
 {
   if (!cfg->pipe_zmq) cfg->pipe_homegrown = TRUE;
 }
+
+void P_zmq_pipe_init(void *zh, int *pipe_fd, u_int32_t *seq)
+{
+  plugin_pipe_zmq_compile_check();
+
+#ifdef WITH_ZMQ
+  if (zh) {
+    struct p_zmq_host *zmq_host = zh;
+    char log_id[SHORTBUFLEN];
+
+    p_zmq_plugin_pipe_init_plugin(zmq_host);
+
+    snprintf(log_id, sizeof(log_id), "%s/%s", config.name, config.type);
+    p_zmq_set_log_id(zmq_host, log_id);
+
+    p_zmq_set_hwm(zmq_host, config.pipe_zmq_hwm);
+    p_zmq_sub_setup(zmq_host);
+    p_zmq_set_retry_timeout(zmq_host, config.pipe_zmq_retry);
+
+    if (pipe_fd) (*pipe_fd) = p_zmq_get_fd(zmq_host);
+    if (seq) (*seq) = 0;
+  }
+#endif
+}
