@@ -1,6 +1,6 @@
 /*
     pmacct (Promiscuous mode IP Accounting package)
-    pmacct is Copyright (C) 2003-2017 by Paolo Lucente
+    pmacct is Copyright (C) 2003-2019 by Paolo Lucente
 */
 
 /*
@@ -20,27 +20,28 @@
 */
 
 /* defines */
-#define ARGS_NFACCTD "n:dDhP:b:f:F:c:m:p:r:s:S:L:l:o:t:O:uRVaA:E:I:W"
-#define ARGS_SFACCTD "n:dDhP:b:f:F:c:m:p:r:s:S:L:l:o:t:O:uRVaA:E:I:W"
-#define ARGS_PMACCTD "n:NdDhP:b:f:F:c:i:I:m:p:r:s:S:o:t:O:uwWL:RVazA:E:"
-#define ARGS_UACCTD "n:NdDhP:b:f:F:c:m:p:r:s:S:o:t:O:uRg:L:VaA:E:"
-#define ARGS_PMTELEMETRYD "hVL:l:f:dDS:F:o:O:i:"
-#define ARGS_PMBGPD "hVL:l:f:dDS:F:o:O:i:"
+#define ARGS_NFACCTD "n:dDhP:b:f:F:c:m:p:r:s:S:L:l:o:t:O:MuRVaAE:I:WZ:Y:"
+#define ARGS_SFACCTD "n:dDhP:b:f:F:c:m:p:r:s:S:L:l:o:t:O:MuRVaAE:I:WZ:Y:"
+#define ARGS_PMACCTD "n:NdDhP:b:f:F:c:i:I:m:p:r:s:S:o:t:O:MuwWZ:Y:L:RVazAE:"
+#define ARGS_UACCTD "n:NdDhP:b:f:F:c:m:p:r:s:S:o:t:O:MuRg:L:VaAE:"
+#define ARGS_PMTELEMETRYD "hVL:u:t:Z:f:dDS:F:o:O:i:"
+#define ARGS_PMBGPD "hVL:l:f:dDS:F:o:O:i:gm:"
 #define ARGS_PMBMPD "hVL:l:f:dDS:F:o:O:i:"
-#define ARGS_PMACCT "Ssc:Cetm:p:P:M:arN:n:lT:O:E:uDVUiI"
-#define N_PRIMITIVES 75
+#define ARGS_PMACCT "hSsc:Cetm:p:P:M:arN:n:lT:O:E:uVUiI0"
+#define N_PRIMITIVES 128
 #define N_FUNCS 10 
 #define MAX_N_PLUGINS 32
 #define PROTO_LEN 12
+#define PROTO_NUM_STRLEN 4
 #define MAX_MAP_ENTRIES 2048 /* allow maps */
 #define BGP_MD5_MAP_ENTRIES 8192
 #define AGG_FILTER_ENTRIES 128 
 #define FOLLOW_BGP_NH_ENTRIES 32 
-#define MAX_PROTOCOL_LEN 16
-#define MAX_PKT_LEN_DISTRIB_BINS 255
-#define MAX_PKT_LEN_DISTRIB_LEN 15
+#define MAX_PROTOCOL_LEN 32 
 #define DEFAULT_AVRO_SCHEMA_REFRESH_TIME 60
+#define MAX_AVRO_SCHEMA 16
 #define DEFAULT_IMT_PLUGIN_POLL_TIMEOUT 5
+#define DEFAULT_SLOTH_SLEEP_TIME 5
 #define UINT32T_THRESHOLD 4290000000UL
 #define UINT64T_THRESHOLD 18446744073709551360ULL
 #define INT64T_THRESHOLD 9223372036854775807ULL
@@ -48,6 +49,14 @@
 #define PM_COUNTRY_T_STRLEN 4
 #define PM_POCODE_T_STRLEN 12
 #define PCAP_SAVEFILE_MAX_ERRORS 10
+#define PCAP_MAX_INTERFACES 1000
+#define PCAP_MAX_ATTEMPTS 3
+#define PCAP_RETRY_PERIOD 5
+#define PCAP_IFINDEX_NONE 0
+#define PCAP_IFINDEX_SYS 1
+#define PCAP_IFINDEX_HASH 2 
+#define PCAP_IFINDEX_MAP 3
+#define PORT_STRLEN 6
 #ifndef UINT8_MAX
 #define UINT8_MAX (255U)
 #endif
@@ -69,11 +78,7 @@
 
 #define LONGLONG_RETRY INT_MAX
 
-#if defined ENABLE_IPV6
 #define DEFAULT_SNAPLEN 128
-#else
-#define DEFAULT_SNAPLEN 68
-#endif
 #define SNAPLEN_ISIS_MIN 512
 #define SNAPLEN_ISIS_DEFAULT 1476
 
@@ -90,16 +95,17 @@
 #define PRIMITIVE_LEN 		32
 #define PRIMITIVE_DESC_LEN	64
 
+#define PMACCT_VERSION "1.7.4-git"
 #define MANTAINER "Paolo Lucente <paolo@pmacct.net>"
 #define GET_IN_TOUCH_MSG "If you see this message, please get in touch"
-#define PMACCTD_USAGE_HEADER "Promiscuous Mode Accounting Daemon, pmacctd 1.7.0"
-#define UACCTD_USAGE_HEADER "Linux NetFilter NFLOG Accounting Daemon, uacctd 1.7.0"
-#define PMACCT_USAGE_HEADER "pmacct, pmacct client 1.7.0"
-#define NFACCTD_USAGE_HEADER "NetFlow Accounting Daemon, nfacctd 1.7.0"
-#define SFACCTD_USAGE_HEADER "sFlow Accounting Daemon, sfacctd 1.7.0"
-#define PMTELEMETRYD_USAGE_HEADER "Streaming Network Telemetry Daemon, pmtelemetryd 1.7.0"
-#define PMBGPD_USAGE_HEADER "pmacct BGP Collector Daemon, pmbgpd 1.7.0"
-#define PMBMPD_USAGE_HEADER "pmacct BMP Collector Daemon, pmbmpd 1.7.0"
+#define PMACCTD_USAGE_HEADER "Promiscuous Mode Accounting Daemon, pmacctd"
+#define UACCTD_USAGE_HEADER "Linux NetFilter NFLOG Accounting Daemon, uacctd"
+#define PMACCT_USAGE_HEADER "pmacct IMT plugin client, pmacct"
+#define NFACCTD_USAGE_HEADER "NetFlow Accounting Daemon, nfacctd"
+#define SFACCTD_USAGE_HEADER "sFlow Accounting Daemon, sfacctd"
+#define PMTELEMETRYD_USAGE_HEADER "Streaming Network Telemetry Daemon, pmtelemetryd"
+#define PMBGPD_USAGE_HEADER "pmacct BGP Collector Daemon, pmbgpd"
+#define PMBMPD_USAGE_HEADER "pmacct BMP Collector Daemon, pmbmpd"
 #define PMACCT_COMPILE_ARGS COMPILE_ARGS
 #ifndef TRUE
 #define TRUE 1
@@ -154,6 +160,8 @@
 #define MAP_TEE_RECVS		106	/* tee_receivers */
 #define MAP_IGP			107	/* igp_daemon_map */
 #define MAP_CUSTOM_PRIMITIVES	108	/* aggregate_primitives */
+#define MAP_BGP_XCS		109	/* bgp_xconnect_map */
+#define MAP_PCAP_INTERFACES	110	/* pcap_interfaces_map */
 
 /* PRIMITIVES DEFINITION: START */
 /* internal: first registry, ie. what_to_count, aggregation, etc. */
@@ -210,7 +218,7 @@
 #define COUNT_INT_SAMPLING_RATE		0x0002000000000001ULL
 #define COUNT_INT_SRC_HOST_COUNTRY	0x0002000000000002ULL
 #define COUNT_INT_DST_HOST_COUNTRY	0x0002000000000004ULL
-#define COUNT_INT_PKT_LEN_DISTRIB	0x0002000000000008ULL
+#define COUNT_INT_EXPORT_PROTO_SYSID	0x0002000000000008ULL
 #define COUNT_INT_POST_NAT_SRC_HOST	0x0002000000000010ULL
 #define COUNT_INT_POST_NAT_DST_HOST	0x0002000000000020ULL
 #define COUNT_INT_POST_NAT_SRC_PORT	0x0002000000000040ULL
@@ -234,7 +242,18 @@
 #define COUNT_INT_TUNNEL_IP_PROTO	0x0002000001000000ULL
 #define COUNT_INT_TUNNEL_IP_TOS		0x0002000002000000ULL
 #define COUNT_INT_NDPI_CLASS		0x0002000004000000ULL
-#define COUNT_INT_CUSTOM_PRIMITIVES	0x0002000008000000ULL
+#define COUNT_INT_SRC_HOST_COORDS	0x0002000008000000ULL
+#define COUNT_INT_DST_HOST_COORDS	0x0002000010000000ULL
+#define COUNT_INT_SAMPLING_DIRECTION	0x0002000020000000ULL
+#define COUNT_INT_MPLS_PW_ID		0x0002000040000000ULL
+#define COUNT_INT_SRC_ROA		0x0002000080000000ULL
+#define COUNT_INT_DST_ROA		0x0002000100000000ULL
+#define COUNT_INT_VXLAN			0x0002000200000000ULL
+#define COUNT_INT_TUNNEL_SRC_MAC	0x0002000400000000ULL
+#define COUNT_INT_TUNNEL_DST_MAC	0x0002000800000000ULL
+#define COUNT_INT_TUNNEL_SRC_PORT	0x0002001000000000ULL
+#define COUNT_INT_TUNNEL_DST_PORT	0x0002002000000000ULL
+#define COUNT_INT_CUSTOM_PRIMITIVES	0x0002800000000000ULL
 
 #define COUNT_INDEX_MASK	0xFFFF
 #define COUNT_INDEX_CP		0xFFFF000000000000ULL  /* index 0xffff reserved to custom primitives */
@@ -295,7 +314,7 @@
 #define COUNT_SAMPLING_RATE		(COUNT_INT_SAMPLING_RATE & COUNT_REGISTRY_MASK)
 #define COUNT_SRC_HOST_COUNTRY		(COUNT_INT_SRC_HOST_COUNTRY & COUNT_REGISTRY_MASK)
 #define COUNT_DST_HOST_COUNTRY		(COUNT_INT_DST_HOST_COUNTRY & COUNT_REGISTRY_MASK)
-#define COUNT_PKT_LEN_DISTRIB		(COUNT_INT_PKT_LEN_DISTRIB & COUNT_REGISTRY_MASK)
+#define COUNT_EXPORT_PROTO_SYSID	(COUNT_INT_EXPORT_PROTO_SYSID & COUNT_REGISTRY_MASK)
 #define COUNT_POST_NAT_SRC_HOST		(COUNT_INT_POST_NAT_SRC_HOST & COUNT_REGISTRY_MASK)
 #define COUNT_POST_NAT_DST_HOST		(COUNT_INT_POST_NAT_DST_HOST & COUNT_REGISTRY_MASK)
 #define COUNT_POST_NAT_SRC_PORT		(COUNT_INT_POST_NAT_SRC_PORT & COUNT_REGISTRY_MASK)
@@ -319,6 +338,17 @@
 #define COUNT_TUNNEL_IP_PROTO		(COUNT_INT_TUNNEL_IP_PROTO & COUNT_REGISTRY_MASK)
 #define COUNT_TUNNEL_IP_TOS		(COUNT_INT_TUNNEL_IP_TOS & COUNT_REGISTRY_MASK)
 #define COUNT_NDPI_CLASS                (COUNT_INT_NDPI_CLASS & COUNT_REGISTRY_MASK)
+#define COUNT_SRC_HOST_COORDS		(COUNT_INT_SRC_HOST_COORDS & COUNT_REGISTRY_MASK)
+#define COUNT_DST_HOST_COORDS		(COUNT_INT_DST_HOST_COORDS & COUNT_REGISTRY_MASK)
+#define COUNT_SAMPLING_DIRECTION	(COUNT_INT_SAMPLING_DIRECTION & COUNT_REGISTRY_MASK)
+#define COUNT_MPLS_PW_ID		(COUNT_INT_MPLS_PW_ID & COUNT_REGISTRY_MASK)
+#define COUNT_SRC_ROA			(COUNT_INT_SRC_ROA & COUNT_REGISTRY_MASK)
+#define COUNT_DST_ROA			(COUNT_INT_DST_ROA & COUNT_REGISTRY_MASK)
+#define COUNT_VXLAN			(COUNT_INT_VXLAN & COUNT_REGISTRY_MASK)
+#define COUNT_TUNNEL_SRC_MAC		(COUNT_INT_TUNNEL_SRC_MAC & COUNT_REGISTRY_MASK)
+#define COUNT_TUNNEL_DST_MAC		(COUNT_INT_TUNNEL_DST_MAC & COUNT_REGISTRY_MASK)
+#define COUNT_TUNNEL_SRC_PORT		(COUNT_INT_TUNNEL_SRC_PORT & COUNT_REGISTRY_MASK)
+#define COUNT_TUNNEL_DST_PORT		(COUNT_INT_TUNNEL_DST_PORT & COUNT_REGISTRY_MASK)
 #define COUNT_CUSTOM_PRIMITIVES		(COUNT_INT_CUSTOM_PRIMITIVES & COUNT_REGISTRY_MASK)
 /* PRIMITIVES DEFINITION: END */
 
@@ -356,7 +386,7 @@
 #define WANT_MATCH			0x00000010
 #define WANT_RESET			0x00000020
 #define WANT_CLASS_TABLE		0x00000040
-#define WANT_PKT_LEN_DISTRIB_TABLE	0x00000080
+/* #define XXX 				0x00000080 */
 #define WANT_LOCK_OP			0x00000100
 #define WANT_CUSTOM_PRIMITIVES_TABLE	0x00000200
 #define WANT_ERASE_LAST_TSTAMP		0x00000400
@@ -375,6 +405,7 @@
 #define CHLD_WARNING		0x00000001
 #define CHLD_ALERT		0x00000002
 
+#define BGP_SRC_PRIMITIVES_UNK  0x00000000
 #define BGP_SRC_PRIMITIVES_KEEP	0x00000001
 #define BGP_SRC_PRIMITIVES_MAP	0x00000002
 #define BGP_SRC_PRIMITIVES_BGP	0x00000004
@@ -385,7 +416,9 @@
 #define PRINT_OUTPUT_CSV	0x00000002
 #define PRINT_OUTPUT_JSON	0x00000004
 #define PRINT_OUTPUT_EVENT	0x00000008
-#define PRINT_OUTPUT_AVRO  	0x00000010
+#define PRINT_OUTPUT_AVRO_BIN  	0x00000010
+#define PRINT_OUTPUT_AVRO_JSON	0x00000020
+#define PRINT_OUTPUT_CUSTOM	0x00000040
 
 #define DIRECTION_UNKNOWN	0x00000000
 #define DIRECTION_IN		0x00000001
@@ -407,12 +440,23 @@
 #define FUNC_TYPE_NULL			0
 #define FUNC_TYPE_BGP			1
 #define FUNC_TYPE_BMP			2
-#define FUNC_TYPE_SFLOW_COUNTER		3
-#define FUNC_TYPE_TELEMETRY		4
-#define FUNC_TYPE_MAX			5
+#define FUNC_TYPE_RPKI			3
+#define FUNC_TYPE_SFLOW_COUNTER		4
+#define FUNC_TYPE_TELEMETRY		5
+#define FUNC_TYPE_BGP_BLACKHOLE		6
+#define FUNC_TYPE_MAX			7
 
 #define PM_MSG_BIN_COPY			0
 #define PM_MSG_STR_COPY			1
+#define PM_MSG_STR_COPY_ZERO		2
+
+#define DYN_STR_UNKNOWN			0
+#define DYN_STR_KAFKA_TOPIC		1
+#define DYN_STR_KAFKA_PART		2
+#define DYN_STR_RABBITMQ_RK		3
+#define DYN_STR_MONGODB_TABLE		4
+#define DYN_STR_SQL_TABLE		5
+#define DYN_STR_PRINT_FILE		6
 
 typedef u_int32_t pm_class_t;
 typedef u_int64_t pm_id_t;
@@ -485,6 +529,7 @@ typedef u_int32_t pm_counter_t;
 #define NF9_FTYPE_TRAFFIC_IPV6		1
 #define NF9_FTYPE_IPV4                  1
 #define NF9_FTYPE_IPV6                  2
+#define NF9_FTYPE_DLFS			3
 #define NF9_FTYPE_VLAN                  5
 #define NF9_FTYPE_VLAN_IPV4             6
 #define NF9_FTYPE_VLAN_IPV6             7 
@@ -494,6 +539,7 @@ typedef u_int32_t pm_counter_t;
 #define NF9_FTYPE_VLAN_MPLS             15      
 #define NF9_FTYPE_VLAN_MPLS_IPV4        16
 #define NF9_FTYPE_VLAN_MPLS_IPV6        17
+#define NF9_FTYPE_TRAFFIC_MAX		99  /* temporary: re-coding needed */
 #define NF9_FTYPE_EVENT			100 /* temporary: re-coding needed */
 #define NF9_FTYPE_NAT_EVENT             100
 #define NF9_FTYPE_OPTION		200
