@@ -1,6 +1,6 @@
 /*  
     pmacct (Promiscuous mode IP Accounting package)
-    pmacct is Copyright (C) 2003-2017 by Paolo Lucente
+    pmacct is Copyright (C) 2003-2019 by Paolo Lucente
 */
 
 /*
@@ -30,9 +30,11 @@
 #include <pcap.h>
 #endif
 
+#include <inttypes.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <unistd.h>
+#include <stdbool.h>
 #include <string.h>
 #include <errno.h>
 #include <assert.h>
@@ -45,10 +47,6 @@
 #include <malloc.h>
 #endif
 
-#if defined (HAVE_ZLIB)
-#include <zlib.h>
-#endif
-
 #include <ctype.h>
 #include <fcntl.h>
 #include <sys/ioctl.h>
@@ -67,7 +65,12 @@
 #include <sys/resource.h>
 #include <dirent.h>
 #include <limits.h>
+#include <pwd.h>
+#include <grp.h>
+#include <dlfcn.h>
+#include <math.h>
 #include "pmsearch.h"
+#include "filters/bloom.h"
 
 #include <sys/mman.h>
 #if !defined (MAP_ANONYMOUS)
@@ -90,7 +93,12 @@
 #endif
 
 #if defined (WITH_NDPI)
+/* NDPI_LIB_COMPILATION definition appears to be new in 2.5 */
+#ifndef NDPI_LIB_COMPILATION
+#define NDPI_LIB_COMPILATION
+#endif
 #include <ndpi_main.h>
+#undef NDPI_LIB_COMPILATION
 #endif
 
 #if defined (WITH_ZMQ)
@@ -210,7 +218,7 @@ struct plugin_requests {
 };
 
 typedef struct {
-  char *val;
+  u_char *val;
   u_int16_t len;
 } pm_hash_key_t;
 
@@ -227,11 +235,16 @@ typedef struct {
 #include <avro.h>
 #endif
 
+#if (defined WITH_SERDES)
+#if (!defined WITH_AVRO)
+#error "--enable-serdes requires --enable-avro"
+#endif
+#endif
+
 #include "pmacct-defines.h"
 #include "network.h"
 #include "pretag.h"
 #include "cfg.h"
-#include "util.h"
 #include "xflow_status.h"
 #include "log.h"
 #include "once.h"
@@ -255,12 +268,37 @@ typedef struct {
 #endif
 
 /* structures */
+struct pcap_interface {
+  u_int32_t ifindex;
+  char ifname[IFNAMSIZ];
+  int direction;
+};
+
+struct pcap_interfaces {
+  struct pcap_interface *list;
+  int num;
+};
+
 struct pcap_device {
+  char str[IFNAMSIZ];
+  u_int32_t id;
   pcap_t *dev_desc;
   int link_type;
   int active;
   int errors; /* error count when reading from a savefile */
+  int fd;
   struct _devices_struct *data; 
+  struct pcap_interface *pcap_if;
+};
+
+struct pcap_devices {
+  struct pcap_device list[PCAP_MAX_INTERFACES];
+  int num;
+};
+
+struct pcap_callback_signals {
+  int is_set;
+  sigset_t set;
 };
 
 struct pcap_callback_data {
@@ -273,6 +311,8 @@ struct pcap_callback_data {
   struct pcap_device *device;
   u_int32_t ifindex_in;
   u_int32_t ifindex_out;
+  u_int8_t has_tun_prims;
+  struct pcap_callback_signals sig;
 };
 
 struct _protocols_struct {
@@ -321,62 +361,55 @@ struct child_ctl2 {
 	x.end = x.base+sizeof(x.base); \
 	x.ptr = x.base;
 
+#include "util.h"
+
 /* prototypes */
 void startup_handle_falling_child();
 void handle_falling_child();
 void ignore_falling_child();
-void my_sigint_handler();
+void PM_sigint_handler();
 void reload();
 void push_stats();
 void reload_maps();
+extern void pm_pcap_device_initialize(struct pcap_devices *);
+extern void pm_pcap_device_copy_all(struct pcap_devices *, struct pcap_devices *);
+extern void pm_pcap_device_copy_entry(struct pcap_devices *, struct pcap_devices *, int);
+extern int pm_pcap_device_getindex_byifname(struct pcap_devices *, char *);
+extern pcap_t *pm_pcap_open(const char *, int, int, int, int, int, char *);
+extern void pm_pcap_add_filter(struct pcap_device *);
+extern int pm_pcap_add_interface(struct pcap_device *, char *, struct pcap_interface *, int);
 
-#if (!defined __LL_C)
-#define EXT extern
-#else
-#define EXT
-#endif
-EXT char sll_mac[2][ETH_ADDR_LEN];
 
-EXT void null_handler(const struct pcap_pkthdr *, register struct packet_ptrs *);
-EXT void eth_handler(const struct pcap_pkthdr *, register struct packet_ptrs *);
-EXT void fddi_handler(const struct pcap_pkthdr *, register struct packet_ptrs *);
-EXT void tr_handler(const struct pcap_pkthdr *, register struct packet_ptrs *);
-EXT u_int16_t mpls_handler(u_char *, u_int16_t *, u_int16_t *, register struct packet_ptrs *);
-EXT void ppp_handler(const struct pcap_pkthdr *, register struct packet_ptrs *);
-EXT void ieee_802_11_handler(const struct pcap_pkthdr *, register struct packet_ptrs *);
-EXT void sll_handler(const struct pcap_pkthdr *, register struct packet_ptrs *);
-EXT void raw_handler(const struct pcap_pkthdr *, register struct packet_ptrs *);
-EXT u_char *llc_handler(const struct pcap_pkthdr *, u_int, register u_char *, register struct packet_ptrs *);
-EXT void chdlc_handler(const struct pcap_pkthdr *, register struct packet_ptrs *);
-#undef EXT
+extern void null_handler(const struct pcap_pkthdr *, register struct packet_ptrs *);
+extern void eth_handler(const struct pcap_pkthdr *, register struct packet_ptrs *);
+extern void fddi_handler(const struct pcap_pkthdr *, register struct packet_ptrs *);
+extern void tr_handler(const struct pcap_pkthdr *, register struct packet_ptrs *);
+extern u_int16_t mpls_handler(u_char *, u_int16_t *, u_int16_t *, register struct packet_ptrs *);
+extern void ppp_handler(const struct pcap_pkthdr *, register struct packet_ptrs *);
+extern void ieee_802_11_handler(const struct pcap_pkthdr *, register struct packet_ptrs *);
+extern void sll_handler(const struct pcap_pkthdr *, register struct packet_ptrs *);
+extern void raw_handler(const struct pcap_pkthdr *, register struct packet_ptrs *);
+extern u_char *llc_handler(const struct pcap_pkthdr *, u_int, register u_char *, register struct packet_ptrs *);
+extern void chdlc_handler(const struct pcap_pkthdr *, register struct packet_ptrs *);
 
-#if (!defined __NL_C)
-#define EXT extern
-#else
-#define EXT
-#endif
-EXT int ip_handler(register struct packet_ptrs *);
-EXT int ip6_handler(register struct packet_ptrs *);
-EXT int gtp_tunnel_func(register struct packet_ptrs *);
-EXT int gtp_tunnel_configurator(struct tunnel_handler *, char *);
-EXT void tunnel_registry_init();
-EXT void pcap_cb(u_char *, const struct pcap_pkthdr *, const u_char *);
-EXT int PM_find_id(struct id_table *, struct packet_ptrs *, pm_id_t *, pm_id_t *);
-EXT void compute_once();
-EXT void set_index_pkt_ptrs(struct packet_ptrs *);
-EXT ssize_t recvfrom_savefile(struct pcap_device *, void **, struct sockaddr *, struct timeval **);
-#undef EXT
+extern int ip_handler(register struct packet_ptrs *);
+extern int ip6_handler(register struct packet_ptrs *);
+extern int gtp_tunnel_func(register struct packet_ptrs *);
+extern int gtp_tunnel_configurator(struct tunnel_handler *, char *);
+extern void tunnel_registry_init();
+extern void pcap_cb(u_char *, const struct pcap_pkthdr *, const u_char *);
+extern int PM_find_id(struct id_table *, struct packet_ptrs *, pm_id_t *, pm_id_t *);
+extern void PM_print_stats(time_t);
+extern void compute_once();
+extern void reset_index_pkt_ptrs(struct packet_ptrs *);
+extern void set_index_pkt_ptrs(struct packet_ptrs *);
+extern ssize_t recvfrom_savefile(struct pcap_device *, void **, struct sockaddr *, struct timeval **, int *, struct packet_ptrs *);
+extern ssize_t recvfrom_rawip(unsigned char *, size_t, struct sockaddr *, struct packet_ptrs *);
 
 #ifndef HAVE_STRLCPY
 size_t strlcpy(char *, const char *, size_t);
 #endif
 
-#if (defined WITH_JANSSON)
-#if (!defined HAVE_JSON_OBJECT_UPDATE_MISSING)
-int json_object_update_missing(json_t *, json_t *);
-#endif
-#endif
-
 void
 #ifdef __STDC__
 pm_setproctitle(const char *fmt, ...);
@@ -389,24 +422,27 @@ void
 initsetproctitle(int, char**, char**);
 
 /* global variables */
-#if (!defined __PMACCTD_C) && (!defined __NFACCTD_C) && (!defined __SFACCTD_C) && (!defined __UACCTD_C) && (!defined __PMTELEMETRYD_C) && (!defined __PMBGPD_C) && (!defined __PMBMPD_C)
-#define EXT extern
-#else
-#define EXT
-#endif
-EXT struct host_addr mcast_groups[MAX_MCAST_GROUPS];
-EXT int reload_map, reload_map_exec_plugins, reload_geoipv2_file;
-EXT int reload_map_bgp_thread, reload_log_bgp_thread, reload_log_bmp_thread;
-EXT int reload_log_sf_cnt, reload_log_telemetry_thread;
-EXT int data_plugins, tee_plugins;
-EXT struct timeval reload_map_tstamp;
-EXT struct child_ctl2 dump_writers;
-EXT int debug;
-EXT struct configuration config; /* global configuration structure */
-EXT struct plugins_list_entry *plugins_list; /* linked list of each plugin configuration */
-EXT pid_t failed_plugins[MAX_N_PLUGINS]; /* plugins failed during startup phase */
-EXT u_char dummy_tlhdr[16];
-EXT pcap_t *glob_pcapt;
-EXT struct pcap_stat ps;
-#undef EXT
+extern char sll_mac[2][ETH_ADDR_LEN];
+extern struct host_addr mcast_groups[MAX_MCAST_GROUPS];
+extern int reload_map, reload_map_exec_plugins, reload_geoipv2_file;
+extern int reload_map_bgp_thread, reload_log, reload_log_bgp_thread;
+extern int reload_map_bmp_thread, reload_log_bmp_thread;
+extern int reload_map_rpki_thread, reload_log_rpki_thread;
+extern int reload_map_telemetry_thread, reload_log_telemetry_thread;
+extern int reload_map_pmacctd;
+extern int print_stats;
+extern int reload_log_sf_cnt;
+extern int data_plugins, tee_plugins;
+extern struct timeval reload_map_tstamp;
+extern struct child_ctl2 dump_writers;
+extern int debug;
+extern struct configuration config; /* global configuration structure */
+extern struct plugins_list_entry *plugins_list; /* linked list of each plugin configuration */
+extern pid_t failed_plugins[MAX_N_PLUGINS]; /* plugins failed during startup phase */
+extern u_char dummy_tlhdr[16];
+extern struct pcap_device device;
+extern struct pcap_devices devices, bkp_devices;
+extern struct pcap_interfaces pcap_if_map, bkp_pcap_if_map;
+extern struct pcap_stat ps;
+extern struct sigaction sighandler_action;
 #endif /* _PMACCT_H_ */
