@@ -1,6 +1,6 @@
 /*
     pmacct (Promiscuous mode IP Accounting package)
-    pmacct is Copyright (C) 2003-2016 by Paolo Lucente
+    pmacct is Copyright (C) 2003-2019 by Paolo Lucente
 */
 
 /*
@@ -19,12 +19,17 @@
     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 
-#define __XFLOW_STATUS_C
-
 /* includes */
 #include "pmacct.h"
 #include "addr.h"
 
+/* Global variables */
+struct xflow_status_entry *xflow_status_table[XFLOW_STATUS_TABLE_SZ];
+u_int32_t xflow_status_table_entries;
+u_int8_t xflow_status_table_error;
+u_int32_t xflow_tot_bad_datagrams;
+u_int8_t smp_entry_status_table_memerr, class_entry_status_table_memerr;
+
 /* functions */
 u_int32_t hash_status_table(u_int32_t data, struct sockaddr *sa, u_int32_t size)
 {
@@ -32,7 +37,6 @@ u_int32_t hash_status_table(u_int32_t data, struct sockaddr *sa, u_int32_t size)
 
   if (sa->sa_family == AF_INET)
     hash = (data ^ ((struct sockaddr_in *)sa)->sin_addr.s_addr) % size;
-#if defined ENABLE_IPV6
   else if (sa->sa_family == AF_INET6) {
     u_int32_t tmp;
 
@@ -40,7 +44,6 @@ u_int32_t hash_status_table(u_int32_t data, struct sockaddr *sa, u_int32_t size)
     hash = (data ^ tmp) % size;
     // hash = (data ^ ((struct sockaddr_in6 *)sa)->sin6_addr.s6_addr32[3]) % size;
   }
-#endif
 
   return hash;
 }
@@ -89,19 +92,18 @@ struct xflow_status_entry *search_status_table(struct sockaddr *sa, u_int32_t au
   return entry;
 }
 
-void update_status_table(struct xflow_status_entry *entry, u_int32_t seqno)
+void update_status_table(struct xflow_status_entry *entry, u_int32_t seqno, int bytes)
 {
   if (!entry) return;
 
+  entry->counters.total++;
+  entry->counters.bytes += bytes;
+
   if (!entry->seqno || config.nfacctd_disable_checks) {
-    // entry->seqno = seqno; /* Init */
     entry->counters.good++;
   }
   else {
-    if (seqno == entry->seqno+entry->inc) {
-      // entry->seqno = seqno;
-      entry->counters.good++;
-    }
+    if (seqno == entry->seqno + entry->inc) entry->counters.good++;
     else {
       char agent_ip_address[INET6_ADDRSTRLEN];
       char collector_ip_address[INET6_ADDRSTRLEN];
@@ -116,16 +118,8 @@ void update_status_table(struct xflow_status_entry *entry, u_int32_t seqno)
       Log(LOG_INFO, "INFO ( %s/%s ): expecting flow '%u' but received '%u' collector=%s:%u agent=%s:%u\n",
 		config.name, config.type, entry->seqno+entry->inc, seqno, collector_ip_address,
 		config.nfacctd_port, agent_ip_address, entry->aux1);
-      if (seqno > entry->seqno+entry->inc) {
-        // entry->counters.missed += (seqno-entry->seqno);
-        entry->counters.jumps_f++;
-	// entry->seqno = seqno;
-      }
-      else {
-	entry->counters.jumps_b++;
-	// entry->seqno--;
-	// entry->seqno = seqno;
-      }
+      if (seqno > entry->seqno+entry->inc) entry->counters.jumps_f++;
+      else entry->counters.jumps_b++;
     }
   }
 
@@ -135,38 +129,27 @@ void update_status_table(struct xflow_status_entry *entry, u_int32_t seqno)
 void print_status_table(time_t now, int buckets)
 {
   struct xflow_status_entry *entry; 
-  char nf [] = "NetFlow";
-  char sf [] = "sFlow";
-  char uf [] = "unknown";
-  char *ftype = uf; 
   int idx;
   char agent_ip_address[INET6_ADDRSTRLEN];
   char collector_ip_address[INET6_ADDRSTRLEN];
   char null_ip_address[] = "0.0.0.0";
 
+  Log(LOG_NOTICE, "NOTICE ( %s/%s ): +++\n", config.name, config.type);
 
-  if (config.acct_type == ACCT_NF) ftype = nf; 
-  if (config.acct_type == ACCT_SF) ftype = sf; 
+  if (config.nfacctd_ip) memcpy(collector_ip_address, config.nfacctd_ip, MAX(strlen(config.nfacctd_ip), INET6_ADDRSTRLEN));
+  else strcpy(collector_ip_address, null_ip_address);
   
   for (idx = 0; idx < buckets; idx++) {
     entry = xflow_status_table[idx];
 
     bucket_cycle:
-    if (entry) {
+    if (entry && entry->counters.total && entry->counters.bytes) {
       addr_to_str(agent_ip_address, &entry->agent_addr);
-      if (config.nfacctd_ip)
-	memcpy(collector_ip_address, config.nfacctd_ip, MAX(strlen(config.nfacctd_ip), INET6_ADDRSTRLEN));
-      else
-        strcpy(collector_ip_address, null_ip_address);
 
-      Log(LOG_NOTICE, "NOTICE ( %s/%s ): +++\n", config.name, config.type);
-      Log(LOG_NOTICE, "NOTICE ( %s/%s ): %s statistics collector=%s:%u agent=%s:%u (%u):\n",
-		config.name, config.type, ftype, collector_ip_address, config.nfacctd_port,
-		agent_ip_address, entry->aux1, now);
-      Log(LOG_NOTICE, "NOTICE ( %s/%s ): Good datagrams:  %u\n", config.name, config.type, entry->counters.good);
-      Log(LOG_NOTICE, "NOTICE ( %s/%s ): Forward jumps:   %u\n", config.name, config.type, entry->counters.jumps_f);
-      Log(LOG_NOTICE, "NOTICE ( %s/%s ): Backward jumps:  %u\n", config.name, config.type, entry->counters.jumps_b);
-      Log(LOG_NOTICE, "NOTICE ( %s/%s ): ---\n", config.name, config.type);
+      Log(LOG_NOTICE, "NOTICE ( %s/%s ): stats [%s:%u] agent=%s:%u time=%ld packets=%" PRIu64 " bytes=%" PRIu64 " seq_good=%u seq_jmp_fwd=%u seq_jmp_bck=%u\n",
+		config.name, config.type, collector_ip_address, config.nfacctd_port,
+		agent_ip_address, entry->aux1, (long)now, entry->counters.total, entry->counters.bytes,
+		entry->counters.good, entry->counters.jumps_f, entry->counters.jumps_b);
 
       if (entry->next) {
 	entry = entry->next;
@@ -175,8 +158,10 @@ void print_status_table(time_t now, int buckets)
     } 
   }
 
-  Log(LOG_NOTICE, "NOTICE ( %s/%s ): +++\n", config.name, config.type);
-  Log(LOG_NOTICE, "NOTICE ( %s/%s ): Total bad %s datagrams: %u (%u)\n", config.name, config.type, ftype, xflow_tot_bad_datagrams, now);
+  Log(LOG_NOTICE, "NOTICE ( %s/%s ): stats [%s:%u] time=%ld discarded_packets=%u\n",
+		config.name, config.type, collector_ip_address, config.nfacctd_port,
+		(long)now, xflow_tot_bad_datagrams);
+
   Log(LOG_NOTICE, "NOTICE ( %s/%s ): ---\n", config.name, config.type);
 }
 
@@ -243,7 +228,7 @@ search_class_id_status_table(struct xflow_status_entry_class *centry, pm_class_t
   while (centry) {
     haystack = ntohl(centry->class_id);
 
-    if (centry->class_id == class_id) return centry;
+    if (haystack == needle) return centry;
     centry = centry->next;
   }
 
@@ -284,12 +269,11 @@ void set_vector_f_status(struct packet_ptrs_vector *pptrsv)
   pptrsv->vlan4.f_status = pptrsv->v4.f_status;
   pptrsv->mpls4.f_status = pptrsv->v4.f_status;
   pptrsv->vlanmpls4.f_status = pptrsv->v4.f_status;
-#if defined ENABLE_IPV6
+
   pptrsv->v6.f_status = pptrsv->v4.f_status;
   pptrsv->vlan6.f_status = pptrsv->v4.f_status;
   pptrsv->vlanmpls6.f_status = pptrsv->v4.f_status;
   pptrsv->mpls6.f_status = pptrsv->v4.f_status;
-#endif
 }
 
 void set_vector_f_status_g(struct packet_ptrs_vector *pptrsv)
@@ -297,10 +281,9 @@ void set_vector_f_status_g(struct packet_ptrs_vector *pptrsv)
   pptrsv->vlan4.f_status_g = pptrsv->v4.f_status_g;
   pptrsv->mpls4.f_status_g = pptrsv->v4.f_status_g;
   pptrsv->vlanmpls4.f_status_g = pptrsv->v4.f_status_g;
-#if defined ENABLE_IPV6
+
   pptrsv->v6.f_status_g = pptrsv->v4.f_status_g;
   pptrsv->vlan6.f_status_g = pptrsv->v4.f_status_g;
   pptrsv->vlanmpls6.f_status_g = pptrsv->v4.f_status_g;
   pptrsv->mpls6.f_status_g = pptrsv->v4.f_status_g;
-#endif
 }
